library(codetools)

flowControlFunctions <- c("if","while","repeat","for",
	#rest is special
	"function"
	)
operatorFunctions <- c(
	"+", "-", "*", "/", "%", "^",
	">", ">=", "<", "<=", "==", "!=",
	"!", "&", "|",
	"~",
	"->", "<-",
	"$",
	":",
	"%%", "%*%", "%/%", "%in%", "%o%", "%x%",
	"(", "{",
	"[", "[[")
	
deparseTreeExp <- function(e){
	s <- NULL
	d <- deparse(e, width.cutoff = 400)
	if(length(d) > 1){
		for(x in d){
			s <- paste(s, x, "\n" ,sep="")
		}
		s <- substring(s, 1, nchar(s, type="chars")-1)
		return(s)
	}
	return(d)
}

parseToNodes <- function(text = NULL, file = ""){
	
	codes <- parse(text = text, file = file)
	
	lines <- list()
	
	for(i in 1:length(codes)){
		e <- codes[[i]]
		flist <- retrieveFunction(e)
		lines[[i]] <- flist
	}
	
	lines
}


retrieveFunction <- function(e){
	env <- new.env()
	functionCollect <- function(v, e){
		assign(v, e, envir = env)
	}
	env$line = deparseTreeExp(e)
	
	w <- functionWalker(functionCollect)
	w$top = TRUE
	
	walkCode(e, w)
	
	as.list(env)
}


functionHandler <- function(v, w){
	#if(w$top){
		switch(v, "="=,"<-"=assignHandler)
	#}
}

assignHandler <- function(e, w) {
	w$collect("symbol", deparseTreeExp(e[[2]]))
	w$top = FALSE
	walkCode(e[[3]], w)
}

functionCall <- function(e, w){
	w$collect("sentence", deparseTreeExp(e))
	fun <- deparseTreeExp(e[[1]])
	if(fun %in% flowControlFunctions){
		w$collect("control", TRUE)
	}else{
		if(fun %in% operatorFunctions){
			#do nothing
		}else{
			w$collect("func", fun)
			args <- as.list(e[-1])
			args <- lapply(args, function(x){x <- deparseTreeExp(x)})
			w$collect("args", args)
		}
	}
}

functionLeaf <- function(e, w){
	w$collect("sentence", deparseTreeExp(e))
}

functionWalker <- function(collect){
	w <- makeCodeWalker(handler = functionHandler, call = functionCall, leaf = functionLeaf)
	w$collect = collect
	w
}
