
###########################
#r -> java function
###########################
.rflow.pager <- function(file, header, title, delete.file) {
  invisible(.jnew("com/ef_prime/rflow/tool/SimplePager",as.character(file), as.character(header), as.character(title), as.logical(delete.file)))
}
#options(pager=.rflow.pager)

###########################
#unsupported function
###########################
#.unsupported <- function(){stop("Unsupported Function")}
#windows <- .unsupported
#X11 <- .unsupported

###########################
#Rflow function
###########################

#get function arguments
.getFormals <- function (fun)
{
	formals <- formals(fun)
	for(i in 1:length(formals))
	{
		formals[[i]] <- deparse(formals[[i]])
	}
	return(formals)
}

#get function's local variables
.findLocals <- function (fun, merge = FALSE) 
{
    vars <- new.env(TRUE)
    funs <- new.env(TRUE)
    enter <- function(type, v, e, w) if (type == "function") 
        assign(v, TRUE, funs)
    else assign(v, TRUE, vars)
    collectUsage(fun, enterLocal = enter)
    fnames <- ls(funs, all.names = TRUE)
    vnames <- ls(vars, all.names = TRUE)
    if (merge) 
        sort(unique(c(vnames, fnames)))
    else list(functions = fnames, variables = vnames)
}

#clear all objects of envir
.clearObjects <- function(envir = .GlobalEnv)
{
	rm(list = ls(envir = envir, all.names = TRUE), envir = envir)
}

#do expr in envir
.doInLocal <- function(expr, envir, envchar, ...) 
{
	#if(is.null(envir)) {
	#  envir <- new.env(..., parent=.GlobalEnv)
	#  eval(expr, envir=envir)
	#  return(envir)
	#}
	
	if(!exists(envchar)){
		env <- new.env(parent=.GlobalEnv)
		assign(envchar, env, envir = .GlobalEnv)
		eval(expr, envir)
	}
	else{
		eval(expr, envir)
	}
}

#command support
.supportCommand <- function(x, envir = NULL)
{
	ret <- NULL
	if((a <- regexpr('\\$$', x)) > 1){
		func <- substr(x, 1, a-1)
		if(exists(func, where=1))
			ret <- names(get(func, pos=1))
	}else if((a <- regexpr('\\$', x)) > 1){
		func <- substr(x, 1, a-1)
		if(exists(func, where=1)){
			v <- names(get(func, pos=1))
			y <- substr(x, a+1, nchar(x))
			pat <- paste('^', y, sep="")
			match <- grep(pat, v, ignore.case=TRUE)
			for(i in match){
				ret <- c(ret, v[i])
			}
		}
	}else{
		pat <- paste("^", x, ".*", sep="")
		if(is.null(envir)){
			n <- length(search())
			for(i in 1:n){
				namez <- grep(pat, ls(pos=i, all.names=TRUE), ignore.case=TRUE, value=TRUE)
				ret <- c(ret, namez)
			}
		}else{
			namez <- grep(pat, ls(envir=envir, all.names=TRUE), ignore.case=TRUE, value=TRUE)
			ret <- c(ret, namez)
		}
	}
	sort(ret)
}

#get objects filtered by FILTERFUN
#FILTERFUN must return TRUE/FALSE
.getObjectsFiltered <- function(envir = .GlobalEnv, FILTERFUN)
{
	a <- unlist(eapply(envir, FILTERFUN))
	if(!is.null(a)){
		a <- sort(names(which(a)))
	}
	a
}

#get help text filepath
.getHelpTextFile <- function(topic)
{
	h <- help(topic, verbose = FALSE, try.all.packages = FALSE, chmhelp = FALSE, htmlhelp = FALSE)
	file <- as.character(h)
	zfile <- zip.file.extract(file, "Rhelp.zip")
	if(!file.exists(zfile)){
		return(NULL)
	}
	attr(zfile, "delete") <- (zfile != file)
	zfile
}

#get length or rownum of obj
.getLength <- function(obj)
{
	tip <- dim(obj)
	if(is.null(tip)){
		tip <- length(obj)
		#if(len == 1){
		#	tip <- NULL
		#}else{
		#	tip <- len
		#}
	}else{
		tip <- tip[1]
	}
	
	return(tip)
}

#get objects informations
#all.names: see ls
#limit: maximum of obj infos to be returned
.getObjects <- function(parent = .GlobalEnv, all.names = FALSE, limit = 200)
{
	objects <- list()
	#attr name
	overlimit <- "overlimit"
	
	getObjInfo <- function(objs, name, obj){
		objs[[name]] <- list()
		objs[[name]]$type <- typeof(obj)
		objs[[name]]$class <- class(obj)
		tip <- dim(obj)
		if(is.null(tip)){
			len <- length(obj)
			if(len == 1){
				tip <- substring(deparse(obj, width.cutoff = 100), 1, 100)
			}else{
				tip <- paste("[", as.character(len), "]", sep="")
			}
		}else{
			tip <- paste("[", as.character(tip[1]), ",", as.character(tip[2]), "]", sep="")
		}
		objs[[name]]$tip <- tip
		objs[[name]]$validname <- ifelse(is.numeric(name), TRUE, (make.names(name)==name))
		objs
	}
	
	if(typeof(parent) == "environment"){
		objs <- ls(parent, all.names = all.names)
		if(length(objs) > 0){
			len <- ifelse(limit > 0 && length(objs) > limit, limit, length(objs))
			for(i in 1:len){
				objects <- getObjInfo(objects, objs[i], parent[[objs[i]]])
			}
			if(length(objs) > limit){
				attr(objects, overlimit) <- length(objs)
			}
		}
	}else if(typeof(parent) == "list" || typeof(parent) == "pairlist"){
		namez <- names(parent)
		if(is.null(namez)){
			#no names
			if(length(parent) > 0){
				len <- ifelse(limit > 0 && length(parent) > limit, limit, length(parent))
				for(i in 1:len){
					objects <- getObjInfo(objects, i, parent[[i]])
				}
				if(length(parent) > limit){
					attr(objects, overlimit) <- length(parent)
				}
			}
		}else{
			if(length(namez) > 0){
				len <- ifelse(limit > 0 && length(namez) > limit, limit, length(namez))
				for(i in 1:len){
					#empty name is index
					x <- ifelse(namez[i] == "", i, namez[i])
					objects <- getObjInfo(objects, x, parent[[x]])
				}
				if(length(namez) > limit){
					attr(objects, overlimit) <- length(namez)
				}
			}
		}
	}
	
	return(objects)
}

#variable for error check
.errorObject <- NULL

