package com.ef_prime.rflow;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.List;

import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;
import javax.swing.undo.UndoManager;

import org.rosuda.JRI.REXP;

import com.ef_prime.rflow.console.BusyListener;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.exception.ReadException;
import com.ef_prime.rflow.flow.FlowExecuter;
import com.ef_prime.rflow.flow.FlowSetting;
import com.ef_prime.rflow.flow.FlowUtil;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.GraphReader;
import com.ef_prime.rflow.flow.GraphWriter;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.FlowViewer.ViewerActionKey;
import com.ef_prime.rflow.flow.FlowViewer.ViewerActionManager;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.ElementWriter;
import com.ef_prime.rflow.io.FlowDataReader;
import com.ef_prime.rflow.io.FlowDataWriter;
import com.ef_prime.rflow.object.ObjectBrowser;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.AboutDialog;
import com.ef_prime.rflow.tool.DirectoryChooser;
import com.ef_prime.rflow.tool.FlowSettingDialog;
import com.ef_prime.rflow.tool.RflowFileFilterFactory;
import com.ef_prime.rflow.tool.RflowOptionPane;
import com.ef_prime.rflow.tool.SearchDialog;
import com.ef_prime.rflow.util.AbstractActionManager;
import com.ef_prime.rflow.util.ExtensionUtility;
import com.ef_prime.rflow.util.FileChooserUtility;
import com.ef_prime.rflow.util.PropertiesUtility;

public class ActionManager extends AbstractActionManager<ActionManager.ActionKey>
	implements ItemListener, ActiveViewerListener, BusyListener
{

	private final Rflow rflow;
	private ViewerActionManager viewerActionManager;
	
	private boolean notBusy = true;
	
	public static enum ActionKey{
		NEW,
		OPEN,
		SAVE,
		SAVE_AS,
		FLOW_OPTION,
		IMPORT_SOURCE,
		EXIT,
		UNDO,
		REDO,
		CUT,
		COPY,
		PASTE,
		FIND,
		COPY_SCRIPT,
		CONSOLE_WINDOW,
		//CODE_VIEWER,
		
		NEW_NODE,
		NEW_FREE_NODE,
		CREATE_SUBFLOW_NODE,
		CREATE_IF_NODE,
		CREATE_FOR_NODE,
		CREATE_WHILE_NODE,
		CREATE_REPEAT_NODE,
		CREATE_FUNCTION_NODE,
		
		EDIT_NODE,
		REMOVE_NODE,
		SELECT_ALL,
		RUN,
		RUN_CLEAR,
		RUN_SELECTED,
		STOP,
		//UPDATE_CACHE,
		//CLEAN_CACHE,
		CHANGE_DIRECTORY,
		PREFERENCES,
		MINIMIZE_WINDOWS,
		CLOSE_ALL_WINDOWS,
		HELP,
		ABOUT,
		TEST_ACTION;
		
		public static final String SELECTED_KEY = "selected_key"; //$NON-NLS-1$
		
	}
	
	ActionManager(final Rflow rFlow){
		this.rflow = rFlow;
		
		rflow.addActiveViewerListener(this);

		viewerActionManager = new ViewerActionManager(rflow.viewer);
		
		putAction(ActionKey.NEW, new NewAction());
		putAction(ActionKey.OPEN, new OpenAction());
		putAction(ActionKey.SAVE, new SaveAction());
		putAction(ActionKey.SAVE_AS, new SaveAsAction());
		putAction(ActionKey.FLOW_OPTION, new FlowOptionAction());
		putAction(ActionKey.IMPORT_SOURCE, new ImportSourceAction());
		putAction(ActionKey.EXIT, new ExitAction());
		putAction(ActionKey.UNDO, viewerActionManager.getAction(ViewerActionKey.UNDO));
		putAction(ActionKey.REDO, viewerActionManager.getAction(ViewerActionKey.REDO));
		putAction(ActionKey.CUT, viewerActionManager.getAction(ViewerActionKey.CUT));
		putAction(ActionKey.COPY, viewerActionManager.getAction(ViewerActionKey.COPY));
		putAction(ActionKey.PASTE, viewerActionManager.getAction(ViewerActionKey.PASTE));
		putAction(ActionKey.FIND, new FindAction());
		putAction(ActionKey.COPY_SCRIPT, viewerActionManager.getAction(ViewerActionKey.COPY_SCRIPT));
		
		putAction(ActionKey.NEW_NODE, viewerActionManager.getAction(ViewerActionKey.NEW_NODE));
		putAction(ActionKey.NEW_FREE_NODE, viewerActionManager.getAction(ViewerActionKey.NEW_FREE_NODE));
		putAction(ActionKey.CREATE_SUBFLOW_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_SUBFLOW_NODE));
		putAction(ActionKey.CREATE_FUNCTION_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_FUNCTION_NODE));
		putAction(ActionKey.CREATE_IF_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_IF_NODE));
		putAction(ActionKey.CREATE_FOR_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_FOR_NODE));
		putAction(ActionKey.CREATE_WHILE_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_WHILE_NODE));
		putAction(ActionKey.CREATE_REPEAT_NODE, viewerActionManager.getAction(ViewerActionKey.CREATE_REPEAT_NODE));
		
		putAction(ActionKey.EDIT_NODE, viewerActionManager.getAction(ViewerActionKey.EDIT_NODE));
		putAction(ActionKey.REMOVE_NODE, viewerActionManager.getAction(ViewerActionKey.REMOVE_NODE));
		putAction(ActionKey.SELECT_ALL, viewerActionManager.getAction(ViewerActionKey.SELECT_ALL));
		putAction(ActionKey.RUN, viewerActionManager.getAction(ViewerActionKey.RUN));
		putAction(ActionKey.RUN_CLEAR, viewerActionManager.getAction(ViewerActionKey.RUN_CLEAR));
		putAction(ActionKey.RUN_SELECTED, viewerActionManager.getAction(ViewerActionKey.RUN_SELECTED));
		putAction(ActionKey.STOP, new StopAction());
		//putAction(ActionKey.UPDATE_CACHE, viewerActionManager.getAction(ViewerActionKey.UPDATE_CACHE));
		//putAction(ActionKey.CLEAN_CACHE, viewerActionManager.getAction(ViewerActionKey.CLEAN_CACHE));
		putAction(ActionKey.CHANGE_DIRECTORY, new ChangeDirAction());
		putAction(ActionKey.PREFERENCES, new PreferencesAction());
		putAction(ActionKey.MINIMIZE_WINDOWS, viewerActionManager.getAction(ViewerActionKey.MINIMIZE_WINDOWS));
		putAction(ActionKey.CLOSE_ALL_WINDOWS, viewerActionManager.getAction(ViewerActionKey.CLOSE_ALL_WINDOWS));
		putAction(ActionKey.HELP, new HelpAction());
		putAction(ActionKey.ABOUT, new AboutAction());
		putAction(ActionKey.TEST_ACTION, new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				JDialog dialog = new JDialog(rflow.frame);
				dialog.getContentPane().add(new ObjectBrowser());
				dialog.pack();
				dialog.setVisible(true);
			}
		});
		
		
		/*
		putAction(ActionKey.CONSOLE_WINDOW, new AbstractAction("コンソール表示"){
			public void actionPerformed(ActionEvent e) {
				rflow.console.setVisible(!rflow.console.isVisible());
				//getAction(Actions.CONSOLE_WINDOW).putValue(Action.SELECTED_KEY, rflow.console.isVisible());
			}
		});
		getAction(ActionKey.CONSOLE_WINDOW).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0));
		getAction(ActionKey.CONSOLE_WINDOW).addPropertyChangeListener(new PropertyChangeListener(){
			public void propertyChange(PropertyChangeEvent evt) {
				if(evt.getPropertyName().equals(ActionKey.SELECTED_KEY)){
					if(((Boolean)evt.getNewValue()).booleanValue())
						getAction(ActionKey.CONSOLE_WINDOW).putValue(Action.SMALL_ICON, FileResource.ACTION_REC);
					else
						getAction(ActionKey.CONSOLE_WINDOW).putValue(Action.SMALL_ICON, null);
				}					
			}
		});
		*/

		/*putAction(ActionKey.CODE_VIEWER, new AbstractAction("コードビュー"){
			public void actionPerformed(ActionEvent e) {
				rflow.codeViewer.getDialog().setVisible(true);
			}
		});*/
		
		//初期不可
		getAction(ActionKey.UNDO).setEnabled(false);
		getAction(ActionKey.REDO).setEnabled(false);
		getAction(ActionKey.CUT).setEnabled(false);
		getAction(ActionKey.COPY).setEnabled(false);
		getAction(ActionKey.PASTE).setEnabled(false);
		getAction(ActionKey.REMOVE_NODE).setEnabled(false);
		getAction(ActionKey.COPY_SCRIPT).setEnabled(false);
		getAction(ActionKey.EDIT_NODE).setEnabled(false);
		getAction(ActionKey.RUN).setEnabled(false);
		getAction(ActionKey.RUN_CLEAR).setEnabled(false);
		getAction(ActionKey.RUN_SELECTED).setEnabled(false);
	}
	
	
	JMenuBar createMenuBar(){
		JMenuBar menuBar = new JMenuBar();
		
		JMenu file = new JMenu(Messages.getString("Menu.file")); //$NON-NLS-1$
		file.add(getAction(ActionKey.NEW));
		file.add(getAction(ActionKey.OPEN));
		file.addSeparator();
		file.add(getAction(ActionKey.SAVE));
		file.add(getAction(ActionKey.SAVE_AS));
		file.addSeparator();
		file.add(getAction(ActionKey.IMPORT_SOURCE));
		file.addSeparator();
		file.add(getAction(ActionKey.EXIT));

		JMenu edit = new JMenu(Messages.getString("Menu.edit")); //$NON-NLS-1$
		edit.add(getAction(ActionKey.UNDO));
		edit.add(getAction(ActionKey.REDO));
		edit.addSeparator();
		edit.add(getAction(ActionKey.CUT));
		edit.add(getAction(ActionKey.COPY));
		edit.add(getAction(ActionKey.PASTE));
		edit.addSeparator();
		edit.add(getAction(ActionKey.REMOVE_NODE));
		edit.add(getAction(ActionKey.SELECT_ALL));
		edit.addSeparator();
		edit.add(getAction(ActionKey.FIND));
		edit.addSeparator();
		edit.add(getAction(ActionKey.COPY_SCRIPT));
		
//		JMenu display = new JMenu("表示");
//		display.add(getAction(Actions.CONSOLE_WINDOW));

		JMenu node = new JMenu(Messages.getString("Menu.node")); //$NON-NLS-1$
		node.add(getAction(ActionKey.EDIT_NODE));
		node.addSeparator();
		node.add(getAction(ActionKey.NEW_NODE));
		node.add(getAction(ActionKey.NEW_FREE_NODE));
		node.addSeparator();
		node.add(getAction(ActionKey.CREATE_SUBFLOW_NODE));
		JMenu special = new JMenu(Messages.getString("Menu.create_special_box")); //$NON-NLS-1$
		special.add(getAction(ActionKey.CREATE_FUNCTION_NODE));
		special.add(getAction(ActionKey.CREATE_IF_NODE));
		special.add(getAction(ActionKey.CREATE_FOR_NODE));
		special.add(getAction(ActionKey.CREATE_WHILE_NODE));
		special.add(getAction(ActionKey.CREATE_REPEAT_NODE));
		node.add(special);
		
		JMenu exec = new JMenu(Messages.getString("Menu.run")); //$NON-NLS-1$
		exec.add(getAction(ActionKey.RUN));
		exec.add(getAction(ActionKey.RUN_CLEAR));
		exec.add(getAction(ActionKey.RUN_SELECTED));
		exec.add(getAction(ActionKey.STOP));
		
		//JMenu tool = new JMenu(Messages.getString("Menu.tool")); //$NON-NLS-1$
		//JMenu cache = new JMenu(Messages.getString("Menu.cache")); //$NON-NLS-1$
		//cache.add(getAction(ActionKey.UPDATE_CACHE));
		//cache.add(getAction(ActionKey.CLEAN_CACHE));
		//tool.add(cache);

		JMenu option = new JMenu(Messages.getString("Menu.option")); //$NON-NLS-1$
		option.add(getAction(ActionKey.CHANGE_DIRECTORY));
		option.add(getAction(ActionKey.FLOW_OPTION));
		option.addSeparator();
		option.add(getAction(ActionKey.PREFERENCES));

		JMenu window = new JMenu(Messages.getString("Menu.window"));  //$NON-NLS-1$
		window.add(getAction(ActionKey.MINIMIZE_WINDOWS));
		window.add(getAction(ActionKey.CLOSE_ALL_WINDOWS));

		JMenu help = new JMenu(Messages.getString("Menu.help")); //$NON-NLS-1$
		help.add(getAction(ActionKey.HELP));
		help.add(getAction(ActionKey.ABOUT));
		//if(Rflow.TEST)help.add(getAction(ActionKey.TEST_ACTION));
		

		menuBar.add(file);
		menuBar.add(edit);
//		menuBar.add(display);
		menuBar.add(node);
		menuBar.add(exec);
		//menuBar.add(tool);
		menuBar.add(option);
		menuBar.add(window);
		menuBar.add(help);
		
		
		return menuBar;
	}
	
	JToolBar createToolBar(){
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);

		toolBar.add(getAction(ActionKey.NEW));
		toolBar.add(getAction(ActionKey.SAVE));
		toolBar.addSeparator();
		toolBar.add(getAction(ActionKey.RUN));
		toolBar.add(getAction(ActionKey.STOP));
		//toolBar.add(getAction(Actions.REC));//test
		
		return toolBar;
	}
	
	
	void setViewer(FlowViewer viewer){
		viewerActionManager.setViewer(viewer);
	}
	
	public void itemStateChanged(ItemEvent e) {
		updateActionState();
	}
	
	public void activeViewerChanged(FlowViewer activeViewer) {
		setViewer(activeViewer);
		updateActionState();
		updateUndoState();
	}
	
	public void busyStateChanged(int which) {
		notBusy = (which == BusyListener.NOT_BUSY);
		getAction(ActionKey.STOP).setEnabled(!notBusy);
		updateActionState();
	}
	
	public void updateActionState(){
		FlowViewer viewer = rflow.getActiveViewer();

		getAction(ActionKey.IMPORT_SOURCE).setEnabled(notBusy);
		getAction(ActionKey.CHANGE_DIRECTORY).setEnabled(notBusy);
		
		//TODO null pointer
		boolean isSelection = (viewer.getPickedState().getPickedVertices().size() > 0);
		getAction(ActionKey.CUT).setEnabled(isSelection);
		getAction(ActionKey.COPY).setEnabled(isSelection);
		getAction(ActionKey.REMOVE_NODE).setEnabled(isSelection);
		
		boolean singleSelection = isSelection && (viewer.getPickedState().getPickedVertices().size() == 1);
		getAction(ActionKey.COPY_SCRIPT).setEnabled(singleSelection && notBusy);
		getAction(ActionKey.EDIT_NODE).setEnabled(singleSelection);
		
		boolean canExec = false;
		boolean isCache = false;
		if(singleSelection){
			FlowNode node = (FlowNode)viewer.getPickedState().getPickedVertices().iterator().next();
			canExec = viewer.isExecutable(node);
			if(canExec){
				List<FlowNode> flow = FlowUtil.getStream(node, true, false);
				for(FlowNode fn :  flow)
					if(fn.isCacheData() && fn.getCacheData().isValid())
					{
						isCache = true;
						break;
					}
			}
		}
		getAction(ActionKey.RUN).setEnabled(canExec && notBusy);
		getAction(ActionKey.RUN_CLEAR).setEnabled(canExec && notBusy);
		
		getAction(ActionKey.RUN_SELECTED).setEnabled(notBusy && FlowExecuter.isSelectionExectuable(viewer.getPickedState().getPickedVertices()));
		
		String runClearName = Messages.getString("Action.run_clear"); //$NON-NLS-1$
		if(isCache)runClearName += Messages.getString("Action.cache_notice"); //$NON-NLS-1$
		getAction(ActionKey.RUN_CLEAR).putValue(Action.NAME, runClearName);
		
		String runName = Messages.getString("Action.run"); //$NON-NLS-1$
		if(isCache)runName += Messages.getString("Action.cache_notice"); //$NON-NLS-1$
		getAction(ActionKey.RUN).putValue(Action.NAME, runName);
	}

	public void updateUndoState() {
		UndoManager undoManager = rflow.getActiveViewer().getUndoManager();
		getAction(ActionKey.UNDO).setEnabled(undoManager.canUndo());
		getAction(ActionKey.REDO).setEnabled(undoManager.canRedo());

		getAction(ActionKey.UNDO).putValue(Action.NAME,	undoManager.getUndoPresentationName());
		getAction(ActionKey.REDO).putValue(Action.NAME,	undoManager.getRedoPresentationName());
	}
	
	/**
	 * .rflowファイルを開く
	 * @param file .rflowファイル
	 */
	public void open(File file, boolean clearObject){
		if(file == null)return;
		try {			
			//old file
			File oldFile = rflow.getWorkingFile();
			File oldFlowFolder = rflow.getFlowDir();
			
			FlowDataReader reader = new FlowDataReader();
			GraphReader graphReader = new GraphReader(rflow.frame, rflow.viewer.getSize());
			
			reader.read(file, new ElementReader[]{graphReader});
			
			//初期化
			rflow.init(clearObject, file, graphReader.getFlowSettings(),
					file.getParentFile(), graphReader.getViewLayout(), graphReader.getFlowState());

			//ファイルが変わった場合、古いファイルのフローフォルダが空なら消去
			if(oldFile != null && !oldFile.equals(file)){
				FlowFolderSupport.cleanFlowDir(rflow.fileControl, oldFlowFolder);
			}
		} catch (ReadException e1) {
			if(Rflow.DEBUG > 0)e1.printStackTrace();
			JOptionPane.showMessageDialog(rflow.frame, e1.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			return;
		} catch (IOException e1) {
			if(Rflow.DEBUG > 0)e1.printStackTrace();
			JOptionPane.showMessageDialog(rflow.frame, e1.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			return;
		} catch (Exception e1){
			if(Rflow.DEBUG > 0)e1.printStackTrace();
			JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.read"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
	}
	
	/**
	 * 上書き保存
	 * @return　false if failed or save canceled
	 */
	private boolean save(){
		File file = rflow.getWorkingFile();
		if(file == null)
			return save_as();
		else
			return save_internal(file);
	}
	
	/**
	 * 名前を付けて保存
	 * @return false if failed or save canceled
	 */
	private boolean save_as(){
		FileFilter[] filters = new FileFilter[]{RflowFileFilterFactory.createRflowFileFilter()};
		
		File file = FileChooserUtility.chooseSaveFile(Messages.getString("Message.save"), Prefs.WORKING_DIRECTORY, filters, rflow.frame); //$NON-NLS-1$
		return save_internal(file);
	}
	
	private boolean save_internal(File file){
		if(file == null)return false;
		
		rflow.backupThread.pause();
		try {
			boolean save_as = (rflow.getWorkingFile() == null) || !file.equals(rflow.getWorkingFile());
			boolean save_as_overwrite = save_as && file.exists();
			
			boolean copyAdditionalData = true;
			if(save_as && !rflow.isNewFlow() && FlowFolderSupport.isAdditionalData(rflow.viewer.getPickedState())){
				int ret = JOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.ask_copy_caches"), //$NON-NLS-1$
						Messages.getString("Message.confirm"), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE); //$NON-NLS-1$
				switch(ret){
				case JOptionPane.YES_OPTION:
					copyAdditionalData = true;
					break;
				case JOptionPane.NO_OPTION:
					copyAdditionalData = false;
					break;
				default://cancel
					return false;
				}
			}
			
			//get temp dir
			File temp = rflow.getTempDir();
			//名前をつけて保存の場合、新しいフローフォルダ名をフロー設定に設定する
			if(save_as)
				FlowFolderSupport.setNewFolder(rflow.getFlowSetting(), file, save_as_overwrite);
			//old file
			File oldFile = rflow.getWorkingFile();
			File oldFlowFolder = rflow.getFlowDir();
			
			//書き込み処理
			ElementWriter[] elementWriters = new ElementWriter[]{
					new GraphWriter(rflow.viewer.getViewLayout().fullCopy(), rflow.getFlowSetting().copy())
					};
			new FlowDataWriter().write(file, elementWriters);

			
			rflow.setWorkingFile(file);
			
			/*
			 * in .raf files
			 */
			//file name file
			//save cache
			if(copyAdditionalData)
				rflow.viewer.getCacheHandler().saveCaches(oldFile, file, temp);
			if(rflow.getFlowDir().exists()){
				//キャッシュの移動等でフォルダが作成されていた場合
				File fileNameFile = new File(rflow.getFlowDir(), Constants.NAME_FILE_PREFIX+ExtensionUtility.removeExtension(file.getName()));
				if(!fileNameFile.exists())
					fileNameFile.createNewFile();//ignore return value
			}
			
			if(!copyAdditionalData){
				//付随データをコピーしなかった場合
				//キャッシュをすべて無効にする
				rflow.viewer.getCacheHandler().clearAllCache();
			}
			
			
			//remove temp dir
			rflow.fileControl.delete_file(temp, false);//ignore failed
			
			
			//clean cache before changing workfile
			rflow.viewer.getCacheHandler().cleanCaches();
			
			//ファイルが変わった場合、古いファイルのフローフォルダが空なら消去
			if(oldFile != null && !oldFile.equals(file)){
				FlowFolderSupport.cleanFlowDir(rflow.fileControl, oldFlowFolder);
			}
			
			rflow.setWorkingDirectory(file.getParentFile());
			rflow.setDirty(false);
			//フォルダログの更新
			rflow.updateFolderlog();
			
			
			return true;
		} catch (IOException e1) {
			if(Rflow.DEBUG > 0)e1.printStackTrace();
			JOptionPane.showMessageDialog(rflow.frame, e1.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			return false;
		} catch (Exception e) {
			if(Rflow.DEBUG > 0)e.printStackTrace();
			JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.save"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		} finally {
			rflow.backupThread.restart();
		}
	}
	
	/**
	 * 保存の確認 および 計算の中断の確認
	 * @return false if canceled or save failed
	 */
	public boolean askSave(){
		if(Rflow.isBusy()){
			int ret = JOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.comfirm_interrupt"), Messages.getString("Message.comfirm"), JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			if(ret == JOptionPane.OK_OPTION)
				stop();
			else
				return false;
		}
		
		if(!rflow.isDirty())return true;
		
		int ret = RflowOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.confirm_save"), Messages.getString("Message.confirm"), JOptionPane.QUESTION_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
		
		switch(ret){
		case JOptionPane.YES_OPTION:
			return save();
		case JOptionPane.NO_OPTION:
			return true;
		default:
			return false;
		}
	}


	void changeDirectory() {
		File currentDir = Prefs.WORKING_DIRECTORY; //$NON-NLS-1$
		REXP wdEXP = RCommand.silentIdleEval("ifelse(Encoding(getwd())==\"UTF-8\", iconv(getwd(), \"UTF-8\", localeToCharset()), getwd())"); //$NON-NLS-1$
		
		if(wdEXP != null){
			currentDir = new File(Constants.replaceFileSepR2System(wdEXP.asString()));
		}
		
		File ret = DirectoryChooser.chooseDirectory(rflow.frame, Messages.getString("Action.change_directory"), currentDir); //$NON-NLS-1$
		
		if(ret != null){
			rflow.setWorkingDirectory(ret);
		}
	}
	
	public Command stop(){
		//if(!Rflow.isBusy())return null;
//		if(Platform.getPlatform() == Platform.MAC)return null;
		//TODO windows以外のOSでの動作の確認が必要
		rflow.R.rniStop(0);
		return rflow.console.cancel();
	}
	
	private class NewAction extends AbstractAction{
		NewAction(){
			putValue(Action.NAME, Messages.getString("Action.new")); //$NON-NLS-1$
			putValue(Action.SMALL_ICON, FileResource.ACTION_NEW);
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_N, Constants.MENU_KEY_MASK));
		}
		
		public void actionPerformed(ActionEvent e) {
			if(!askSave())
				return;
			rflow.init();
		}
	}
	
	private class OpenAction extends AbstractAction{
		OpenAction(){
			putValue(Action.NAME, Messages.getString("Action.open")); //$NON-NLS-1$
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, Constants.MENU_KEY_MASK));
		}
		
		public void actionPerformed(ActionEvent e) {
			if(!askSave())
				return;
			
			FileFilter[] filters = new FileFilter[]{RflowFileFilterFactory.createRflowFileFilter()};
			
			File file = FileChooserUtility.chooseOpenFile(Messages.getString("Action.open"), Prefs.WORKING_DIRECTORY, filters, rflow.frame); //$NON-NLS-1$
			if(file == null)return;
			
			open(file, true);
		}
	}
	
	private class SaveAction extends AbstractAction{
		SaveAction(){
			putValue(Action.NAME, Messages.getString("Action.save")); //$NON-NLS-1$
			putValue(Action.SMALL_ICON, FileResource.ACTION_SAVE);
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, Constants.MENU_KEY_MASK));
		}
		
		public void actionPerformed(ActionEvent e) {
			save();
		}
	}
	
	private class SaveAsAction extends AbstractAction{
		SaveAsAction(){
			putValue(Action.NAME, Messages.getString("Action.save_as")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			save_as();
		}
	}
	
	private class FlowOptionAction extends AbstractAction{
		FlowOptionAction(){
			putValue(Action.NAME, Messages.getString("Action.file_option")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			FlowSetting setting = FlowSettingDialog.showSettingDialog(rflow.frame, rflow.getFlowSetting());
			if(setting != null)
				rflow.setFlowSetting(setting);
		}
	}
	
	private class ImportSourceAction extends AbstractAction{
		ImportSourceAction(){
			putValue(Action.NAME, Messages.getString("Action.import.source")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			if(!askSave())
				return;
			
			ViewLayout layout = RflowUtil.importSource(rflow.frame);
			if(layout == null)return;
			
			rflow.init();
			rflow.viewer.addSubLayout(layout, null);
		}
	}
	
	private class ExitAction extends AbstractAction{
		ExitAction(){
			putValue(Action.NAME, Messages.getString("Action.exit")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			rflow.quit();
		}
	}
	
	private class FindAction extends AbstractAction{
		private SearchDialog dialog;
		
		FindAction(){
			putValue(Action.NAME, Messages.getString("Action.find")); //$NON-NLS-1$
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F, Constants.MENU_KEY_MASK));
		}
		
		public void actionPerformed(ActionEvent e) {
			if(dialog == null){
				dialog = new SearchDialog(rflow.frame, rflow);
				dialog.setLocationRelativeTo(rflow.frame);
			}
			dialog.setVisible(true);
		}
	}
	
	private class StopAction extends AbstractAction{
		StopAction(){
			putValue(Action.NAME, Messages.getString("Action.stop")); //$NON-NLS-1$
			putValue(Action.SMALL_ICON, FileResource.ACTION_STOP);
			putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0));
		}
		
		public void actionPerformed(ActionEvent e) {
			stop();
		}
	}
	
	
	private class ChangeDirAction extends AbstractAction{
		ChangeDirAction(){
			putValue(Action.NAME, Messages.getString("Action.change_directory")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			changeDirectory();
		}
	}
	
	private class PreferencesAction extends AbstractAction{
		PreferencesAction(){
			putValue(Action.NAME, Messages.getString("Action.preferences")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			Setting setting = SettingDialog.showSettingDialog(rflow.frame, Rflow.setting, false);
			if(setting != null){
				try {
					PropertiesUtility.store(setting, FileResource.SETTING_FILE);
				} catch (Exception e1) {
					JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.write_option"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
					return;
				}
				Rflow.setting.setPropertyAll(setting);
			}
		}
	}
	
	private class HelpAction extends AbstractAction{
		HelpAction(){
			putValue(Action.NAME, Messages.getString("Action.help")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
	  		//OSの確認
			switch(Platform.getPlatform()){
			case WINDOWS:
			case WINDOWS_VISTA:
		  		if(!FileResource.CHTML_HELP_FILE.exists()){
		  			JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.no_help")); //$NON-NLS-1$
		  			return;
		  		}
		  		try {
					Runtime.getRuntime().exec("hh.exe "+FileResource.CHTML_HELP_FILE.getAbsolutePath()); //$NON-NLS-1$
				} catch (IOException ex) {
		  			JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.run_help")); //$NON-NLS-1$
					ex.printStackTrace();
					return;
				}
				break;
			default://not windows
				//javahelp
				if(FileResource.JHELP_FILE == null || !FileResource.JHELP_FILE.exists()){
					JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.no_help")); //$NON-NLS-1$
					return;
				}
				try{
					URL url = FileResource.JHELP_FILE.toURI().toURL();
					HelpSet helpset = new HelpSet(ClassLoader.getSystemClassLoader(), url);

					HelpBroker broker = helpset.createHelpBroker();
					ActionListener helper = new CSH.DisplayHelpFromSource(broker);
					helper.actionPerformed(e);
				}catch (Exception e1) {
		  			JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.run_help")); //$NON-NLS-1$
		  			if(Rflow.DEBUG > 0)e1.printStackTrace();
					return;
				}
			}
		}
	}
	
	private class AboutAction extends AbstractAction{
		AboutAction(){
			putValue(Action.NAME, Messages.getString("Action.about")); //$NON-NLS-1$
		}
		
		public void actionPerformed(ActionEvent e) {
			AboutDialog dialog = new AboutDialog(rflow.frame);
			dialog.setLocationRelativeTo(rflow.frame);
			dialog.setVisible(true);
		}
	}
}
