package com.ef_prime.rflow;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import com.ef_prime.rflow.flow.GraphWriter;
import com.ef_prime.rflow.io.ElementWriter;
import com.ef_prime.rflow.io.FlowDataWriter;
import com.ef_prime.rflow.xml.XMLException;

public class BackupThread extends Thread implements PropertyChangeListener{
	
	private Rflow rflow;
	
	private boolean inProgress;
	private int pause;
	
	public static final String BACKUP_EXT = ".bak"; //$NON-NLS-1$

	private long WAIT = 1 * 60 * 1000;
	private boolean doBackup = true;
	
	public BackupThread(Rflow rflow){
		this.rflow = rflow;
		this.setDaemon(true);
		Rflow.setting.addPropertyChangeListener(Setting.Key.AUTO_BACKUP, this);
		Rflow.setting.addPropertyChangeListener(Setting.Key.AUTO_BACKUP_INTERVAL, this);
	}
	
	@Override
	public void run() {
		
		while(true){
			
			try {
				Thread.sleep(WAIT);
			} catch (InterruptedException e) {
			}
			
			if(doBackup && pause < 1 && rflow.isDirtySub()){
				inProgress = true;
				try{
					String name = rflow.isNewFlow()?"new":rflow.getWorkingFile().getName(); //$NON-NLS-1$
					File backupFile = new File(rflow.getTempDir(), name + BACKUP_EXT);
	
					if(!backupFile.getParentFile().exists() && !backupFile.getParentFile().mkdirs()){
						//failed create backup folder
						continue;
					}
	
					ElementWriter[] elementWriters = new ElementWriter[]{
							new GraphWriter(rflow.viewer.getViewLayout().fullCopy(), rflow.getFlowSetting().copy())
					};
					try {
						//if(Rflow.DEBUG > 0)
						//	System.out.println("backup!"); //$NON-NLS-1$
						new FlowDataWriter().write(backupFile, elementWriters);
						rflow.setDirtySub(false);
					} catch (IOException e) {
						if(Rflow.DEBUG > 0)
							e.printStackTrace();
					} catch (XMLException e) {
						if(Rflow.DEBUG > 0)
							e.printStackTrace();
					}
				}finally{
					inProgress = false;
				}
			}
		}
	}
	
	public void doBackup(){
		interrupt();
	}
	
	public void restart(){
		if(pause > 0)
			pause--;
	}
	
	public void pause(){
		pause++;
		while(inProgress){
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
			}
		}
	}
	
	public void propertyChange(PropertyChangeEvent evt) {
		try{
			Setting setting = (Setting)evt.getSource();
			if(Setting.Key.AUTO_BACKUP.getKeyString().equals(evt.getPropertyName())){
				doBackup = (Boolean)setting.getProperty(Setting.Key.AUTO_BACKUP);
			}else if(Setting.Key.AUTO_BACKUP_INTERVAL.getKeyString().equals(evt.getPropertyName())){
				Integer i = (Integer)setting.getProperty(Setting.Key.AUTO_BACKUP_INTERVAL);
				WAIT = i * 1000;
			}
		}catch(Exception e){}
	}
}
