package com.ef_prime.rflow;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;

import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.node.NodeProperty;
import com.ef_prime.rflow.node.NodePropertyChangeListener;
import com.ef_prime.rflow.tool.CodingTextPane;
import com.ef_prime.rflow.util.NoWrapEditorKit;
import com.ef_prime.rflow.util.SideBorder;

import edu.uci.ics.jung.visualization.PickedState;

public class CodeBar extends JPanel implements ItemListener, NodeModelChangeListener, NodePropertyChangeListener{
	
	private CodingTextPane codeField = new CodingTextPane();
	private FlowNode node;
	private FlowViewer viewer;
	private JLabel iconLabel = new JLabel();
	
	private static final Color background = UIManager.getColor("Panel.background"); //$NON-NLS-1$
	
	public CodeBar(FlowViewer viewer){
		super();
		this.viewer = viewer;
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		//int space = 3;
		//setBorder(new EmptyBorder(space,space,space,space));
		//setBorder(new EmptyBorder(0,0,0,0));
		//final Border focusBorder = new LineBorder(Color.GRAY, 1);
		final Border focusBorder = new SideBorder(SideBorder.SIDE_BOTTOM, Color.GRAY);
//		final Border noFocusBorder = new LineBorder(Color.LIGHT_GRAY, 1);
		final Border noFocusBorder = new SideBorder(SideBorder.SIDE_BOTTOM, Color.LIGHT_GRAY);
		setBorder(noFocusBorder);

		int height = 25;
		setPreferredSize(new Dimension(Integer.MAX_VALUE, height));
		setMaximumSize(new Dimension(Integer.MAX_VALUE, height));
		setMinimumSize(new Dimension(0, height));
		
		//codeField.setBorder(new LineBorder(Color.GRAY));
		iconLabel.setPreferredSize(new Dimension(25,25));
		//iconLabel.setBackground(Color.WHITE);
		iconLabel.setHorizontalAlignment(JLabel.CENTER);
		//iconLabel.setIcon(FileResource.ACTION_RUN);
		//iconLabel.setText("TEST");
		//iconLabel.setBounds(3, 0, 25, 25);
		add(iconLabel);
		
		
		//codeField.setBorder(null);
		codeField.setBorder(new EmptyBorder(4,3,3,3));
		codeField.setEditorKit(new NoWrapEditorKit());
		codeField.setIntend(false);
		codeField.setFont(Prefs.CODE_FONT_12);
		codeField.addFocusListener(new FocusListener(){
			public void focusGained(FocusEvent e) {
				CodeBar.this.setBorder(focusBorder);
			}
			
			public void focusLost(FocusEvent e) {
				CodeBar.this.setBorder(noFocusBorder);
				sendCode2Model();
			}
		});
		codeField.addKeyListener(new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
				if(e.isConsumed())return;
				switch(e.getKeyCode()){
				case KeyEvent.VK_ENTER:
					//sendCode2Model();
					CodeBar.this.requestFocus();
					e.consume();
					break;
				}
			}
		});
		
		JScrollPane sp = new JScrollPane(codeField);
		sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		sp.setBorder(null);
		add(sp);
	}
	
	public void clear(){
		try {
			codeField.getDocument().remove(0, codeField.getDocument().getLength());
		} catch (BadLocationException e) {}
		
		codeField.setEditable(false);
		setBackground(background);
		codeField.setBackground(background);
		iconLabel.setIcon(null);
		repaint();
	}
	
	public void setText(String text){
		if(text == null || text.length() < 1)
			codeField.setText(" "); //$NON-NLS-1$
		else
			codeField.setText(text);
		
		codeField.setCaretPosition(0);
		codeField.getUndoManager().discardAllEdits();
	}
	
	private void sendCode2Model(){
		NodeModel model = (node == null)?null:node.getModel();
		if(model != null && model.isEditable()){
			try {
				NodeModel newModel = model.copy();
				newModel.setCommand(codeField.getRawText());
				node.setModel(newModel);
				requestFocus();
			} catch (IncompatibleCommandException e1) {
				codeField.selectAll();
				codeField.requestFocus();
			}
		}
	}
	
	private void setModel(FlowNode node){
		this.node = node;
		NodeModel model = (node == null)?null:node.getModel();
		if(model != null){
			codeField.setEditable(model.isEditable());
			if(!model.isEditable()){
				setBackground(background);
				codeField.setBackground(background);
			}else{
				setBackground(Color.WHITE);
				codeField.setBackground(Color.WHITE);
			}
			String c = model.getCommand();
			setText((model.isEditable())?c:omitCommand(c));
			
			Icon icon = node.getSmallIcon();
			iconLabel.setIcon(icon);
			repaint();
		}else{
			clear();
		}
	} 
	
	private String omitCommand(String command){
		String omitted = command.replaceAll(Constants.NEWLINE_STR, "  "); //$NON-NLS-1$
		omitted = omitted.replaceAll("\t", ""); //$NON-NLS-1$ //$NON-NLS-2$
		if(omitted.length() > 50)
			omitted = omitted.substring(0, 50)+"..."; //$NON-NLS-1$
		return omitted;
	}

	/*-----------------------ItemStateChangeListener---------------------------*/
	public void itemStateChanged(ItemEvent e) {
		PickedState pickedState = (PickedState)e.getSource();
		if(pickedState.getPickedVertices().size() == 1){
			FlowNode node = (FlowNode)pickedState.getPickedVertices().iterator().next();
			if(viewer == null || viewer.getGraph().getVertices().contains(node)){
				setModel(node);
				return;
			}
		}

		setModel(null);
	}

	/*-------------------------NodeModelChangeListener------------------------------*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		if(node == CodeBar.this.node)
			setModel(node);
	}
	public void subflowNodeModelChanged(FlowNode node) {}

	/*-------------------------NodePropertyChangeListener------------------------------*/
	public void nodePropertyChanged(FlowNode node, NodeProperty old, NodeProperty property) {
		iconLabel.setIcon(node.getSmallIcon());
		repaint();
	}
}
