package com.ef_prime.rflow;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.Caret;
import javax.swing.text.DefaultCaret;
import javax.swing.text.JTextComponent;

import com.ef_prime.rflow.flow.FlowState;
import com.ef_prime.rflow.flow.FlowUtil;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.tool.CodingTextPane;
import com.ef_prime.rflow.util.NoWrapEditorKit;

import edu.uci.ics.jung.graph.event.GraphEvent;
import edu.uci.ics.jung.graph.event.GraphEventListener;

public class CodeViewer extends CodingTextPane implements ItemListener, GraphEventListener, NodeModelChangeListener
{

	public static final int VIEW_NODE = 0;
	public static final int VIEW_FLOW = 1;
	public static final int VIEW_FLOW_WITH_BRANCH = 2;
	public static final int VIEW_FLOW_TO_LAST = 3;

	private Rflow rflow;
	private JDialog dialog;

	private Map<FlowNode, Integer> lineCountMap = new HashMap<FlowNode, Integer>();
	private List<FlowNode> nodeList = new ArrayList<FlowNode>();
	private Integer[] lineCumsum = new Integer[0];
	private int currentLineNumber;

	private int viewState = VIEW_FLOW;

	public CodeViewer(final Rflow rflow){
		super();
		this.rflow = rflow;
		
		setEditorKit(new NoWrapEditorKit());
		
		addCaretListener(new CaretHandler());
	    setOpaque(false);
	    Caret caret = new DefaultCaret() {
	      protected synchronized void damage(Rectangle r) {
	        if(r!=null) {
	          JTextComponent c = getComponent();
	          x = 0; y = r.y;
	          width  = c.getSize().width;
	          height = r.height;
	          repaint();
	        }
	      }
	    };
	    caret.setBlinkRate(getCaret().getBlinkRate());
	    setCaret(caret);

		
		
		dialog = new JDialog(rflow.frame, "コード", false);
		dialog.addComponentListener(new ComponentAdapter(){
			@Override
			public void componentShown(ComponentEvent e) {
				update();
			}
			@Override
			public void componentHidden(ComponentEvent e) {
				clear();
				rflow.getActiveViewer().repaint();
			}
		});
		JScrollPane pane = new JScrollPane(this);
		pane.getViewport().setBackground(Color.WHITE);
		dialog.getContentPane().add(pane);
		dialog.pack();

	}

	public JDialog getDialog() {
		return dialog;
	}
	
	public boolean isEmpty(){
		return nodeList.isEmpty();
	}
	
	private void clear(){
		currentLineNumber = -1;
		nodeList.clear();
		rflow.getActiveViewer().getPickedState().setHighlightNode(null);
	}

	public void update(){
		if(!dialog.isVisible())return;
		
		clear();
		
		List<Integer> lineCumsumList = new ArrayList<Integer>();
		FlowState state = rflow.getActiveViewer().getPickedState(); 
		String command = "";
		switch(viewState){
		case VIEW_NODE:
			FlowNode node = state.getSelectedNode();
			if(node != null){
				command = node.getModel().getCommandNewline();
				nodeList.add(node);
				lineCumsumList.add(lineCountMap.get(node));
			}
			break;
		case VIEW_FLOW:
		case VIEW_FLOW_WITH_BRANCH:
			List<FlowNode> nodes = state.getSelectionNodeFlow();//DO NOT WORK
			if(!nodes.isEmpty()){
				command = FlowUtil.getCommandFromStream(nodes, true);
				//cum sum line num
				int line = -1;
				for(FlowNode flowNode : nodes){
					nodeList.add(flowNode);
					line += lineCountMap.get(flowNode);
					lineCumsumList.add(line);
				}
			}
			break;
		}

		lineCumsum = lineCumsumList.toArray(new Integer[0]);
		
		setText(command);
		
		setCaretPosition(getDocument().getLength());
	}
	
	public void flowModelChanged(){
		for(Object o : rflow.getActiveViewer().getGraph().getVertices())
			putLineCount((FlowNode)o);
	}

	void init(){
		setText("");
		getUndoManager().discardAllEdits();
	}

	public void setViewState(int state){
		if(viewState != state){
			viewState = state;
			update();
		}
	}
	
	private void putLineCount(FlowNode node){
		char[] command = node.getModel().getCommandNewline().toCharArray();
		//count lines
		int lineCount = 0;
		for(char c : command)
			if(c == Constants.NEWLINE)lineCount++;
		//count comment lines
		String comment = node.getProperty().getComment();
		if(comment != null){
			if(!comment.endsWith(Constants.NEWLINE_STR))
				comment += Constants.NEWLINE;
			for(char c : comment.toCharArray())
				if(c == Constants.NEWLINE)lineCount++;
		}
		lineCountMap.put(node, lineCount);
	}
	
	private class CaretHandler implements CaretListener{
		public void caretUpdate(CaretEvent e) {
			if(nodeList.isEmpty())return;
			
			int lineNum = getDocument().getDefaultRootElement().getElementIndex(e.getDot());
			if(currentLineNumber == lineNum)return;
			//System.out.println(lineNum);
			
			int i = 0;
			try {
				while (lineCumsum[i] < lineNum)
					i++;
			} catch (ArrayIndexOutOfBoundsException e1) {
				return;
			}
			if(i >= nodeList.size()){
				currentLineNumber = -1;
				return;
			}
			
			currentLineNumber = lineNum;
			//System.out.println(nodeList.get(i).getModel().getTitle());
			
			//描画用
			Graphics2D g2 = (Graphics2D)getGraphics();
			Insets inset = getInsets();
			int topLine = (i == 0)?0:(lineCumsum[i-1]+1);
			int lines = lineCumsum[i] - topLine + 1;
			int lineHeight = g2.getFontMetrics().getHeight();
			int h = lineHeight*lines;
			int y = topLine*lineHeight+inset.top;
			hilightRectangle.setRect(inset.left, y, getSize().width-inset.left-inset.right, h);
			
			((Component)e.getSource()).repaint();
			rflow.getActiveViewer().getPickedState().setHighlightNode(nodeList.get(i));
			rflow.getActiveViewer().repaint();
		}
	}

	/*--------------------------ItemListener----------------------------*/
	/*
	 * ノードの選択状態の変化
	 */
	public void itemStateChanged(ItemEvent e) {
		update();
	}

	/*--------------------------GraphEventListener----------------------------*/
	public void edgeAdded(GraphEvent event) {
	}

	public void edgeRemoved(GraphEvent event) {
	}

	public void vertexAdded(GraphEvent event) {
		putLineCount((FlowNode)event.getGraphElement());
		update();
	}

	public void vertexRemoved(GraphEvent event) {
		lineCountMap.remove((FlowNode)event.getGraphElement());
	}

	/*--------------------------NodeModelChangeListener----------------------------*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		putLineCount(node);
		update();
	}
	public void subflowNodeModelChanged(FlowNode node) {}
	
	private final Color targetNodeColor = new Color(232,242,254);
	private Rectangle hilightRectangle = new Rectangle();
	protected void paintComponent(Graphics g) {
		if(!isEmpty() && currentLineNumber >= 0){
			Graphics2D g2 = (Graphics2D)g;
			g2.setPaint(targetNodeColor);
			g2.fillRect(hilightRectangle.x, hilightRectangle.y, hilightRectangle.width, hilightRectangle.height);
		}
		super.paintComponent(g);
	}
	
}
