package com.ef_prime.rflow;

import java.awt.Toolkit;
import java.io.File;
import java.util.Locale;

import com.ef_prime.rflow.util.ExtensionUtility;

public class Constants {
	
	public static final String PROGRAM_DIR = new File("").getAbsolutePath(); //$NON-NLS-1$
	public static final String PROGRAM_DIR_SLASH = replaceFileSeparator(PROGRAM_DIR)+'/';
	
	public static final String RFLOW_FILE_EXTENSION = ".rflow"; //$NON-NLS-1$
	public static final String RFILE_EXTENSION = ".R"; //$NON-NLS-1$
	public static final String RDATA_EXTENSION = ".RData"; //$NON-NLS-1$
	
	public static final String RFLOW_FOLDER = ".raf"; //$NON-NLS-1$

	public static final String CACHE_DIR = "cache"; //$NON-NLS-1$
	public static final String FILE_TEMP_DIR = "tmp"; //$NON-NLS-1$
	public static final File TEMP_DIR = new File(Platform.getUserDirWithVersion(), "temp"); //$NON-NLS-1$
	
	public static final String NAME_FILE_PREFIX = "FILE_"; //$NON-NLS-1$
	
	/**
	 * Rflow用の環境の名前
	 */
	public static final String RFLOW_ENV = ".RflowEnv"; //$NON-NLS-1$
	
	/**
	 * Ｒ用の改行文字
	 */
	public static final char NEWLINE = '\n';
	public static final String NEWLINE_STR = "\n"; //$NON-NLS-1$
	
	/** //TODO コメント文字は変更できるのでＲから取得するようにする */
	public static final char COMMENT_CHAR = '#';
	
	/** R用のlogical TRUE*/
	public static final String TRUE = "TRUE"; //$NON-NLS-1$
	/** R用のlogical FALSE*/
	public static final String FALSE = "FALSE"; //$NON-NLS-1$
	
	/** 数字始まりの変数の頭につける文字 */
	public static final String VAR_PREFIX = "X"; //$NON-NLS-1$
	
	/**
	 * Ｒ用のファイルパスの区切り文字
	 */
	public static final char FILE_SEPARATOR_CHAR = '/';
	public static final String FILE_SEPARATOR = "/"; //$NON-NLS-1$
	
	public static final String QUOTE = "\""; //$NON-NLS-1$
	
	/** 内部処理用の文字列区切り文字 */
	public static final String DELIM = "\b"; //$NON-NLS-1$
	
	public static final int MENU_KEY_MASK = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
	
	/**
	 * 両端を"で囲み、"をエスケープします
	 * @param s
	 * @return
	 */
	public static String quote(String s){
		s = s.replaceAll("\\\\", "\\\\\\\\"); //$NON-NLS-1$ //$NON-NLS-2$
		s = s.replaceAll("\"", "\\\\\""); //$NON-NLS-1$ //$NON-NLS-2$
		return QUOTE + s + QUOTE;
	}
	
	public static String unquote(String s){
		if(s.startsWith(QUOTE) && s.endsWith(QUOTE) && s.length() > 1){
			if(s.length() == 2)
				return ""; //$NON-NLS-1$
			s = s.substring(1, s.length()-1);
			s = s.replaceAll("\\\\\"", "\""); //$NON-NLS-1$ //$NON-NLS-2$
			return s.replaceAll("\\\\\\\\", "\\\\"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return s;
	}
	
	/**
	 * Ｒ用のlogicalの文字列をbooleanに変換します
	 * @param s
	 * @return
	 */
	public static boolean toBoolean(String s){
		if(s.equals("TRUE") || s.equals("T")) //$NON-NLS-1$ //$NON-NLS-2$
			return true;
		if(s.equals("FALSE") || s.equals("F")) //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		throw new IllegalArgumentException("can not parse to boolean"); //$NON-NLS-1$
	}
	
	/**
	 * booleanからＲ用のlogicalの文字列に変換します
	 * @param b
	 * @return
	 */
	public static String toRBoolean(boolean b){
		return (b)?"TRUE":"FALSE"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public static String combine(String[] objects){
		String ret = "c("; //$NON-NLS-1$
		for(String o : objects){
			ret += o+", "; //$NON-NLS-1$
		}
		ret = ret.substring(0, ret.length()-2);
		ret += ")"; //$NON-NLS-1$
		return ret;
	}
	
	/** Rで使用できる変数名を取得します */
	public static String getVarName(String var){
		//数字で始まる場合頭に文字を加える
		if(var.matches("^\\d.*")){ //$NON-NLS-1$
			var = VAR_PREFIX + var;
		}
		var = var.replace(' ', '_');
		//System.out.println(var);
		return var;
	}
	
	/**
	 * ロケールの言語名がついたファイルを返します。
	 * もし、ロケールの言語名のついたファイルが存在しなかった場合は、
	 * 元のファイルを返します。
	 * help.chm -> help_ja.chm
	 * @param file
	 * @return
	 */
	public static File getLocaleFile(File file){
		String name = ExtensionUtility.removeExtension(file.getName());
		String ext = ExtensionUtility.getExtension(file.getName());
		
		//String lang = Locale.getDefault().getLanguage();
		String lang = Locale.getDefault().getLanguage();
		File localFile = new File(file.getParent(), name+'_'+lang+'.'+ext);
		
		return (localFile.exists())?localFile:file;
	}
	
	/**
	 * ネームファイルを見つけて、名前を抜き出して返します。
	 * @param flowDir
	 * @return null if no name file found
	 */
	public static String getFileName(File flowDir){
		if(flowDir == null || !flowDir.exists())
			return null;
		for(File f : flowDir.listFiles()){
			if(f.getName().startsWith(NAME_FILE_PREFIX)){
				return f.getName().substring(NAME_FILE_PREFIX.length());
			}
		}
		return null;
	}
	
	/**
	 * プログラムからの相対パスを絶対パスに変換する
	 * @param relativePath
	 * @return
	 */
	public static String getPath(String relativePath){
		return PROGRAM_DIR_SLASH+relativePath;
	}
	
	public static File getUserFile(String filename){
		return new File(Platform.getUserDirWithVersion(), filename);
	}
	
	/**
	 * システムの区切り文字をR用の区切り文字に変換する
	 * @param s
	 * @return
	 */
	public static String replaceFileSeparator(String s){
		return s.replace(File.separatorChar, Constants.FILE_SEPARATOR_CHAR);
	}
	
	/**
	 * R用の区切り文字をシステムの区切り文字に変換する
	 * @param s
	 * @return
	 */
	public static String replaceFileSepR2System(String s){
		return s.replace(Constants.FILE_SEPARATOR_CHAR, File.separatorChar);
	}
}
