package com.ef_prime.rflow;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.Locale;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import com.ef_prime.rflow.Setting.Key;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.ErrorDialog;
import com.ef_prime.rflow.tool.WorkingDirSetter;
import com.ef_prime.rflow.util.FileControl;
import com.ef_prime.rflow.util.PropertiesUtility;

public class Main {

	public static void main(String[] args)
	{
		try {
			init(args);
		} catch (Exception e) {
			showErrorDialog(e);
		}
	}
	
	static void init(String[] args) throws IOException{

		File runScriptFile = new File(Platform.getUserDirWithVersion(),"run.vbs"); //$NON-NLS-1$
		File settingsFile = FileResource.SETTING_FILE;
		
		final Setting setting = Setting.defaultSetting();

		if(!settingsFile.exists())
		{
			//setLookAndFeel
			new JPanel();
	        try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (Exception e) {}
			
			//1.(古いバージョンの設定を削除。その前にダイアログを出す) 
			//この処理は、全OS共通の処理なのでRflowのmainに移動しました
			/*
			File file = new File(Platform.getUserDir(),"rflow.settings"); //$NON-NLS-1$
			
			if(file.exists())
				try {
					SwingUtilities.invokeAndWait(new Runnable(){
						public void run() {
							int value = JOptionPane.showConfirmDialog(null, Messages.getString("Main.ask_del_preview_setting"),Messages.getString("Message.confirm"),JOptionPane.YES_NO_CANCEL_OPTION); //$NON-NLS-1$ //$NON-NLS-2$
							
							if(value == JOptionPane.YES_OPTION)
							{
								try
								{
									new File(Platform.getUserDir(),"rflow.settings").delete(); //$NON-NLS-1$
									new File(Platform.getUserDir(),"run.vbs").delete(); //$NON-NLS-1$
									new File(Platform.getUserDir(),"function.xml").delete(); //$NON-NLS-1$
								}
								catch(RuntimeException ex)
								{
								}
							}
						}});
				} catch (InterruptedException e1) {
				} catch (InvocationTargetException e1) {
				}
				*/

			//2.初期設定の取得 
			//3.初期設定が入力されたダイアログを表示してR、作業ディレクトリのパスを入力 
			//4.settingFileに保存 
			SettingDialog.main(new String[0]);

			//キャンセルが押されたら終了
			if(!settingsFile.exists())
			{
					return;
			}

			PropertiesUtility.load(setting, settingsFile);
			setting.setProperty(Setting.Key.IS_FIRST, Boolean.valueOf(false));
			PropertiesUtility.store(setting, settingsFile);

			try {
				SwingUtilities.invokeAndWait(new Runnable(){
					public void run() {
						int value = JOptionPane.showConfirmDialog(null, Messages.getString("Main.ask_copy_sample"), Messages.getString("Message.confirm"), JOptionPane.YES_NO_OPTION); //$NON-NLS-1$ //$NON-NLS-2$
						if(value == JOptionPane.YES_OPTION)
						{
//							File sampleFolder = new File(
//									new File(setting.getProperty(Setting.Key.DEFAULT_WORKING_DIRECTORY).toString())
//									, "SampleAnalysis"); //$NON-NLS-1$
//
//							if(!sampleFolder.exists())
//								sampleFolder.mkdir();
//
//							if(sampleFolder.exists())
//								try {
//									copyFiles(new File(Constants.getPath("sample")), sampleFolder); //$NON-NLS-1$
//								} catch (Exception e) {
//								}
							File workDir = new File(setting.getProperty(Setting.Key.DEFAULT_WORKING_DIRECTORY).toString());
							int max_path = (Integer)setting.getProperty(Setting.Key.MAX_PATH);
							try {
								WorkingDirSetter.copySamples(new FileControl(max_path, null), workDir);
							} catch (IOException e) {
								JOptionPane.showMessageDialog(null, Messages.getString("Error.sample_copy"),  //$NON-NLS-1$
										Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
							}
						}
					}});
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		
		PropertiesUtility.load(setting, settingsFile);
		

		String r_home = (String)setting.getProperty(Key.R_HOME);
		boolean use_library_pack = (Boolean)setting.getProperty(Key.USE_LIBLARY_PACK);
		Locale.setDefault(setting.getLocale());

		
		//Rの存在チェック
		
		File rHome = new File(r_home);
		File rBin = new File(rHome,"bin"); //$NON-NLS-1$
		File rScriptExe = new File(rBin,"Rscript.exe"); //$NON-NLS-1$
		
		if(r_home.equals("") || !rHome.exists() || !rBin.exists() || !rScriptExe.exists()) //$NON-NLS-1$
		{
			System.out.println("DEBUG:Directory R_HOME not found." + r_home); //$NON-NLS-1$
			showSettingDialog(Messages.getString("Error.Main.no_r_installdirectory")); //$NON-NLS-1$
			return;
		}

		
		//Rを起動して使用するRに関する情報を取得。
		String libpath;
		
		String command = "\"" + rScriptExe.getAbsolutePath() + "\" r_library_path.R"; //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println(command);

		Process p = Runtime.getRuntime().exec(command);
		BufferedReader reader = new BufferedReader(
				new InputStreamReader(p.getInputStream()));
		String major = reader.readLine();//2
		String minor = reader.readLine();//7.1 (2.7.1の場合)
		final String version = major + "." + minor; //$NON-NLS-1$
		
		
		
		libpath = reader.readLine();
		reader.close();
			
			
			
		//上記で取得した情報を元に起動スクリプト生成

		File libraryPack = new File(new File(new File("lib"),"R"),version); //$NON-NLS-1$ //$NON-NLS-2$

		try
		{
			String[] temp = minor.split("\\."); //$NON-NLS-1$
			String m1 =  temp[0];
			String m2 =  temp[1];
			int m2i = Integer.parseInt(m2);
			while(!libraryPack.exists() && --m2i >= 0)
			{
				String version2 = major + "." + m1 + "." + m2i; //$NON-NLS-1$ //$NON-NLS-2$
				libraryPack = new File(new File(new File("lib"),"R"),version2); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		catch(Exception ex)
		{
		}
		
		if(use_library_pack && libraryPack.exists())
		{
			//通常のフローはこちら
			
			File jriJar = new File(libraryPack,"JRI.jar"); //$NON-NLS-1$
			makeRunScript(runScriptFile,
					rHome.getAbsolutePath(),
					rBin.getAbsolutePath(),
					libraryPack.getAbsolutePath(),
					jriJar.getAbsolutePath(),
					libpath);
		}
		else
		{
			if(true)
			{
				try {
					SwingUtilities.invokeAndWait(new Runnable(){

						public void run() {
							new JPanel();
					        try {
								UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
							} catch (Exception e) {}
							String rVersions = ""; //$NON-NLS-1$
							String minor = ""; //$NON-NLS-1$
							File libDir = new File(new File("lib"),"R"); //$NON-NLS-1$ //$NON-NLS-2$
							try{
								if(libDir.exists() && libDir.isDirectory()){
									for(File f : libDir.listFiles()){
										if(f.isFile())continue;
										String m = f.getName().split("\\.")[1]; //$NON-NLS-1$
										if(!minor.equals(m))
											rVersions += "\n"; //$NON-NLS-1$
										else
											rVersions += ", "; //$NON-NLS-1$
										rVersions += f.getName();
										minor = m;
									}
								}
							}catch(Exception e){
							}
							if(rVersions.equals("")) //$NON-NLS-1$
								rVersions += "\n [search failed]"; //$NON-NLS-1$
							JOptionPane.showMessageDialog(null, Messages.getString("Error.Main.unsupported_r") //$NON-NLS-1$
									+rVersions);
						}});
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return;
			}
			
			
			//このコード必要かな？該当バージョンがないときはRのライブラリパスにrJavaがないか探す。
			//日本語対応されてないjriが呼び出されてしまうけど新バージョン対応が必要ない。
			
			
			final File jriJar = new File(libpath,"rJava\\jri\\JRI.jar"); //$NON-NLS-1$
			
			if(!jriJar.exists())
			{
				SwingUtilities.invokeLater(new Runnable(){
					public void run() {
						new JPanel();
				        try {
							UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
						} catch (Exception e) {}
						JOptionPane.showMessageDialog(null, Messages.getString("Error.file.file_not_found") + jriJar.getAbsolutePath()); //$NON-NLS-1$
					}});
				return;
			}
			
			makeRunScript(runScriptFile,
					rHome.getAbsolutePath(),
					rBin.getAbsolutePath(),
					jriJar.getParentFile().getAbsolutePath(),
					jriJar.getAbsolutePath(),
					libpath);
		}

		String runCommand = "cscript \"" + runScriptFile.getAbsolutePath() + "\"" + ((args.length > 0)?(" \"" + args[0] + '"'):""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Runtime.getRuntime().exec(runCommand);
	}
	
	/**
	 * 設定ダイアログを表示する。
	 * @param detail
	 */
	static void showSettingDialog(final String detail)
	{
		SwingUtilities.invokeLater(new Runnable(){
			public void run() {
				//setLookAndFeel
				new JPanel();
		        try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception e) {}
				int val = JOptionPane.showConfirmDialog(null, 
						detail + "\n\n" +Messages.getString("Error.Main.error_ask_show_pref"));//$NON-NLS-1$ //$NON-NLS-2$
				if(val == JOptionPane.YES_OPTION)
				{
					SettingDialog.main(new String[0]);
				}
			}});
	}
	
	/**
	 * エラーメッセージをダイアログで表示する
	 * @param e
	 */
	static void showErrorDialog(final Throwable e)
	{

		SwingUtilities.invokeLater(new Runnable(){
			public void run() {
				//setLookAndFeel
				new JPanel();
		        try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception e) {}
				ErrorDialog.show(null, "", e); //$NON-NLS-1$
			}});
	}
	
	
	/**
	 * 起動スクリプト生成
	 * 
	 * @param runScript スクリプトのFile
	 * @param rHome Rの存在するディレクトリ。環境変数R_HOMEとして設定される。
	 * @param rbinDirectory Rの実行ファイル(R.exeなど)が存在するディレクトリ。
	 * @param libpath jri.dllが存在するディレクトリ。
	 * @param jriJar JRI.jarのパス
	 * @param rLib　Rのライブラリパス
	 * @throws IOException
	 */
	static void makeRunScript(
			File runScript,
			String rHome,
			String rbinDirectory,
			String libpath,
			String jriJar,
			String rLib)throws IOException{

		BufferedWriter writer = new BufferedWriter(new FileWriter(runScript)); //$NON-NLS-1$

		writer.write("Set objShell = WScript.CreateObject(\"WScript.Shell\")"); //$NON-NLS-1$
		writer.newLine();
		writer.write("Set WSHArguments = WScript.Arguments"); //$NON-NLS-1$
		writer.newLine();
		writer.write("If WSHArguments.Count > 0 Then"); //$NON-NLS-1$
		writer.newLine();
		writer.write("	For Each I In WSHArguments"); //$NON-NLS-1$
		writer.newLine();
		writer.write("		str=str & \" \"\"\" & I & \"\"\"\""); //$NON-NLS-1$
		writer.newLine();
		writer.write("	Next"); //$NON-NLS-1$
		writer.newLine();
		writer.write("End If"); //$NON-NLS-1$
		writer.newLine();
		writer.write("Set colEnv = objShell.Environment(\"Process\")"); //$NON-NLS-1$
		writer.newLine();
		writer.write("colEnv.Item(\"PATH\") = colEnv.Item(\"PATH\") & \";" + rbinDirectory + ";" + libpath + "\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		writer.newLine();
		writer.write("colEnv.Item(\"R_HOME\") = \"" + rHome + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.newLine();
		writer.write("colEnv.Item(\"JAVA_HOME\") = \".\\jre\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.newLine();
		writer.write("colEnv.Item(\"R_LIBS\") = colEnv.Item(\"R_LIBS\") & \";" + rLib + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.newLine();
		writer.write("library = \"-Djava.library.path=\"\"" + libpath + "\"\"\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.newLine();
		writer.write("jrijar = \"" + jriJar + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		writer.newLine();
		writer.write("objShell.Run \".\\jre\\bin\\javaw.exe -splash:img\\R-flow-splash.png \" & library & \" -classpath .\\lib\\commons-collections-3.1.jar;.\\lib\\colt.jar;.\\lib\\resolver.jar;.\\lib\\rsyntaxtextarea.jar;.\\lib\\jung-1.7.6.jar;.\\lib\\concurrent.jar;.\\lib\\xercesImpl.jar;.\\lib\\xml-apis.jar;.\\lib\\rflow.jar;.\\bin;\"\"\" & jrijar & \"\"\"; com.ef_prime.rflow.Rflow\" & str"); //$NON-NLS-1$
		writer.newLine();
		writer.flush();
		writer.close();
	}
	
	
	/**
	 * This function will copy files or directories from one location to another.
	 * note that the source and the destination must be mutually exclusive. This 
	 * function can not be used to copy a directory to a sub directory of itself.
	 * The function will also have problems if the destination files already exist.
	 * @param src -- A File object that represents the source for the copy
	 * @param dest -- A File object that represnts the destination for the copy.
	 * @throws IOException if unable to copy.
	 */
	public static void copyFiles(File src, File dest) throws IOException {
		//Check to ensure that the source is valid...
		if (!src.exists()) {
			throw new IOException("copyFiles: Can not find source: " + src.getAbsolutePath()+"."); //$NON-NLS-1$ //$NON-NLS-2$
		} else if (!src.canRead()) { //check to ensure we have rights to the source...
			throw new IOException("copyFiles: No right to source: " + src.getAbsolutePath()+"."); //$NON-NLS-1$ //$NON-NLS-2$
		}
		//is this a directory copy?
		if (src.isDirectory()) 	{
			if (!dest.exists()) { //does the destination already exist?
				//if not we need to make it exist if possible (note this is mkdirs not mkdir)
				if (!dest.mkdirs()) {
					throw new IOException("copyFiles: Could not create direcotry: " + dest.getAbsolutePath() + "."); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			//get a listing of files...
			String list[] = src.list();
			//copy all the files in the list.
			for (int i = 0; i < list.length; i++)
			{
				File dest1 = new File(dest, list[i]);
				File src1 = new File(src, list[i]);
				copyFiles(src1 , dest1);
			}
		} else { 
			//This was not a directory, so lets just copy the file
			FileInputStream fin = null;
			FileOutputStream fout = null;
			byte[] buffer = new byte[4096]; //Buffer 4K at a time (you can change this).
			int bytesRead;
			try {
				//open the files for input and output
				fin =  new FileInputStream(src);
				fout = new FileOutputStream (dest);
				//while bytesRead indicates a successful read, lets write...
				while ((bytesRead = fin.read(buffer)) >= 0) {
					fout.write(buffer,0,bytesRead);
				}
			} catch (IOException e) { //Error copying file... 
				IOException wrapper = new IOException("copyFiles: Unable to copy file: " +  //$NON-NLS-1$
							src.getAbsolutePath() + "to" + dest.getAbsolutePath()+"."); //$NON-NLS-1$ //$NON-NLS-2$
				wrapper.initCause(e);
				wrapper.setStackTrace(e.getStackTrace());
				throw wrapper;
			} finally { //Ensure that the files are closed (if they were open).
				if (fin != null) { fin.close(); }
				if (fout != null) { fin.close(); }
			}
		}
	}
}
/*
#Tue Feb 26 11:42:02 JST 2008
USE_LIBLARY_PACK=true
MAX_PATH=255
IS_FIRST=false
GRID_DISTANCE=10
IS_AUTO_UPDATE=false
USE_CONSOLE_BUFFER=true
R_HOME=C\:\\R\\R-2.6.2\\bin
LANGUAGE=ja
CHECKED_VERSION=0.3.0
USE_GRID=true
DEFAULT_WORKING_DIRECTORY=C\:\\Documents and Settings\\nagai\\My Documents\\eclipse\\Rflow\\data
 */