package com.ef_prime.rflow;

import java.io.File;

import javax.swing.JOptionPane;

import com.ef_prime.rflow.resource.Messages;

public enum Platform {
	LINUX,
	MAC,
	WINDOWS,
	WINDOWS_VISTA,
	OTHER;
	
	private static Platform platform;
	
	/*user folder*/
	private static final String FOLDER_RANALYICFLOW = "RAnalyticFlow"; //$NON-NLS-1$
	private static final String FOLDER_DOT_RANALYICFLOW = ".ranalyticflow"; //$NON-NLS-1$

	
	public static Platform getPlatform(){
		if(platform ==null){
			String osname = System.getProperty("os.name"); //$NON-NLS-1$
			//UNIX?
			if(osname == null)
				platform = Platform.OTHER;
			else if(osname.startsWith("Windows")){ //$NON-NLS-1$
				if(osname.contains("Vista")){ //$NON-NLS-1$
					platform = Platform.WINDOWS_VISTA;
				}else
					platform = Platform.WINDOWS;
			}else if(osname.contains("Linux")) //$NON-NLS-1$
				platform = Platform.LINUX;
			else if(osname.contains("Mac")) //$NON-NLS-1$
				platform = Platform.MAC;
			else
				platform = Platform.OTHER;
		}
		return platform;
	}
	
	public static File getUserDirWithVersion()
	{
		File f = getUserDir();
		File g = new File(f, Version.getVersionMinor());
		if(!g.exists())
			if(!g.mkdir())
			{
				JOptionPane.showMessageDialog(
						null,
						new FolderCreationFailedException(g).getMessage(),
						Messages.getString("Error.title.error"),//$NON-NLS-1$
						JOptionPane.ERROR_MESSAGE); 
				//RAF cannot run without this folder
				System.exit(1);
				return null;
			}
		return g;
	}
	
	public static boolean isWindowsFamily()
	{
		return getPlatform() == Platform.WINDOWS_VISTA || getPlatform() == Platform.WINDOWS;
	}
	
	public static File getUserDir(){
		//TODO フォルダの作成の仕方
		File userHome = new File(System.getProperty("user.home")); //$NON-NLS-1$
		try{
		switch(Platform.getPlatform()){
		case WINDOWS:
			File app = new File(userHome, "Application Data"); //$NON-NLS-1$
			if(!app.exists())
				if(!app.mkdir())throw new FolderCreationFailedException(app);
			File rflow = new File(app, FOLDER_RANALYICFLOW);
			if(!rflow.exists())
				if(!rflow.mkdir())throw new FolderCreationFailedException(rflow);
			return rflow;
		case WINDOWS_VISTA:
			File vista_app = new File(userHome, "AppData"); //$NON-NLS-1$
			if(!vista_app.exists())
				if(!vista_app.mkdir())throw new FolderCreationFailedException(vista_app);
			File vista_roaming = new File(vista_app, "Roaming"); //$NON-NLS-1$
			if(!vista_roaming.exists())
				if(!vista_roaming.mkdir())throw new FolderCreationFailedException(vista_roaming);
			File vista_rflow = new File(vista_roaming, FOLDER_RANALYICFLOW);
			if(!vista_rflow.exists())
				if(!vista_rflow.mkdir())throw new FolderCreationFailedException(vista_rflow);
			return vista_rflow;
		default:
			File dot_rflow = new File(userHome, FOLDER_DOT_RANALYICFLOW);
			if(!dot_rflow.exists())
				if(!dot_rflow.mkdir())throw new FolderCreationFailedException(dot_rflow);
			return dot_rflow;
		}
		}catch(FolderCreationFailedException e){
			JOptionPane.showMessageDialog(null, e.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			//RAF cannot run without this folder
			System.exit(1);
			return null;
		}
	}
	
	public static File getUserWorkingDir(){
		File userHome = new File(System.getProperty("user.home")); //$NON-NLS-1$
		switch(Platform.getPlatform()){
		case WINDOWS:
			return new File(userHome, "My Documents"+File.separatorChar+FOLDER_RANALYICFLOW); //$NON-NLS-1$
		case MAC:
		case WINDOWS_VISTA:
			return new File(userHome, "Documents"+File.separatorChar+FOLDER_RANALYICFLOW); //$NON-NLS-1$
		default:
			return new File(userHome, FOLDER_RANALYICFLOW);
		}		
	}
	
	private static class FolderCreationFailedException extends Exception{
		public FolderCreationFailedException(File dir){
			super(Messages.getString("Error.file.create_folder") + ":" + dir.getAbsolutePath()); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
}
