package com.ef_prime.rflow;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.rosuda.JRI.RBool;
import org.rosuda.JRI.REXP;
import org.rosuda.JRI.RList;
import org.rosuda.JRI.RVector;
import org.rosuda.JRI.Rengine;

import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.console.CommandReceiver;
import com.ef_prime.rflow.node.SentenceInfo;
import com.ef_prime.rflow.node.base.ArgEntry;
import com.ef_prime.rflow.node.base.ArgListMod;
import com.ef_prime.rflow.object.RObject;

public class RCommand {
	
	private static Rengine R;
	private static CommandReceiver receiver;
	

	public static void setR(Rengine R) {
		RCommand.R = R;
	}
	
	public static void setReceiver(CommandReceiver receiver) {
		RCommand.receiver = receiver;
	}
	
	public static Rengine getR(){
		return R;
	}
	
	public static CommandReceiver getReceiver() {
		return receiver;
	}
	
	public static String getRflowEnvCommand(){
		return "as.environment(\""+ Constants.RFLOW_ENV +"\")"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public static REXP silentIdleEval(String command){
		return R.idleEval("try("+command+", silent = TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	private static final String ERROR_HEAD = "tryCatch({"; //$NON-NLS-1$ 
	private static final String ERROR_TAIL = "}, error=.errorHandler)"; //$NON-NLS-1$ 
	public static String getErrorCheckCommand(String s){
		//空文字の場合、そのまま返す
		//debugが空文字を使用するので、空文字を空文字のまま渡すためにこの対応をしました
		//if(s.length() == 0)
		//	return s;
		if(s.endsWith(Constants.NEWLINE_STR))
			s = s.substring(0, s.length()-1);
		return ERROR_HEAD+s+ERROR_TAIL;
	}
	
	public static String trimErrorCommandFirst(String s){
		return s.substring(ERROR_HEAD.length());
	}
	
	public static String trimErrorCommandLast(String s){
		return s.substring(0, s.length()-(ERROR_TAIL.length()+1))+Constants.NEWLINE;
	}
	
	public static boolean isErrorOccurred(){
		//REXP e = R.idleEval("eval(.errorObject, envir = "+ getRflowEnvCommand() +"))");
		REXP e = R.idleEval(".errorObject"); //$NON-NLS-1$
		if(e == null)return false;
		if(e.rtype == REXP.NILSXP)return false;
		return true;
	}
	
	public static void clearErrorObject(){
		R.idleEval("assign(\".errorObject\", NULL, envir = "+ getRflowEnvCommand() +")"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**
	 * 
	 * @param s
	 * @return error message. null if no error
	 */
	public static String checkSyntax(String s){
		if(s.length() < 1)return null;
		REXP e = R.idleEval("tryCatch(is.expression(parse(text="+Constants.quote(s)+")), error=function(e){conditionMessage(e)})"); //$NON-NLS-1$ //$NON-NLS-2$
		if(e == null)return null;
		if(e.rtype == REXP.STRSXP)return e.asString();
		else
			return null;
	}
	
	public static String[] supportList(String s){
		if(!checkCommandString(s))return null;
		
		REXP cmds = R.idleEval("try(.supportCommand(\""+s+"\"), silent = TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$
		String[] c = null;
        if (cmds != null && (c=cmds.asStringArray()) != null) return c;
		return null;
	}
	
	public static String[] supportListGlobal(String s){
		if(s == null)
			s = ""; //$NON-NLS-1$
		
		REXP cmds = R.idleEval("try(.supportCommand(\""+s+"\", envir=.GlobalEnv), silent = TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$
		String[] c = null;
        if (cmds != null && (c=cmds.asStringArray()) != null) return c;
		return null;
	}	
	
	public static ArgListMod argList(String function){
		if(!checkCommandString(function))return null;
		
		REXP formals = R.idleEval("try(.getFormals("+function+"), silent = TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$
		if(formals == null)return null;
		RList list = formals.asList();
		if(list == null)return null;
		
		ArgListMod argList = new ArgListMod();
		String[] keys = list.keys();
		for(int i = 0; i < keys.length; i++){
			String key = keys[i];
			String[] values = list.at(i).asStringArray();
			String value = ""; //$NON-NLS-1$
			for(String v : values){
				value += v + Constants.NEWLINE;
			}
			argList.add(new ArgEntry(key, value.trim()));
		}
		
		return argList;
	}
	
	public static String[] getAllVars(String expression){
		if(!checkCommandString(expression))return null;
		
		REXP vars = R.idleEval("all.vars(expression("+expression+"))"); //$NON-NLS-1$ //$NON-NLS-2$
		if(vars == null)return null;
		return vars.asStringArray();		
	}
	
	public static String[] getLocalVariables(String function){
		if(!checkCommandString(function))return null;
		
		REXP locals = R.idleEval(".findLocals("+function+")"); //$NON-NLS-1$ //$NON-NLS-2$
		if(locals == null)return null;
		RList list = locals.asList();
		if(list == null)return null;
		
		REXP variables = list.at("variables"); //$NON-NLS-1$
		String[] c = null;
        if (variables != null && (c=variables.asStringArray()) != null) return c;
		
		return null;
	}
	
	public static void clearObjects(){
		receiver.passCommand(getClearObjectsCommand());
	}
	
	public static Command getClearObjectsCommand(){
		return new BasicCommand(".clearObjects()"); //$NON-NLS-1$
	}
	
	public static String[] getObjectsFiltered(String filter){
		REXP objects = R.idleEval(".getObjectsFiltered(FILTERFUN="+filter+")"); //$NON-NLS-1$ //$NON-NLS-2$
		if(objects == null)return null;
		return objects.asStringArray();
	}
	
	public static Integer getLength(String obj){
		REXP exp = silentIdleEval(".getLength("+obj+")"); //$NON-NLS-1$ //$NON-NLS-2$
		if(exp == null)return null;
		return exp.asInt();
	}
	
	/**
	 * 指定した親のオブジェクトの一覧を返します
	 * @param parent parent of objects. null if parent is global env
	 * @param allNames get Objects start with "."
	 * @param limit maximum of obj infos to be returned
	 * @return null if failed
	 */
	public static RObject[] getObjects(RObject parent, boolean allNames, int limit){
		String p = ""; //$NON-NLS-1$
		if(parent != null){
			if(parent.isAtomic())return null;
			p = "parent="+parent.getFullName()+", "; //$NON-NLS-1$ //$NON-NLS-2$
		}
		REXP objects = silentIdleEval(".getObjects("+ p + "all.names="+Constants.toRBoolean(allNames)+", limit=" + limit + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		if(objects == null)return null;
		RVector objVector = objects.asVector();
		
		if(objVector == null)return null;
		
		List<RObject> robjList = new ArrayList<RObject>();
		Vector names = objVector.getNames();
		boolean isNames = (names != null);
		for(int i = 0; i < objVector.size(); i++){
			RList l = objVector.at(i).asList();
			if(l == null)continue;
			REXP type = l.at("type"); //$NON-NLS-1$
			REXP clas = l.at("class"); //$NON-NLS-1$
			REXP tip = l.at("tip"); //$NON-NLS-1$
			REXP validname = l.at("validname"); //$NON-NLS-1$
			if(type == null || clas == null || tip == null || validname==null)continue;
			RBool bool = validname.asBool();
			if(bool == null)continue;
			boolean bValidName = bool.isTRUE();
			String name = (isNames)?(String)names.get(i):String.valueOf(i+1);
			boolean nameIsIndex = !isNames || name.length() == 0;
			if(name.length() == 0)name = String.valueOf(i+1);
			robjList.add(new RObject(parent, name, type.asString(), clas.asString(), tip.asString(), bValidName, nameIsIndex));
		}
		//exceed limit?
		REXP overlimit = objects.getAttribute("overlimit"); //$NON-NLS-1$
		if(overlimit != null && overlimit.asInt() != 0){
			robjList.add(new RObject(parent, (overlimit.asInt()-limit)+" more objects", "", "", "", true, false)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		
		return robjList.toArray(new RObject[0]);
	}

	private static final String localEnv = ".localEnv"; //$NON-NLS-1$
	public static void clearLocalEnv(){
		R.eval("if(exists(\""+localEnv+"\"))rm("+localEnv+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public static String getLocalExcutionCode(String command){
		if(!checkCommandString(command))return null;
		
		return ".doInLocal(parse(text=\""+command+"\"), envir="+localEnv+", envchar=\""+ localEnv+"\")\n"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}
	
	public static String[] getPackages(){
		REXP exp = R.idleEval(".packages(all.available=TRUE)"); //$NON-NLS-1$
		if(exp == null)return null;
		return exp.asStringArray();
	}
	
	public static void help(final String function){
		R.eval("help(\""+function+"\")"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public static String makenames(String varname){
		REXP exp = silentIdleEval("make.names(\""+varname+"\")");
		if(exp == null)return null;
		return exp.asString();
	}
	
	public static String getHelpFile(String topic){
		REXP helpFile = R.idleEval(".getHelpTextFile(\""+topic+"\")"); //$NON-NLS-1$ //$NON-NLS-2$
		if(helpFile == null)return null;
		
		REXP del = helpFile.getAttribute("delete"); //$NON-NLS-1$
		if(del == null)return null;
		
		Boolean delete = del.asBool().isTRUE();
		String file = helpFile.asString();
		
		return file + Constants.DELIM + delete;
	}
	
	public static boolean setWD(String wd){
		REXP fail = R.eval(getErrorCheckCommand2("setwd(\""+wd+"\")")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-2
		if(fail == null)return false;
		return fail.asBool().isFALSE();
	}
	
	/*
	 * コードからフローのモデルを作成する
	 */
	private static final String KEY_LINE = "line"; //$NON-NLS-1$
	private static final String KEY_FUNC = "func"; //$NON-NLS-1$
	private static final String KEY_ARGS = "args"; //$NON-NLS-1$
	private static final String KEY_SYMBOL = "symbol"; //$NON-NLS-1$
	//private static final String KEY_SENTENCE = "sentence"; //$NON-NLS-1$
	private static final String KEY_CONTROL = "control"; //$NON-NLS-1$
	
	public static SentenceInfo[] parseToNodes(File file){
		return parseToNodesInternal(null, file);
	}

	public static SentenceInfo[] parseToNodes(String code){
		return parseToNodesInternal(code, null);
	}
	
	private static SentenceInfo[] parseToNodesInternal(String code, File file){
		REXP codeInfos;
		
		if(code != null){
			codeInfos = R.idleEval("suppressWarnings(try(parseToNodes("+ Constants.quote(code) +"), silent=TRUE))"); //$NON-NLS-1$ //$NON-NLS-2$
		}else if(file != null){
			String filePath;
			try {
				filePath = Constants.replaceFileSeparator(file.getCanonicalPath());
			} catch (IOException e) {
				return null;
			} 
			codeInfos = R.idleEval("try(parseToNodes(file="+ Constants.quote(filePath) +"), silent=TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			return null;
		}
		
		
		if(codeInfos == null)return null;
		
		RVector codeInfoV = codeInfos.asVector();
		if(codeInfoV == null)return null;
		//RList codeInfoList = codeInfos.asList();
		//if(codeInfoList == null)return null;
		
		int nLine = codeInfoV.size();
		
		//NodeModel[] models = new NodeModel[nLine];
		//boolean[] isFreeNode = new boolean[models.length];
		SentenceInfo[] infos = new SentenceInfo[nLine];
		
		for(int i = 0; i < nLine; i++){
			REXP codeInfo = codeInfoV.at(i);
			//NodeModel model;
			
			if(!(codeInfo.getType() == REXP.XT_VECTOR || codeInfo.getType() == REXP.XT_LIST))
				return null;
			
			RList funcInfo = codeInfo.asList();

			REXP line_exp = funcInfo.at(KEY_LINE);
			if(line_exp == null)return null;
			String line = line_exp.asString();
			
			//REXP sentence_exp = funcInfo.at(KEY_SENTENCE);
			//if(sentence_exp == null)return null;
			//String sentence = sentence_exp.asString();
			//boolean isSentence = true;
			//isFreeNode[i] = !isSentence;
			
			//if(isSentence){
			
				boolean isControl = false;
				//if(flowControlIsFreeNode){
					REXP control = funcInfo.at(KEY_CONTROL);
					if(control != null){
						RBool b =  control.asBool();
						isControl = (b != null && b.isTRUE());
					}
				//}
				
				/*if(isControl){
					FreeNodeModel fModel = new FreeNodeModel();
					fModel.command = line;
					model = fModel;
					isFreeNode[i] = true;
				}else{*/
					//SentenceNodeModel sModel = new SentenceNodeModel();
					
					//sModel.sentence = sentence;
	
					String func = ""; //$NON-NLS-1$
					REXP function = funcInfo.at(KEY_FUNC);
					if(function != null){
						//sModel.function = funcInfo.at(KEY_FUNC).asString();
						func = funcInfo.at(KEY_FUNC).asString();
					}
					
					String syb = ""; //$NON-NLS-1$
					REXP symbol = funcInfo.at(KEY_SYMBOL);
					if(symbol != null){
						syb = symbol.asString();
					}
					
					ArgListMod list = new ArgListMod();
					REXP args = funcInfo.at(KEY_ARGS);
					if(args != null){
						RVector argsList = args.asVector();
						if(argsList != null){
							Vector names = argsList.getNames();
							boolean hasName = (names != null && names.size() > 0);
							for(int j = 0; j < argsList.size(); j++){
								String key = (hasName)?names.get(j).toString():""; //$NON-NLS-1$
								String v = argsList.at(j).asString();
								//sModel.argList.add(new ArgEntry(key, v.trim()));
								list.add(new ArgEntry(key, v.trim()));
							}
						}
					}
					
					//model = sModel;
				//}
//			}else{
//				//unreachable now
//				//free node if not list(one character)
//				FreeNodeModel fModel = new FreeNodeModel();
//				fModel.command = line;
//				model = fModel;
//			}

			
			//models[i] = model;
			infos[i] = new SentenceInfo(line, syb, func, isControl, list);
		}
		
		//連続するFreeNodeModelを結合する
		/*LinkedList<NodeModel> modelList = new LinkedList<NodeModel>();
		modelList.add(models[0]);
		for(int i = 1; i< models.length; i++){
			if(isFreeNode[i] && isFreeNode[i-1]){
				FreeNodeModel lastFreeModel = (FreeNodeModel)modelList.getLast();
				FreeNodeModel currentFreeModel = (FreeNodeModel)models[i];
				
				String c = lastFreeModel.getCommandNewline();
				FreeNodeModel fm = new FreeNodeModel();
				fm.command = c + currentFreeModel.command;
				
				modelList.removeLast();
				modelList.add(fm);
			}else{
				modelList.add(models[i]);
			}
		}*/
		
		//return modelList.toArray(new NodeModel[0]);
		return infos;
	}

	
	private static boolean checkCommandString(String command){
		if(command == null || (command.trim().length() < 1))
			return false;
		return true;		
	}
	
	private static String getErrorCheckCommand2(String command){
		return "inherits(try(" + command + ", silent=TRUE), \"try-error\")"; //$NON-NLS-1$ //$NON-NLS-2$
	}
}
