package com.ef_prime.rflow;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JDesktopPane;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;

import org.rosuda.JRI.Rengine;

import com.ef_prime.rflow.Setting.Key;
import com.ef_prime.rflow.console.BusyListener;
import com.ef_prime.rflow.console.CommandReceiver;
import com.ef_prime.rflow.console.RConsole;
import com.ef_prime.rflow.exception.ReadException;
import com.ef_prime.rflow.explorer.Explorer;
import com.ef_prime.rflow.flow.FlowSetting;
import com.ef_prime.rflow.flow.FlowState;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.FlowViewerScrollPane;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.group.AppearanceGroupManager;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.FlowDataReader;
import com.ef_prime.rflow.node.DefaultNodeModelController;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.node.NodeModelController;
import com.ef_prime.rflow.node.NodeModelManager;
import com.ef_prime.rflow.node.NodeProperty;
import com.ef_prime.rflow.node.NodePropertyChangeListener;
import com.ef_prime.rflow.node.base.FreeNodeModel;
import com.ef_prime.rflow.node.base.FreeNodeModelController;
import com.ef_prime.rflow.node.base.FunctionNodeModel;
import com.ef_prime.rflow.node.base.FunctionNodeModelController;
import com.ef_prime.rflow.node.base.IfNodeModel;
import com.ef_prime.rflow.node.base.IfNodeModelController;
import com.ef_prime.rflow.node.base.LoopNodeModel;
import com.ef_prime.rflow.node.base.LoopNodeModelController;
import com.ef_prime.rflow.node.base.SentenceNodeModel;
import com.ef_prime.rflow.node.base.SentenceNodeModelController;
import com.ef_prime.rflow.node.base.SubflowNodeModel;
import com.ef_prime.rflow.node.base.SubflowNodeModelController;
import com.ef_prime.rflow.node.base.TunnelNodeModel;
import com.ef_prime.rflow.node.base.TunnelNodeModelController;
import com.ef_prime.rflow.node.pack.UIPack;
import com.ef_prime.rflow.node.pack.UIPackManager;
import com.ef_prime.rflow.node.pack.UIPackReader;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.BusyLabel;
import com.ef_prime.rflow.tool.UpdateChecker;
import com.ef_prime.rflow.tool.WorkingDirSetter;
import com.ef_prime.rflow.util.BackgroundLayout;
import com.ef_prime.rflow.util.FileControl;
import com.ef_prime.rflow.util.PropertiesUtility;
import com.ef_prime.rflow.xml.XMLException;

public class Rflow {
	
	/**
	 * デバッグ用定数
	 * この値はそのままRengine.DEBUGにも代入されます
	 * <p>0: デバッグ出力なし
	 * <br>>0:デバッグ出力あり
	 * <p>例：
	 * <code>if(Rflow.DEBUG > 0)
	 *     System.out.println(e.getMessage());</code>
	 */
	public static final int DEBUG = 0;
	
	/** 
	 * テスト用定数
	 */
	public static final boolean TEST = false;
		
	public final MainFrame frame;
	public final Rengine R;
	public final FileControl fileControl;
	public final CommandReceiver receiver;
	//public final FlowState state;
	public final FlowViewer viewer;
	public final JDesktopPane desktop;
	public final CodeBar codeBar;
	//public final CodeViewer codeViewer;
	public final Explorer explorer;
	public final RConsole console;
	public final ActionManager actionManager;
	public final AppearanceGroupManager groupManager = new AppearanceGroupManager();
	public static final Setting setting = new Setting();
	public static LinkedHashSet<String> folderlog = new LinkedHashSet<String>();
	public final BackupThread backupThread = new BackupThread(this);
	
	private static boolean IS_BUSY;
	
	private List<FlowViewer> viewers = new Vector<FlowViewer>();
	private FlowViewer activeViewer;
	private List<ActiveViewerListener> activeViewerListenerList = new ArrayList<ActiveViewerListener>();
	
	private File workingFile;
	private FlowSetting flowSetting = new FlowSetting();
	private boolean isDirty;
	private boolean isDirtySub;//dirty flag for backup
	
	
	static{
		Rengine.DEBUG = Rflow.DEBUG;	//set debug
	}
	
	public Rflow(){		
		/*SplashScreen splashScreen = SplashScreen.getSplashScreen();
		try {
			splashScreen.setImageURL(new File("img/R-flow-splash.png").toURI().toURL());
		}　catch (Exception e) {
			e.printStackTrace();
		}*/
		
		//System.getProperties().list(System.out);
		//System.out.println(System.getProperty("user.dir"));
		
		//Look&Feel
		setLookAndFeel();

		//設定ファイルの読み込み
		try {
			PropertiesUtility.load(setting, FileResource.SETTING_FILE);
			setting.update();
		} catch (IOException e2) {
			e2.printStackTrace();
			JOptionPane.showMessageDialog(null, Messages.getString("Error.read_option"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE);  //$NON-NLS-1$//$NON-NLS-2$
		}

		//ロケールの設定
		Locale.setDefault(setting.getLocale());

		//各種ファイルを設定をロケールにしたがって設定
		FileResource.prepareFiles();
		
		//リソースの設定（設定ファイルの言語設定を使う）
		setResource();
		
		//フォルダログの読み込み
		try {
			if(FileResource.FOLDERLOG_FILE.exists())
				folderlog = new LinkedHashSet<String>(PropertiesUtility.readList(FileResource.FOLDERLOG_FILE));
		} catch (IOException e2) {
			//TODO
			//エラーは無視する
			e2.printStackTrace();
		}

		
		//メインフレーム
		frame = new MainFrame();
		frame.addWindowListener(new WindowAdapter(){
			@Override
			public void windowClosing(WindowEvent e) {
				quit();
			}
		});
		//file control
		fileControl = new FileControl((Integer)Rflow.setting.getProperty(Setting.Key.MAX_PATH), frame);
		
		//ノードモデル
		initializeNodeModelManager();
		
		//パネル
		JPanel panel = new JPanel(new BorderLayout());
		//レシーバー
		receiver = new CommandReceiver(this);
		//フロー
		Box flowBox = Box.createVerticalBox();
		//ビュー
		desktop = new JDesktopPane();
		viewer = new FlowViewer(this);
		activeViewer = viewer;
		FlowViewerScrollPane graphPane = new FlowViewerScrollPane(viewer);
		desktop.add(graphPane);
		desktop.setLayer(graphPane, 0, -1);
		desktop.setLayout(new BackgroundLayout(graphPane));
		//コードバー
		codeBar = new CodeBar(viewer);
		flowBox.add(codeBar);
		flowBox.add(desktop);
		//関数
		explorer = new Explorer(this);
		//中央スプリットペイン
		JSplitPane centerSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, explorer, flowBox);
		centerSplitPane.setDividerLocation(200);
		panel.add(centerSplitPane, BorderLayout.CENTER);
		//アクション
		actionManager = new ActionManager(this);
		//ツールバー
		JToolBar toolbar = actionManager.createToolBar();
		panel.add(toolbar, BorderLayout.NORTH);
		//メニューバー
		JMenuBar menuBar = actionManager.createMenuBar();
		
		frame.add(panel, BorderLayout.CENTER);
		//frame.add(menuBar, BorderLayout.NORTH);
		frame.setJMenuBar(menuBar);
		//frame.setLocationRelativeTo(null);

		//コードビューワ
		//codeViewer = new CodeViewer(this);
		//viewer.getGraph().addListener(codeViewer, GraphEventType.ALL_SINGLE_EVENTS);
		
		//
		addViewer(viewer);
		
		//node model change listener
		FlowNode.addNodeModelChangeListener(codeBar);
		FlowNode.addNodePropertyChangeListener(codeBar);
		//FlowNode.addNodeModelChangeListener(codeViewer);
		FlowNode.addNodeModelChangeListener(new NodeModelChangeListener(){
			public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
				setDirty(true);
			}
			public void subflowNodeModelChanged(FlowNode node) {}
		});
		FlowNode.addNodePropertyChangeListener(new NodePropertyChangeListener(){
			public void nodePropertyChanged(FlowNode node, NodeProperty old, NodeProperty property) {
				setDirty(true);
			}
		});
		
		
		//R console
		console = new RConsole(this);
		console.addBusyListener(new BusyListener(){
			public void busyStateChanged(int which) {
				IS_BUSY = (which == BusyListener.BUSY);
			}
		});
		console.addBusyListener(actionManager);
		
		//busy label
		BusyLabel busyLabel = new BusyLabel();
		console.addBusyListener(busyLabel);
		toolbar.add(Box.createHorizontalGlue());
		toolbar.add(busyLabel);
		toolbar.add(Box.createHorizontalStrut(5));
		
		//R
		R = new Rengine(new String[]{"--vanilla"}, false, console); //$NON-NLS-1$
		if(!R.waitForR()){
			return;
		}
		
		//
		RCommand.setR(R);
		RCommand.setReceiver(receiver);
		
		//警告表示の抑制
		//R.eval("old.warn <- getOption(\"warn\")");
		//R.eval("options(warn=-1)");


		//デフォルトの作業ディレクトリ
		String dir = (String)setting.getProperty(Setting.Key.DEFAULT_WORKING_DIRECTORY);
		if(!WorkingDirSetter.setWorkingDir(
				fileControl, frame, (dir.equals(""))?null:new File(dir))) //$NON-NLS-1$
			System.exit(0);


		//この位置に過去にinit.Rに関する記述があった。

		R.eval("source(\"initRflow.R\", encoding=\"UTF-8\")"); //$NON-NLS-1$
		
		//linuxの場合pagerを変更するために、rJavaを読み込む
		if(Platform.getPlatform() == Platform.LINUX || Platform.getPlatform() == Platform.MAC){
			R.eval("library(rJava)");
		}
		//rflow用の関数群の登録
		R.eval("attach(new.env(hash=TRUE, parent=.GlobalEnv), name=\""+ Constants.RFLOW_ENV +"\")"); //$NON-NLS-1$ //$NON-NLS-2$
		R.eval("eval(quote(source(\"rflow.R\", encoding=\"UTF-8\", local=TRUE)), envir="+RCommand.getRflowEnvCommand()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		R.eval("eval(quote(source(\"func.R\", encoding=\"UTF-8\", local=TRUE)), envir="+RCommand.getRflowEnvCommand()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		R.eval("assign(\".errorHandler\", function(e){" + //$NON-NLS-1$
				"assign(\".errorObject\", e, envir = "+ RCommand.getRflowEnvCommand() +")" + 	Constants.NEWLINE + //$NON-NLS-1$ //$NON-NLS-2$
						"if(deparse(conditionCall(e)[[1]]) == \"doTryCatch\"){" +				Constants.NEWLINE + //$NON-NLS-1$
						"msg <- paste(\"Error : \", conditionMessage(e), \"\n\", sep = \"\")" + Constants.NEWLINE + //$NON-NLS-1$
						".Internal(seterrmessage(msg[1]))" +									Constants.NEWLINE + //$NON-NLS-1$
						"if (identical(getOption(\"show.error.messages\"), TRUE)) {" + 			Constants.NEWLINE + //$NON-NLS-1$
						"    cat(msg, file = stderr())" +										Constants.NEWLINE + //$NON-NLS-1$
						"    .Internal(printDeferredWarnings())" +								Constants.NEWLINE + //$NON-NLS-1$
						"}" +																	Constants.NEWLINE + //$NON-NLS-1$
						"}else{" +																Constants.NEWLINE + //$NON-NLS-1$
						"stop(e)}" +															Constants.NEWLINE + //$NON-NLS-1$
						"}, envir="+RCommand.getRflowEnvCommand()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		R.eval("lockEnvironment("+RCommand.getRflowEnvCommand()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		//linuxの場合pagerを変更
		if(Platform.getPlatform() == Platform.LINUX || Platform.getPlatform() == Platform.MAC){
			R.eval("options(pager=.rflow.pager)");
			R.eval(".jinit()");
		}
		
		//ツリー初期化
		try {
			File file = FileResource.USER_FUNCTION_FILE;
			if(!file.exists())
				file = FileResource.FUNCTION_FILE;
			
			if(file.exists())
				explorer.getFunctionExplorer().importFunctionTree(file, groupManager);
		} catch (Exception e) {
			//JOptionPane.showMessageDialog(frame, "関数ツリーの読み込みに失敗しました。", "エラー", JOptionPane.ERROR_MESSAGE);
		}
		
		//警告表示を元に戻す
		//R.eval("options(warn=old.warn)");
		//R.eval("rm(old.warn)");
		
		//初期化
		init(false, null, null, new File((String)setting.getProperty(Setting.Key.DEFAULT_WORKING_DIRECTORY)), null, null);
		
		//コンソール用のメインループを開始
		R.startMainLoop();
		//receiver.passCommand("Sys.setlocale(\"LC_ALL\", \""+Locale.getDefault().getLanguage()+"\")");
		
		try {
			SwingUtilities.invokeAndWait(new Runnable(){
				public void run() {
					frame.setVisible(true);
					console.setVisible(true);
				}});
		} catch (InterruptedException e1) {
		} catch (InvocationTargetException e1) {
		}

		checkUpdate();
		
		//run backup thread
		backupThread.start();
	}
	
	private static void setLookAndFeel(){
		//Linux対応
		new JPanel();
		//look&feel
		try{
			if(System.getProperty("swing.defaultlaf") == null){
				if(Platform.getPlatform() == Platform.LINUX){
					//GDK lookandfeel　の場合、desktoppaneで下にタスクバーが作成されて、
					//水平スクロールバーが見えなくなってしまうので、linuxにおいてはデフォルトでメタルを使用する
			        UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
				}else{
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
			}
	        //if(Rflow.TEST)
//	        	UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
//	        UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
//	        UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
//	        UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
//	        UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel");
			
			//tooltip
			ToolTipManager.sharedInstance().setInitialDelay(0);
			ToolTipManager.sharedInstance().setDismissDelay(Integer.MAX_VALUE);
			
			
	    }catch(Exception ex){
	    	ex.printStackTrace();
	    }
	}
	
	private void setResource(){
		//リソースバンドルを言語設定にしたがって取得
		Messages.prepareResourceBundle();
		//Swing Component のリソースを設定
		UIManager.put("AbstractUndoableEdit.undoText", Messages.getString("Action.undo")); //$NON-NLS-1$ //$NON-NLS-2$
		UIManager.put("AbstractUndoableEdit.redoText", Messages.getString("Action.redo")); //$NON-NLS-1$ //$NON-NLS-2$
		//UIManager.put("OptionPane.yesButtonText", Messages.getString("Button.yes"));
		//UIManager.put("OptionPane.noButtonText", Messages.getString("Button.no"));
		UIManager.put("OptionPane.cancelButtonText", Messages.getString("Button.cancel")); //$NON-NLS-1$ //$NON-NLS-2$
		UIManager.put("OptionPane.okButtonText", Messages.getString("Button.ok")); //$NON-NLS-1$ //$NON-NLS-2$
		UIManager.put("FileChooser.cancelButtonText", Messages.getString("Button.cancel")); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	private void initializeNodeModelManager(){
		NodeModelManager.addNodeModel(SentenceNodeModel.class, new SentenceNodeModelController());
		NodeModelManager.addNodeModel(FreeNodeModel.class, new FreeNodeModelController());

		NodeModelManager.addNodeModel(SubflowNodeModel.class, new SubflowNodeModelController());
		NodeModelManager.addNodeModel(LoopNodeModel.class, new LoopNodeModelController());
		NodeModelManager.addNodeModel(FunctionNodeModel.class, new FunctionNodeModelController());
		NodeModelManager.addNodeModel(IfNodeModel.class, new IfNodeModelController());
		NodeModelManager.addNodeModel(TunnelNodeModel.class, new TunnelNodeModelController());
		
		for(NodeModelController controller : NodeModelManager.getModelControllerList()){
			if (controller instanceof DefaultNodeModelController) {
				((DefaultNodeModelController) controller).install(this);
			}
		}
		
		if(Rflow.TEST){
			//for(UIModelPack pack : UIModelPackFactory.getPacks())
			//	UIPackManager.addUIPack(pack);
			File uiFolder = new File(Constants.getPath("plugins/ui")); //$NON-NLS-1$
			if(uiFolder.exists())
				for(File ui : uiFolder.listFiles()){
					if(ui.getName().startsWith("."))continue; //$NON-NLS-1$
					UIPackReader reader = new UIPackReader();
					try {
						new FlowDataReader().read(ui, new ElementReader[]{reader});
						for(UIPack pack : reader.getUIPacks())
							UIPackManager.addUIPack(pack);
					} catch (ReadException e) {
						// TODO Auto-generated catch block
						if(Rflow.DEBUG > 0)e.printStackTrace();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						if(Rflow.DEBUG > 0)e.printStackTrace();
					} catch (Exception e){
						if(Rflow.DEBUG > 0)e.printStackTrace();
					}
				}
		}
	}
	
	public FlowViewer getActiveViewer() {
		return activeViewer;
	}
	
	public void setActiveViewer(FlowViewer viewer){
		if(activeViewer != viewer){
			activeViewer = viewer;
			for(ActiveViewerListener l : activeViewerListenerList){
				l.activeViewerChanged(activeViewer);
			}
		}
	}
	
	public void addActiveViewerListener(ActiveViewerListener listener){
		activeViewerListenerList.add(listener);
	}
	
	public void removeActiveViewerListener(ActiveViewerListener listener){
		activeViewerListenerList.remove(listener);
	}
	
	public List<FlowViewer> getViewers(){
		return viewers;
	}
	
	public void addViewer(FlowViewer viewer){
		viewers.add(viewer);

		if(viewer.isRootViewer()){
			FlowState state = viewer.getPickedState();
			state.addItemListener(codeBar);
			state.addItemListener(actionManager);
			//state.addItemListener(codeViewer);
			state.addItemListener(explorer.getHierarchyExplorer());
			FlowNode.addNodeModelChangeListener(state);
			FlowNode.addNodeModelChangeListener(viewer.getCacheHandler());
		}
		FlowNode.addNodeModelChangeListener(viewer);
		FlowNode.addNodePropertyChangeListener(viewer);
	}
	
	public void removeViewer(FlowViewer viewer){
		viewers.remove(viewer);
		
		FlowNode.removeNodeModelChangeListener(viewer);
		FlowNode.removeNodePropertyChangeListener(viewer);
	}
	
	void quit(){
		if(!actionManager.askSave()){
			return;
		}
		
		onExit();
		
		//wait for r quit
		try {
			Thread.sleep(500);
		} catch (InterruptedException e1) {
		}
		
		
		if(Rflow.DEBUG > 0)
			System.out.println("system exit"); //$NON-NLS-1$
		System.exit(0);
	}
	
	private void onExit(){
		try {
			//save user function tree
			explorer.getFunctionExplorer().save(FileResource.USER_FUNCTION_FILE);
			
			//save setting
			PropertiesUtility.store(setting, FileResource.SETTING_FILE);
		} catch (IOException e) {
			// TODO メッセージの表示を検討
			if(Rflow.DEBUG > 0)e.printStackTrace();
		} catch (XMLException e) {
			// TODO メッセージの表示を検討
			if(Rflow.DEBUG > 0)e.printStackTrace();
		}
		
		//delete temp files
		//ignore if failed
		fileControl.delete_file(Constants.TEMP_DIR, false);
		fileControl.delete_file(getTempDir(), false);
		
		//delete empty cache folder
		File cacheDir = viewer.getCacheHandler().getCacheDir(false);
		if(cacheDir.exists() && cacheDir.listFiles().length == 0){
			//ignore if failed
			fileControl.delete_file(cacheDir, false);
		}
		
		//空のフローフォルダの消去
		if(!isNewFlow())
			FlowFolderSupport.cleanFlowDir(fileControl, getFlowDir());
		
		//terminate R
		//on quit R deletes RTmp dir
		//this command seems to terminates whole program process
//		R.eval("q(save=\"no\")");
		receiver.passCommand("q(save=\"no\")"); //$NON-NLS-1$
		
	}


	/**
	 * 新規ファイル時の初期化
	 */
	void init(){
		init(true);
	}
	
	/**
	 * 新規ファイル時の初期化
	 * @param clearObject objectをクリアするか
	 */
	void init(boolean clearObject){
		init(clearObject, null, null, null, null, null);
	}
	
	/**
	 * 新規ファイル時の初期化
	 * @param clearObject objectをクリアするか
	 * @param workingFile 作業ファイル
	 * @param flowSetting ファイルの設定
	 * @param workingDirectory 作業ディレクトリ
	 * @param layout 新しいフロー null if empty
	 * @parem state 新しいフローstate null if empty
	 */
	void init(boolean clearObject, 
			File workingFile, 
			FlowSetting flowSetting, 
			File workingDirectory,
			ViewLayout layout,
			FlowState state)
	{
		//rflow.R.rniStop(0);
		if(clearObject)
			RCommand.clearObjects();
		viewer.clearFlow();
		
		if(clearObject)
			console.clearConsole();
		codeBar.clear();
		
		//clear last flow temp
		fileControl.delete_file(getTempDir(), false);//ignore failed
		
		
		setFlowSetting(flowSetting);
		setWorkingFile(workingFile);
		setWorkingDirectory(workingDirectory);
		
		//フローフォルダの設定
		//フローフォルダが保存されていなかった場合、ファイル名に設定
		if(!isNewFlow())
			FlowFolderSupport.setFolderDefault(getFlowSetting(), getWorkingFile());
		//ネームファイルとファイル名の一致
		if(!isNewFlow())
			if(!FlowFolderSupport.checkFlowFolder(this, layout, state)){
				//キャンセルされた場合、新規フロー
				init();
				return;
			}
			
		
		//getFlowSetting().setCacheDir(workingFile);//
		setDirty(false);
		
		if(layout != null){
			viewer.init(layout);
			//キャッシュ情報をステートに追加
			if(state != null)
				viewer.getPickedState().add(state);
			viewer.getCacheHandler().init();//set command to cache data
			//if((Boolean)setting.getProperty(Setting.Key.CHECK_CACHE_WHEN_OPENED))
			viewer.getCacheHandler().update();
			viewer.repaint();
		}
		
		explorer.getHierarchyExplorer().update();
		
		//フォルダログの更新
		updateFolderlog();
	}
	
	public void setWorkingFile(File workingFile) {
		this.workingFile = workingFile;
		frame.setFileName(workingFile, isDirty());
	}
	
	public File getWorkingFile() {
		return workingFile;
	}
	
	public boolean isNewFlow(){
		return workingFile == null;
	}
	
	public File getFlowDir(){
		if(isNewFlow()){
			return new File(Constants.TEMP_DIR, "new_flow"); //$NON-NLS-1$
		}else{
			String folder = Constants.RFLOW_FOLDER + "/"; //$NON-NLS-1$
			folder += (String)getFlowSetting().getProperty(FlowSetting.Key.FOLDER);
			return new File(getWorkingFile().getParent(), folder);
		}
	}
	
	public File getTempDir(){
		return new File(getFlowDir(), Constants.FILE_TEMP_DIR);
	}
	
	public void setFlowSetting(FlowSetting flowSetting) {
		if(flowSetting == null){
			flowSetting = FlowSetting.createDefaultSettings();
		}
		if(!this.flowSetting.equals(flowSetting)){	
			this.flowSetting = flowSetting;
			for(FlowViewer fv : viewers)
				fv.updateFlowSetting();
			setDirty(true);
		}
	}
	
	public FlowSetting getFlowSetting() {
		return flowSetting;
	}
	

	/**
	 * 作業ディレクトリを変更します。
	 * nullを渡すとデフォルト値に設定します。
	 * @param file 
	 */
	public void setWorkingDirectory(File file){
		setWD(file);
		explorer.getFileExplorer().update();
	}
	
	private static void setWD(File file){
		//nullの場合変更しない
		if(file == null){
			return;
		}

		String dir = Constants.replaceFileSeparator(file.getAbsolutePath());
		//RCommand.getR().eval("try(setwd(\""+dir+"\"), silent=TRUE)"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-2
		//すぐにgetwd()を実行し変更が正常に終了したかチェックする
		boolean success = false;
		/*REXP wdEXP = RCommand.getR().eval("try(getwd(), silent=TRUE)"); //$NON-NLS-1$
		if(wdEXP != null){
			//success = dir.equals(wdEXP.asString());
			success = dir.equals(Constants.replaceFileSeparator(wdEXP.asString()));
		}*/		
		success = RCommand.setWD(dir);
		
		if(success){
			Prefs.WORKING_DIRECTORY = file;
		}
	}
	
	public void setDirty(boolean isDirty) {
		if(this.isDirty != isDirty){
			frame.setFileName(workingFile, isDirty);
		}
		this.isDirty = isDirty;
		setDirtySub(isDirty);
	}
	
	public boolean isDirty() {
		return isDirty;
	}
	
	/**
	 * 自動バックアップ用のダーティフラグの設定
	 * @param isDirtySub
	 */
	public void setDirtySub(boolean isDirtySub) {
		this.isDirtySub = isDirtySub;
	}

	/**
	 * 自動バックアップ用のダーティフラグ
	 * @param isDirtySub
	 */
	public boolean isDirtySub() {
		return isDirtySub;
	}
	
	public static boolean isBusy(){
		return IS_BUSY;
	}
	
	/**
	 * フォルダログの更新
	 * @param workingFile
	 */
	public void updateFolderlog() {
		if(!isNewFlow() && getFlowDir().exists()){
			if(!folderlog.contains(workingFile.getParentFile().getAbsolutePath())){
				folderlog.add(workingFile.getParentFile().getAbsolutePath());
				try {
					PropertiesUtility.writeList(FileResource.FOLDERLOG_FILE, new ArrayList<String>(folderlog));
				} catch (IOException e) {
					//エラーは無視する
					e.printStackTrace();
				}
			}
		}
	}

	public static void main(String[] args) {	

		checkIsFirst();
		
		Rflow rflow = 
			new Rflow();
			
		//.rflowファイルを開く
		if(args.length == 1 && args[0].endsWith(Constants.RFLOW_FILE_EXTENSION))
		{        	
			File file = new File(args[0]);
			
			if(file.exists()){
				rflow.actionManager.open(file, true);
			}else{
				JOptionPane.showMessageDialog(rflow.frame, 
						Messages.getString("Error.file.file_not_found")+" : "+file.getAbsolutePath(),  //$NON-NLS-1$ //$NON-NLS-2$
						Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			}
		}
	}


	private static void checkUpdate() {
		try {
			if((Boolean)setting.getProperty(Key.IS_AUTO_UPDATE))
			{
				UpdateChecker.updateCheck();
			}
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			if(Rflow.DEBUG > 0)e.printStackTrace();
		}
	}

	private static final String NEW_LINE_CODE =
		System.getProperty("line.separator"); //$NON-NLS-1$
	
	private static void checkIsFirst() {

		Setting setting = Setting.defaultSetting();
		try {
			PropertiesUtility.load(setting, FileResource.SETTING_FILE);
		} catch (FileNotFoundException e1) {
			//do nothing
		} catch (IOException e1) {
			//do nothing
		}
		
		Locale.setDefault(setting.getLocale());
				
		if((Boolean)setting.getProperty(Key.IS_FIRST))
		{
			//設定ファイルが存在しない
			
			setLookAndFeel();

			//プレビュー版の設定ファイルが存在する場合、確認して消去する
			File file = new File(Platform.getUserDir(),"rflow.settings"); //$NON-NLS-1$
			if(file.exists())
				try {
					SwingUtilities.invokeAndWait(new Runnable(){
						public void run() {
							int value = JOptionPane.showConfirmDialog(null, Messages.getString("Main.ask_del_preview_setting"),Messages.getString("Message.confirm"),JOptionPane.YES_NO_OPTION); //$NON-NLS-1$ //$NON-NLS-2$
							
							if(value == JOptionPane.YES_OPTION)
							{
								try
								{
									new File(Platform.getUserDir(),"rflow.settings").delete(); //$NON-NLS-1$
									new File(Platform.getUserDir(),"run.vbs").delete(); //$NON-NLS-1$
									new File(Platform.getUserDir(),"function.xml").delete(); //$NON-NLS-1$
								}
								catch(RuntimeException ex)
								{
								}
							}
						}});
				} catch (InterruptedException e1) {
				} catch (InvocationTargetException e1) {
				}

			//更新をチェックするか確認する
			int ans = 
				JOptionPane.showConfirmDialog(null, 
						Messages.getString("Rflow.first_update_option_check.1") + //$NON-NLS-1$
						NEW_LINE_CODE +
						Messages.getString("Rflow.first_update_option_check.2") + //$NON-NLS-1$
						NEW_LINE_CODE +
						NEW_LINE_CODE +
						Messages.getString("Rflow.first_update_option_check.3") //$NON-NLS-1$
						,

						Messages.getString("Message.select"), //$NON-NLS-1$
						JOptionPane.YES_NO_OPTION
				);

			setting.setProperty(Key.IS_AUTO_UPDATE, (ans == JOptionPane.YES_OPTION));
			setting.setProperty(Key.IS_FIRST, false);

			try {
				PropertiesUtility.store(setting, FileResource.SETTING_FILE);
			} catch (IOException e) {
				//TODO
				if(Rflow.DEBUG > 0)e.printStackTrace();
			}
			
			
		}
	}
	
	
	
}
