package com.ef_prime.rflow;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.SentenceInfo;
import com.ef_prime.rflow.node.base.FreeNodeModel;
import com.ef_prime.rflow.node.base.SentenceNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowFileFilterFactory;
import com.ef_prime.rflow.util.FileChooserUtility;

public class RflowUtil {
	
	//v1よりv2の方が新しかったらtrue;
	public static boolean checkVersion(String v1,String v2)
	{
		String[] s1 = v1.split("\\."); //$NON-NLS-1$
		String[] s2 = v2.split("\\."); //$NON-NLS-1$
		try
		{
			for(int i = 0;i < s1.length && i < s2.length;i++)
			{
				int t1 = Integer.valueOf(s1[i]);
				int t2 = Integer.valueOf(s2[i]);
				if(t1 < t2)return true;
				if(t1 > t2)return false;
			}
		}
		catch(RuntimeException ex)
		{
			//ex.printStackTrace();
		}
		return false;
	}
	
	/**
	 * choose source file and parse to ViewLayout.
	 * @param parent dialog parent
	 * @return ViewLayout of parsed source file. null if canceled or failed
	 */
	public static ViewLayout importSource(Component parent){

		FileFilter[] filters = new FileFilter[]{RflowFileFilterFactory.createRFileFilter()};
		File file = FileChooserUtility.chooseOpenFile(Messages.getString("Message.open"), Prefs.WORKING_DIRECTORY, filters, parent); //$NON-NLS-1$
		if(file == null)
			return null;
		
		NodeModel[] models = convertToModel(RCommand.parseToNodes(file), true);
		if(models == null){
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.read"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return null;
		}

		return new ViewLayout(models);
	}
	
	public static ViewLayout importSourceFromClipboard(Component parent){
		Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
		if(clip.isDataFlavorAvailable(DataFlavor.stringFlavor)){
			String data;
			try {
				data = (String)clip.getData(DataFlavor.stringFlavor);
			} catch (UnsupportedFlavorException e) {
				return null;
			} catch (IOException e) {
				JOptionPane.showMessageDialog(parent, e.getMessage(),
						Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
				return null;
			}
			
			NodeModel[] models = convertToModel(RCommand.parseToNodes(data), true);
			if(models == null){
				JOptionPane.showMessageDialog(parent, Messages.getString("Error.read"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
				return null;
			}
			
			return new ViewLayout(models);
		}
		return null;
	}

	public static NodeModel[] convertToModel(String command){
		return convertToModel(command, true);
	}
	
	public static NodeModel[] convertToModel(String command, boolean ctrlFuncIsFreeNode){
		return convertToModel(RCommand.parseToNodes(command), ctrlFuncIsFreeNode);
	}
	
	public static NodeModel[] convertToModel(SentenceInfo[] infos, boolean ctrlFuncIsFreeNode){
		if(infos == null)return null;
		NodeModel[] models = new NodeModel[infos.length];
		for(int i = 0; i < infos.length; i++){
			try {
				SentenceInfo info = infos[i];
				if(info.isControl() && ctrlFuncIsFreeNode){
					FreeNodeModel free = new FreeNodeModel();
					free.setCommand(info.getLine());
					models[i] = free;
				}else{
					SentenceNodeModel sentence = new SentenceNodeModel();
					sentence.setCommand(info);
					models[i] = sentence;
				}
			} catch (IncompatibleCommandException e) {
				return null;
			}
		}
		return models;
	}

	public static SentenceInfo retrieveSentenceInfo(String command){
		SentenceInfo[] infos = RCommand.parseToNodes(command);
		if(infos == null || infos.length > 1){
			return null;
		}
		return infos[0];
	}
	

	/**
	 * command without assign part
	 * @return
	 */
	public static String getSentence(String command){
		SentenceInfo info = retrieveSentenceInfo(command);
		if(info == null)return command;
		
		if(info.isAssign()){
			String c = command.substring(info.getSymbol().length(), command.length());
			c = c.trim();
			c = c.substring((c.charAt(0)=='=')?1:2, c.length());
			return c.trim();
		}else{
			return command;
		}
	}
	
	
	public static Map<String, String> getArgHelp(String topic){
		String s = RCommand.getHelpFile(topic);
		if(s == null)return null;
		String[] val = s.split(Constants.DELIM);
		
		File file = new File(val[0]);
		boolean delete = Boolean.parseBoolean(val[1]);
		
		String b = "_\b"; //$NON-NLS-1$
		String arg = "_\bA_\br_\bg_\bu_\bm_\be_\bn_\bt_\bs"; //$NON-NLS-1$
		boolean inArg = false;
		
		Map<String, String> map = new HashMap<String, String>();
		String param = null;
		String des = null;
		
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8")); //$NON-NLS-1$
			while(reader.ready()){
				String line = reader.readLine();
				if(!inArg && line.startsWith(arg))
					inArg = true;
				else if(inArg){
					if(line.equals("")){ //$NON-NLS-1$
						if(param != null){
							map.put(param, des);
							param = null;
							des = null;
						}
					}else{
						if(!line.startsWith("          ")){ //$NON-NLS-1$
							int index = line.indexOf(':');
							param = line.substring(0, index).trim();
							des = line.substring(index+1);
						}else{
							des += Constants.NEWLINE + line.trim();
						}
					}
					if(line.startsWith(b))
						break;
				}
			}
		} catch (IOException e) {
			
		} finally{
			try {
				if(reader != null)reader.close();
			} catch (IOException e) {
			}
		}
		
		if(delete)
			file.delete();
		
		return map;
	}
}
