package com.ef_prime.rflow;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Locale;

import com.ef_prime.rflow.util.TypeDefProperties;

public class Setting extends TypeDefProperties{

	public static enum Key implements TypeDefProperties.Key{
		
		/** 接続するＲのディレクトリ */
		R_HOME(Type.STRING, ""), //$NON-NLS-1$
		
		/** ライブラリーパックを使用するか */
		USE_LIBLARY_PACK(Type.BOOLEAN, true),
		
		CHECKED_VERSION(Type.STRING, Version.VERSION),
		IS_FIRST(Type.BOOLEAN, true),
		IS_AUTO_UPDATE(Type.BOOLEAN, true),
		
		/**
		 * デフォルトの作業ディレクトリ
		 * Rflow起動後に空文字だった場合にRflow側から
		 * 適切な初期値に置き換えることとする
		 */
		DEFAULT_WORKING_DIRECTORY(Type.STRING, ""), //$NON-NLS-1$

		/** 言語 */
		LANGUAGE(Type.STRING, LANG_SYSTEM),
		
		/** エラーのチェックをするか */
		CHECK_ERROR(Type.BOOLEAN, true),
		
		/** グリッドのon/off */
		USE_GRID(Type.BOOLEAN, true),
		
		/**  グリッドの幅 */
		GRID_DISTANCE(Type.INTEGER, 10),
		
		/** コンソールで表示にバッファーを使用するか */
		USE_CONSOLE_BUFFER(Type.BOOLEAN, true),
		
		///** 分析を開いた時に自動的にキャッシュファイルを検証するか */
		//CHECK_CACHE_WHEN_OPENED(Type.BOOLEAN, true),
		
		/** キャッシュのバックアップ（元に戻すで復元）を使用するか */
		USE_CACHE_BACKUP(Type.BOOLEAN, true),
		
		///** 更新日時をキャッシュのチェックに使う */
		//CHECK_CACHE_LASTMODIFIED(Type.BOOLEAN, false),
		
		/** 自動バックアップするか */
		AUTO_BACKUP(Type.BOOLEAN, true),
		
		/** 自動バックアップの間隔（sec） */
		AUTO_BACKUP_INTERVAL(Type.INTEGER, 60),
		
		/** ファイルのパスの最大長(バイト数) */
		MAX_PATH(Type.INTEGER, 255);
		
		private Object defaultValue;
		private Type type;
		
		private Key(Type type, Object defaultValue){
			this.defaultValue = defaultValue;
			this.type = type;
		}

		public Object getDefaultValue() {
			return defaultValue;
		}

		public Type getType() {
			return type;
		}
		
		public String getKeyString() {
			return toString();
		}
	}

    private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
    
    public static final String LANG_SYSTEM = "system"; //$NON-NLS-1$
    
    /**
     * 言語設定の取得
     * @return
     */
    public Locale getLocale(){
    	String locale = (String)getProperty(Key.LANGUAGE);
    	if(locale.equals(LANG_SYSTEM))
    		return Locale.getDefault();
    	else
    		return new Locale(locale);
    }
    
	/**
	 * 初期設定を返します。
	 * @return
	 */
	public static Setting defaultSetting()
	{
		return (Setting)setDefaultProperties(new Setting(), Key.values());
	}
	
	/**
	 * 保存されていたSettingと現在のSettingクラスのバージョンの違いに
	 * 対応するための更新メソッドです。
	 * ・現在使用されていないキーの消去
	 * ・新しく追加されたキーをデフォルト値で登録
	 */
	public void update(){
		update(Key.values());
		
		if(RflowUtil.checkVersion((String)getProperty(Key.CHECKED_VERSION), Version.VERSION)){
			setProperty(Key.CHECKED_VERSION, Version.VERSION);
		}
	}
	
	public Setting copy(){
		Setting setting = new Setting();
		setting.setPropertyAll(this);
		return setting;
	}
	
	@Override
	public synchronized Object setProperty(String key, String value) {
		Object o = super.setProperty(key, value);
		changeSupport.firePropertyChange(key, o, value);
		return o;
	}


    public synchronized void addPropertyChangeListener(PropertyChangeListener listener){
        changeSupport.addPropertyChangeListener(listener);
    }

    public synchronized void removePropertyChangeListener(PropertyChangeListener listener){
        changeSupport.removePropertyChangeListener(listener);
    }

    public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
        return changeSupport.getPropertyChangeListeners();
    }

    public synchronized void addPropertyChangeListener(Key key, PropertyChangeListener listener){
    	addPropertyChangeListener(key.getKeyString(), listener);
    }
    
    public synchronized void addPropertyChangeListener(String propertyName, PropertyChangeListener listener){
        changeSupport.addPropertyChangeListener(propertyName, listener);
    }

    public synchronized void removePropertyChangeListener(String propertyName, PropertyChangeListener listener){
        changeSupport.removePropertyChangeListener(propertyName, listener);
    }
    
    public synchronized PropertyChangeListener[] getPropertyChangeListeners(String propertyName){
        return changeSupport.getPropertyChangeListeners(propertyName);
    }
    
}
