package com.ef_prime.rflow.console;

import java.util.ArrayList;
import java.util.List;

public abstract class AbstractCommand implements Command{

	private Command parent;
	private boolean checkError = false;
	private boolean isStarted = false;
	private boolean isFinished = false;
	private boolean isCanceled = false;
	private boolean isErrorOccurred = false;
	private List<CommandListener> listeners;

	public AbstractCommand() {
		super();
	}
	
	public Command getParent() {
		return parent;
	}
	
	public void setParent(Command parent) {
		this.parent = parent;
	}
	
	public boolean isRoot() {
		return getParent() == null;
	}
	
	public boolean checkError() {
		return checkError;
	}
	
	public void setCheckError(boolean check) {
		checkError = check;
		if(getCommands() != null)
		for(Command c : getCommands()){
			c.setCheckError(check);
		}
	}
	
	public boolean isStarted() {
		return isStarted;
	}

	public void start() {
		if(!isStarted){
			isStarted = true;
			fireCommandStart();
		}
	}

	public final synchronized boolean isFinished() {
		return isFinished;
	}

	public final synchronized void finish() {
		isFinished = true;
		fireCommandFinished();
		notifyAll();
	}
	
	public boolean isCanceled() {
		return isCanceled;
	}
	
	public void cancel() {
		if(!isFinished){
			isCanceled = true;
			fireCommandCanceled();
		}
	}
	
	public boolean isErrorOccurred() {
		return isErrorOccurred;
	}
	
	public void error() {
		isErrorOccurred = true;
		Command c = this;
		if (!c.isRoot()) {
			c.getParent().error();
		}
		fireErrorOccurred();
	}

	public final synchronized void waitForCommand() {
		while(!isFinished){
			try {
				wait(50);
			} catch (InterruptedException e) {
			}
		}
	}

	public final synchronized void addCommandListener(CommandListener listener) {
		if(listeners == null){
			listeners = new ArrayList<CommandListener>();
		}
		listeners.add(listener);
	}

	public final synchronized void removeCommandListener(CommandListener listener) {
		if(listeners != null){
			listeners.remove(listener);
		}
	}

	protected final void fireCommandFinished() {
		if(listeners != null){
			for(CommandListener l : listeners)
				l.commandFinished(this);
		}
	}
	
	protected final void fireCommandStart(){
		if(listeners != null){
			for(CommandListener l : listeners)
				l.commandStart(this);
		}
	}

	protected final void fireCommandCanceled(){
		if(listeners != null){
			for(CommandListener l : listeners)
				l.commandCanceled(this);
		}
	}

	protected final void fireErrorOccurred(){
		if(listeners != null){
			for(CommandListener l : listeners)
				l.errorOccurred(this);
		}
	}
}