package com.ef_prime.rflow.console;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;

import org.rosuda.JRI.RMainLoopCallbacks;
import org.rosuda.JRI.Rengine;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Platform;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.base.SentenceNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.CodingTextPane;
import com.ef_prime.rflow.util.AbstractActionManager;
import com.ef_prime.rflow.util.FileChooserUtility;
import com.ef_prime.rflow.util.NoWrapEditorKit;
import com.ef_prime.rflow.util.SizeLimitVector;
import com.ef_prime.rflow.util.SwingUtils;

public class RConsole implements RMainLoopCallbacks, PropertyChangeListener{

	/**
	 * コマンドの履歴
	 */
	private final SizeLimitVector<String> commandHistory = new SizeLimitVector<String>(512);

	private int histPos = commandHistory.size();
	
	private Rflow rflow; 
	
//	private JDialog dialog;
	private JFrame dialog;
	private JTextPane output;
	private CodingTextPane input;
	private JScrollPane outputSp;
	private CommandReceiver receiver;
	
	private boolean useBuffer;
	private static final int BUFFER_SIZE = 1024;//number of char
	private StringBuffer buffer = new StringBuffer(BUFFER_SIZE);
	private int currentOutputType = 0;
	
	private Command currentCommand = new BasicCommand(""); //$NON-NLS-1$
	
	private ConsoleActionManager manager = new ConsoleActionManager();
	
	private Queue<String> commandQueue = new LinkedList<String>();
	
	private Object mutex = new Object();
	
	public RConsole(Rflow rflow){
		this.rflow = rflow;
		this.receiver = rflow.receiver;
		useBuffer = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CONSOLE_BUFFER);
		Rflow.setting.addPropertyChangeListener(Setting.Key.USE_CONSOLE_BUFFER, this);
		
		addBusyListener(manager);
		
		output = new JTextPane();
		output.setFont(Prefs.CODE_FONT);
		output.setEditable(false);
		output.setEditorKit(new NoWrapEditorKit());
		SwingUtils.setTabs(output, Prefs.CHAR_PER_TAB);
		output.addMouseListener(new MouseAdapter(){//ポップアップ動作
			@Override
			public void mouseClicked(MouseEvent e) {
				if(SwingUtilities.isRightMouseButton(e)){
					createPopup().show((Component)e.getSource(), e.getX(), e.getY());
				}
			}
			
			private JPopupMenu createPopup(){
				final int mask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

				Action copy = new DefaultEditorKit.CopyAction();
				copy.putValue(Action.NAME, Messages.getString("Action.copy")); //$NON-NLS-1$
				copy.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, mask));

				JPopupMenu popup = new JPopupMenu();
				popup.add(copy);
				
				return popup; 
			}
		});
		
		outputSp = new JScrollPane(output, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		outputSp.setBorder(new EmptyBorder(0,0,0,0));
		
		input = new CodingTextPane();
		input.setIntend(false);
		input.setPreferredSize(new Dimension(400, 50));
		input.setMaximumSize(new Dimension(400, 50));
		input.addKeyListener(new InputKeyHandler());
		input.setBorder(new LineBorder(Color.GRAY, 1));
		
		//dialog = new JDialog(rflow.frame, "R console", ModalityType.MODELESS){
//		dialog = new JDialog(rflow.frame, "R console"){
		dialog = new JFrame("R console"); //$NON-NLS-1$
//		{
//			@Override
//			public void setVisible(boolean b) {
//				super.setVisible(b);
//				Actions.CONSOLE_WINDOW.putValue(Actions.SELECTED_KEY, b);
//			}
//		};
		dialog.setIconImage(FileResource.DESKTOP_16.getImage());
		dialog.getContentPane().setLayout(new BorderLayout());
		//dialog.getContentPane().add(createMenuBar(), BorderLayout.NORTH);
		dialog.getContentPane().add(outputSp, BorderLayout.CENTER);
		dialog.getContentPane().add(input, BorderLayout.SOUTH);
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		dialog.setJMenuBar(manager.createMenuBar());
		//dialog.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		//dialog.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
		//サイズの変更をＲに通知
		dialog.addComponentListener(new ComponentAdapter(){
			@Override
			public void componentResized(ComponentEvent e) {
				int charWidth = output.getFontMetrics(output.getFont()).charWidth('R');
				
				int width = output.getVisibleRect().width / charWidth - 1;
				if(width < 10)width = 10;
				else if(width > 10000)width = 10000;
				if(RCommand.getR() != null)
					RCommand.getR().idleEval("options(width="+width+")"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
		

		Rectangle rect = SwingUtils.getScreenBounds();
		dialog.setSize(600, 400);
		dialog.setLocation(25, rect.height-dialog.getHeight()-50);
		//dialog.setLocationRelativeTo(parent);
		//dialog.setVisible(true);
	}
	
	private static enum ConsoleAction{
		RUN,
		CREATE_NODE,
		STOP
		;
	}
	
	private class ConsoleActionManager extends AbstractActionManager<ConsoleAction> implements BusyListener{
		ConsoleActionManager(){
			putAction(ConsoleAction.RUN, new RunAction());
			putAction(ConsoleAction.CREATE_NODE, new CreateNodeAction());
			putAction(ConsoleAction.STOP, new StopAction());
		}
		
		public void busyStateChanged(int which) {
			boolean notBusy = (which == BusyListener.NOT_BUSY);
			getAction(ConsoleAction.RUN).setEnabled(notBusy);
			getAction(ConsoleAction.CREATE_NODE).setEnabled(notBusy);
		}
		
		public JMenuBar createMenuBar(){
			JMenuBar menuBar = new JMenuBar();
			JMenu command = new JMenu(Messages.getString("Menu.command")); //$NON-NLS-1$
			command.add(getAction(ConsoleAction.RUN));
			command.add(getAction(ConsoleAction.CREATE_NODE));
			command.add(getAction(ConsoleAction.STOP));
			//command.add(rflow.actionManager.getAction(ActionKey.STOP));
			
			menuBar.add(command);
			
			return menuBar;
		}
		
		private class RunAction extends AbstractAction{
			public RunAction() {
				putValue(Action.NAME, Messages.getString("Action.run")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
			}
			public void actionPerformed(ActionEvent e) {
				submitCommand();
			}
		}
		
		private class CreateNodeAction extends AbstractAction{
			public CreateNodeAction() {
				putValue(Action.NAME, Messages.getString("Action.new_node")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, Constants.MENU_KEY_MASK));
			}
			public void actionPerformed(ActionEvent e) {
				createNewNode();
			}
		}
		
		private class StopAction extends AbstractAction{
			StopAction(){
				putValue(Action.NAME, Messages.getString("Action.stop")); //$NON-NLS-1$
				putValue(Action.SMALL_ICON, FileResource.ACTION_STOP);
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0));
			}
			
			public void actionPerformed(ActionEvent e) {
    			if(//!(Platform.getPlatform() == Platform.MAC) || 
    					Rflow.isBusy()){
    				rflow.actionManager.stop();
    			}
			}
		}
	}
	
	/*private JMenuBar createMenuBar(){
		final int mask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		
		JMenuBar menuBar = new JMenuBar();
		
		JMenu command = new JMenu(Messages.getString("Menu.command")); //$NON-NLS-1$
		
		Action submit = new AbstractAction(Messages.getString("Action.run")){ //$NON-NLS-1$
			public void actionPerformed(ActionEvent e) {
				submitCommand();
			}
		};
		submit.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
		
		Action create = new AbstractAction(Messages.getString("Action.new_node")){ //$NON-NLS-1$
			public void actionPerformed(ActionEvent e) {
				createNewNode();
			}
		};
		create.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, mask));
		
		command.add(submit);
		command.add(create);
		command.add(rflow.actionManager.getAction(ActionKey.STOP));
		
//		JMenu display = new JMenu("表示");
//		display.add(Actions.CONSOLE_WINDOW);

		menuBar.add(command);
//		menuBar.add(display);
		
		return menuBar;
	}*/
	
	public boolean isVisible(){
//		return dialog.getHeight() > 50;
		return dialog.isVisible();
	}
	
	//private int previousHeight = 400;
	public void setVisible(boolean visible){
		//if(visible)
		dialog.setVisible(visible);
		/*
		if(visible)
			dialog.setSize(dialog.getWidth(), previousHeight);
		else
			dialog.setSize(dialog.getWidth(), 0);
			*/
	}
	
	public void clearConsole(){
		try {
			Document outputDoc = output.getDocument();
			int len = outputDoc.getLength();
			if(len > 1 && outputDoc.getText(len-2, 2).equals("> ")) //$NON-NLS-1$
				len -= 2;
			outputDoc.remove(0, len);
			input.getDocument().remove(0, input.getDocument().getLength());
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}
	
	private void append(final String text, final AttributeSet attributeSet){
		SwingUtilities.invokeLater(new Runnable(){
			public void run() {
				Document document = output.getDocument();
				try {
					document.insertString(document.getLength(), text, attributeSet);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}		

				if(document.getLength() > Prefs.CONSOLE_CHAR_MAX)
				{
					int delChar = document.getLength() - Prefs.CONSOLE_CHAR_MAX;
					try {
						String s = document.getText(0, document.getLength());

						int index = s.indexOf(Constants.NEWLINE, delChar) + 1;
						if(index > 0){
							document.remove(0, index);
						}else{
							//改行なしで制限超えた場合
							document.remove(0, delChar);
						}
					} catch (BadLocationException e) {
						e.printStackTrace();
					}
				}

				output.setCaretPosition(document.getLength());

			}
		});
	}
	
	private void addToHistory(String command){
		String s = command.trim();
		if(s.length() > 0){
	    	commandHistory.add(command.trim());
	    	histPos = commandHistory.size();
		}
	}
	
	private void submitCommand(){
		BasicCommand command = new BasicCommand(input.getRawText().trim());
		command.addCommandListener(new CommandEndListener(){
			@Override
			public void commandEnded(Command c) {
				rflow.explorer.getObjectBrowser().update();
			}
			
			@Override
			public void commandCanceled(Command c) {
				rflow.explorer.getObjectBrowser().update();
			}
			
			@Override
			public void errorOccurred(Command c) {
				rflow.explorer.getObjectBrowser().update();
			}
		});
        input.setText(""); //$NON-NLS-1$
    	receiver.passCommand(command);
	}
	
	private void createNewNode(){
        String command = input.getText().trim();
        input.setText(""); //$NON-NLS-1$
    	//複数行を許す場合
    	/*NodeModel[] models = RCommand.parseToNodes(command);
    	if(models != null){
    		ViewLayout layout = new ViewLayout(models);
    		rflow.viewer.addSubLayout(layout, new Point(0,0));
    	}*/
    	SentenceNodeModel model = new SentenceNodeModel();
		boolean lastCommand = (command.length() < 1 && histPos == commandHistory.size());
    	try {
    		if(lastCommand)
    			command = commandHistory.get(histPos - 1);
			model.setCommand(command);
			rflow.getActiveViewer().addNode(new FlowNode(model), null, true);
		} catch (IncompatibleCommandException e1) {
			// TODO 再読み込み検討
			e1.printStackTrace();
		}
    	
    	//histPos = commandHistory.size();
		if(!lastCommand)addToHistory(command);
	}
	
	private void flushBuffer(){
    	if(buffer.length() > 0){
	    	if(currentOutputType > 0)
	    		append(buffer.toString(), Prefs.ERROR_ATTR);
	    	else
	    		append(buffer.toString(), Prefs.OUTPUT_ATTR);
	    	buffer.delete(0, buffer.length());
    	}
	}
	
	public Command cancel(){
		synchronized (mutex) {
			//start command
			if (currentCommand != null) {
				currentCommand.cancel();
				Command c = currentCommand;
				while (!c.isRoot()) {
					c = c.getParent();
					c.cancel();
				}
			}
		}    	
		receiver.discardAllCommand();
		//if(Rflow.DEBUG > 0)rWriteConsole(rflow.R, "[CANCELED]", 1);
		final Command c = new BasicCommand(Constants.NEWLINE_STR);
		receiver.passCommand(c);
		return c;
	}
	
	public void executionFinished(){
		ignoreAll = false;
	}
    
    public void rBusy(Rengine re, int which) {
    	if(Rflow.DEBUG > 0)
    		System.out.println("rBusy("+which+")"); //$NON-NLS-1$ //$NON-NLS-2$
    	fireBusyStateChanged(which);
    }
    
    public void rWriteConsole(Rengine re, String text, int oType) {
    	if(useBuffer){
	    	if(currentOutputType != oType)
	            flushBuffer();
	    	currentOutputType = oType;
	    	
	    	if(buffer.length()+text.length() > buffer.capacity())
	            flushBuffer();
	    		
	    	buffer.append(text);
    	}else{
	    	if(oType > 0)
	    		append(text, Prefs.ERROR_ATTR);
	    	else
	    		append(text, Prefs.OUTPUT_ATTR);
    	}
    }
    
    private static final int CANCEL_OPTION = 0;
    private static final int IGNORE_OPTION = 1;
    private static final int IGNORE_ALL_OPTION = 2;
    private static final String[] errorOptions = new String[]{Messages.getString("Button.stop"),Messages.getString("Button.ignore"),Messages.getString("Button.ignore_all")}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    private boolean ignoreAll = false;
    
    private void errorMessage(){
		if(!ignoreAll){
			//ask ignore
			FlowNode errorNode = null;
			for(FlowNode node : rflow.viewer.getRootFlowState().getExecutingNode()){
				if(!node.getModel().isOpenSubflow()){
					errorNode = node;break;
				}
			}
			JOptionPane optionPane = new JOptionPane(Messages.getString("Message.node_name")+ " : "+errorNode.getTitle()+"\n"+Messages.getString("Message.error_occurred"), JOptionPane.ERROR_MESSAGE);  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			optionPane.setOptions(errorOptions);
			JDialog dialog = optionPane.createDialog(rflow.frame, Messages.getString("Error.title.error")); //$NON-NLS-1$
			dialog.setVisible(true);
			dialog.dispose();
			String s = (String)optionPane.getValue();

			int i;
			if(s == null)
				i = CANCEL_OPTION;
			else
				for(i = 0; i < errorOptions.length; i++){
					if(errorOptions[i].equals(s))
						break;
				}
			switch(i){
			case CANCEL_OPTION:
				cancel();
				//TODO clear flag
				break;
			case IGNORE_OPTION:
				break;
			case IGNORE_ALL_OPTION:
				ignoreAll = true;
				break;
			}
    	}
    }

    private boolean firstLineOfQueue = false;
    public String rReadConsole(Rengine re, String prompt, int addToHistory) {
        try {
        	if(useBuffer)flushBuffer();
        	
        	//if multi line queue
        	if(!commandQueue.isEmpty()){
        		if(!firstLineOfQueue){
        			append(prompt, Prefs.INPUT_ATTR);
        		}
        		String s2 = commandQueue.poll();
        		String s = s2;
        		if(currentCommand.checkError()){
        			if(firstLineOfQueue){
        				s = RCommand.trimErrorCommandFirst(s);
        			}
        			if(commandQueue.isEmpty()){
        				s = RCommand.trimErrorCommandLast(s);
        			}
        		}
            	append(s, Prefs.INPUT_ATTR);
            	addToHistory(s);
        		firstLineOfQueue = false;
    			return (s2==null||s2.length()==0)?s2:s2+Constants.NEWLINE_STR;
        	}
        	
        	if(currentCommand.checkError() && RCommand.isErrorOccurred()){
        		currentCommand.error();
        		errorMessage();
        	}else
        		currentCommand.finish();
        	currentCommand = null;
        	
        	//
        	append(prompt, Prefs.INPUT_ATTR);
        	
        	//複数plotのreturnおくり
        	/*if(isBusy){
    		waitReturn();
    		rWriteConsole(re, Constants.NEWLINE, 0);
    		return Constants.NEWLINE;
    		}*/

        	//next command
        	Command c = receiver.waitForCommand();
        	currentCommand = c;
        	String s = c.getCommand();
        	String s2 = s;
			if(c.checkError()){
				RCommand.clearErrorObject();
				s2 = RCommand.getErrorCheckCommand(s2);
			}
			//start command
			synchronized (mutex) {
				//start command
				c.start();
				Command com = c;
				while (!com.isRoot()) {
					com = com.getParent();
					com.start();
				}
			}
        	//multi line command
        	int index = s.indexOf(Constants.NEWLINE);
        	if(index > -1 && index < s.length()-2){
        		if(Rflow.DEBUG > 0)
        			System.out.println("mulit line command"); //$NON-NLS-1$
        		String[] commands = s2.split(Constants.NEWLINE_STR);
        		for(String command : commands)
        			commandQueue.offer(command+Constants.NEWLINE);
        		firstLineOfQueue = true;
        		return rReadConsole(re, prompt, addToHistory);
        	}
        	
        	append(s, Prefs.INPUT_ATTR);
        	addToHistory(s);
        	
        	//現在コマンドは改行で終わるようになっているので、
        	//空文字を空文字のまま送れるようにするためにこの処理があります
        	//ただし、空文字を送るとq()と同じ扱いになってしまう
			//if(s.equals("\n"))
			//	return "";
			return (s2==null||s2.length()==0)?s2:s2+Constants.NEWLINE_STR;
        } catch (Exception e) {
        	if(Rflow.DEBUG > 0)
        		System.out.println("jriReadConsole exception: "+e.getMessage()); //$NON-NLS-1$
        }
        return null;
    }
    
    public void rShowMessage(Rengine re, String message) {
    	if(Rflow.DEBUG > 0)
    		System.out.println("rShowMessage \""+message+"\""); //$NON-NLS-1$ //$NON-NLS-2$
    }
	
    public String rChooseFile(Rengine re, int newFile) {
    	if(Rflow.DEBUG > 0)
    		System.out.println("choose file"); //$NON-NLS-1$
    	
    	File file = null;
    	if(newFile == 0){
    		//load
    		file = FileChooserUtility.chooseOpenFile(Messages.getString("Message.select"), new File(""), new FileFilter[0], dialog); //$NON-NLS-1$ //$NON-NLS-2$
    	}else{
    		//save
    		file = FileChooserUtility.chooseSaveFile(Messages.getString("Message.select"), new File(""), new FileFilter[0], dialog); //$NON-NLS-1$ //$NON-NLS-2$
    	}
    	if(file == null)return null;
    	return file.getAbsolutePath();
    }
    
    public void   rFlushConsole (Rengine re) {
    	flushBuffer();
    }
	
    public void   rLoadHistory  (Rengine re, String filename) {
    	if(Rflow.DEBUG > 0)
    		System.out.println("load history"); //$NON-NLS-1$
    }			
    
    public void   rSaveHistory  (Rengine re, String filename) {
    	if(Rflow.DEBUG > 0)
    		System.out.println("save history"); //$NON-NLS-1$
    }

    /*--------------------------------------------------------------*/
    //wait return
    /*
    private boolean hitReturn = false;
    private boolean waitingReturn = false;
    synchronized void waitReturn(){
		//System.out.println("wait for return!");
		waitingReturn = true;
    	while(!hitReturn){
			try {
				wait(200);
			} catch (InterruptedException e) {
			}
    	}
    	hitReturn = false;
    	waitingReturn = false;
    }
    
    synchronized void notifyReturn(){
    	hitReturn = true;
    	notifyAll();
    }
    */
    /*--------------------------------------------------------------*/

    private class InputKeyHandler extends KeyAdapter{
    	
    	private final int mask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
    	
    	@Override
    	public void keyPressed(KeyEvent e) {
    		if(e.isConsumed())return;

    		
    		CodingTextPane console = (CodingTextPane)e.getSource();
    		switch(e.getKeyCode()){
    		case KeyEvent.VK_ENTER:
//    			if(waitingReturn){
//    				notifyReturn();
//    				e.consume();
//    				return;
//  			}
    			//if(!Rflow.isBusy()){
    				if(e.getModifiers() == mask){
    					createNewNode();
    				}else{
    					submitCommand();                	
    				}
    			//}
                e.consume();
    			break;
    		case KeyEvent.VK_P:
    			if(!Rflow.TEST || !e.isControlDown())break;
    		case KeyEvent.VK_UP:
    			if(--histPos < 0){
    				histPos = 0;
    				console.setText(""); //$NON-NLS-1$
    				return;
    			}
    			console.setText(getHistory(histPos));
    			e.consume();
    			break;
    		case KeyEvent.VK_N:
    			if(!Rflow.TEST || !e.isControlDown())break;
    		case KeyEvent.VK_DOWN:
    			if(++histPos > commandHistory.size()){
    				histPos = commandHistory.size();
    				return;
    			}
    			console.setText(getHistory(histPos));
    			e.consume();
    			break;
    		case KeyEvent.VK_ESCAPE:
    			//中断
    			if(//!(Platform.getPlatform() == Platform.MAC) || 
    					Rflow.isBusy()){
    				rflow.actionManager.stop();
    				e.consume();
    			}
    			break;
    		}
    	}
    	
    	private String getHistory(int histPos){
    		if(histPos == commandHistory.size())
    			return ""; //$NON-NLS-1$
			return commandHistory.get(histPos);
    	}
    }
    
    /*-----------------------------BusyListener--------------------------------*/
    private List<BusyListener> busyListeners = new ArrayList<BusyListener>();
    public void addBusyListener(BusyListener l){
    	busyListeners.add(l);
    }
    public void removeBusyListener(BusyListener l){
    	busyListeners.remove(l);
    }
    private void fireBusyStateChanged(final int which){
    	//new Thread(new Runnable(){
    		//public void run() {
    	    	for(BusyListener l : busyListeners){
    	    		l.busyStateChanged(which);
    	    	}
    		//}
    	//}).start();
    }

    /*-------------PropertyChangeListener-------------*/
	public void propertyChange(PropertyChangeEvent e) {
		if(Setting.Key.USE_CONSOLE_BUFFER.getKeyString().equals(e.getPropertyName())){
			Setting setting = (Setting)e.getSource();
			flushBuffer();
			useBuffer = (Boolean)setting.getProperty(Setting.Key.USE_CONSOLE_BUFFER);
		}
	}
}
