package com.ef_prime.rflow.explorer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceAdapter;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;

import sun.awt.shell.ShellFolder;

import com.ef_prime.rflow.ActionManager;
import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.TreeDnDMoveHandler;
import com.ef_prime.rflow.util.AbstractActionManager;
import com.ef_prime.rflow.util.FileControl;
import com.ef_prime.rflow.util.TreeUtils;

public class FileExplorer extends JPanel implements ClipboardOwner{

	private Rflow rflow;
	private JTree tree;
	private DefaultTreeModel model;
	private DragSourceHandler dragSourceHandler;
	private DropTargetHandler dropHandler;
	 
	private final FileActionManager fileActionManager = new FileActionManager();
	
	private FileControl fileControl;
	
	public FileExplorer(Rflow rflow) {
		super();
		this.rflow = rflow;
		this.fileControl = rflow.fileControl;
		
		setLayout(new BorderLayout());
		model = new DefaultTreeModel(new FileNode(Prefs.WORKING_DIRECTORY, null));
		tree = new JTree(model){
			@Override
			public String getToolTipText(MouseEvent e) {
			    TreePath path = getPathForLocation(e.getX(), e.getY());
			    if(path == null)return null;
			    FileNode node = (FileNode)path.getLastPathComponent();
			    return (node==null || !model.getRoot().equals(node))?null:node.getFile().getAbsolutePath();
			}
		};
		ToolTipManager.sharedInstance().registerComponent(tree);
		//SwingUtils.insertMouseListener(tree, new MouseHandler());
		tree.addMouseListener(new MouseHandler());
		tree.addFocusListener(new FocusAdapter(){
			@Override
			public void focusGained(FocusEvent e) {
				update();
			}
		});
		tree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener(){
			public void valueChanged(TreeSelectionEvent e) {
				fileActionManager.updateTarget();
			}
		});
		//キー
		tree.setActionMap(fileActionManager);
		tree.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, fileActionManager.getInputMap());
		//ドラッグ動作
		DragSource dragSource = new DragSource();
		dragSourceHandler = new DragSourceHandler();
		dragSource.createDefaultDragGestureRecognizer(tree, DnDConstants.ACTION_COPY_OR_MOVE, dragSourceHandler);
		dropHandler = new DropTargetHandler(tree);
		new DropTarget(tree, dropHandler);

		//レンダラー
		tree.setCellRenderer(new FileTreeRenderer());
		
		JScrollPane sp = new JScrollPane(tree);
		//sp.setBorder(BorderFactory.createEmptyBorder());
		add(sp, BorderLayout.CENTER);
	}
	
	/**
	 * 現在の作業ディレクトリに表示を変更
	 *
	 */
	public synchronized void update(){
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		boolean dirChanged = true;
		if(model.getRoot() != null){
			dirChanged = !((FileNode)model.getRoot()).getFile().equals(Prefs.WORKING_DIRECTORY);
		}
		
		Enumeration<TreePath> expanded = null;
		TreePath[] selection = null;
		if(!dirChanged){
			expanded = tree.getExpandedDescendants(new TreePath(model.getRoot()));
			selection = tree.getSelectionPaths();
		}
		
		if(Prefs.WORKING_DIRECTORY != null)
			model.setRoot(new FileNode(Prefs.WORKING_DIRECTORY, null));
		else
			model.setRoot(null);

		if(!dirChanged){
			while(expanded.hasMoreElements())
				tree.expandPath(expanded.nextElement());
			tree.setSelectionPaths(selection);
		}

		setCursor(Cursor.getDefaultCursor());
	}
	
	public TreePath[] getSelectedPaths(TreePath invokePath){
		TreePath[] paths = tree.getSelectionPaths();
		if(paths == null)
			return new TreePath[]{invokePath};
		boolean inSelection = tree.getSelectionModel().isPathSelected(invokePath);
		
		if(inSelection && TreeUtils.isSiblings(paths))
			return paths;
		else{
			if(inSelection){
				tree.clearSelection();
				tree.setSelectionPath(invokePath);
			}
			return new TreePath[]{invokePath};
		}
	}
	
	void open(File file){
		if(!fileControl.checkFileExists(file))
			return;
		if(file.isFile()){
			if(file.getPath().endsWith(Constants.RFLOW_FILE_EXTENSION)){
				if(rflow.getWorkingFile() != null && rflow.getWorkingFile().equals(file))
					return;
				if(!rflow.actionManager.askSave())
					return;
				rflow.actionManager.open(file, false);
			}else{
				try {
					//TODO ShellFolderを使用しない方法の検討
					ShellFolder sf = ShellFolder.getShellFolder(file);
					if(sf.getExecutableType() == null){
						JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.file.get_app"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
						return;
					}
						
					Runtime.getRuntime().exec("\""+sf.getExecutableType()+"\" \""+file.getAbsolutePath()+"\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	
				} catch (IOException e1) {
					JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.file.run_app"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
	}
	
	void delete(TreePath[] paths){
		int ret = JOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.confirm_delete"), //$NON-NLS-1$
				Messages.getString("Message.confirm"), JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
		if(ret == JOptionPane.YES_OPTION){
			for(TreePath path : paths){
				FileNode node = (FileNode)path.getLastPathComponent();
				File file = node.getFile();
				if(fileControl.delete_file(file)){
					update();
				}else{
					break;
				}
			}
		}
	}
	
	void copyToClipboard(TreePath[] paths){
		Toolkit kit = Toolkit.getDefaultToolkit();
		Clipboard clip = kit.getSystemClipboard();
		clip.setContents(new FileNodeTransferable(getLastComponents(paths)), this);
	}
		
	private FileNode[] getLastComponents(TreePath[] paths){
		FileNode[] nodes = new FileNode[paths.length];
		for(int i = 0; i < nodes.length; i++)
			nodes[i] = (FileNode)paths[i].getLastPathComponent();
		return nodes;
	}
	/*
	private Image getNodeImage(TreePath[] paths){
		
		Component[] pathsComponent = new Component[paths.length];
		for(int i = 0; i< paths.length; i++){
			TreePath path = paths[i];
			FileNode node = (FileNode)path.getLastPathComponent();
			
			Component c = tree.getCellRenderer().getTreeCellRendererComponent(tree, node, false,
					false, node.isLeaf(), tree.getRowForPath(path), false);
			
			pathsComponent[i] = c;
		}
		
		Image image = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D)image.getGraphics();
		//int x = 0;
		int y = 0;
		for(Component c : pathsComponent){
			c.paint(g);
			y += c.getHeight();
			g.translate(0, y);
		}
		g.translate(0, -y);
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC, 0.5f));
		pathsComponent[0].paint(g);
		
		return image;
	}*/
	
	private enum FileActionKey{
		OPEN,
		DELETE,
		RENAME,
		NEW_FOLDER,
		//NEW_FILE,
		COPY,
		PASTE,
		SORT_NAME,
		SORT_TYPE,
		SET_WORKDIR,
		REFRESH;
	}
	
	private class FileActionManager extends AbstractActionManager<FileActionKey>{
		boolean actionEnabled;
		
		boolean isSingleSelection;
		boolean isRoot;
		private TreePath[] paths;
		private TreePath path;
		private FileNode node;
		
		FileActionManager(){
			final int mask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
			
			putAction(FileActionKey.OPEN, new AbstractAction(Messages.getString("Action.open")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					open(node.getFile());
				}
			});
			getAction(FileActionKey.OPEN).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
			
			putAction(FileActionKey.DELETE, new AbstractAction(Messages.getString("Action.remove")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					delete(paths);
				}
			});
			getAction(FileActionKey.DELETE).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
			
			putAction(FileActionKey.COPY, new AbstractAction(Messages.getString("Action.copy")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					copyToClipboard(paths);
				}
			});
			getAction(FileActionKey.COPY).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, mask));
			
			putAction(FileActionKey.PASTE, new AbstractAction(Messages.getString("Action.paste")){ //$NON-NLS-1$
				@SuppressWarnings("unchecked") //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
					if(clip.isDataFlavorAvailable(DataFlavor.javaFileListFlavor)){
						List<File> list;
						try {
							list = (List<File>)clip.getData(DataFlavor.javaFileListFlavor);
							for(File file : list)
								if(!fileControl.copyFile(file, node.getFile(), FileControl.OVERWRITE_ASK, list.size() > 1))
									break;
						} catch (IOException e1) {
							JOptionPane.showMessageDialog(FileExplorer.this, e1.getMessage(),
									Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
						} catch (UnsupportedFlavorException e1) {}
						update();
					}
				}
			});
			getAction(FileActionKey.PASTE).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, mask));
			
			//
			Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
			clip.addFlavorListener(new FlavorListener(){
				public void flavorsChanged(FlavorEvent e) {
					try{
						Clipboard clip = (Clipboard)e.getSource();
						boolean b = (clip.isDataFlavorAvailable(DataFlavor.javaFileListFlavor));
						getAction(FileActionKey.PASTE).setEnabled(b);
					}catch(Exception e1){
					}
				}
			});
			
			putAction(FileActionKey.RENAME, new AbstractAction(Messages.getString("Action.rename")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					fileControl.rename(node.getFile());
					update();
				}
			});
			getAction(FileActionKey.RENAME).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
			
			putAction(FileActionKey.NEW_FOLDER, new AbstractAction(Messages.getString("Action.new_folder")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					fileControl.createFolderAskName(node.getFile());
					update();
				}
			});

			putAction(FileActionKey.SORT_NAME, new AbstractAction(Messages.getString("Action.sort_name")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					FileNode.setComparator(FileNode.NAME_COMPARATOR);
					update();
				}
			});

			putAction(FileActionKey.SORT_TYPE, new AbstractAction(Messages.getString("Action.sort_type")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					FileNode.setComparator(FileNode.TYPE_COMPARATOR);
					update();
				}
			});
			
			putAction(FileActionKey.SET_WORKDIR, new AbstractAction(Messages.getString("Action.set_dir0")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					File dir = node.getFile();
					if(!dir.isDirectory() || !dir.exists())return;
					rflow.setWorkingDirectory(dir);
				}
			});
			
			
			putAction(FileActionKey.REFRESH, new AbstractAction(Messages.getString("Action.refresh")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					update();
				}
			});
			getAction(FileActionKey.REFRESH).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
			
		}
		
		public InputMap getInputMap(){
			return getInputMap(FileActionKey.values());
		}
		
		void setActionEnabled(boolean actionEnabled) {
			if(this.actionEnabled != actionEnabled)
				for(FileActionKey key : FileActionKey.values()){
					getAction(key).setEnabled(actionEnabled);
				}
			this.actionEnabled = actionEnabled;
		}
		
		void updateTarget(){
			paths = tree.getSelectionPaths();
			if(paths == null || !TreeUtils.isSiblings(paths)){
				setActionEnabled(false);
				return;
			}
			setActionEnabled(true);
			
			isSingleSelection = paths.length == 1;
			this.path = paths[0];
			this.node = (FileNode)path.getLastPathComponent();
			isRoot = isSingleSelection && model.getRoot().equals(node);
			
			getAction(FileActionKey.OPEN).setEnabled(node.getFile().isFile() && isSingleSelection);
			getAction(FileActionKey.RENAME).setEnabled(isSingleSelection);
			getAction(FileActionKey.NEW_FOLDER).setEnabled(isSingleSelection);
			getAction(FileActionKey.PASTE).setEnabled(isSingleSelection);
		}

		public void showPopup(Component invoker, int x, int y) {
			if(!actionEnabled)return;
			
			//set enabled
			getAction(FileActionKey.SET_WORKDIR).setEnabled(!Rflow.isBusy());
			Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
			getAction(FileActionKey.PASTE).setEnabled(clip.isDataFlavorAvailable(DataFlavor.javaFileListFlavor));
			
			JPopupMenu popup = new JPopupMenu();
			if(getAction(FileActionKey.OPEN).isEnabled())
				popup.add(getAction(FileActionKey.OPEN));
			if(isSingleSelection){
				popup.add(getAction(FileActionKey.RENAME));
				popup.addSeparator();
				popup.add(getAction(FileActionKey.NEW_FOLDER));
				popup.addSeparator();//be careful. when menu change
			}
			popup.add(getAction(FileActionKey.COPY));
			if(isSingleSelection)
				popup.add(getAction(FileActionKey.PASTE));
			popup.add(getAction(FileActionKey.DELETE));
			if(isRoot){
				popup.addSeparator();
				popup.add(rflow.actionManager.getAction(ActionManager.ActionKey.CHANGE_DIRECTORY));
			}else if(isSingleSelection && !node.isLeaf()){
				popup.addSeparator();
				popup.add(getAction(FileActionKey.SET_WORKDIR));
			}
			popup.addSeparator();
			JMenu sortMenu = new JMenu(Messages.getString("Menu.sort"));  //$NON-NLS-1$
			sortMenu.add(getAction(FileActionKey.SORT_NAME));
			sortMenu.add(getAction(FileActionKey.SORT_TYPE));
			popup.add(sortMenu);
			popup.addSeparator();
			popup.add(getAction(FileActionKey.REFRESH));
			
			popup.show(invoker, x, y);
		}
	}
	
	private class MouseHandler extends MouseAdapter{
		@Override
		public void mousePressed(MouseEvent e) {
			/*
			 * cosume event if pressed path was already selected
			 * to avoid change of selection for multiple drag
			 */
			JTree tree = (JTree)e.getSource();
			Point p = e.getPoint();
			TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
			if(path == null)return;
			if(tree.isPathSelected(path))
				e.consume();
		}
		
		@Override
		public void mouseClicked(MouseEvent e) {
			JTree tree = (JTree)e.getSource();
			Point p = e.getPoint();
			TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
			if(path == null)return;
			FileNode node = (FileNode)path.getLastPathComponent();
			
			if(SwingUtilities.isLeftMouseButton(e)){
				if(e.getClickCount() == 2){
					open(node.getFile());
				}
			}else if(SwingUtilities.isRightMouseButton(e)){
				//ポップアップ
				if(!tree.getSelectionModel().isPathSelected(path))
					tree.setSelectionPath(path);
				fileActionManager.showPopup(tree, p.x, p.y);
			}
		}
	}
	
	private class DragSourceHandler extends DragSourceAdapter implements DragGestureListener{

		private Set<MutableTreeNode> dragNodeSet = new HashSet<MutableTreeNode>();
		
		private Cursor MOVE_CURSOR;
		private Cursor COPY_CURSOR;
		private Cursor INVALID_CURSOR;
		
		DragSourceHandler(){
			try {
				MOVE_CURSOR = Cursor.getSystemCustomCursor("MoveDrop.32x32"); //$NON-NLS-1$
				COPY_CURSOR = Cursor.getSystemCustomCursor("CopyDrop.32x32"); //$NON-NLS-1$
				INVALID_CURSOR = Cursor.getSystemCustomCursor("InvalidDrop.32x32"); //$NON-NLS-1$
			} catch (Exception e) {
				MOVE_CURSOR = COPY_CURSOR = INVALID_CURSOR = Cursor.getDefaultCursor();
			}			
		}
		
		public Set<MutableTreeNode> getDragNodeSet() {
			return dragNodeSet;
		}
		
		@Override
		public void dragExit(DragSourceEvent dse) {
//			System.out.println("drag exit");
			dse.getDragSourceContext().setCursor(INVALID_CURSOR);
		}
		
		@Override
		public void dragDropEnd(DragSourceDropEvent dsde) {
			dragNodeSet.clear();
			tree.repaint();
		}		
		
		@Override
		public void dragEnter(DragSourceDragEvent dsde) {
			//System.out.println("drag enter");
			setCursor(dsde);
		}
		
		@Override
		public void dropActionChanged(DragSourceDragEvent dsde) {
			setCursor(dsde);
		}

		private void setCursor(DragSourceDragEvent dsde) {
			if(dsde.getTargetActions() != DnDConstants.ACTION_NONE){
				switch(dsde.getUserAction()){
				case DnDConstants.ACTION_COPY:
					dsde.getDragSourceContext().setCursor(COPY_CURSOR);
					break;
				case DnDConstants.ACTION_MOVE:
					dsde.getDragSourceContext().setCursor(MOVE_CURSOR);
					break;
				case DnDConstants.ACTION_NONE:
					dsde.getDragSourceContext().setCursor(INVALID_CURSOR);
					break;
				}
			}else{
				dsde.getDragSourceContext().setCursor(INVALID_CURSOR);
			}
		}

		public void dragGestureRecognized(DragGestureEvent dge) {
			//左クリック
			if((dge.getTriggerEvent().getModifiers() & InputEvent.BUTTON1_MASK) == 0)
				return;
			
	        Point clickPoint = dge.getDragOrigin();
	        TreePath path = tree.getPathForLocation(clickPoint.x, clickPoint.y);
	        if (path == null)return;
	        
	        TreePath[] paths = getSelectedPaths(path);
	        FileNode[] nodes = getLastComponents(paths);
        	Transferable trans = new FileNodeTransferable(nodes);
	        try {
				dge.getDragSource().startDrag(dge, 
						Cursor.getDefaultCursor(), 
						//getNodeImage(paths), new Point(),
				        trans, this);
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			dragNodeSet.clear();
	        for(FileNode node : nodes){
	        	dragNodeSet.add(node);
	        }
		}
	}
	
	private class DropTargetHandler extends TreeDnDMoveHandler{
		
		public DropTargetHandler(JTree tree) {
			super(tree, false);
		}

		
		@Override
		public void dragOver(DropTargetDragEvent dtde) {
			if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
				Point p = dtde.getLocation();
		        TreePath path = tree.getPathForLocation(p.x, p.y);
		        tree.setSelectionPath(path);
		        //from external
		        if(!dtde.isDataFlavorSupported(getNodeFlavor())){
		        	if(path == null)
		        		dtde.rejectDrag();
		        	else
		        		dtde.acceptDrag(dtde.getDropAction());
		        	return;
		        }
			}
			super.dragOver(dtde);
		}
		
		@SuppressWarnings("unchecked") //$NON-NLS-1$
		@Override
		public void drop(DropTargetDropEvent dtde){
			if(dtde.isLocalTransfer())
				super.drop(dtde);
			else{
				//外部からのドラッグ
				if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
				{

					switch(dtde.getDropAction()){
					case DnDConstants.ACTION_COPY:
					case DnDConstants.ACTION_MOVE:
						dtde.acceptDrop(dtde.getDropAction());
						break;
					default:
						dtde.dropComplete(false);
						return;
					}
					
					Transferable trans = dtde.getTransferable();
					List<File> list;
					try {
						list = (List<File>)trans.getTransferData(DataFlavor.javaFileListFlavor);
					} catch (Exception e) {e.printStackTrace();
						return;}

					Point p = dtde.getLocation();
			        TreePath path = tree.getPathForLocation(p.x, p.y);
			        tree.setSelectionPath(path);
			        if(path == null){
						dtde.dropComplete(false);
						return;
			        }
			        FileNode target = (FileNode)path.getLastPathComponent();
					
					switch(dtde.getDropAction()){
					case DnDConstants.ACTION_COPY:
						for(File file : list){
							if(!fileControl.copyFile(file, target.getFile(), FileControl.OVERWRITE_ASK, list.size() > 1))
								break;
						}
						break;
					case DnDConstants.ACTION_MOVE:
						File parentFile = (target.getFile().isDirectory())?target.getFile():target.getFile().getParentFile();
						if(parentFile == null){
							dtde.dropComplete(false);
							return;
						}
						//位置が変わらない場合
						if(parentFile.equals(list.get(0).getParentFile()))
							return;
						
						for(File file : list){
							if(!fileControl.moveFile(file, new File(parentFile, file.getName()), FileControl.OVERWRITE_ASK, list.size() > 1))
								break;
						}
						break;
					default:
						dtde.dropComplete(false);
					}
					dtde.dropComplete(true);
					update();
				}
			}
		}
		
		/*@Override
		protected void removeDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes) {
			//do nothing
		}*/
		
		@Override
		protected void moveDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes, MutableTreeNode parent, int index) {
			File parentFile = ((FileNode)parent).getFile();
			//位置が変わらない場合
			if(parentFile.equals(((FileNode)nodes[0].getParent()).getFile()))
				return;
			
			if(parentFile.exists()){
				for(MutableTreeNode node : nodes){
					File file =((FileNode)node).getFile();
					if(!fileControl.moveFile(file, new File(parentFile, file.getName()), FileControl.OVERWRITE_ASK, nodes.length > 1))
						break;
				}
				update();
			}
		}
		
		@Override
		protected void copyDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes, MutableTreeNode parent, int index) {
			File parentFile = ((FileNode)parent).getFile();

			if(parentFile.exists())
				for(MutableTreeNode node : nodes){
					File file =((FileNode)node).getFile();
					if(!fileControl.copyFile(file, parentFile, FileControl.OVERWRITE_ASK, nodes.length > 1))
						break;
				}
			update();
		}
		
		@Override
		protected boolean checkDragAccept(DropTargetDragEvent dtde, MutableTreeNode[] nodes) {
			if(dtde.getDropAction() == DnDConstants.ACTION_MOVE
				&&
				getCurrentTargetParent().equals(nodes[0].getParent()))
				return false;
			return true;
		}
		
		@Override
		protected void onDropComplete(MutableTreeNode[] dragNodes) {
			update();
		}
	}
	
	private class FileTreeRenderer extends DefaultTreeCellRenderer{
		
		//for getIcon - using icon cache(in FileView)
		private JFileChooser chooser = new JFileChooser();
		
		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {

			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
					row, hasFocus);
			
			FileNode node = (FileNode)value;

			selected = selected || (dragSourceHandler.getDragNodeSet().contains(node));
			
			try{
				//Icon icon = fileSystemView.getSystemIcon(node.getFile());
				Icon icon = chooser.getIcon(node.getFile());
				if(icon != null)
					setIcon(icon);
			}catch(Exception e){
			}

			if(model.getRoot().equals(node)){
				setToolTipText(node.getFile().getAbsolutePath());
			}
			
			return this;
		}
	}

	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		
	}
	
	/*public void busyStateChanged(final int which) {
    	SwingUtilities.invokeLater(new Runnable(){
    		public void run() {
    			if(which == BusyListener.NOT_BUSY)
    				update();
    		}
    	});
	}*/
}
