package com.ef_prime.rflow.explorer;

import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import com.ef_prime.rflow.util.ObjectUtilities;

public class FileNode implements MutableTreeNode{
	
	private File file;
	private File[] children;
	private FileNode parent;
	public static final Comparator<File> NAME_COMPARATOR = new Comparator<File>(){
		public int compare(File o1, File o2) {
			if(o1.isDirectory() ^ o2.isDirectory())
				return (o1.isDirectory())?-1:1;
			return o1.getName().compareToIgnoreCase(o2.getName());
		}
	};
	public static final Comparator<File> TYPE_COMPARATOR = new Comparator<File>(){
		public int compare(File o1, File o2) {
			if(o1.isDirectory() ^ o2.isDirectory())
				return (o1.isDirectory())?-1:1;
			if(!o1.isDirectory()){
				int extComp = getExtension(o1.getName()).compareToIgnoreCase(getExtension(o2.getName()));
				if(extComp != 0)
					return extComp;
			}
			return o1.getName().compareToIgnoreCase(o2.getName());
		}
		
		private String getExtension(String name){
			int index = name.lastIndexOf('.');
			if(index < 0)
				return ""; //$NON-NLS-1$
			return name.substring(index+1, name.length());
		}
	};
	private static Comparator<File> comparator = TYPE_COMPARATOR;
	
	public FileNode(File file, FileNode parent){
		this.file = file;
		this.parent = parent;
		updateChildren();
	}
	
	public static Comparator<File> getComparator() {
		return comparator;
	}
	
	public static void setComparator(Comparator<File> comparator) {
		FileNode.comparator = comparator;
	}
	
	void updateChildren(){
		children = file.listFiles();
		if(children == null)
			children = new File[0];
		Arrays.sort(children, comparator);
	}
	
	public File getFile() {
		return file;
	}

	public Enumeration children() {
		Vector<FileNode> v = new Vector<FileNode>();
		for(File file : children)
			v.add(new FileNode(file, this));
		return v.elements();
	}

	public boolean getAllowsChildren() {
		return file.isDirectory();
	}

	public TreeNode getChildAt(int childIndex) {
		return new FileNode(children[childIndex], this);
	}

	public int getChildCount() {
		return children.length;
	}

	public int getIndex(TreeNode node) {
		FileNode fNode = (FileNode)node;
		for(int i = 0; i < children.length; i++)
			if(children[i].equals(fNode.getFile()))
				return i;
		return -1;
	}

	public TreeNode getParent() {
		return parent;
	}

	public boolean isLeaf() {
		return file.isFile();
	}

	
	@Override
	public boolean equals(Object obj) {
		if(this == obj)
			return true;
		if (obj instanceof FileNode) {
			FileNode node = (FileNode) obj;
			return ObjectUtilities.equals(getFile(), node.getFile());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return getFile().hashCode();
	}
	
	@Override
	public String toString() {
		return file.getName();
	}

	public void insert(MutableTreeNode child, int index) {
		updateChildren();
	}

	public void remove(int index) {
		updateChildren();
	}

	public void remove(MutableTreeNode node) {
		updateChildren();
	}

	public void removeFromParent() {
		((FileNode)getParent()).updateChildren();
	}

	public void setParent(MutableTreeNode newParent) {
		((FileNode)getParent()).updateChildren();
		((FileNode)newParent).updateChildren();
	}

	public void setUserObject(Object object) {
		
	}
}
