package com.ef_prime.rflow.explorer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceAdapter;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.exception.ReadException;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.group.AppearanceGroupManager;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.ElementWriter;
import com.ef_prime.rflow.io.FlowDataReader;
import com.ef_prime.rflow.io.FlowDataWriter;
import com.ef_prime.rflow.node.pack.UIPackManager;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.ButtonPanel;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.tool.TreeDnDMoveHandler;
import com.ef_prime.rflow.util.AbstractActionManager;
import com.ef_prime.rflow.util.TreeUtils;
import com.ef_prime.rflow.xml.XMLException;

public class FunctionExplorer extends JPanel{
	
	private Rflow rflow;
	private AppearanceGroupManager manager; 
	private JTree tree;
	private MoveDropHandler moveDropHandler;
	
	public FunctionExplorer(Rflow rflow){
		super();
		this.rflow = rflow;
		this.manager = rflow.groupManager;
		setLayout(new BorderLayout());
		
		//tree = new JTree(createSampleTreeNode());
		tree = new JTree(new DefaultTreeModel(new FunctionNode("(Empty)"))); //$NON-NLS-1$
		//tree.setDragEnabled(true);
		tree.setEditable(false);
		//マウス
		tree.addMouseListener(new MouseHandler());
		//モデルリスナー
		getTreeModel().addTreeModelListener(new FunctionTreeModelListener());
		//ドラッグ動作
		DragSource dragSource = new DragSource();
		dragSource.createDefaultDragGestureRecognizer(tree, DnDConstants.ACTION_MOVE, new DragGesture());
		//dragSource.addDragSourceListener(new DragSourceHandler());
		moveDropHandler = new MoveDropHandler(tree);
		new DropTarget(tree, moveDropHandler);
		//レンダラーの設定
		tree.setCellRenderer(new TreeRenderer());
		
		add(new JScrollPane(tree), BorderLayout.CENTER);
	}
	
	/*
	public FunctionNode createSampleTreeNode(){
		FunctionNode root = new FunctionNode("すべて");
		
		FunctionNode analysis = new FunctionNode("分析");
		FunctionNode regression = new FunctionNode("回帰分析");
		FunctionNode lm = new FunctionNode("線形回帰", "lm");
		FunctionNode glm = new FunctionNode("一般線形回帰", "glm");
		regression.add(lm);regression.add(glm);
		analysis.add(regression);
		
		FunctionNode user = new FunctionNode("ユーザー定義", null, null);
		FunctionNode tablepie = new FunctionNode("円グラフ", "tablepie", 
				"function(x = iris$Species){pie(table(x))}");
		user.add(tablepie);
		
		root.add(analysis);
		root.add(user);
		
		return root;
	}
	*/
	
	public void importFunctionTree(File file, AppearanceGroupManager manager) throws ReadException, IOException{
		FlowDataReader reader = new FlowDataReader();
		FunctionTreeReader functionTreeReader = new FunctionTreeReader(manager);
		
		reader.read(file, new ElementReader[]{functionTreeReader});
		
		//初期化
		//Rflow.GROUP_MANAGER = functionTreeReader.getFunctionGroupManager();
		init(functionTreeReader.getRootNode());
	}
	
	public void init(FunctionNode root){
		getTreeModel().setRoot(root);
	}
	
	
	public FunctionNode getRoot(){
		return (FunctionNode)tree.getModel().getRoot();
	}
	
	public DefaultTreeModel getTreeModel(){
		return (DefaultTreeModel)tree.getModel();
	}
	
	public void save(File file) throws IOException, XMLException{
		new FlowDataWriter().write(file, new ElementWriter[]{
				new FunctionTreeElementWriter(manager, getRoot())
		});
	}
	
	public FunctionNode[] getSelectedNodes(FunctionNode invokeNode){
		FunctionNode[] nodes = new FunctionNode[tree.getSelectionCount()];
		TreePath[] paths = tree.getSelectionPaths();
		boolean inSelection = false;
		for(int i = 0; i < paths.length; i++){
			nodes[i] = (FunctionNode)paths[i].getLastPathComponent();
			if(!inSelection && nodes[i].equals(invokeNode))
				inSelection = true;
		}
		
		if(inSelection && TreeUtils.isSiblings(nodes))
			return nodes;
		else{
			if(inSelection){
				tree.clearSelection();
				tree.setSelectionPath(new TreePath(invokeNode.getPath()));
			}
			return new FunctionNode[]{invokeNode};
		}
	}

	/**
	 * parentNodeが関数ノードでなかった場合は子供に新しいノードを作成
	 * parentNodeが関数ノードだった場合は下に新しいノードを作成
	 * @param parentNode
	 * @param p invoked point for edit dialog location
	 */
	void newNode(FunctionNode parentNode, boolean isFunction, Point p) {
		FunctionNode emptyNode = (isFunction)?new FunctionNode("", "", ""):new FunctionNode(""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		FunctionNode newNode = editNode_internal(emptyNode, p);
		if(newNode == null)return;
		
		FunctionNode parent = parentNode;
		if(!parent.hasFunction() || parent.isRoot()){
			getTreeModel().insertNodeInto(newNode, parent, parent.getChildCount());
		}else{
			parent = (FunctionNode)parent.getParent();
			getTreeModel().insertNodeInto(newNode, parent, parent.getIndex(parentNode)+1);
		}
		manager.addFunction(newNode.getFunction(), newNode.getGroupBelongTo(), true);
		updateGroup();
	}

	/**
	 * ノードを消去します
	 * @param node
	 */
	void removeNode(FunctionNode node) {
		MutableTreeNode parent = (MutableTreeNode)node.getParent();
		if(parent == null)return;
		
		removeFunctionFromGroup(node);
		getTreeModel().removeNodeFromParent(node);
	}
	
	/**
	 * ノードの編集をします
	 * @param node
	 * @param p invoked point for edit dialog location
	 */
	void editNode(FunctionNode node, Point p) {
		FunctionNode newNode = editNode_internal(node, p);
		if(newNode == null)return;
		removeFunctionFromGroup(node);
		node.setData(newNode);
		addFunctionToGroup(node);
		getTreeModel().nodeChanged(node);
	}
	
	private FunctionNode editNode_internal(FunctionNode node, Point p){
		Point panelPoint = FunctionExplorer.this.getLocationOnScreen();
		EditDialog dialog = new EditDialog(rflow.frame, node);
		dialog.setLocation(panelPoint.x+p.x+50, panelPoint.y+p.y-dialog.getHeight()/2);
		dialog.setVisible(true);
		dialog.dispose();
		
		return dialog.getFunctionNode();
	}
	
	void moveNode(MutableTreeNode node, boolean up){
		MutableTreeNode parent = (MutableTreeNode)node.getParent();
		if(parent == null)return;
		int index = parent.getIndex(node);
		index = (up)?index-1:index+1;
		if(index < 0 || index > parent.getChildCount()-1)
			return;
		getTreeModel().removeNodeFromParent(node);
		getTreeModel().insertNodeInto(node, parent, index);
	}
	
	private void removeFunctionFromGroup(FunctionNode node){
		if(node.hasFunction())
			manager.removeFunction(node.getFunction());
		for(int i = 0; i < node.getChildCount(); i++)
			removeFunctionFromGroup((FunctionNode)node.getChildAt(i));
	}
	
	private void addFunctionToGroup(FunctionNode node){
		if(node.hasFunction()){
			AppearanceGroup group = node.getGroupBelongTo();
			if(group != null)
				manager.addFunction(node.getFunction(), group, true);
		}
		for(int i = 0; i < node.getChildCount(); i++)
			addFunctionToGroup((FunctionNode)node.getChildAt(i));
	}

	private void updateGroup() {
		for(FlowViewer fv : rflow.getViewers())
			fv.updateGroup();
	}
	
	/**
	 * ApperanceGroupManagerに変更の通知をする
	 * 
	 */
	private class FunctionTreeModelListener implements TreeModelListener{

		public void treeNodesChanged(TreeModelEvent e) {
			updateGroup();
		}

		public void treeNodesInserted(TreeModelEvent e) {
			updateGroup();
		}

		public void treeNodesRemoved(TreeModelEvent e) {
			updateGroup();
		}

		public void treeStructureChanged(TreeModelEvent e) {
			update();
			updateGroup();
		}

		
		private void update(){
			manager.clearFunctions();
			registFunction(getRoot());
		}
		
		private void registFunction(FunctionNode node){
			if(node.hasFunction()){
				AppearanceGroup group = node.getGroupBelongTo();
				if(group != null)
					manager.addFunction(node.getFunction(), group, false);
			}
			for(int i = 0; i < node.getChildCount(); i++){
				registFunction((FunctionNode)node.getChildAt(i));
			}
		}
		
	}
	
	private enum FunctionExplorerActions{
		NEW_FUNCTION,
		NEW_FOLDER,
		REMOVE,
		EDIT,
		MOVE_UP,
		MOVE_DOWN,
		HELP,
		SOURCE,
		DEFAULT,
		;
	}
	
	private class TreePopup extends AbstractActionManager<FunctionExplorerActions>{
		
		private FunctionNode node;
		private Point p;
		
		TreePopup(){
			
			putAction(FunctionExplorerActions.NEW_FUNCTION, new AbstractAction(Messages.getString("Action.new_function")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					newNode(node, true, p);
				}
			});

			putAction(FunctionExplorerActions.NEW_FOLDER, new AbstractAction(Messages.getString("Action.new_folder")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					newNode(node, false, p);
				}
			});

			putAction(FunctionExplorerActions.REMOVE, new AbstractAction(Messages.getString("Action.remove")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					int ret = JOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.confirm_delete"), //$NON-NLS-1$
							Messages.getString("Message.confirm"), JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
					if(ret == JOptionPane.YES_OPTION)
						removeNode(node);
				}
			});
			
			putAction(FunctionExplorerActions.EDIT, new AbstractAction(Messages.getString("Action.edit")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					editNode(node, p);
				}
			});

			putAction(FunctionExplorerActions.MOVE_UP, new AbstractAction(Messages.getString("Action.move_up")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					moveNode(node, true);
				}
			});
			
			putAction(FunctionExplorerActions.MOVE_DOWN, new AbstractAction(Messages.getString("Action.move_down")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					moveNode(node, false);
				}
			});
			
			
			putAction(FunctionExplorerActions.HELP, new AbstractAction(Messages.getString("Action.help")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					RCommand.getReceiver().passCommand("help("+node.getFunction()+")"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			});
			
			putAction(FunctionExplorerActions.SOURCE, new AbstractAction(Messages.getString("FunctionExplorer.show_source")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					RCommand.getReceiver().passCommand(node.getFunction()+Constants.NEWLINE_STR);
				}
			});
			
			putAction(FunctionExplorerActions.DEFAULT, new AbstractAction(Messages.getString("Action.restore_default_function_tree")){ //$NON-NLS-1$
				public void actionPerformed(ActionEvent e) {
					try {
						int ret = JOptionPane.showConfirmDialog(rflow.frame, Messages.getString("Message.confirm_lost_setting"), //$NON-NLS-1$
								Messages.getString("Message.confirm"), JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
						if(ret == JOptionPane.YES_OPTION){
							manager.clear();
							importFunctionTree(FileResource.FUNCTION_FILE, manager);
						}
					} catch (Exception e1) {
						JOptionPane.showMessageDialog(rflow.frame, Messages.getString("Error.read"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			});
		}
		
		
		public void show(Component invoker, FunctionNode node, int x, int y) {
			this.node = node;
			this.p = new Point(x, y);
			
			//set enabled
			boolean notBusy = !Rflow.isBusy();
			getAction(FunctionExplorerActions.EDIT).setEnabled(notBusy);
			getAction(FunctionExplorerActions.NEW_FUNCTION).setEnabled(notBusy);
			getAction(FunctionExplorerActions.HELP).setEnabled(notBusy);
			getAction(FunctionExplorerActions.SOURCE).setEnabled(notBusy);
			
			
			//create popup
			JPopupMenu popup = new JPopupMenu();

			popup.add(getAction(FunctionExplorerActions.EDIT));
			popup.addSeparator();
			popup.add(getAction(FunctionExplorerActions.NEW_FUNCTION));
			popup.add(getAction(FunctionExplorerActions.NEW_FOLDER));
			popup.addSeparator();
			popup.add(getAction(FunctionExplorerActions.REMOVE));
			popup.addSeparator();
			popup.add(getAction(FunctionExplorerActions.MOVE_UP));
			popup.add(getAction(FunctionExplorerActions.MOVE_DOWN));
			if(node.hasFunction()){
				popup.addSeparator();
				popup.add(getAction(FunctionExplorerActions.HELP));
				popup.add(getAction(FunctionExplorerActions.SOURCE));
			}
			popup.addSeparator();
			popup.add(getAction(FunctionExplorerActions.DEFAULT));
			
			popup.show(invoker, x, y);
		}
	}
	
	private class EditDialog extends JDialog{
		
		private boolean hasFunction;
		private FunctionNode node;
		
		public EditDialog(Dialog owner, FunctionNode node) throws HeadlessException {
			super(owner, true);
			init(node);
		}

		public EditDialog(Frame owner, FunctionNode node) throws HeadlessException {
			super(owner, true);
			init(node);
		}
		
		private void init(FunctionNode node_orginal){
			final FunctionNode node = node_orginal.copy();
			AppearanceGroup group = node_orginal.getGroupBelongTo();
			hasFunction = node.hasFunction();
			
			setTitle(Messages.getString("Message.edit")); //$NON-NLS-1$
			
			Dimension labelSize = new Dimension(60, 20);
			Dimension textSize = new Dimension(200, 20);
			
			//タイトル
			JLabel titleLabel = new JLabel(Messages.getString("Message.name")); //$NON-NLS-1$
			titleLabel.setMaximumSize(labelSize);
			titleLabel.setPreferredSize(labelSize);
			final JTextField titleField = new RflowTextField();
			titleField.setMaximumSize(textSize);
			titleField.setPreferredSize(textSize);
			titleField.setText(node.getTitle());
			Box titleBox = Box.createHorizontalBox();
			titleBox.add(titleLabel);
			titleBox.add(titleField);
			
			//パッケージ名
			JLabel packageLabel = new JLabel(Messages.getString("FunctionExplorer.package")); //$NON-NLS-1$
			packageLabel.setMaximumSize(labelSize);
			packageLabel.setPreferredSize(labelSize);
			String[] packages = RCommand.getPackages();
			if(packages == null)
				packages = new String[0];
			Vector<String> pkgs = new Vector<String>(Arrays.asList(packages));
			pkgs.add(0, ""); //$NON-NLS-1$
			final JComboBox packageField = new JComboBox(pkgs);
			packageField.setMaximumSize(textSize);
			packageField.setPreferredSize(textSize);
			if(node.hasPackage())
				packageField.setSelectedItem(node.getPackage());
			Box packageBox = Box.createHorizontalBox();
			packageBox.add(packageLabel);
			packageBox.add(packageField);
			
			//関数名
			JLabel functionLabel = new JLabel(Messages.getString("FunctionExplorer.function")); //$NON-NLS-1$
			functionLabel.setMaximumSize(labelSize);
			functionLabel.setPreferredSize(labelSize);
			final JTextField functionField = new RflowTextField();
			functionField.setMaximumSize(textSize);
			functionField.setPreferredSize(textSize);
			if(hasFunction)
				functionField.setText(node.getFunction());
			Box functionBox = Box.createHorizontalBox();
			functionBox.add(functionLabel);
			functionBox.add(functionField);
			
			//グループ
			JLabel groupLabel = new JLabel(Messages.getString("FunctionExplorer.group")); //$NON-NLS-1$
			groupLabel.setMaximumSize(labelSize);
			groupLabel.setPreferredSize(labelSize);
			Vector<AppearanceGroup> groups = new Vector<AppearanceGroup>();
			groups.add(null);
			groups.addAll(rflow.groupManager.getGroups());
			final JComboBox groupComboBox = new JComboBox(groups);
			groupComboBox.setMaximumSize(textSize);
			groupComboBox.setPreferredSize(textSize);
			groupComboBox.setRenderer(new DefaultListCellRenderer(){
				@Override
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
					super.getListCellRendererComponent(list, value, index, isSelected,
							cellHasFocus);
					
					AppearanceGroup group = (AppearanceGroup)value;
					if(group == null){
						setText(Messages.getString("FunctionExplorer.inherit_from_parent")); //$NON-NLS-1$
					}else{
						setIcon(group.getSmallIcon());
					}
					
					return this;
				}
			});
			groupComboBox.setSelectedItem(node.getGroup());
			if(node.getGroup() == null && group != null)
				groupComboBox.removeItem(group);
			
			Box groupBox = Box.createHorizontalBox();
			groupBox.add(groupLabel);
			groupBox.add(groupComboBox);
			
			
			ActionListener ok = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					//関数のチェック
					if(hasFunction){
						String function = functionField.getText();
						if(function.length() < 1){
							JOptionPane.showMessageDialog(EditDialog.this, Messages.getString("Error.function_is_empty"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
							return;
						}
						if(!node.getFunction().equals(function) && manager.hasFunction(function)){
							JOptionPane.showMessageDialog(EditDialog.this, Messages.getString("Error.duplicate_function"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
							functionField.requestFocus();
							functionField.selectAll();
							return;
						}
					}
					String pkg = (String)packageField.getSelectedItem();
					if(pkg.equals(""))pkg = null; //$NON-NLS-1$
					
					EditDialog.this.node = 
						new FunctionNode(titleField.getText(), 
										(hasFunction)?functionField.getText():null,
										(hasFunction)?pkg:null,
										null,
										(AppearanceGroup)groupComboBox.getSelectedItem());
					dispose();
				}
			};
			ActionListener cancel = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					EditDialog.this.node = null;
					dispose();
				}
			};
			JPanel buttonPanel = new ButtonPanel(ok, cancel, new Dimension(80, 22)); 

			Box box = Box.createVerticalBox();
			box.setBorder(new EmptyBorder(10,10,10,10));
			box.add(titleBox);
			if(hasFunction){
				//TODO package
				//box.add(Box.createVerticalStrut(5));
				//box.add(packageBox);
				box.add(Box.createVerticalStrut(5));
				box.add(functionBox);
			}
			box.add(Box.createVerticalStrut(5));
			box.add(groupBox);
			box.add(Box.createVerticalStrut(15));
			box.add(buttonPanel);
			
			getContentPane().add(box, BorderLayout.CENTER);
			
			pack();
		}
		
		public FunctionNode getFunctionNode() {
			return node;
		}
	}
	
	private class MouseHandler extends MouseAdapter{
		private TreePopup popup = new TreePopup();
		
		@Override
		public void mousePressed(MouseEvent e) {
			JTree tree = (JTree)e.getSource();
			Point p = e.getPoint();
			TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
			if(path == null)return;
			FunctionNode node = (FunctionNode)path.getLastPathComponent();
			
			//右クリック
			if(SwingUtilities.isRightMouseButton(e)){
				popup.show(tree, node, p.x, p.y);
			}
		}
	}
	
	private class DragGesture extends DragSourceAdapter implements DragGestureListener{
		

		private Cursor MOVE_CURSOR;
		private Cursor INVALID_CURSOR;
		
		DragGesture(){
			try {
				MOVE_CURSOR = Cursor.getSystemCustomCursor("MoveDrop.32x32"); //$NON-NLS-1$
				INVALID_CURSOR = Cursor.getSystemCustomCursor("InvalidDrop.32x32"); //$NON-NLS-1$
			} catch (Exception e) {
				MOVE_CURSOR = INVALID_CURSOR = Cursor.getDefaultCursor();
			}			
		}
		
		@Override
		public void dragEnter(DragSourceDragEvent dsde) {
			dsde.getDragSourceContext().setCursor(MOVE_CURSOR);
		}

		@Override
		public void dragExit(DragSourceEvent dse) {
			dse.getDragSourceContext().setCursor(INVALID_CURSOR);
		}
		
		
		public void dragGestureRecognized(DragGestureEvent dge) {
	        Point clickPoint = dge.getDragOrigin();
	        TreePath path = tree.getPathForLocation(clickPoint.x, clickPoint.y);
	        if (path == null)return;
	        
	        FunctionNode draggedNode = (FunctionNode) path.getLastPathComponent();
	        FunctionNode[] nodes = getSelectedNodes(draggedNode);
        	Transferable trans = new FunctionNodeTransferable(nodes);
	        try {
				dge.getDragSource().startDrag(dge, 
						Cursor.getSystemCustomCursor("MoveDrop.32x32"), //$NON-NLS-1$
				        trans, this);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class MoveDropHandler extends TreeDnDMoveHandler{

		public MoveDropHandler(JTree tree) {
			super(tree, true);
		}
		
		@Override
		protected boolean checkDragAccept(DropTargetDragEvent dtde, MutableTreeNode[] nodes) {
			for(MutableTreeNode node : nodes){
				if(node == getTargetNode())return false;
			}
			//変わらない場合
			if(getCurrentTargetParent().getIndex(nodes[0]) > -1){
				int index = getCurrentTargetIndex()-1;
				TreeNode parent = nodes[0].getParent();
				for(MutableTreeNode node : nodes){
					int nodeIndex = parent.getIndex(node);
					if(index == nodeIndex || index+1 == nodeIndex){
						return false;
					}
				}
			}
			return true;
		}
		
		@Override
		protected void copyDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes, MutableTreeNode parent, int index) {
			//no copy action
			//unreachable
		}
		
		@Override
		protected void onDropComplete(MutableTreeNode[] dragNodes) {
			for(MutableTreeNode node : dragNodes)
				addFunctionToGroup((FunctionNode)node);
			updateGroup();
		}

	}
	
	private class TreeRenderer extends DefaultTreeCellRenderer{
		
		private boolean isTarget;
		private boolean isFolder;
		
		private ImageIcon uiIcon;
		
		TreeRenderer(){
			if(Rflow.TEST){
				ImageIcon icon = FileResource.WARNING;

				if(icon != null)
					uiIcon = new ImageIcon(icon.getImage().getScaledInstance(16, 16, Image.SCALE_SMOOTH));
				else
					uiIcon = null;
			}	
		}
		
		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
					row, hasFocus);
			
			FunctionNode node = (FunctionNode)value;
			if(node.hasGroup()){
				setIcon(node.getGroup().getSmallIcon());
			}else if(!node.hasFunction()){
				setIcon((expanded)?getOpenIcon():getClosedIcon());
			}else{
				if(UIPackManager.contains(node.getFunction())){
					if(uiIcon != null)
						setIcon(uiIcon);
				}
			}
			
			isTarget = (node == moveDropHandler.getTargetNode());
			isFolder = !node.hasFunction();
			
			return this;
		}
		
        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            if(isTarget) {
                g.setColor(Color.BLACK);
                if(!isFolder) {
                	if(moveDropHandler.isUpperSide())
                		g.drawLine(0, 0, getSize().width, 0);
                	else
                		g.drawLine(0, getSize().height-1, getSize().width, getSize().height-1);
                }else{
                	if(moveDropHandler.isUpperSide()){
    	        		if(moveDropHandler.getTargetNode().getParent() != null)
    	        			g.drawLine(0, 0, getSize().width, 0);
                	}else
                		g.drawRect(0, 0, getSize().width-1, getSize().height-1);
                }
            }
        }
	}
}
