package com.ef_prime.rflow.explorer;

import javax.swing.tree.DefaultMutableTreeNode;

import com.ef_prime.rflow.group.AppearanceGroup;

public class FunctionNode extends DefaultMutableTreeNode{
	
	private String function; 
	private String pkg;//package
	private String source;
	private AppearanceGroup group;
	
	public FunctionNode(String title) {
		this(title, null, null, null, null);
	}
	
	public FunctionNode(String title, AppearanceGroup group){
		this(title, null, null, null, group);
	}
	
	public FunctionNode(String title, String function, String pkg){
		this(title, function, pkg, null, null);
	}
	

	public FunctionNode(String title, String function, String pkg, String source) {
		this(title, function, pkg, source, null);
	}
	
	public FunctionNode(String title, String function, String pkg, String source, AppearanceGroup group) {
		super(title);
		this.function = function;
		this.pkg = pkg;
		this.source = source;
		this.group = group;
	}
	
	private FunctionNode(FunctionNode node){
		super();
		setData(node);
	}
	
	@Override
	public boolean getAllowsChildren() {
		return !hasFunction();
	}
	
	@Override
	public boolean isLeaf() {
		return hasFunction();
	}
	
	public void setData(FunctionNode node){
		setUserObject(node.getTitle());
		this.function = node.getFunction();
		this.source = node.getSource();
		this.group = node.getGroup();
		this.pkg = node.getPackage();
	}
	
	public String getTitle(){
		return getUserObject().toString();
	}
	
	public boolean hasFunction(){
		return (function != null) ;
	}

	public String getFunction() {
		if(hasFunction())
			return function;
		else
			return null;
	}
	
	public boolean hasPackage(){
		return pkg != null;
	}
	
	public String getPackage(){
		return pkg;
	}

	public boolean hasSource() {
		return (source != null);
	}

	public String getSource() {
		if(hasSource())
			return source;
		else
			return null;
	}
	
	public boolean hasGroup() {
		return (group != null);
	}
	
	public AppearanceGroup getGroup() {
		return group;
	}
	
	/**
	 * 属するグループを返す。
	 * グループを持っていない時は、親のグループを返す。
	 * @return
	 */
	public AppearanceGroup getGroupBelongTo(){
		return getGroupBelongTo(this);
	}
	
	private AppearanceGroup getGroupBelongTo(FunctionNode node){
		AppearanceGroup group = node.getGroup();
		if(group != null){
			return group;
		}else{
			FunctionNode parent = (FunctionNode)node.getParent();
			if(parent == null)
				return null;
			else
				return getGroupBelongTo(parent);
		}
	}
	
	public FunctionNode copy(){
		return new FunctionNode(this);
	}

	@Override
	public int hashCode() {
		return getTitle().hashCode();
	}
}
