package com.ef_prime.rflow.explorer;

import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.group.AppearanceGroupManager;
import com.ef_prime.rflow.group.IconAppearanceGroup;
import com.ef_prime.rflow.group.ShapeAppearanceGroup;
import com.ef_prime.rflow.io.ElementWriter;
import com.ef_prime.rflow.xml.XMLException;
import com.ef_prime.rflow.xml.XMLWriter;

public class FunctionTreeElementWriter implements ElementWriter, FunctionTreeElements{

	private AppearanceGroupManager groupManager; 
	private FunctionNode functionRoot;
	
	public FunctionTreeElementWriter(AppearanceGroupManager groupManager, FunctionNode functionRoot) {
		this.functionRoot = functionRoot;
		this.groupManager = groupManager;
	}

	public void write(XMLWriter out) throws XMLException {
		if(functionRoot == null)return;
	
		out.writeStartElement(FUNCTION_TREE);
		out.writeAttribute(VERSION, FUNCTION_TREE_VERSION);
		writeFunctionGroup(out);
		writeFunctionTreeNode(out, functionRoot);
		out.writeEndElement();
	}
	
	private void writeFunctionGroup(XMLWriter out) throws XMLException{
		for(AppearanceGroup group : groupManager.getGroups()){
			out.writeEmptyElement(FUNCTION_GROUP);
			out.writeAttribute(TITLE, group.getTitle());
			if (group instanceof IconAppearanceGroup) {
				IconAppearanceGroup iconGroup = (IconAppearanceGroup) group;
				out.writeAttribute(ICON, iconGroup.getIconPath());
			}else if (group instanceof ShapeAppearanceGroup) {
				ShapeAppearanceGroup shapeGroup = (ShapeAppearanceGroup) group;
				out.writeAttribute(SHAPE, shapeGroup.getNodeShape().toString());
			}
		}
	}

	private void writeFunctionTreeNode(XMLWriter out, FunctionNode node) throws XMLException{
		out.writeStartElement(FUNCTION_NODE);
		out.writeAttribute(TITLE, node.getTitle());
		
		if(node.hasFunction()){
			out.writeAttribute(FUNCTION, node.getFunction());
			if(node.hasPackage())
				out.writeAttribute(PACKAGE, node.getPackage());
		}
		if(node.hasGroup()){
			out.writeAttribute(GROUP, node.getGroup().getTitle());
		}
		if(node.hasSource()){
			out.writeStartElement(SOURCE);
			out.writeCharacters(node.getSource());
			out.writeEndElement();
		}
		
		if(!node.isLeaf()){
			for(int i = 0; i < node.getChildCount(); i++){
				FunctionNode child = (FunctionNode)node.getChildAt(i);
				writeFunctionTreeNode(out, child);
			}
		}
		
		out.writeEndElement();
	}
	
}
