package com.ef_prime.rflow.explorer;

import java.util.HashMap;
import java.util.Map;

import org.xml.sax.Attributes;

import com.ef_prime.rflow.flow.rendering.NodeShape;
import com.ef_prime.rflow.group.AppearanceGroupManager;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.SingleElementReader;

public class FunctionTreeReader extends ElementReader implements FunctionTreeElements{
	
	private AppearanceGroupManager manager;
	private FunctionNode parentNode;
	private Map<String, String> nodeInfoMap = new HashMap<String, String>();
	
	public FunctionTreeReader(AppearanceGroupManager agm){
		this.manager = agm;
		
		addSingleElementReader(FUNCTION_GROUP, new SingleElementReader(){
			@Override
			public
			void startElement(Attributes attrs) {
				String title = attrs.getValue(TITLE);
				String icon = attrs.getValue(ICON);
				if(icon != null)
					manager.addFunctionGroup(title, icon);
				else{
					String shape = attrs.getValue(SHAPE);
					if(shape != null){
						try{
							manager.addFunctionGroup(title, NodeShape.valueOf(shape));
						}catch(Exception e){
							//TODO 未定義のNodeShapeだった場合
							e.printStackTrace();
						}
					}
				}
			}
		});
		
		addSingleElementReader(FUNCTION_NODE, new SingleElementReader(){
			@Override
			public
			void startElement(Attributes attrs) {
				if(!nodeInfoMap.isEmpty())
					createNode();
				nodeInfoMap.put(TITLE, attrs.getValue(TITLE));
				nodeInfoMap.put(GROUP, attrs.getValue(GROUP));
				nodeInfoMap.put(FUNCTION, attrs.getValue(FUNCTION));
				nodeInfoMap.put(PACKAGE, attrs.getValue(PACKAGE));
			}
			@Override
			public
			void endElement(String text) {
				if(!nodeInfoMap.isEmpty())
					createNode();
				if(!parentNode.isRoot())
					parentNode = (FunctionNode)parentNode.getParent();
			}
		});
		/*
		addElementReader(CHILDREN, new ElementReader(true, false){
			@Override
			void startElement(Attributes attrs) {
				createNode();
			}
		});
		*/
		
		addSingleElementReader(SOURCE, new SingleElementReader(){
			@Override
			public
			void endElement(String text) {
				nodeInfoMap.put(SOURCE, text);
			}
		});
	}

	private void createNode() {
		FunctionNode node = new FunctionNode(
				nodeInfoMap.get(TITLE), 
				nodeInfoMap.get(FUNCTION), 
				nodeInfoMap.get(PACKAGE),
				nodeInfoMap.get(SOURCE),
				manager.getAppearanceGroupFromGroupTitle(nodeInfoMap.get(GROUP)));
		nodeInfoMap.clear();
		
		if(parentNode != null)
			parentNode.add(node);
		
		//グループの登録
		/*if(node.hasFunction()){
			AppearanceGroup group = node.getGroupBelongTo();
			if(group != null)
				manager.addFunction(node.getFunction(), group, false);
		}*/
		
		parentNode = node;
	}
	
/*	private AppearanceGroup getGroup(FunctionNode node){
		AppearanceGroup group = node.getGroup();
		if(group != null){
			return group;
		}else{
			FunctionNode parent = (FunctionNode)node.getParent();
			if(parent == null)
				return null;
			else
				return getGroup(parent);
		}
	}*/
	
	public FunctionNode getRootNode(){
		return parentNode;
	}
	
	public AppearanceGroupManager getFunctionGroupManager(){
		return manager;
	}
}
