package com.ef_prime.rflow.explorer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import com.ef_prime.rflow.ActiveViewerListener;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.flow.rendering.NodeRendering;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.AbstractActionManager;

import edu.uci.ics.jung.graph.event.GraphEvent;
import edu.uci.ics.jung.graph.event.GraphEventListener;

public class HierarchyExplorer extends JPanel implements 
ActiveViewerListener, GraphEventListener, ItemListener
{

	private Rflow rflow;
	private JTree tree;
	private HierarchyActionManager actionManager = new HierarchyActionManager();
	
	public HierarchyExplorer(Rflow rflow){
		super();
		this.rflow = rflow;
		rflow.addActiveViewerListener(this);
		FlowViewer.addGraphEventListener(this);
		
		setLayout(new BorderLayout());

		tree = new JTree(new HierarchyNode(rflow.viewer.getViewLayout()));
		tree.setEditable(false);
		tree.setCellRenderer(new TreeRenderer());
		tree.addMouseListener(new MouseHandler());
		tree.setToggleClickCount(3);
		tree.setActionMap(actionManager);
		tree.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, actionManager.createInputMap());
		add(new JScrollPane(tree), BorderLayout.CENTER);
		
	}
	
	public void update(){
		HierarchyNode root = new HierarchyNode(rflow.viewer.getViewLayout());
		/*FlowGraph g = rflow.viewer.getGraph();
		update(root, g);*/
		tree.setModel(new DefaultTreeModel(root));
	}
	
	/*private void update(HierarchyNode parent, FlowGraph g){
		for(Object o : g.getVertices()){
			FlowNode node = (FlowNode)o;
			if(node.getModel().isSubflow()){
				HierarchyNode child = new HierarchyNode(node);
				parent.add(child);
				SubflowNodeModel model = (SubflowNodeModel)node.getModel();
				for(FlowGraph graph : model.getGraphs()){
					update(child, graph);
				}
			}
		}
	}*/
	
	private class MouseHandler extends MouseAdapter{
		@Override
		public void mouseClicked(MouseEvent e) {
			JTree tree = (JTree)e.getSource();
			Point p = e.getPoint();
			TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
			if(path == null)return;
			HierarchyNode node = (HierarchyNode)path.getLastPathComponent();
			if(node.isRoot())return;
			//左ダブルクリック
			if(e.getClickCount() == 2 && SwingUtilities.isLeftMouseButton(e)){
				rflow.viewer.editNode(node.getFlowNode());
			}
			//右クリック
			else if(SwingUtilities.isRightMouseButton(e)){
				actionManager.showPopup(tree, node, p.x, p.y);
			}
		}
	}
	
	private class TreeRenderer extends DefaultTreeCellRenderer{
		
		NodeRendering nodeRendering = rflow.viewer.getRenderer().getNodeRendering();
		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			
			HierarchyNode node = (HierarchyNode)value;
			if(node.isRoot())
				setText(Messages.getString("Explorer.hierarchy.root")); //$NON-NLS-1$
			else{
				setText(node.getFlowNode().getTitle());
				setIcon(nodeRendering.getSmallIcon(node.getFlowNode()));
			}
			
			return this;
		}
	}
	//----------------------------------ItemListener------------------------------//
	public void itemStateChanged(ItemEvent e) {
		tree.repaint();
	}

	//-----------------------------ActiveViewerListener---------------------------//
	public void activeViewerChanged(FlowViewer activeViewer) {
		FlowNode node = activeViewer.getGraph().getParentNode();
		HierarchyNode hNode = (HierarchyNode)tree.getModel().getRoot();
		List<HierarchyNode> hit = new ArrayList<HierarchyNode>();
		search(hNode, node, hit);
		
		if(hit.size() > 0){
			HierarchyNode hitNode = hit.get(0);
			TreePath path = hitNode.getPath();
			//expandPath(path);
			tree.setSelectionPath(path);
			tree.scrollPathToVisible(path);
		}
	}
	
	private void search(HierarchyNode hNode, FlowNode node, List<HierarchyNode> hit){
		if(hit.size() > 0)return;
		if(hNode.getFlowNode() == node)
			hit.add(hNode);
		for(int i = 0; i < hNode.getChildCount(); i++){
			search((HierarchyNode)hNode.getChildAt(i), node, hit);
		}
	}
	
	//-----------------------------ViewerEventListener-----------------------------//
	public void vertexRemoved(GraphEvent event) {
		FlowNode node = (FlowNode)event.getGraphElement();
		if(node.getModel().isSubflow())
			update();
	}
	public void vertexAdded(GraphEvent event) {
		FlowNode node = (FlowNode)event.getGraphElement();
		if(node.getModel().isSubflow())
			update();
	}
	public void edgeAdded(GraphEvent event) {}
	public void edgeRemoved(GraphEvent event) {}
	
	//-----------------------------------------------------------------------------//
	

	private enum HierarchyActions{
		REFRESH
		;
	}
	
	private class HierarchyActionManager extends AbstractActionManager<HierarchyActions>{
		public HierarchyActionManager(){
			putAction(HierarchyActions.REFRESH, new RefreshAction());
		}
		
		public InputMap createInputMap(){
			return getInputMap(HierarchyActions.values());
		}

		public void showPopup(Component invoker, HierarchyNode node, int x, int y) {
			JPopupMenu popup = new JPopupMenu();
			popup.add(getAction(HierarchyActions.REFRESH));
			popup.show(invoker, x, y);
		}
		
		private class RefreshAction extends AbstractAction{
			public RefreshAction() {
				putValue(Action.NAME, Messages.getString("Action.refresh")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
			}
			public void actionPerformed(ActionEvent e) {
				update();
			}
		}
	}
		
		
}
