package com.ef_prime.rflow.explorer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.base.SubflowNodeModel;

public class HierarchyNode implements TreeNode{
	
	private FlowNode flowNode;
	private FlowNode[] children;
	private ViewLayout[] layouts;//layout of children
	private HierarchyNode parent;

	/**
	 * for root node
	 * @param layout
	 */
	public HierarchyNode(ViewLayout layout){
		this(null, null, layout);
	}
	
	/**
	 * for non root node
	 * @param flowNode
	 */
	public HierarchyNode(FlowNode flowNode, HierarchyNode parent) {
		this(flowNode, parent, null);
	}
	
	/**
	 * 
	 * @param flowNode node. model must be instance of SubflowNodeModel.
	 * or null if root.
	 * @param layout layout of children
	 */
	private HierarchyNode(FlowNode flowNode, HierarchyNode parent, ViewLayout layout) {
		this.flowNode = flowNode;
		this.parent = parent;
		if(layout != null)
			this.layouts = new ViewLayout[]{layout};
		else{
			SubflowNodeModel model = (SubflowNodeModel)flowNode.getModel();
			layouts = model.getLayouts();
		}
		
		List<FlowNode> list = new ArrayList<FlowNode>();
		for(ViewLayout viewLayout : layouts){
			for(Object o : viewLayout.getGraph().getVertices()){
				FlowNode node = (FlowNode)o;
				if(node.getModel().isSubflow()){
					list.add(node);
				}
			}
		}
		children = list.toArray(new FlowNode[0]);
		//Arrays.sort(children, new LocationCompartor(layouts));
		Arrays.sort(children, new TitleCompartor());
	}
	
	public FlowNode getFlowNode(){
		return flowNode;
	}
	
	public boolean isRoot(){
		return parent == null;
	}
	
	public TreePath getPath(){
		LinkedList<HierarchyNode> list = new LinkedList<HierarchyNode>();
		
		HierarchyNode parent = this;
		while(parent != null){
			list.addFirst(parent);
			parent = (HierarchyNode)parent.getParent();
		}
		
		return new TreePath(list.toArray(new HierarchyNode[0]));
	}

	public Enumeration children() {
		Vector<HierarchyNode> v = new Vector<HierarchyNode>();
		for(FlowNode node : children)
			v.add(new HierarchyNode(node, this));
		return v.elements();
	}

	public boolean getAllowsChildren() {
		return true;
	}

	public TreeNode getChildAt(int childIndex) {
		return new HierarchyNode(children[childIndex], this);
	}

	public int getChildCount() {
		return children.length;
	}

	public int getIndex(TreeNode node) {
		HierarchyNode hNode = (HierarchyNode)node;
		for(int i = 0; i < children.length; i++)
			if(children[i] == hNode.getFlowNode())
				return i;
		return -1;
	}

	public TreeNode getParent() {
		return parent;
	}

	public boolean isLeaf() {
		return getChildCount() == 0;
	}
	

	/*private static final class LocationCompartor implements Comparator<FlowNode>{
		
		private Map<Vertex, Point2D> locationMap;

		public LocationCompartor(ViewLayout[] layouts) {
			locationMap = layouts[0].getLocationMap();
			for(int i = 1; i < layouts.length; i++){
				locationMap.putAll(layouts[i].getLocationMap());
			}
		}

		public int compare(FlowNode o1, FlowNode o2) {
			Point2D p1 = locationMap.get(o1);
			Point2D p2 = locationMap.get(o2);
			if(p1.getY() > p2.getY())
				return 1;
			else if(p1.getY() < p2.getY())
				return -1;

			if(p1.getX() > p2.getX())
				return 1;
			else
				return -1;
		}
	};*/


	private static final class TitleCompartor implements Comparator<FlowNode>{

		public int compare(FlowNode o1, FlowNode o2) {
			String t1 = o1.getTitle().toLowerCase();
			String t2 = o2.getTitle().toLowerCase();
			return t1.compareTo(t2);
		}
	};
	
}
