package com.ef_prime.rflow.explorer;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceAdapter;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.LayoutTransferable;
import com.ef_prime.rflow.flow.ViewLayout;

public class PartsExplorerPanel extends JPanel{

	Rflow rflow;
	JTree tree;
	
	public PartsExplorerPanel(Rflow rflow){
		super();
		this.rflow = rflow;
		setLayout(new BorderLayout());

		tree = new JTree(createSampleTreeNode());
		tree.setEditable(true);
		tree.addMouseListener(new MouseHandler());
		JScrollPane sp = new JScrollPane(tree);
		
		add(sp);
		

		//ドラッグ動作
		final DragSource dragSource = new DragSource();
		dragSource.createDefaultDragGestureRecognizer(tree, DnDConstants.ACTION_COPY, new DragGestureListener(){
			public void dragGestureRecognized(DragGestureEvent dge) {
		        Point clickPoint = dge.getDragOrigin();
		        TreePath path = tree.getPathForLocation(clickPoint.x, clickPoint.y);
		        if (path == null)return;
		        
				PartsNode node = (PartsNode) path.getLastPathComponent();
				if (!node.hasLayout())return;

		        Transferable trans = new LayoutTransferable(node.getViewLayout(), node.getLayoutImage(PartsExplorerPanel.this.rflow.viewer));
		        try {
					dge.getDragSource().startDrag(dge, Cursor.getSystemCustomCursor("MoveDrop.32x32"),
					        trans, new DragSourceAdapter(){
					});
				} catch (Exception e) {
				}
		        
			}
		});
		
	}
	
	public void init(PartsNode partsRoot){
		tree.setModel(new DefaultTreeModel(partsRoot));
	}

	public PartsNode getRoot(){
		return (PartsNode)tree.getModel().getRoot();
	}
	
	private PartsNode createSampleTreeNode(){
		PartsNode root = new PartsNode("パーツ");
		
		return root;
	}
	
	public void addParts(String title, ViewLayout partsLayout){
		PartsNode node = new PartsNode(title, partsLayout);
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)tree.getModel().getRoot();
		root.add(node);
		DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
		model.nodesWereInserted(root, new int[]{root.getChildCount()-1});
	}
	
	private class MouseHandler extends MouseAdapter{
		@Override
		public void mousePressed(MouseEvent e) {
			JTree tree = (JTree)e.getSource();
			Point p = e.getPoint();
			TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
			if(path == null)return;
			
			PartsNode node = (PartsNode) path.getLastPathComponent();
			
			if(SwingUtilities.isLeftMouseButton(e)){
				//左ダブルクリック
				if(e.getClickCount() == 2){
					if (!node.hasLayout())return;
					//node.showContentsDialog(rflow, e.getLocationOnScreen());
					Point screenP = e.getPoint();
					SwingUtilities.convertPointToScreen(screenP, e.getComponent());
					node.showContentsDialog(rflow, screenP);
				}
			}
			
		}
	}
}
