package com.ef_prime.rflow.explorer;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;

import javax.swing.JDialog;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.tree.DefaultMutableTreeNode;

import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowLayout;

import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.visualization.VertexLocationFunction;

public class PartsNode extends DefaultMutableTreeNode{

	private ViewLayout layout;
	
	private JDialog partsDialog;
	private FlowViewer smallViewer;

	public PartsNode(String title) {
		this(title, null);
	}
	
	public PartsNode(String title, ViewLayout layout) {
		super(title);
		this.layout = layout;
	}
	
	public boolean hasLayout(){
		return layout != null;
	}
	
	public ViewLayout getViewLayout() {
		if(smallViewer != null && smallViewer.isVisible())
			layout = smallViewer.getViewLayout();
		return layout;
	}
	
	public String getTitle(){
		return getUserObject().toString();
	}
	
	public void showContentsDialog(Rflow rflow, Point p){
		if(partsDialog != null && partsDialog.isVisible()){
			partsDialog.requestFocus();
			return;
		}
		
		smallViewer = createViewer(rflow.viewer);
		
		partsDialog = new JDialog(rflow.frame);
		partsDialog.getContentPane().add(smallViewer);
		partsDialog.pack();
		partsDialog.setLocation(p.x + 50, p.y - partsDialog.getHeight()/2);
		partsDialog.setVisible(true);
	}

	private FlowViewer createViewer(FlowViewer viewer) {
		FlowLayout slayout = new FlowLayout(layout.getGraph());
		slayout.initialize(layout.getSize(), new VertexLocationFunction(){
			public Point2D getLocation(ArchetypeVertex v) {
				Point2D p = layout.getLocationMap().get(v);
				if(p == null)
					return new Point2D.Double();
				return p;
			}

			public Iterator getVertexIterator() {
				return layout.getGraph().getVertices().iterator();
			}
		});
		
		final FlowViewer fv = viewer.createNewViewer();
		fv.setPreferredSize(layout.getSize());
		fv.setGraphLayout(slayout, false);
		fv.addComponentListener(new ComponentAdapter(){
			@Override
			public void componentResized(ComponentEvent e) {
				FlowViewer viewer = (FlowViewer)e.getSource();
				viewer.getGraphLayout().resize(viewer.getSize());
			}
		});
		fv.addAncestorListener(new AncestorListener(){
			public void ancestorAdded(AncestorEvent event) {}
			public void ancestorMoved(AncestorEvent event) {}
			public void ancestorRemoved(AncestorEvent event) {
				layout = fv.getViewLayout();
				smallViewer = null;
			}			
		});
		
		return fv;
	}
	
	public Image getLayoutImage(FlowViewer sv){
		FlowViewer viewer;
		/*if(smallViewer == null){
			viewer = createViewer(sv);
		}else{
			viewer = smallViewer;
		}*/
		viewer = createViewer(sv);
		viewer.setSize(layout.getSize());
		viewer.setBackground(new Color(255,255,255,0));
		
	    BufferedImage image = new BufferedImage(
	    		layout.getSize().width,layout.getSize().height,BufferedImage.TYPE_INT_ARGB);
	    
	    Graphics2D g = image.createGraphics();
	    g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC, 0.6f));
	    viewer.paintToGraphics(g);
	    g.dispose();
	    
	    return image;
	}
}
