package com.ef_prime.rflow.explorer;

import java.awt.Frame;
import java.io.IOException;

import org.xml.sax.Attributes;

import com.ef_prime.rflow.exception.ReadException;
import com.ef_prime.rflow.flow.GraphReader;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.FlowDataReader;
import com.ef_prime.rflow.io.SingleElementReader;

public class PartsTreeReader extends ElementReader implements PartsTreeElements {

	PartsNode parentNode;

	NodeData nodeData = new NodeData();

	public PartsTreeReader(final Frame parent) {
		addSingleElementReader(PARTS_NODE, new SingleElementReader() {
			@Override
			public void startElement(Attributes attrs) {
				if (!nodeData.isEmpty())
					createNode();
				nodeData.title = attrs.getValue(TITLE);
			}

			@Override
			public void endElement(String text) {
				if (!nodeData.isEmpty())
					createNode();
				if (!parentNode.isRoot())
					parentNode = (PartsNode) parentNode.getParent();
			}
		});

		addSingleElementReader(PARTS, new SingleElementReader() {
			@Override
			public void endElement(String text) {
				FlowDataReader reader = new FlowDataReader();
				GraphReader graphElementManager = new GraphReader(parent, null);
				try {
					reader.read(text,
							new ElementReader[] { graphElementManager });
				} catch (ReadException e) {
					// TODOAuto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODOAuto-generated catch block
					e.printStackTrace();
				}
				nodeData.layout = graphElementManager.getViewLayout();
			}
		});
	}

	private void createNode() {
		PartsNode node = new PartsNode(nodeData.title, nodeData.layout);
		nodeData.clear();
		if (parentNode != null)
			parentNode.add(node);
		parentNode = node;
	}

	private static class NodeData {
		String title;

		ViewLayout layout;

		boolean isEmpty() {
			return ((title == null) && (layout == null));
		}

		void clear() {
			title = null;
			layout = null;
		}
	}

	public PartsNode getRoot() {
		return parentNode;
	}
}
