package com.ef_prime.rflow.explorer;

import com.ef_prime.rflow.flow.GraphWriter;
import com.ef_prime.rflow.io.ElementWriter;
import com.ef_prime.rflow.io.FlowDataWriter;
import com.ef_prime.rflow.xml.XMLException;
import com.ef_prime.rflow.xml.XMLWriter;

public class PartsTreeWriter implements ElementWriter, PartsTreeElements{
	
	private PartsNode partsRoot;
	
	public PartsTreeWriter(PartsNode partsRoot) {
		this.partsRoot = partsRoot;
	}
	
	public void write(XMLWriter out) throws XMLException {
		if(partsRoot == null)return;
	
		out.writeStartElement(PARTS_TREE);
		writePartsNode(out, partsRoot);
		out.writeEndElement();
	}

	private void writePartsNode(XMLWriter out, PartsNode node) throws XMLException{
		out.writeStartElement(PARTS_NODE);
		out.writeAttribute(TITLE, node.getTitle());
		
		if(node.hasLayout()){
			out.writeStartElement(PARTS);
			ElementWriter writer = new GraphWriter(node.getViewLayout(), null);
			String s = new FlowDataWriter().writeToString(new ElementWriter[]{writer});
			out.writeCharacters(s);
			out.writeEndElement();
		}
		
		if(!node.isLeaf()){
			for(int i = 0; i < node.getChildCount(); i++){
				PartsNode child = (PartsNode)node.getChildAt(i);
				writePartsNode(out, child);
			}
		}
		
		out.writeEndElement();
	}
}
