package com.ef_prime.rflow.flow;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JOptionPane;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.console.CommandAdapter;
import com.ef_prime.rflow.console.CompoundCommand;
import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.node.base.SubflowNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowOptionPane;
import com.ef_prime.rflow.util.FileUtility;

import edu.uci.ics.jung.graph.event.GraphEvent;
import edu.uci.ics.jung.graph.event.GraphEventListener;

public class CacheHandler implements GraphEventListener, NodeModelChangeListener{
	
	private static final String BACKUP_EXTENSION = ".bak"; //$NON-NLS-1$
	
	private Rflow rflow;
	private FlowViewer viewer;//viewer is root viewer
	private FlowState state;
	
	private boolean lockCacheState = false;

	//private Set<FlowNode> cacheSet = new HashSet<FlowNode>();
	
	private static final int CACHE_STATE_OK = 0;
	private static final int CACHE_STATE_NOT_EXIST = 1;
	private static final int CACHE_STATE_COMMAND_MISMATCH = 2;
	//private static final int CACHE_STATE_HASH_MISMATCH = 3;
	private static final int CACHE_STATE_SIZE_MISMATCH = 4;
	
	
	public CacheHandler(Rflow rflow, FlowViewer viewer) {
		this.rflow = rflow;
		this.viewer = viewer;
		state = viewer.getRootFlowState();
	}
	
	public void init(){
		//init cache command and file data
		//set command to cache data
		for(FlowNode node : state.getCacheNodes()){
			CacheData old = node.getCacheData();
			String command = (old.isCommand())?old.getCommand():FlowUtil.getFlowCommand(node);
			File file = new File(getCacheDir(false), old.getFile().getName());
			node.setCacheData(new CacheData(file, command, old.getDate(), old.getSize(), old.isValid()));
		}
	}
	
	public void update(){
		update(state.getCacheNodes().toArray(new FlowNode[0]));
	}

	public void update(final FlowNode[] cachedNodes){
		//check all nodes cache state
		if(isCacheEmpty())
			return;
		
//		final ProgressMonitor pm = new ProgressMonitor(rflow.frame, Messages.getString("Message.cache_verification"), true); //$NON-NLS-1$
//		pm.setLabelText(Messages.getString("Message.verifying_cache")); //$NON-NLS-1$
//		Thread checkThread = new Thread(new Runnable(){
//			public void run() {
//				try {
					String message  = ""; //$NON-NLS-1$
					for(int i = 0; i < cachedNodes.length; i++){
						FlowNode node = cachedNodes[i];
						//グレーのキャッシュはとばす
						if(!node.getCacheData().isValid())continue;
						String msg = checkCacheState(node);
						if(msg != null){
							message += Messages.getString("Message.node_name")+" : "+node.getTitle()+"\n";  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
							message += msg + "\n\n"; //$NON-NLS-1$
						}
					}
						//if(b != CACHE_STATE_OK){
							/*case CACHE_STATE_HASH_MISMATCH:
							msg = Messages.getString("Error.cache_hash_mismatch"); //$NON-NLS-1$
							break;*/
							/*String s = msg + " : " + node.getTitle() + "\n"; //$NON-NLS-1$ //$NON-NLS-2$
							String title = Messages.getString("Error.title.error"); //$NON-NLS-1$
							if(i < cachedNodes.length-1){
								int ret = JOptionPane.showConfirmDialog(viewer, 
										s + Messages.getString("Message.ask_continue_verify"), title, //$NON-NLS-1$
										JOptionPane.YES_NO_OPTION, JOptionPane.ERROR_MESSAGE);
								if(ret == JOptionPane.NO_OPTION)
									break;
							}else{
								JOptionPane.showMessageDialog(viewer, s, title, JOptionPane.YES_OPTION);
							}*/
						//}
//				} catch (CanceledException e) {
//				}
//				pm.close();
//			}
//		});
//		checkThread.start();
//		pm.open();
					
		//エラーが合った場合、メッセージを表示
		if(message.length() > 0){
			String s = Messages.getString("Error.cache_load_failed"); //$NON-NLS-1$
			RflowOptionPane.showTextAreaDialog(viewer, Messages.getString("Error.title.error"), //$NON-NLS-1$
					s, message, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
		}
	}
	
	/*private void update(FlowGraph g) throws CanceledException{
		for(Object o : g.getVertices()){
			FlowNode node = (FlowNode)o;
			if(node.isCacheData()){
				checkCacheExists(node);
			}
			
			if(node.getModel().isSubflow() && node.getModel().isOpenSubflow()){
				SubflowNodeModel model = (SubflowNodeModel)node.getModel();
				for(FlowGraph child : model.getGraphs())
					update(child);
			}
		}
	}*/

	public void saveCaches(File before, File after, File beforeTempDir){
		//assume flowsetting FOLDER is set "after"
		
		boolean save_as = before == null || !before.equals(after);
		for(FlowNode node : state.getCacheNodes()){
			CacheData data = node.getCacheData();
			File file = null;
			try {
				file = data.getFile();
				if(file.exists()){
					boolean isTemp = FileUtility.isChild(beforeTempDir, file);
					
					File cacheFile = new File(getCacheDir(false), file.getName());
					if(isTemp && data.isValid()){
						//if cache is in temp dir, move

						if(cacheFile.exists() && !cacheFile.delete())
							throw new IOException();
						if(!cacheFile.getParentFile().exists() && !cacheFile.getParentFile().mkdirs())
							throw new IOException();
						if(!file.renameTo(cacheFile))
							throw new IOException();
						CacheData old = node.getCacheData();
						//String name = ExtensionUtility.removeExtension(cacheFile.getName());
						state.setCache(node, new CacheData(cacheFile, old.getCommand(), old.getDate(), old.getSize(), old.isValid()));
					}else if(!isTemp){
						if(!save_as && cacheFile.exists() && !data.isValid())
							//remove invalid cache if overwrite
							cacheFile.delete();//ignore failed
						else if(file.exists() && save_as && data.isValid()){
							if(!cacheFile.getParentFile().exists() && !cacheFile.getParentFile().mkdirs())
								throw new IOException();
							//copy valid cache if save as
							FileUtility.copy(file, cacheFile);
						}
					}
				}
			} catch (IOException e) {
				if(file != null)
					JOptionPane.showMessageDialog(viewer, Messages.getString("Error.cache_move") + ":" + file.getAbsolutePath(),  //$NON-NLS-1$//$NON-NLS-2$
							Messages.getString("Error.title.warning"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
				if(Rflow.DEBUG > 0)
					e.printStackTrace();
			}
		}
		
		if(save_as){
			init();
		}
	}
	
	/**
	 * ノードにキャッシュを設定します
	 * @param node
	 * @throws IOException
	 */
	public void markCache(final FlowNode node) throws IOException{
		final File rdataFile = getCacheFileNew(node);
		String command = FlowUtil.getFlowCommand(node);
		state.setCache(node, new CacheData(rdataFile, command, null, 0L, false));
		//
		rflow.getActiveViewer().repaint();
		rflow.actionManager.updateActionState();
		rflow.setDirty(true);
	}

	/**
	 * キャッシュを作成するコマンドを返します。
	 * @param node
	 * @return
	 * @throws IOException
	 */
	public Command cacheNode(final FlowNode node) throws IOException{
		CompoundCommand command = new CompoundCommand();
		
		//clear and execute
		//if(exec)
		//	command.add(rflow.getActiveViewer().getFlowExecuter().getCommand(node, null, false, true, true));

		//cache file
		//final File rdataFile = (node.isCacheData())?new File(getCacheDir(true), node.getCacheData().getFile().getName()):getCacheFileNew(node);
		
		//新しく作成されるキャッシュファイル
		final File rdataFile = getCacheFileNew(node);
		//キャッシュのバックアップファイル(存在しない可能性もある)
		final File backupFile = (node.isCacheData())?getBackupFile(node.getCacheData().getFile().getName()):null;
		
		if(!rdataFile.getParentFile().exists() && !rdataFile.getParentFile().mkdirs()){
			throw new IOException("failed to make cache folder"); //$NON-NLS-1$
		}
		
		
		//write RData (save image)
		String rdata = Constants.replaceFileSeparator(rdataFile.getAbsolutePath());
		BasicCommand c = new BasicCommand("save.image(file="+ Constants.quote(rdata) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
		c.addCommandListener(new CommandAdapter(){
			public void commandFinished(Command c) {
				//set flow state
				String command = FlowUtil.getFlowCommand(node);
				//String name = ExtensionUtility.removeExtension(rdataFile.getName());
				GregorianCalendar date = new GregorianCalendar();
				long size = rdataFile.length();

				//hash of cache data
				/*String hash = null;
				try {
					hash = FileUtility.getHash(rdataFile, null);
				} catch (IOException e) {
					if(Rflow.DEBUG > 0)e.printStackTrace();
				} catch (CanceledException e) {
				}*/

				state.setCache(node, new CacheData(rdataFile, command, date, size, true));

				//
				rflow.getActiveViewer().repaint();
				rflow.actionManager.updateActionState();
				rflow.setDirty(true);
				
				//バックアップの消去
				if(backupFile != null && backupFile.exists()){
					//失敗は無視する
					backupFile.delete();
				}
			}
		});
		command.add(c);
		return command;
	}

	
	public Command loadCache(FlowNode node) throws FileNotFoundException{
//		File rdataFile = getCacheFile(node.getCacheData());
		File rdataFile = node.getCacheData().getFile();
		/*if(!rdataFile.isAbsolute()){
			rdataFile = new File(rflow.getWorkingFile().getParent(), rdataFile.getPath());
		}*/
		String msg = checkCacheState(node);
		if(msg != null){
			//checkCacheExists(node);
			node.getCacheData().setValid(false);
			viewer.repaintFamiliy();
			throw new FileNotFoundException(msg); //$NON-NLS-1$
		}
		
		//read rdata (load image)
		String rdata = Constants.replaceFileSeparator(rdataFile.getAbsolutePath());
		//String rdata = Constants.replaceFileSeparator(rdataFile.getPath());
		//if(!RCommand.load(rdata))
		//	throw new IOException("failed to load image");
		
		return new BasicCommand("load(file="+ Constants.quote(rdata) + ")"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public void removeCache(FlowNode node, boolean removeMark, boolean backup){
		try{
			//cache file
			File rdataFile = node.getCacheData().getFile();
			boolean isTempFile = FileUtility.isChild(getCacheDir(true), rdataFile);
			
			if(!backup){
				//don't backup
				if(rdataFile.exists() && isTempFile && !rdataFile.delete())
					throw new IOException("failed to delete RData cache file"); //$NON-NLS-1$
			}else{
				//do backup
				File backupFile = getBackupFile(rdataFile.getName());

				if(rdataFile.exists()){
					//backup one cache in temp dir

					//delete last backup file
					if(backupFile.exists() && !backupFile.delete())
						throw new IOException("failed to delete RData cache file"); //$NON-NLS-1$

					//create backup dir
					if(!backupFile.getParentFile().exists() && !backupFile.getParentFile().mkdirs())
						throw new IOException("failed to create backup folder"); //$NON-NLS-1$

					if(isTempFile){
						//move and rename current cache to backup
						if(!rdataFile.renameTo(backupFile))
							throw new IOException("failed to move cache file"); //$NON-NLS-1$
					}else{
						//create dummy backup
						if(!backupFile.createNewFile())
							throw new IOException("failed to create dummy cache flag"); //$NON-NLS-1$
					}
				}
			}

			if(removeMark)
				state.removeCacheNode(node);
			else
				node.getCacheData().setValid(false);
			//
			//rflow.explorer.getFileExplorer().update();
			rflow.actionManager.updateActionState();
			rflow.setDirty(true);
			viewer.repaintFamiliy();
		} catch (IOException e) {
			if(Rflow.DEBUG > 0)e.printStackTrace();
			JOptionPane.showMessageDialog(viewer, Messages.getString("Error.cache_delete") + ": " + node.getTitle(), "Error.title.warning", JOptionPane.WARNING_MESSAGE);  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
		}
	}
	
	/**
	 * キャッシュファイルの検証を行います。
	 * @param node
	 * @return エラーが合った場合、エラーメッセージ。ない場合は、null
	 */
	private String checkCacheState(FlowNode node){
		String msg = ""; //$NON-NLS-1$
		switch(checkCacheExists(node)){
		case CACHE_STATE_NOT_EXIST:
			msg = Messages.getString("Error.no_cache"); //$NON-NLS-1$
			break;
		case CACHE_STATE_COMMAND_MISMATCH:
			msg = Messages.getString("Error.cache_command_mismatch"); //$NON-NLS-1$
			break;
		case CACHE_STATE_SIZE_MISMATCH:
			msg = Messages.getString("Error.cache_size_mismatch"); //$NON-NLS-1$
			break;
		}
		if(msg.length() == 0)
			return null;
		else
			return msg;
	}
	
	/**
	 * 
	 * @param node
	 * @return hash equals; true if no hash verification
	 */
	private int checkCacheExists(FlowNode node){
		File rdataFile;
		//boolean valid = false;
		//try {
			rdataFile = node.getCacheData().getFile();
			CacheData data = node.getCacheData();
			
			//not created
			if(!data.isDate()){
				node.getCacheData().setValid(false);
				return CACHE_STATE_OK;
			}
			
			//exists
			if(!rdataFile.exists()){
				node.getCacheData().setValid(false);
				return CACHE_STATE_NOT_EXIST;
			}
			
			//command
			if(!FlowUtil.getFlowCommand(node).equals(data.getCommand())){
				node.getCacheData().setValid(false);
				return CACHE_STATE_COMMAND_MISMATCH;
			}
			
			//size
			if(data.isSize() && data.getSize() != rdataFile.length()){
				node.getCacheData().setValid(false);
				return CACHE_STATE_SIZE_MISMATCH;
			}
			//boolean lastmodMatch = false;
			//check hash
			/*if(!lastmodMatch && data.isHash()){
				String hash = data.getHash();
				String hash2 = FileUtility.getHash(rdataFile, pm);
				if(!hash.equals(hash2)){
					valid = false;
				}else{
					valid = true;
					if((Boolean)Rflow.setting.getProperty(Setting.Key.CHECK_CACHE_LASTMODIFIED))
						node.getCacheData().setLastModified(rdataFile.lastModified());
				}
			}*/
		//} catch (IOException e) {
		//	valid = false;
		//}
		node.getCacheData().setValid(true);
		viewer.repaintFamiliy();
		return CACHE_STATE_OK;
	}
	
	public void restoreCache(FlowNode node, boolean restoreFlag){
		if(restoreFlag){
			//just restore flag only
			state.setCache(node, node.getCacheData());
			viewer.repaintFamiliy();
			return;
		}
		
		try {
			
			File f = node.getCacheData().getFile();//getCacheFile(node.getCacheData(), false);
			File backup = getBackupFile(f.getName());
			boolean backupIsOrginal = (backup.length() == 0);

			if(backup.exists()){
				//check command equals
				if(node.getCacheData().isCommand()){
					String c = FlowUtil.getFlowCommand(node);
					if(!c.equals(node.getCacheData().getCommand())){
						throw new IOException("saved cache command conflicts with current flow command"); //$NON-NLS-1$
					}
				}
				if(!backupIsOrginal){
					if(f.exists() && !f.delete())
						throw new IOException("failed to delete cache"); //$NON-NLS-1$
					if(!f.getParentFile().exists() && !f.getParentFile().mkdirs())
						throw new IOException("failed to create cache folder"); //$NON-NLS-1$
					if(!backup.renameTo(f))
						throw new IOException("failed to move backup cache"); //$NON-NLS-1$
				}else{
					backup.delete();//delete backup dummy
				}
				
				node.getCacheData().setValid(true);
				state.setCache(node, node.getCacheData());
			}
			
			viewer.repaintFamiliy();
		} catch (IOException e) {
			if(Rflow.DEBUG > 0)e.printStackTrace();
		}
	}
	
	public void cleanCaches(){
		//try {
			Set<String> names = new HashSet<String>();
			for(FlowNode node : state.getCacheNodes()){
				names.add(node.getCacheData().getFile().getName());
			}
			File dir = getCacheDir(false);
			
			if(dir.exists()){
				File[] files = dir.listFiles();
				for(File cacheFile : files){
					if(cacheFile.isFile()){
						if(cacheFile.getName().endsWith(Constants.RDATA_EXTENSION) 
								&& !names.contains(cacheFile.getName()))
						{
							if(Rflow.DEBUG > 0)
								System.out.println("delete cachefile:" + cacheFile.getName()); //$NON-NLS-1$
							cacheFile.delete();
						}
					}
				}
				if(dir.listFiles().length == 0)
					dir.delete();
			}
		/*} catch (IOException e) {
			//do nothing
			if(Rflow.DEBUG > 0)e.printStackTrace();
		}*/
	}
	
	private File getCacheFileNew(FlowNode node) throws IOException{
		//get command of node
		String command = FlowUtil.getFlowCommand(node);

		//cache file
		File cacheDir = getCacheDir(true);
		if(!cacheDir.exists() && !cacheDir.mkdirs()){
			throw new IOException("failed to create cache dir"); //$NON-NLS-1$
		}

		File rFile;
		int i = 0;
		while(true){
			String num = (i == 0)?"":String.valueOf(i); //$NON-NLS-1$
			Long currentTime = new Long(System.currentTimeMillis());
			String hash = Integer.toHexString(command.hashCode()^currentTime.hashCode());
			String cache = "cache" + num + "_" + hash; //$NON-NLS-1$ //$NON-NLS-2$
			rFile = new File(cacheDir, cache + Constants.RDATA_EXTENSION);
			//if cache already exists, check command equals
			if(rFile.exists() || state.isCacheFile(cache + Constants.RDATA_EXTENSION)){
				i++;
				continue;
			}
			break;
		}

		return rFile;
	}
	
	public void clearAllCache(){
		for(FlowNode node : state.getCacheNodes()){
			if(node.getCacheData().isValid())
				node.getCacheData().setValid(false);
		}
		viewer.repaint();
	}

	/**
	 * キャッシュファイルの置かれるディレクトリを返します
	 * tempDirがtrueの場合、テンポラリディレクトリが返されます
	 * @param tempDir
	 * @return
	 */
	public File getCacheDir(boolean tempDir){
		File f = (tempDir)?rflow.getTempDir():rflow.getFlowDir();
		return new File(f, "cache"); //$NON-NLS-1$
	}
	
	private File getBackupFile(String name){
		File tempFile = new File(getCacheDir(true), name);
		return new File(tempFile.getParentFile(), tempFile.getName()+BACKUP_EXTENSION);
	}

	public synchronized void setCacheStateLock(boolean lock){
		lockCacheState = lock;
	}
	
	/*----------------------------------GraphEventListener----------------------------------*/

	public void edgeAdded(GraphEvent event) {
		FlowEdge edge = (FlowEdge)event.getGraphElement();
		/*FlowNode closed = FlowNode.getTopClosedSubflowNode(edge);
		if(closed != null){
			checkCache(closed);
		}else{*/
			//checkCacheFirst((FlowNode)edge.getSource());
			checkCache((FlowNode)edge.getDest());
		//}
	}

	public void edgeRemoved(GraphEvent event) {
		FlowEdge edge = (FlowEdge)event.getGraphElement();
		/*FlowNode closed = FlowNode.getTopClosedSubflowNode(edge);
		if(closed != null){
			checkCache(closed);
		}else{*/
			//checkCacheFirst((FlowNode)edge.getDest());
			checkCache((FlowNode)edge.getDest());
		//}
	}

	public void vertexAdded(GraphEvent event) {}
	
	public void vertexRemoved(GraphEvent event) {
		//note that if vertex was connected with edge, 
		//removing edge may clear this vertex cache
		FlowNode node = (FlowNode)event.getGraphElement();
		boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
		FlowViewer viewer = rflow.getActiveViewer();
		if(!isCacheEmpty()){
			if(node.isCacheData() && node.getCacheData().isValid()){
				removeCache(node, false, backup);
				if (backup) {
					viewer.getUndoableEditSupport().beginUpdateOnce();
					viewer.fireUndoableEditUpdate(viewer.undoableEditFactory
							.createClearCache(node, false, false));
				}				
			}
			if(node.getModel().isSubflow()){
				SubflowNodeModel model = (SubflowNodeModel)node.getModel();
				Set<FlowNode> set = FlowUtil.getAllDescendants(model);
				for(FlowNode fn : set)
					if(fn.isCacheData() && node.getCacheData().isValid()){
						removeCache(fn, false, backup);
						if (backup) {
							viewer.getUndoableEditSupport().beginUpdateOnce();
							viewer.fireUndoableEditUpdate(viewer.undoableEditFactory
											.createClearCache(fn, false, false));
						}						
					}
						
			}
		}
	}

	/*------------------------------NodeModelChangeListener-------------------------------*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		if(isCacheEmpty())return;		
		checkCache(node);
	}
	
	public void subflowNodeModelChanged(FlowNode node) {
		if(isCacheEmpty())return;		
		checkCache(node);
	}
	

	/*private void checkCacheFirst(FlowNode node) {
		if(lockCacheState)return;
		if(!isCacheEmpty()){
			//if subflow, first node is in node
			if(node.getModel().isOpenSubflow()){
				node = ((SubflowNodeModel)node.getModel()).getIn();
			}
			boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
			for(FlowNode cacheNode : new LinkedList<FlowNode>(state.getCacheNodes())){
				if(cacheNode.getGraph() == null)continue;
				if(!cacheNode.getCacheData().isValid())continue;
				List<FlowNode> stream = FlowUtil.getStream(cacheNode);
				if(node == stream.get(0)){
					removeCache(cacheNode, false, backup);
					if(backup){
						FlowViewer viewer = rflow.getActiveViewer();
						viewer.getUndoableEditSupport().beginUpdateOnce();
						viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createClearCache(cacheNode, false, false));
					}
				}
			}
		}
	}*/

	private void checkCache(FlowNode node) {
		if(lockCacheState)return;
		boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
		for(FlowNode cacheNode : new LinkedList<FlowNode>(state.getCacheNodes())){
			if(cacheNode.getGraph() == null)continue;
			if(!cacheNode.getCacheData().isValid())continue;
			List<FlowNode> stream = FlowUtil.getStream(cacheNode);
			for(FlowNode fn : stream){
				if(fn == node){
					removeCache(cacheNode, false, backup);
					if(backup){
						FlowViewer viewer = rflow.getActiveViewer();
						viewer.getUndoableEditSupport().beginUpdateOnce();
						viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createClearCache(cacheNode, false, false));
					}
					break;
				}
			}
		}
	}
	

	private boolean isCacheEmpty() {
		return state.getCacheNodes().isEmpty();
	}
}
