package com.ef_prime.rflow.flow;

import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;

import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.NamedCompoundEdit;

public class DefaultUndoableEdits {

	public static class AddNodeEdit extends AbstractUndoableEdit{
		
		private FlowViewer viewer;
		private FlowNode node;
		
		AddNodeEdit(FlowViewer viewer, FlowNode node){
			this.viewer = viewer;
			this.node = node;
		}
		
		@Override
		public void undo() throws CannotUndoException {
			super.undo();
			viewer.getGraph().removeVertex(node);
			viewer.getPickedState().pick(node, false);
		}
		
		@Override
		public void redo() throws CannotRedoException {
			super.redo();
			viewer.getGraph().addVertex(node);
			//viewer.getGraphLayout().update();
			viewer.getPickedState().clearPickedVertices();
			viewer.getPickedState().pick(node, true);
			viewer.repaint();
		}
		
		@Override
		public String getPresentationName() {
			return Messages.getString("Action.new_node"); //$NON-NLS-1$
		}
		
		public FlowNode getNode() {
			return node;
		}
	}
	
	public static class NewNodeEdit extends NamedCompoundEdit{
		public NewNodeEdit(){
			super(Messages.getString("Action.new_node")); //$NON-NLS-1$
		}
		
		public FlowNode getNode(){
			for(UndoableEdit edit : edits){
				if(edit instanceof AddNodeEdit){
					AddNodeEdit addNodeEdit = (AddNodeEdit)edit;
					return addNodeEdit.getNode();
				}
			}
			return null;
		}
	}
	
	
	public static class ClearCacheEdit extends AbstractUndoableEdit{

		private FlowViewer viewer;
		private FlowNode node;
		private boolean removeMark;
		private boolean significant;
		
		public ClearCacheEdit(FlowViewer viewer, FlowNode node, boolean removeMark, boolean significant) {
			super();
			this.viewer = viewer;
			this.node = node;
			this.removeMark = removeMark;
			this.significant = significant;
		}

		@Override
		public void undo() throws CannotUndoException {
			super.undo();
			if(Rflow.DEBUG > 0)
				System.out.println("undo cache"); //$NON-NLS-1$
			viewer.getCacheHandler().restoreCache(node, removeMark && !node.getCacheData().isValid());
		}
		
		@Override
		public void redo() throws CannotRedoException {
			super.redo();

			//boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
			viewer.getCacheHandler().removeCache(node, removeMark, true);
		}
		
		public FlowNode getNode(){
			return node;
		}
		
		public void setInsignificant(){
			significant = false;
		}
		
		@Override
		public boolean isSignificant() {
			return significant;
		}
		
		@Override
		public String getPresentationName() {
			if(isSignificant())
				return (!removeMark)?Messages.getString("Action.cache_clear"):Messages.getString("Action.cache_unmark"); //$NON-NLS-1$ //$NON-NLS-2$
			else
				return super.getPresentationName();
		}
	}
}
