package com.ef_prime.rflow.flow;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.ConcurrentModificationException;
import java.util.LinkedList;
import java.util.ListIterator;

import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.rendering.NodeRendering;

import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.visualization.HasGraphLayout;
import edu.uci.ics.jung.visualization.HasShapeFunctions;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.transform.LayoutTransformer;

public class FlowPickSupport extends ShapePickSupport{

	public FlowPickSupport(float pickSize) {
		super(pickSize);
	}

	public FlowPickSupport(HasGraphLayout hasGraphLayout, LayoutTransformer layoutTransformer, HasShapeFunctions hasShapeFunctions, float pickSize) {
		super(hasGraphLayout, layoutTransformer, hasShapeFunctions, pickSize);
	}
	
	@Override
    public Vertex getVertex(double x, double y) {
        Layout layout = hasGraphLayout.getGraphLayout();
        

        Vertex closest = null;
        while(true) {
            try {
            	//逆順（上のノードから）にまわす
            	//glassshapeでヒットしたら、次からshapeで判定
            	LinkedList<Vertex> list = ((FlowGraph)layout.getGraph()).getVertexList();
                for (ListIterator<Vertex> iter=list.listIterator(list.size()); iter.hasPrevious();	) {
                	if(hasShapeFunctions != null) {
                        Vertex v = iter.previous();
                        NodeRendering nodeRendering = (NodeRendering)hasShapeFunctions.getVertexShapeFunction();
                        Shape shape = (closest == null)?
                        		nodeRendering.getGlassShape(v):nodeRendering.getShape(v);
                        // transform the vertex location to screen coords
                        Point2D p = layoutTransformer.layoutTransform(layout.getLocation(v));
                        if(p == null) continue;
                        AffineTransform xform = AffineTransform.getTranslateInstance(p.getX(), p.getY());
                        shape = xform.createTransformedShape(shape);

                        if(shape.contains(x, y)) {
                        	boolean was = (closest != null);
                        	closest = v;
                        	if(was)
                        		break;
                        	else if(xform.createTransformedShape(nodeRendering.getShape(v)).contains(x, y)){
                        		//in glassshape && in shape
                        		break;
                        	}
                        }
                    } 
                }
                break;
            } catch(ConcurrentModificationException cme) {}
        }
        return closest;
    }
}
