package com.ef_prime.rflow.flow;

import java.awt.Point;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;

import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.base.TunnelNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.NamedCompoundEdit;

import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.PickSupport;
import edu.uci.ics.jung.visualization.PickedState;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.PickingGraphMousePlugin;

public class FlowPickingGraphMousePlugin extends PickingGraphMousePlugin{
	
	private boolean dragStarted = false;
	private Point2D dragStartPoint = null;
	private Map<Vertex, Point2D> beforeMap = new HashMap<Vertex, Point2D>();
	
	public FlowPickingGraphMousePlugin() {
		super();
	}

	public FlowPickingGraphMousePlugin(int selectionModifiers, int addToSelectionModifiers) {
		super(selectionModifiers, addToSelectionModifiers);
	}

	/*
	 * mousePressedが呼び出されずに、呼び出された時のための修正
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	@Override
	public void mouseDragged(MouseEvent e) {
		if(down == null)return;

		/*
		 *super.mouseDragged 
		 */
		if(locked == false) {
			FlowViewer viewer = (FlowViewer)e.getSource();
			Set<Vertex> nodes = viewer.getGraph().getVertices();
            if(vertex != null) {
                Point p = e.getPoint();
                Point2D graphPoint = viewer.inverseTransform(p);
                Point2D graphDown = viewer.inverseTransform(down);
                FlowLayout layout = viewer.getGraphLayout();
                double dx = graphPoint.getX()-graphDown.getX();
                double dy = graphPoint.getY()-graphDown.getY();
                PickedState ps = viewer.getPickedState();
                
                if(!dragStarted){
                	//ドラッグが始まったところで座標を記録
                	for(Object o : ps.getPickedVertices()){
                		Vertex v = (Vertex)o;
                		if(nodes.contains(v))
                			beforeMap.put(v, (Point2D)layout.getLocation(v).clone());
                	}
                	dragStartPoint = graphPoint;
                }
                
                dx = graphDown.getX()-dragStartPoint.getX();
                dy = graphDown.getY()-dragStartPoint.getY();
                
                //つかんだノードを移動
                Point2D p1 = beforeMap.get(vertex);
                if(p1 == null)return;
                int x = (int)p1.getX()+(int)dx;
                int y = (int)p1.getY()+(int)dy;
                boolean moved = true;
                if(viewer.isUseGrid())
                	moved = layout.forceMoveOnGrid(vertex, x, y);
                else
                	layout.forceMove(vertex, x, y);
                
                if(moved){
	                //つかんだノードの移動
	                Point2D p2 = layout.getLocation(vertex);
	                double v_dx = p2.getX() - p1.getX();
	                double v_dy = p2.getY() - p1.getY();
	
	                //その他ノードを相対的に移動
	                for(Vertex v : beforeMap.keySet()) {
	                    if(v == vertex)continue;
	                    Point2D vp = beforeMap.get(v);
	                    layout.forceMove(v, vp.getX()+v_dx, vp.getY()+v_dy);
	                }
	                viewer.setDirty(true);
                }
                
                down = p;

                dragStarted = true;
                
            } else {
                Point2D out = e.getPoint();
                if(e.getModifiers() == this.addToSelectionModifiers ||
                        e.getModifiers() == modifiers) {
                    rect.setFrameFromDiagonal(down,out);
                }
            }
            if(vertex != null) e.consume();
        }
		
		
		
		//挿入が可能な状態のエッジの表現
		FlowViewer sv = (FlowViewer)e.getSource();
		sv.getPickedState().clearInsertTargetEdge();
		if(vertex != null 
				&& sv.getPickedState().getPickedVertices().size() == 1 
				&& vertex.getIncidentEdges().isEmpty()
				&& !(((FlowNode)vertex).getModel() instanceof TunnelNodeModel))
		{
			Shape vShape = sv.getRenderer().getVertexShapeFunction().getShape(vertex);
			Point2D p = sv.getGraphLayout().getLocation(vertex);
			p = sv.layoutTransform(p);
			vShape = AffineTransform.getTranslateInstance(p.getX(), p.getY()).createTransformedShape(vShape);
			Rectangle2D rect = vShape.getBounds2D();
			
			for(Object o : sv.getGraph().getEdges()){
				FlowEdge edge = (FlowEdge)o;
				Shape eShape = sv.getRenderer().getEdgeShapes().get(edge);
				if(eShape != null && eShape.intersects(rect)){
					sv.getPickedState().setInsertTargetEdge(edge);
					break;
				}
			}
		}
    }
	
	@Override
	public void mousePressed(MouseEvent e) {
		FlowViewer sv = (FlowViewer)e.getSource();
		sv.getPickedState().clearInsertTargetEdge();
		
		
		//以下super.mousePressed(e)
		//エッジに関する部分をコメントアウト
		down = e.getPoint();
        VisualizationViewer vv = (VisualizationViewer)e.getSource();
        PickSupport pickSupport = vv.getPickSupport();
        PickedState pickedState = vv.getPickedState();
        if(pickSupport != null && pickedState != null) {
            Layout layout = vv.getGraphLayout();
            if(e.getModifiers() == modifiers) {
                vv.addPostRenderPaintable(lensPaintable);
                rect.setFrameFromDiagonal(down,down);
                // p is the screen point for the mouse event
                Point2D p = e.getPoint();
                // take away the view transform
                Point2D ip = vv.inverseViewTransform(p);
                
                vertex = pickSupport.getVertex(ip.getX(), ip.getY());
                if(vertex != null) {
                    if(pickedState.isPicked(vertex) == false) {
                        pickedState.clearPickedVertices();
                        pickedState.pick(vertex, true);
                    }
                    // layout.getLocation applies the layout transformer so
                    // q is transformed by the layout transformer only
                    Point2D q = layout.getLocation(vertex);
                    // transform the mouse point to graph coordinate system
                    Point2D gp = vv.inverseLayoutTransform(ip);

                    offsetx = (float) (gp.getX()-q.getX());
                    offsety = (float) (gp.getY()-q.getY());
                } /*else if((edge = pickSupport.getEdge(ip.getX(), ip.getY())) != null) {
                    pickedState.clearPickedEdges();
                    pickedState.pick(edge, true);
                } */else {
                    pickedState.clearPickedEdges();
                    pickedState.clearPickedVertices();
                }
                
            } else if(e.getModifiers() == addToSelectionModifiers) {
                vv.addPostRenderPaintable(lensPaintable);
                rect.setFrameFromDiagonal(down,down);
                Point2D p = e.getPoint();
                // remove view transform
                Point2D ip = vv.inverseViewTransform(p);
                vertex = pickSupport.getVertex(ip.getX(), ip.getY());
                if(vertex != null) {
                    boolean wasThere = pickedState.pick(vertex, !pickedState.isPicked(vertex));
                    if(wasThere) {
                        vertex = null;
                    } else {
                        // layout.getLocation applies the layout transformer so
                        // q is transformed by the layout transformer only
                        Point2D q = layout.getLocation(vertex);
                        // translate mouse point to graph coord system
                        Point2D gp = vv.inverseLayoutTransform(ip);

                        offsetx = (float) (gp.getX()-q.getX());
                        offsety = (float) (gp.getY()-q.getY());
                    }
                }/* else if((edge = pickSupport.getEdge(ip.getX(), ip.getY())) != null) {
                    pickedState.pick(edge, !pickedState.isPicked(edge));
                }*/
            }
        }
        if(vertex != null) e.consume();
	}
	
	@Override
	public void mouseReleased(MouseEvent e) {
		FlowViewer fv = (FlowViewer)e.getSource();
		FlowEdge edge = fv.getPickedState().getInsertTargetEdge();
		//ノードの挿入
		CompoundEdit insertEdit = null;
		if(edge != null && vertex != null){
			if(!(((FlowNode)vertex).getModel() instanceof TunnelNodeModel)){
				fv.getGraph().removeEdge(edge);
				FlowEdge e1 = (FlowEdge)fv.getGraph().addEdge(new FlowEdge(edge.getSource(), vertex));
				FlowEdge e2 = (FlowEdge)fv.getGraph().addEdge(new FlowEdge(vertex, edge.getDest()));

				insertEdit = new NamedCompoundEdit(Messages.getString("Action.insert")); //$NON-NLS-1$
				insertEdit.addEdit(fv.undoableEditFactory.createDeleteEdit(null, new FlowEdge[]{edge}));
				insertEdit.addEdit(fv.undoableEditFactory.createAddEdgeEdit(new FlowEdge[]{e1,e2}));
			}
		}
		fv.getPickedState().clearInsertTargetEdge();
		
		//移動動作の通知
		if(!beforeMap.isEmpty()){
			UndoableEdit moveEdit = fv.undoableEditFactory.createMoveNodeEdit(new HashMap<Vertex, Point2D>(beforeMap));
			if(insertEdit == null)
				fv.fireUndoableEditUpdate(moveEdit);
			else
				insertEdit.addEdit(moveEdit);
		}
		if(insertEdit != null){
			insertEdit.end();
			fv.fireUndoableEditUpdate(insertEdit);
		}
			
			
		dragStarted = false;
		beforeMap.clear();
		
		super.mouseReleased(e);
	}
	
}
