package com.ef_prime.rflow.flow;

import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.util.TypeDefProperties;

/**
 * フローの設定
 * (.rflowファイルごとの設定です)
 *
 */
public class FlowSetting extends TypeDefProperties{
	public static enum Key implements TypeDefProperties.Key{
		
		/** 
		 * 関連ファイルを保存するフォルダ
		 * (新規フローでない時は、このデフォルト値は不適切であり、
		 * Rflow.init()内でファイル名に置き換えられることになっている)
		 */
		FOLDER(Type.STRING, ";"), //$NON-NLS-1$
		
		/** グリッドのon/off */
		USE_GRID(Type.BOOLEAN, true),
		
		/** グリッドの幅 */
		GRID_DISTANCE(Type.INTEGER, 10);
		
		private Object defaultValue;
		private Type type;
		
		private Key(Type type, Object defaultValue){
			this.defaultValue = defaultValue;
			this.type = type;
		}

		public Object getDefaultValue() {
			return defaultValue;
		}

		public Type getType() {
			return type;
		}
		
		public String getKeyString() {
			return toString();
		}
	}
	

	public static FlowSetting createDefaultSettings()
	{
		FlowSetting def = (FlowSetting)setDefaultProperties(new FlowSetting(), Key.values());
		//デフォルト値をプログラムのSettingから取得
		def.setProperty(Key.USE_GRID, Rflow.setting.getProperty(Setting.Key.USE_GRID));
		def.setProperty(Key.GRID_DISTANCE, Rflow.setting.getProperty(Setting.Key.GRID_DISTANCE));
		
		return def;
	}

	public void update(){
		update(Key.values());
	}
	
	/*public boolean isCacheDir(){
		return !((String)getProperty(Key.FOLDER)).equals(Key.FOLDER.getDefaultValue());
	}*/
	
	/*public void setCacheDir(File workingFile){
		//String old = (String)getProperty(Key.FOLDER);
		boolean wasDefault = !isCacheDir();
		if(workingFile == null){
			//new flow
			//set to default
			if(!wasDefault)
				setProperty(Key.FOLDER, Key.FOLDER.getDefaultValue());
		}else{
			String name = ExtensionUtility.removeExtension(workingFile.getName());
			//TODO すでに同じ名前のフォルダがあるかチェック
			setProperty(Key.FOLDER, name);
			
			//if(wasDefault){
				cacheHandler.renameTempCaches();
			//}

			if(!wasDefault){
				//rename cache folder
				File cacheDir = new File(Constants.RFLOW_FOLDER + "/" + old);
				if(cacheDir.exists()){
					File newCacheDir = new File((String)getProperty(Key.FOLDER));
					if(!cacheDir.renameTo(newCacheDir)){
						//TODO 警告の表示を検討
						setProperty(Key.FOLDER, old);//元に戻す
					}
				}
			}
		}
	}*/
	
	public FlowSetting copy(){
		FlowSetting setting = new FlowSetting();
		setting.setPropertyAll(this);
		return setting;
	}
}
