package com.ef_prime.rflow.flow;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.flow.FlowError.ErrorType;
import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.node.base.SubflowNodeModel;

import edu.uci.ics.jung.graph.event.GraphEvent;
import edu.uci.ics.jung.graph.event.GraphEventListener;
import edu.uci.ics.jung.visualization.MultiPickedState;

public class FlowState extends MultiPickedState 
	implements NodeModelChangeListener, ItemListener, GraphEventListener
{
	/*
	 * note:
	 * if new field is added, must update method 'clearAllState()'
	 */
	
	private FlowNode flagNode;
	private Set<FlowNode> flagBoxSet = new HashSet<FlowNode>();
	private FlowNode highlightNode; //active box
	private Set<FlowNode> executingNode = new HashSet<FlowNode>();
	private List<FlowNode> selectionFlow = new ArrayList<FlowNode>();
	private List<FlowEdge> selectionEdgeFlow = new ArrayList<FlowEdge>();
	
	//cache
	private Set<FlowNode> cacheSet = new HashSet<FlowNode>();
	
	//error
	private Map<FlowNode, Set<FlowError>> errorMap = new HashMap<FlowNode, Set<FlowError>>();
	
	private FlowEdge insertTargetEdge;
	
	private List<FlagNodeChangeListener> flagNodeChangeListeners = new ArrayList<FlagNodeChangeListener>();
	
	
	public FlowState(){
		super();
	}
	
	public void clearAllState(){
		clearPickedVertices();
		clearPickedEdges();
		clearInsertTargetEdge();
		flagNode = null;
		flagBoxSet.clear();
		highlightNode = null;
		executingNode.clear();
		cacheSet.clear();
		errorMap.clear();
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public void clearOutsidePickedVertices(FlowViewer viewer) {
		Set<FlowNode> nodes = viewer.getGraph().getVertices();
		Set<FlowNode> unpick = new HashSet<FlowNode>();
		for(Object o : getPickedVertices()){
			if(!nodes.contains((FlowNode)o))
				unpick.add((FlowNode)o);
		}
		for(FlowNode node : unpick){
			pick(node, false);
		}
	}
	
	/**
	 * 選択されているノードを一つ返す。
	 * 複数選択されている、もしくは一つも選択されていない時はnullが返ります。
	 * @return
	 */
	public FlowNode getSelectedNode(){
		if(getPickedVertices().size() != 1)
			return null;
		return (FlowNode)getPickedVertices().iterator().next();
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	@Override
	public Set<FlowNode> getPickedVertices() {
		return super.getPickedVertices();
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	@Override
	public Set<FlowEdge> getPickedEdges() {
		return super.getPickedEdges();
	}
	
	public boolean isFlowSelected(){
		return selectionFlow.size() > 0;
	}

	public List<FlowNode> getSelectionNodeFlow() {
		//return (List<FlowNode>)Collections.unmodifiableList(selectionFlow);
		return selectionFlow;
	}
	
	public List<FlowEdge> getSelectionEdgeFlow() {
		//return (List<FlowEdge>)Collections.unmodifiableList(selectionEdgeFlow);
		return selectionEdgeFlow;
	}
	
	/**
	 * add cache state now
	 * @param state
	 */
	public void add(FlowState state){
		cacheSet.addAll(state.cacheSet);
	}

	/*---------------------------ItemStateChangeListener------------------------------*/
	public void itemStateChanged(ItemEvent e) {
		selectionFlow.clear();
		selectionEdgeFlow.clear();
		if(getPickedVertices().size() == 1){
			FlowNode picked = (FlowNode)getPickedVertices().iterator().next();
			List<FlowNode> stream = FlowUtil.getStream(picked, true, true);
			FlowNode parent = null;
			Set<FlowNode> boxes = new HashSet<FlowNode>();
			for(int i = stream.size()-1; i >= 0; i--){
				FlowNode node = stream.get(i);
				selectionFlow.add(node);
				
				if(node.getModel().isSubflow())
					boxes.add(node);

				if(parent != null){
					FlowEdge edge = (FlowEdge)node.findEdge(parent);
					if(edge == null){
						for(FlowNode box : boxes){
							edge = (FlowEdge)node.findEdge(box);
							if(edge != null)
								break;
						}
					}
							
					if(edge != null){
						selectionEdgeFlow.add(edge);
						
						//if(node.getModel().isSubflow()){
							//addToFlowSelection(node);
						//}
					}
				}else{
					//if(node.getModel().isSubflow()){
						//addToFlowSelection(node);
					//}
				}

				parent = node;
			}
		}
	}
	
	/*private void addToFlowSelection(FlowNode node){
		SubflowNodeModel model = (SubflowNodeModel)node.getModel();
		//FlowGraph graph = model.getLayout().getGraph();
		List<FlowNode> nodes = model.getFlow();
		for(FlowNode n : nodes){
			NodeModel nm = n.getModel();
			if (nm.isSubflow()) {
				addToFlowSelection(n);
			}
		}
		selectionFlow.addAll(nodes);
		for(FlowNode fn : nodes){
			if(!fn.getInEdges().isEmpty())
				selectionEdgeFlow.add((FlowEdge)fn.getInEdges().iterator().next());
		}
	}*/
	
	///////////////////////////////////////////////////////////////////////////
	//flag node 
	///////////////////////////////////////////////////////////////////////////
	public void addFlagNodeChageListener(FlagNodeChangeListener changeListener){
		flagNodeChangeListeners.add(changeListener);
	}

	public FlowNode getFlagNode() {
		return flagNode;
	}

	public void setFlagNode(FlowNode flagNode) {
		if(flagNode == null)
			clearFlagNode();
		if(this.flagNode != null && flagNode != null && this.flagNode.equals(flagNode))
			return;
		this.flagNode = flagNode;
		
		//flag for parent boxes
		flagBoxSet.clear();
		FlowGraph g = flagNode.getGraph();
		while(!g.isRootGraph()){
			flagBoxSet.add(g.getParentNode());
			g = g.getParentNode().getGraph();
		}
		
		for(FlagNodeChangeListener l : flagNodeChangeListeners){
			l.flagNodeChanged(flagNode);
		}
	}
	
	public void clearFlagNode(){
		if(flagNode == null)
			return;
		flagNode = null;
		flagBoxSet.clear();
		for(FlagNodeChangeListener l : flagNodeChangeListeners){
			l.flagNodeChanged(flagNode);
		}
	}
	
	public boolean isFlagNode(FlowNode node){
		if(flagNode == null)return false;
		return flagNode.equals(node);
	}
	
	public Set<FlowNode> getFlagBoxSet() {
		return flagBoxSet;
	}
	

	public static interface FlagNodeChangeListener{
		void flagNodeChanged(FlowNode newFlagNode);
	}

	///////////////////////////////////////////////////////////////////////////
	//hilight (current box)
	///////////////////////////////////////////////////////////////////////////
	public FlowNode getHighlightNode() {
		return highlightNode;
	}
	
	public void setHighlightNode(FlowNode highlightNode) {
		this.highlightNode = highlightNode;
	}
	
	public boolean isHighlightNode(FlowNode node) {
		if(highlightNode == null)return false;
		return highlightNode.equals(node);
	}

	///////////////////////////////////////////////////////////////////////////
	//executing node
	///////////////////////////////////////////////////////////////////////////
	public void setExecutingNode(FlowNode node){
		if(node == null)
			executingNode.clear();
		else{
			executingNode.clear();
			executingNode.add(node);
			//add parents
			FlowNode fn = node;
			while(!fn.getGraph().isRootGraph()){
				fn = fn.getGraph().getParentNode();
				executingNode.add(fn);
			}
		}
	}
	
	public Set<FlowNode> getExecutingNode() {
		return executingNode;
	}
	
	public boolean isExecutingNode(FlowNode node) {
		if(executingNode.isEmpty())return false;
		return executingNode.contains(node);
	}
	
	///////////////////////////////////////////////////////////////////////////
	//cache
	///////////////////////////////////////////////////////////////////////////

	public Set<FlowNode> getCacheNodes(){
		return cacheSet;
	}

	public boolean isCacheFile(String file){
		for(FlowNode node : getCacheNodes()){
			if(file.equals(node.getCacheData().getFile().getName()))
					return true;
		}
		return false;
	}
	
	
	public void setCache(FlowNode node, CacheData cacheData){
		if(cacheData == null)
			throw new IllegalArgumentException("cacheData must not null"); //$NON-NLS-1$
		cacheSet.add(node);
		cacheData.setFlagExists(true);
		node.setCacheData(cacheData);
	}
	
	public void removeCacheNode(FlowNode node){
		cacheSet.remove(node);
		node.getCacheData().setFlagExists(false);
		//node.setCacheData(null);
	}
	
	///////////////////////////////////////////////////////////////////////////
	//syntax error
	///////////////////////////////////////////////////////////////////////////
	public void addError(FlowNode node, FlowError error, boolean clear){
		if(clear)
			removeError(node, error.getType());
		Set<FlowError> errorSet = errorMap.get(node);
		if(errorSet == null){
			errorSet = new HashSet<FlowError>();
			errorMap.put(node, errorSet);
		}
		errorSet.add(error);
		//add parents
		/*FlowNode fn = node;
		while(!fn.getGraph().isRootGraph()){
			fn = fn.getGraph().getParentNode();
			addError(fn, error);
		}*/
	}
	
	
	public void removeError(FlowNode node, FlowError.ErrorType type){
		Set<FlowError> errorSet = errorMap.get(node);
		if(errorSet != null){
			for(FlowError error : new LinkedHashSet<FlowError>(errorSet))
				if(error.getType() == type)
					errorSet.remove(error);
			
			if(errorSet.isEmpty())
				errorMap.remove(node);
		}
	}
	
	public Set<FlowError> getError(FlowNode node){
		return errorMap.get(node);
	}
	
	/**
	 * 選択ノードからnodeまでのノードを選択する
	 * @param node
	 */
	public void selectInterval(FlowNode node){
		Set<FlowNode> nodes = getPickedVertices();
		if(nodes.size() == 1){
			FlowNode start = nodes.iterator().next();
			
			List<FlowNode> firstStream = FlowUtil.getStreamSimple(start);
			List<FlowNode> lastStream = FlowUtil.getStreamSimple(node);
			
			boolean rev = (firstStream.contains(node));
			//二つノードが区間を作らない場合
			if(!rev && !lastStream.contains(start)){
				clearPickedVertices();
				pick(node, true);
				return;
			}
			
			FlowNode first = (!rev)?start:node;
			List<FlowNode> stream = (!rev)?lastStream:firstStream;
			stream = stream.subList(stream.indexOf(first), stream.size());

			clearPickedVertices();
			for(FlowNode select : stream)
				pick(select, true);
		}else{
			clearPickedVertices();
			pick(node, true);
		}
	}
	

	///////////////////////////////////////////////////////////////////////////
	//InsertTargetEdge
	///////////////////////////////////////////////////////////////////////////
	public void clearInsertTargetEdge(){
		insertTargetEdge = null;
	}
	
	public void setInsertTargetEdge(FlowEdge insertTargetEdge) {
		this.insertTargetEdge = insertTargetEdge;
	}
	
	public FlowEdge getInsertTargetEdge() {
		return insertTargetEdge;
	}
	
	
	public void passState(FlowNode oldNode, FlowNode newNode){
		//cache
		if(oldNode.isCacheData()){
			cacheSet.add(newNode);
			cacheSet.remove(oldNode);
			
			newNode.setCacheData(oldNode.getCacheData());
			oldNode.setCacheData(null);
			/*setCache(newNode, oldNode.getCacheData());
			if(oldNode.getCacheData().isValid()){
				newNode.getCacheData().setValid(true);
				oldNode.getCacheData().setValid(false);
			}

			removeCacheNode(oldNode);*/
		}
		
		//error
		Set<FlowError> errSet = getError(oldNode);
		if(errSet != null){
			for(FlowError err : errSet)
				addError(newNode, err, false);
			errorMap.remove(oldNode);
		}
	}
	/*
	----------------------StreamGraph.GraphWillChangeListener----------------------
	public void graphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph) {
		if(flagNode == null)return;
		List<FlowNode> stream = FlowUtil.getStream(flagNode, false);
		switch(type){
		case ADD:
			if(graphElement instanceof FlowEdge){
				//先頭のチェックだけでよい
				if(stream.get(0) == ((FlowEdge)graphElement).getDest()){
					clearFlagNode();
					break;
				}
			}
			break;
		case REMOVE:
			if(graphElement instanceof FlowEdge){
				FlowNode src = (FlowNode)((FlowEdge)graphElement).getSource();
				if(stream.contains(src))
					clearFlagNode();
			}
			else if(graphElement instanceof FlowNode){
				FlowNode removed = (FlowNode)graphElement;
				if(stream.contains(removed))
					clearFlagNode();
			}
			break;
		}
		
	}*/
	
	
	@Override
	protected void fireItemStateChanged(ItemEvent e) {
        Object[] listeners = listenerList.getListeners(ItemListener.class);//逆順に渡される
        for (int i = listeners.length-1; i >= 0; i--) {
        	((ItemListener)listeners[i]).itemStateChanged(e);
        }
	}

	/*---------------------------NodeModelChangeListener-------------------------*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		if(isFlagNodeValid())
			checkFlag(node);
		checkSyntax(node);
	}
	
	public void subflowNodeModelChanged(FlowNode node) {
		if(isFlagNodeValid())
			checkFlag(node);
		checkSyntax(node);
		//check box connnection
		SubflowNodeModel model = (SubflowNodeModel) node.getModel();
		if(model.isInOutConnected()){
			removeError(node, ErrorType.BOX_CONNECTION);
		}else{
			addError(node, new BoxConnectionError(), true);
		}
	}
	
	private void checkSyntax(FlowNode node){
		String msg = RCommand.checkSyntax(node.getModel().getCommand());
		if(msg == null){
			removeError(node, ErrorType.SYNTAX);
		}else{
			addError(node, new SyntaxError(msg), true);
		}
	}

	/*----------------------------ViewerEventListener----------------------------*/

	public void edgeAdded(GraphEvent event) {
		if(!isFlagNodeValid())return;
		FlowEdge edge = (FlowEdge)event.getGraphElement();
		FlowNode closed = FlowNode.getTopClosedSubflowNode(edge);
		if(closed != null){
			checkFlag(closed);
		}else{
			FlowNode src = (FlowNode)edge.getSource();

			checkFlagFirst(src);
		}
	}

	public void edgeRemoved(GraphEvent event) {
		if(!isFlagNodeValid())return;
		FlowEdge edge = (FlowEdge)event.getGraphElement();
		FlowNode closed = FlowNode.getTopClosedSubflowNode(edge);
		if(closed != null){
			checkFlag(closed);
		}else{
			FlowNode dest = (FlowNode)((FlowEdge)event.getGraphElement()).getDest();
	
			checkFlagFirst(dest);
		}
	}

	public void vertexAdded(GraphEvent event) {
		FlowNode node = (FlowNode)event.getGraphElement();
		String msg = RCommand.checkSyntax(node.getModel().getCommand());
		if(msg != null){
			addError(node, new SyntaxError(msg), true);
		}
	}
	
	public void vertexRemoved(GraphEvent event) {
		FlowNode node = (FlowNode)event.getGraphElement();
		pick(node, false);
		if(errorMap.containsKey(node)){
			removeError(node, ErrorType.SYNTAX);
		}
		if(node.getModel().isSubflow()){
			Set<FlowNode> set = FlowUtil.getAllDescendants((SubflowNodeModel)node.getModel());
			for(FlowNode fn : set)
				if(errorMap.containsKey(fn)){
					removeError(fn, ErrorType.SYNTAX);
					break;
				}
		}
		if(isFlagNodeValid()){
			if(isFlagNode(node)){
				clearFlagNode();
			}
			else if(node.getModel().isSubflow()){
				SubflowNodeModel model = (SubflowNodeModel)node.getModel();
				Set<FlowNode> set = FlowUtil.getAllDescendants(model);
				for(FlowNode fn : set)
					if(isFlagNode(fn)){
						clearFlagNode();
						break;
					}
			}
		}
	}

	private void checkFlag(FlowNode node) {
		List<FlowNode> stream = FlowUtil.getStream(flagNode);
		for(FlowNode nod : stream){
			if(nod == node){
				clearFlagNode();
				break;
			}
		}
	}

	private void checkFlagFirst(FlowNode dest) {
		List<FlowNode> stream = FlowUtil.getStream(flagNode);
		FlowNode first = stream.get(0);
		if(dest == first)clearFlagNode();
	}
	
	private boolean isFlagNodeValid(){
		return flagNode != null && flagNode.getGraph() != null;
	}
	
}
