package com.ef_prime.rflow.flow;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.ImageObserver;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.RflowUtil;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.ActionManager.ActionKey;
import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.explorer.FunctionNode;
import com.ef_prime.rflow.explorer.FunctionNodeFlavor;
import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.model.FlowLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.flow.rendering.FlowRenderer;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeModelChangeListener;
import com.ef_prime.rflow.node.NodeModelController;
import com.ef_prime.rflow.node.NodeModelFactory;
import com.ef_prime.rflow.node.NodeModelManager;
import com.ef_prime.rflow.node.NodeProperty;
import com.ef_prime.rflow.node.NodePropertyChangeListener;
import com.ef_prime.rflow.node.base.ControlNodeModel;
import com.ef_prime.rflow.node.base.FreeNodeModel;
import com.ef_prime.rflow.node.base.FunctionNodeModel;
import com.ef_prime.rflow.node.base.IfNodeModel;
import com.ef_prime.rflow.node.base.LoopNodeModel;
import com.ef_prime.rflow.node.base.LoopType;
import com.ef_prime.rflow.node.base.SentenceNodeModel;
import com.ef_prime.rflow.node.base.SubflowFrame;
import com.ef_prime.rflow.node.base.SubflowNodeModel;
import com.ef_prime.rflow.node.base.TunnelNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.AbstractActionManager;
import com.ef_prime.rflow.util.NamedCompoundEdit;
import com.ef_prime.rflow.util.ObjectUtilities;
import com.ef_prime.rflow.util.SwingUtils;

import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.decorators.ToolTipFunction;
import edu.uci.ics.jung.graph.event.GraphEvent;
import edu.uci.ics.jung.graph.event.GraphEventListener;
import edu.uci.ics.jung.graph.event.GraphEventType;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.VisualizationModel;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.PluggableGraphMouse;

public class FlowViewer extends VisualizationViewer implements
		NodeModelChangeListener, NodePropertyChangeListener {

	private Rflow rflow;
	private FlowViewer parentViewer;
	
	private List<FlowViewer> childViewers = new ArrayList<FlowViewer>();

	private FlowExecuter executer;
	private CacheHandler cacheHandler;

	private static ViewLayout clipedLayout;

	private FlowUndoManager undoManager;

	UndoableEditFactory undoableEditFactory;
	
	//private ViewerActionManager actionManager;
	
	private JDesktopPane desktopPane;

	private boolean useGrid;
	
	private boolean repaintEnabled = true;

	private Point lastClickedPoint = null;

	private FlowUndoableEditSupport undoableEditSupport = new FlowUndoableEditSupport(this);
	//private List<UndoableEditListener> undoableEditListeners = new ArrayList<UndoableEditListener>();
	
	private static List<GraphEventListener> viewerEventListers = new ArrayList<GraphEventListener>();
	
	/**
	 * Root viewer用のコンストラクタ
	 * @param rflow
	 */
	public FlowViewer(Rflow rflow){
		this(rflow, null);
	}
	
	/**
	 * 子viewer用のコンストラクタ
	 * @param rflow
	 * @param parentViewer
	 */
	public FlowViewer(Rflow rflow, FlowViewer parentViewer) {
		super(new FlowLayout(new FlowGraph()), new PluggableRenderer());
		this.rflow = rflow;
		this.parentViewer = parentViewer;
		//this.parentNode = parentNode;
		//getGraph().setParentNode(parentNode);
		
		if(isRootViewer()){
			FlowState state = new FlowState();
			state.addItemListener(state);
			setPickedState(state);
			executer = new FlowExecuter(rflow, this, rflow.receiver);
			cacheHandler = new CacheHandler(rflow, this);
		}else{
			setPickedState(getRootFlowState());
			executer = parentViewer.executer;
			cacheHandler = parentViewer.cacheHandler;
		}
		
		setRenderer(new FlowRenderer(getPickedState()));

		undoManager = new FlowUndoManager(rflow);
		undoableEditFactory = new UndoableEditFactory(this);

		addUndoableEditListener(undoManager);

		setBackground(Color.WHITE);
		setPickSupport(new FlowPickSupport(10));

		setGraphEventListeners();
		// フラグノードが変わったらローカル環境をクリアする
		/*
		state.addFlagNodeChageListener(new FlowState.FlagNodeChangeListener() {
			public void flagNodeChanged(FlowNode newFlagNode) {
				RCommand.clearLocalEnv();
			}
		});
		 */
		//フラグノードが変わったら全体をrepaint
		if(isRootViewer())
			getPickedState().addFlagNodeChageListener(new FlowState.FlagNodeChangeListener() {
				public void flagNodeChanged(FlowNode newFlagNode) {
					repaintFamiliy();
				}
			});
		
		// マウス
		/*
		 * AdjustedPluggableGraphMouse do not invoke mouseClicked mouseEntered
		 * mouseExited mouseMoved
		 */
		//フォーカスリクエスト
		addMouseListener(new MouseAdapter(){
			@Override
			public void mousePressed(MouseEvent e) {
				FlowViewer.this.focusViewer();
			}
		});
		addFocusListener(new FocusAdapter(){
			@Override
			public void focusLost(FocusEvent e) {
				lastClickedPoint = null;
			}
		});
		PluggableGraphMouse graphMouse = new AdjustedPluggableGraphMouse();
		graphMouse.add(new FlowPickingGraphMousePlugin(MouseEvent.BUTTON1_MASK,
				MouseEvent.BUTTON1_MASK | MouseEvent.CTRL_MASK));
//		graphMouse.add(new FlowTranslatingGraphMousePlugin(MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK));
//		graphMouse.add(new ScalingGraphMousePlugin(new LayoutScalingControl(),MouseEvent.NOBUTTON));
//		graphMouse.add(new ScalingGraphMousePlugin(new ViewScalingControl(),MouseEvent.NOBUTTON));
//		graphMouse.add(new RotatingGraphMousePlugin(MouseEvent.BUTTON1_MASK| MouseEvent.ALT_MASK));
		setGraphMouse(graphMouse);
		
		addMouseListener(new MouseHandler());


		// ツールチップ
		setToolTipFunction(new ToolTipHandler(getPickedState()));

		// ドロップ動作
		setDropTarget(new DropTarget(this, new DropTargetHandler()));

		updateFlowSetting();
		
		if(isRootViewer()){
			desktopPane = rflow.desktop;
			desktopPane.setDesktopManager(new FlowDesktopManager(desktopPane.getDesktopManager()));
		}
		//setAction();
		
	}
	
	/*private void setAction(){		
		//actionManager = new ViewerActionManager(this);
		//setActionMap(actionManager);
		//setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, actionManager.getInputMap());
	}*/
	
	public void focusViewer(){
		requestFocus();
		if(isRootViewer()){
			for(JInternalFrame f : desktopPane.getAllFrames()){
				try {
					f.setSelected(false);
				} catch (PropertyVetoException e1) {
					//TODO auto generated
					e1.printStackTrace();
				}
			}

			FlowViewer.this.rflow.setActiveViewer(FlowViewer.this);
			FlowViewer.this.getRootFlowState().clearOutsidePickedVertices(FlowViewer.this);
		}
		
	}

	public FlowViewer createNewViewer() {
		return new FlowViewer(rflow);
	}
	

	public void clearFlow() {
		clearEditWindows();
		//stateからグラフのリスナーを消去
		getRootFlowState().removeItemListener(getGraph());
		setGraphLayout(new FlowLayout(new FlowGraph()));
		getPickedState().clearAllState();
		undoManager.discardAllEdits();
		rflow.actionManager.updateUndoState();
		if(isRootViewer()){
			for(JInternalFrame frame : desktopPane.getAllFrames()){
				FlowViewer[] viewers = ((SubflowFrame)frame).getViewer();
				for(FlowViewer viewer : viewers)
					rflow.removeViewer(viewer);
				desktopPane.remove(frame);
				frame.dispose();
			}
		}
		repaint();
	}

	public void init(ViewLayout viewLayout) {
		setRepaintEnabled(false);
		try{
			FlowLayout layout = new FlowLayout(viewLayout.getGraph());
			setGraphLayout(layout);
			layoutTransformer.setToIdentity();
			for (Map.Entry<Vertex, Point2D> entry : viewLayout.getLocationMap().entrySet()) {
				Point2D p = entry.getValue();
				layout.forceMove(entry.getKey(), p.getX(), p.getY());
			}
	
			Point offset = viewLayout.getOffset();
			if (offset != null)
				layoutTransformer.translate(offset.x, offset.y);
	
			updateFlowSetting();
		}finally{
			setRepaintEnabled(true);
		}
	}

	public void updateFlowSetting() {
		FlowSetting flowSettings = rflow.getFlowSetting();
		FlowLayout layout = getGraphLayout();
		layout.setGridDistance((Integer) flowSettings
				.getProperty(FlowSetting.Key.GRID_DISTANCE));

		setUseGrid((Boolean) flowSettings.getProperty(FlowSetting.Key.USE_GRID));
	}
	
	public synchronized void setRepaintEnabled(boolean repaintEnabled) {
		this.repaintEnabled = repaintEnabled;
	}
	
	public boolean isRootViewer(){
		return parentViewer == null;
	}
	
	public FlowViewer getParentViewer() {
		return parentViewer;
	}
	
	/*public FlowNode getParentNode() {
		return parentNode;
	}*/
	
	public FlowViewer getRootViewer(){
		if(isRootViewer())return this;
		FlowViewer root = this.getParentViewer();
		while(!root.isRootViewer())
			root = root.getParentViewer();
		return root;
	}
	
	void addChildViewer(FlowViewer child){
		childViewers.add(child);
	}
	
	List<FlowViewer> getChildren(){
		return childViewers;
	}

	/*---------------------------NodeModelChangeListener----------------------------*/
	public void nodeModelChanged(FlowNode node, NodeModel old, NodeModel model) {
		if(getGraph().getVertices().contains(node)){
			fireUndoableEditUpdate(undoableEditFactory.createNodeModelChangeEdit(node, old));
			updateGroup(model);
			repaint();
		}
	}
	public void subflowNodeModelChanged(FlowNode node) {repaintFamiliy();}

	/*---------------------------NodePropertyChangeListener----------------------------*/
	public void nodePropertyChanged(FlowNode node, NodeProperty old, NodeProperty property) {
		if(getGraph().getVertices().contains(node)){
			fireUndoableEditUpdate(undoableEditFactory.createNodePropertyChangeEdit(node, old));
			repaint();
		}
	}

	@Override
	public void setGraphLayout(Layout layout) {
		setGraphLayout(layout, false);
	}

	@Override
	public void setGraphLayout(Layout layout, boolean scaleToLayout) {
		//FlowNode node = getGraph().getParentNode();
		super.setGraphLayout(layout, scaleToLayout);
		//getGraph().setParentNode(node);
		setGraphEventListeners();
		updateGroup();
	}

	@Override
	public void setModel(VisualizationModel model) {
		super.setModel(model);
		setGraphEventListeners();
	}

	private void setGraphEventListeners() {
		// TODO
		getGraph().addListener(getPickedState(), GraphEventType.ALL_SINGLE_EVENTS);
		getGraph().addListener(cacheHandler, GraphEventType.ALL_SINGLE_EVENTS);
		getGraph().addListener(new GraphEventHandler(), GraphEventType.ALL_SINGLE_EVENTS);
		getRootFlowState().addItemListener(getGraph());
		
		/*if (Rflow.TEST && rflow.codeViewer != null) {
			getGraph().addListener(rflow.codeViewer,
					GraphEventType.ALL_SINGLE_EVENTS);
			rflow.codeViewer.flowModelChanged();
		}*/
	}

	public FlowGraph getGraph() {
		return (FlowGraph) getGraphLayout().getGraph();
	}

	@Override
	public FlowRenderer getRenderer() {
		return (FlowRenderer) super.getRenderer();
	}

	@Override
	public FlowLayout getGraphLayout() {
		return (FlowLayout) super.getGraphLayout();
	}

	@Override
	public FlowState getPickedState() {
		return (FlowState) super.getPickedState();
	}
	
	public FlowState getRootFlowState(){
		return getRootViewer().getPickedState();
	}
	
	public FlowExecuter getFlowExecuter(){
		return executer;
	}
	
	public CacheHandler getCacheHandler(){
		return cacheHandler;
	}

	public void setUseGrid(boolean useGrid) {
		this.useGrid = useGrid;
	}

	public boolean isUseGrid() {
		return useGrid;
	}

	public void updateGroup() {
		for (Object o : getGraph().getVertices()) {
			updateGroup(((FlowNode) o).getModel());
		}
		repaint();
	}

	public void updateGroup(NodeModel model) {
		model.update(rflow.groupManager);
	}

	public void setDirty(boolean isDirty) {
		rflow.setDirty(isDirty);
	}
	
	public FlowUndoManager getUndoManager(){
		return undoManager;
	}
	
	public FlowUndoableEditSupport getUndoableEditSupport() {
		return undoableEditSupport;
	}

	public void addUndoableEditListener(UndoableEditListener listener) {
		undoableEditSupport.addUndoableEditListener(listener);
	}

	public void removeUndoableEditListener(UndoableEditListener listener) {
		undoableEditSupport.removeUndoableEditListener(listener);
	}

	void fireUndoableEditUpdate(UndoableEdit e) {
		undoableEditSupport.postEdit(e);
	}
	
	public static void addGraphEventListener(GraphEventListener l){
		viewerEventListers.add(l);
	}
	
	public static void removeGraphEventListener(GraphEventListener l){
		viewerEventListers.remove(l);
	}
	
	
	public void clearEditWindows(){
		for(Object o : getGraph().getVertices()){
			((FlowNode)o).clearEditWindow();
		}
	}
	
	public boolean cache(FlowNode node, boolean create){
		try {
			cacheHandler.markCache(node);
			if(create)
				executer.execute(node, null, false, true, true);
			return true;
		} catch (IOException e) {
			if(Rflow.DEBUG > 0)e.printStackTrace();
			JOptionPane.showMessageDialog(this, Messages.getString("Error.create_cache"), "Error.title.error", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
	}
	
	/*public boolean loadCache(FlowNode node){
		try {
			cacheHandler.loadCache(node);
			return true;
		} catch (IOException e) {
			if(Rflow.DEBUG > 0)e.printStackTrace();
			JOptionPane.showMessageDialog(this, "キャッシュの読み込みに失敗しました。", "エラー", JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}*/
	
	public void removeCache(FlowNode node, boolean removeMark, boolean backup){
		cacheHandler.removeCache(node, removeMark, backup);
	}
	
	public boolean isExecutable(FlowNode node){
		boolean canExec = true;
		if(node.getModel() != TunnelNodeModel.outModel 
				&& 
			!isRootViewer() && getGraph().getParentNode().getModel().isSubflow())
		{
			canExec = !(getGraph().getParentNode().getModel() instanceof ControlNodeModel 
					&& 
					FlowUtil.isHeadInNode(node));
		}
		return canExec;
	}

	/**
	 * nodeまでのフローのスクリプトをコピーします。
	 * 
	 * @param node
	 */
	public void copyScript(FlowNode node) {
		if (node == null)
			return;
		executer.copyCommandToClipBoard(node);
	}
	
	public void createNode(Class<? extends NodeModel> modelClass, Point2D p, boolean connect){
		NodeModel model = NodeModelManager.getModelController(modelClass).createNewModel();
		FlowNode node = addNode(new FlowNode(model), p, connect);
		node.editNode(this, false, true);
	}
	
	public void editNode(){
		editNode(getPickedState().getSelectedNode());
	}
	
	public void editNode(FlowNode node){
		editNode(node, false);
	}
	
	public void editNode(FlowNode node, boolean editProperty){
		if(node == null)return;

		if(!editProperty && node.getModel().isSubflow()){
			showBox(node);
		}else{
			node.editNode(this, editProperty, false);
		}
	}

	public void undo() {
		if (undoManager.canUndo())
			undoManager.undo();
		rflow.actionManager.updateUndoState();
	}

	public void redo() {
		if (undoManager.canRedo())
			undoManager.redo();
		rflow.actionManager.updateUndoState();
	}

	/**
	 * 選択されているノードを消去します。
	 * 
	 */
	public void delete() {
		Set<FlowNode> nodes = new HashSet<FlowNode>(getPickedState().getPickedVertices());
		delete_internal(nodes.toArray(new FlowNode[0]));
	}

	private void delete_internal(FlowNode[] nodes) {
		Set<FlowEdge> deletedEdges = new HashSet<FlowEdge>();
		
		Collection<FlowNode> unmods = getUnmodifiableNodes(Arrays.asList(nodes));
		
		for (FlowNode node : nodes) {
			if(!unmods.contains(node)){
				for (Object o : node.getIncidentEdges())
					deletedEdges.add((FlowEdge) o);
	
				getGraph().removeVertex(node);
			}
			getPickedState().pick(node, false);
		}

		if(unmods.size() > 0){
			List<FlowNode> nodeList2 = new ArrayList<FlowNode>();
			Collections.addAll(nodeList2, nodes);
			nodeList2.removeAll(unmods);
			nodes = nodeList2.toArray(new FlowNode[0]);
		}
		
		if(nodes.length > 0){
			fireUndoableEditUpdate(undoableEditFactory.createDeleteEdit(nodes, deletedEdges.toArray(new FlowEdge[0])));
		}
	}
	
	private void deleteEdge(FlowEdge edge){
		getGraph().removeEdge(edge);
		if(getPickedState().getSelectionEdgeFlow().contains(edge))
			getPickedState().clearPickedVertices();
		repaint();

		fireUndoableEditUpdate(
				undoableEditFactory.createDeleteEdit(
						null,
						new FlowEdge[] { edge }));
	}

	public void cut() {
		copy_internal(true);
	}

	public void copy() {
		copy_internal(false);
	}

	private void copy_internal(boolean delete) {
		lastClickedPoint = null;
		ViewLayout layout = getSubLayout(getPickedState().getPickedVertices(), true, null);
		if (layout != null) {
			clipedLayout = layout;
			rflow.actionManager.getAction(ActionKey.PASTE).setEnabled(true);
			if (delete)
				delete();
		}
	}

	public void paste(Point p) {
		if (clipedLayout != null) {
			ViewLayout layout = clipedLayout;
			if(p == null){
				if(lastClickedPoint != null)
					p = getGraphLayout().getPointOnGrid(lastClickedPoint.x, lastClickedPoint.y);
				else
					p = new Point(50,50);
			}
			layout = new ViewLayout(clipedLayout.getGraph(), clipedLayout.getLocationMap(), 
					clipedLayout.getSize(), new Point(0,0));
			//レイアウトの(0,0)一番近いポイントを取得
			double minDist = Double.MAX_VALUE;
			Point2D minPoint = new Point2D.Double(0,0);
			for(Point2D point : layout.getLocationMap().values())
				if(minDist > point.distanceSq(0, 0)){
					minDist = point.distanceSq(0, 0);
					minPoint = point;
				}
			//同じ位置にノードがあるか調べて、ある場合、位置をずらす
			Point pasteP = new Point();
			pasteP.setLocation(inverseTransform(p));
			pasteP.translate((int)(minPoint.getX()+0.5), (int)(minPoint.getY()+0.5));
			int mod = (50 % getGraphLayout().getGridDistance());
			int offset = (mod == 0)?50:50 - mod  + getGraphLayout().getGridDistance();
			a:while(true){
				for(Object o : getGraph().getVertices()){
					Point2D vertexP = getGraphLayout().getLocation((Vertex)o);
					if(vertexP.getX() == pasteP.getX() && vertexP.getY() == pasteP.getY()){
						pasteP.setLocation(pasteP.getX(), pasteP.getY()+offset);
						continue a;
					}
				}
				break;
			}
			pasteP.translate(-(int)(minPoint.getX()+0.5), -(int)(minPoint.getY()+0.5));
			p.setLocation(transform(pasteP));

			Set<FlowNode> set = addSubLayout(layout, p, true, null);
			getPickedState().clearPickedVertices();
			for (FlowNode node : set)
				getPickedState().pick(node, true);
		}
	}
	
	private Collection<FlowNode> getUnmodifiableNodes(Collection<FlowNode> nodeList){

		Set<FlowNode> unmodNodes = new HashSet<FlowNode>();
		
		for (FlowNode node : nodeList) {
			//take out TunnelNodeModel
			if (node.getModel() instanceof TunnelNodeModel) {
				unmodNodes.add(node);
			}
		}
		
//		if(undeletableNodes.size() > 0){
//			List<FlowNode> nodeList2 = new ArrayList<FlowNode>();
//			nodeList2.addAll(nodeList);
//			nodeList2.removeAll(undeletableNodes);
//			nodeList = nodeList2;
//		}
		
		return unmodNodes;
	}

	public void selectAll() {
		for (Object o : getGraph().getVertices()) {
			getPickedState().pick((ArchetypeVertex) o, true);
		}
	}

	/**
	 * 新しいノードをフローに追加します。
	 * 
	 * @param node
	 *            新しいノード
	 * @param p
	 *            新しいノードの位置 nullならば自動設定
	 * @param connect
	 *            選択ノードと接続するか
	 * @return 新しく追加されたノード
	 */
	public FlowNode addNode(FlowNode node, Point2D p, boolean connect) {
		getGraphLayout().initLocation(node);
		getGraph().addVertex(node);
		//getGraphLayout().update();

		CompoundEdit edit = new DefaultUndoableEdits.NewNodeEdit();

		edit.addEdit(undoableEditFactory.createAddNodeEdit(node));
		// fireUndoableEditUpdate(new UndoableEditEvent(this,
		// undoableEditFactory.createAddNodeEdit(node)));

		boolean pointWasNull = (p == null);

		FlowNode src = null;// connect source
		if (connect) {
			src = connectNode(node, edit);
			if (p == null && src != null) {
				p = transform(getGraphLayout().getLocation(src));
				p.setLocation(p.getX() + 100, p.getY());
			}
		}

		if (p == null)
			p = new Point(50, 50);
		p = inverseTransform(p);

		if (isUseGrid())
			getGraphLayout().forceMoveOnGrid(node, p.getX(), p.getY());
		else
			getGraphLayout().forceMove(node, p.getX(), p.getY());

		// 位置が設定されていない場合、重なっていたら下にずらす
		if (pointWasNull) {
			while (intersects(node)) {
				p.setLocation(p.getX(), p.getY() + 80);
				getGraphLayout().forceMove(node, p.getX(), p.getY());
			}
		}

		repaint();

		edit.end();
		fireUndoableEditUpdate(edit);

		return node;
	}
	

	public Set<FlowNode> addSubLayout(ViewLayout layout, Point p) {
		return addSubLayout(layout, p, false, null);
	}

	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public Set<FlowNode> addSubLayout(ViewLayout layout, Point p, boolean modiableOnly, Map<FlowNode, FlowNode> nodeMap) {

		if (p == null) {
			int y = getSize().height / 2 - layout.getSize().height / 2;
			p = new Point(0, y);
		}

		// orginal -> newnode map
		if(nodeMap == null)
			nodeMap = new HashMap<FlowNode, FlowNode>();
		FlowGraph subGraph = layout.getGraph();

		Set<FlowNode> newNodes = new HashSet<FlowNode>(subGraph.getVertices().size());
		Collection<FlowNode> unmods = getUnmodifiableNodes(subGraph.getVertices());
		for (Object o : subGraph.getVertices()) {			
			FlowNode node = (FlowNode)o;
			
			if(!modiableOnly || !unmods.contains(node)){
				FlowNode newNode = node.copy();
				getGraphLayout().initLocation(newNode);
				getGraph().addVertex(newNode);
				newNodes.add(newNode);
				nodeMap.put(node, newNode);

				//getGraphLayout().update();
				// 位置
				Point2D subP = layout.getOffsetLocation(node);
				Point2D newP = new Point2D.Double(subP.getX() + p.getX(), subP.getY() + p.getY());
				newP = inverseTransform(newP);
				getGraphLayout().forceMove(newNode, newP.getX(), newP.getY());
			}
		}

		Set<FlowEdge> newEdges = new HashSet<FlowEdge>(subGraph.getEdges().size());
		for (Object o : subGraph.getEdges()) {
			FlowEdge e = (FlowEdge) o;
			if(!modiableOnly || !unmods.contains(e.getSource()) && !unmods.contains(e.getDest())){
				FlowEdge newEdge = (FlowEdge) getGraph().addEdge(
						new FlowEdge(nodeMap.get(e.getSource()), nodeMap.get(e.getDest())));
				newEdges.add(newEdge);
			}
		}

		if(newNodes.size() > 0)
			fireUndoableEditUpdate(undoableEditFactory.createAddSublayoutEdit(newNodes.toArray(new FlowNode[0]),
							newEdges.toArray(new FlowEdge[0])));

		return newNodes;
	}

	private ViewLayout getSubLayout(final Collection<FlowNode> accept, boolean copy,
			Map<FlowNode, FlowNode> nodeMap) 
	{
		return getSubLayout(accept, new Point(50, 50), copy, nodeMap);
	}

	/**
	 * 指定したノードからサブレイアウトを作成。 
	 * nodeMapには、元のノードをキーとして、新しいレイアウトのノード引く エントリーが入れられる。
	 * space==null場合、画面のゼロポイントからオフセットを設定
	 * 
	 * @param accept
	 * @param nodeMap
	 *            nullで自動生成、orgNode -> newNode Map
	 * @return null if no node
	 */
	private ViewLayout getSubLayout(final Collection<FlowNode> accept, Point space, 
			boolean copy, Map<FlowNode, FlowNode> nodeMap) 
	{
		if (accept.size() < 1)
			return null;

		if(nodeMap == null)
			nodeMap = new HashMap<FlowNode, FlowNode>();

		FlowGraph newGraph;
		if(copy){
			newGraph = (FlowGraph) getGraph().newInstance();
	
			for (FlowNode node : accept) {
				FlowNode newNode = node.copy();
				newGraph.addVertex(newNode);
				nodeMap.put(node, newNode);
			}
	
			for (Object o : getGraph().getEdges()) {
				FlowEdge e = (FlowEdge) o;
				if (accept.containsAll(e.getIncidentVertices())) {
					newGraph.addEdge(new FlowEdge(nodeMap.get(e.getSource()),
							nodeMap.get(e.getDest())));
				}
			}
		}else{
			newGraph = getGraph();
		}

		FlowLayout layout = getGraphLayout();
		Rectangle rect = layout.getVertexRect(accept);

		int gridDist = layout.getGridDistance();
		Point zeroPoint = new Point(rect.x - rect.x % gridDist, rect.y - rect.y
				% gridDist);
		/*
		 * rect.grow(spaceX, spaceY); rect.setFrameFromDiagonal(//ビューへの変換
		 * transform(new Point2D.Double(rect.getMinX(), rect.getMinY())),
		 * transform(new Point2D.Double(rect.getMaxX(), rect.getMaxY())));
		 */

		Map<Vertex, Point2D> locationMap = new HashMap<Vertex, Point2D>();
		for (FlowNode node : accept) {
			Point2D p = layout.getLocation(node);
			Vertex v = (copy)?nodeMap.get(node):node;
			locationMap.put(v, new Point2D.Double(p.getX()
					- zeroPoint.getX(), p.getY() - zeroPoint.getY()));
		}

		Dimension size;
		Point offset;
		if (space != null) {
			rect.grow(space.x, space.y);
			size = new Dimension((int) rect.getWidth(), (int) rect.getHeight());
			offset = new Point(space.x, space.y);
		} else {
			size = getSize();
			Point2D screenZeroP = inverseTransform(new Point2D.Float(0, 0));
			offset = new Point(zeroPoint.x - (int) screenZeroP.getX(),
					zeroPoint.y - (int) screenZeroP.getY());
		}

		return new ViewLayout(newGraph, locationMap, size, offset);
	}

	/**
	 * vと選択されているノードをエッジでつないで、つないだ先のノードに選択を移す。 つないだソースのノード返す。
	 * エッジの追加ができなかった場合は、nullが返る。 自動的にUndoableEditEvent通知を行います。
	 * 
	 * @param v
	 *            destination node
	 * @return source node. null if failed to connect.
	 */
	private void connectNode(Vertex v) {
		CompoundEdit edit = new NamedCompoundEdit(Messages
				.getString("Action.new_edge")); //$NON-NLS-1$
		connectNode(v, edit);
		edit.end();
		fireUndoableEditUpdate(edit);
	}

	/**
	 * vと選択されているノードをエッジでつないで、つないだ先のノードに選択を移す。 つないだソースのノード返す。
	 * エッジの追加ができなかった場合は、nullが返る。
	 * 
	 * @param v
	 *            destination node
	 * @return source node. null if failed to connect.
	 */
	private FlowNode connectNode(Vertex v, CompoundEdit edit) {
		if (pickedState.getPickedVertices().size() == 1
				&& !pickedState.isPicked(v)) {
			FlowNode selectedNode = (FlowNode) pickedState.getPickedVertices()
					.iterator().next();
			connectNode(selectedNode, v, edit);
			pickedState.clearPickedVertices();
			pickedState.pick(v, true);
			return selectedNode;
		}
		pickedState.clearPickedVertices();
		pickedState.pick(v, true);
		return null;
	}

	private void connectNode(Vertex src, Vertex dest, CompoundEdit edit) {
		FlowEdge e = new FlowEdge(src, dest);
		if (FlowUtil.makeEdgeAddable(e, this, edit)) {
			getGraph().addEdge(e);
			repaint();
			edit.addEdit(undoableEditFactory.createAddEdgeEdit(new FlowEdge[] { e }));
		} else {
			JOptionPane
					.showMessageDialog(
							FlowViewer.this,
							Messages.getString("Error.unable_to_add"), Messages.getString("Error.title.error"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * nodeがほかのノードと重ならないかを調べます
	 * 
	 * @param vertex
	 * @return
	 */
	private boolean intersects(Vertex node) {
		Point2D nodeP = layoutTransform(getGraphLayout().getLocation(node));
		if (nodeP == null)
			return false;
		Shape nodeShape = getRenderer().getVertexShapeFunction().getShape(node);
		nodeShape = AffineTransform.getTranslateInstance(nodeP.getX(),
				nodeP.getY()).createTransformedShape(nodeShape);
		Rectangle nodeRect = nodeShape.getBounds();

		for (Object o : getGraph().getVertices()) {
			Vertex v = (Vertex) o;
			if (node.equals(v))
				continue;
			Shape shape = getRenderer().getVertexShapeFunction().getShape(v);
			Point2D p = layoutTransform(getGraphLayout().getLocation(v));
			if (p == null)
				continue;
			shape = AffineTransform.getTranslateInstance(p.getX(), p.getY())
					.createTransformedShape(shape);
			Rectangle rect = shape.getBounds();

			if (nodeRect.intersects(rect))
				return true;
		}
		return false;
	}
	
	private void insertSource(Point p, boolean fromClipboard){
		ViewLayout layout;
		if(fromClipboard)
			layout = RflowUtil.importSourceFromClipboard(rflow.frame);
		else
			layout = RflowUtil.importSource(rflow.frame);
		if (layout == null)
			return;

		Point offset = ViewLayout.getDefaultFirstnodePoint();
		Point point = (Point) p.clone();
		point.translate(-offset.x, -offset.y);
		addSubLayout(layout, point);
	}
	
	private void createSubflowNode(Class<? extends SubflowNodeModel> modelClass, Point2D p){
		NodeModelController controller = NodeModelManager.getModelController(modelClass);
		SubflowNodeModel model = (SubflowNodeModel)controller.createNewModel();
		FlowNode node = addNode(new FlowNode(model), p, true);
		registSubflowFrame(model.getSubflowFrame(rflow, this, node));
	}
	
	private void createLoopNode(LoopType type, Point2D p){
		LoopNodeModel model = new LoopNodeModel(type);
		FlowNode node = addNode(new FlowNode(model), p, true);
		registSubflowFrame(model.getSubflowFrame(rflow, this, node));
	}
	
	private void showBox(FlowNode node){
		final SubflowNodeModel boxNodeModel = (SubflowNodeModel)node.getModel();
		//internal frame
		registSubflowFrame(boxNodeModel.getSubflowFrame(rflow, this, node));
	}
	
	private void registSubflowFrame(SubflowFrame frame){
		if(frame != null){
			getRootViewer().desktopPane.add(frame);
			getRootViewer().desktopPane.setLayer(frame, 1);
			
			FlowViewer[] viewers = frame.getViewer();
			for(FlowViewer viewer :  viewers){
				rflow.addViewer(viewer);
				viewer.getParentViewer().addChildViewer(viewer);
				//menu esc accelerator don't work for some reason
				viewer.addKeyListener(new KeyAdapter(){
					@Override
					public void keyPressed(KeyEvent e) {
						if(e.getKeyCode() == KeyEvent.VK_ESCAPE){
							rflow.actionManager.stop();
						}
					}
				});
				viewer.addFocusListener(new FocusAdapter(){
					@Override
					public void focusGained(FocusEvent e) {
						FlowViewer viewer = (FlowViewer)e.getSource();
						if(rflow.getActiveViewer() != viewer){
							rflow.setActiveViewer(viewer);
							FlowViewer.this.getRootFlowState().clearOutsidePickedVertices(viewer);
							FlowViewer.this.getPickedState().setHighlightNode(viewer.getGraph().getParentNode());
							repaint();
						}
					}
				});
			}
			frame.addInternalFrameListener(new InternalFrameAdapter(){
				@Override
				public void internalFrameActivated(InternalFrameEvent e) {
					SubflowFrame frame = (SubflowFrame)e.getInternalFrame();
					//FlowViewer.this.getRootFlowState().clearPickedVertices();
					FlowViewer activeViewer = frame.getActiveViewer();
					rflow.setActiveViewer(activeViewer);
					FlowViewer.this.getRootFlowState().clearOutsidePickedVertices(activeViewer);
					FlowViewer.this.getPickedState().setHighlightNode(activeViewer.getGraph().getParentNode());
					repaint();
				}
				@Override
				public void internalFrameDeactivated(InternalFrameEvent e) {
					FlowViewer.this.getPickedState().setHighlightNode(null);
					repaint();
				}
			});
			frame.addComponentListener(new ComponentAdapter(){
				@Override
				public void componentHidden(ComponentEvent e) {
					FlowViewer.this.getPickedState().setHighlightNode(null);
					/*//ウィンドウをすべて閉じたらメインにフォーカスを移す
					boolean allClosed = true;
					for(JInternalFrame f : getRootViewer().desktopPane.getAllFrames()){
						if(f.isVisible()){
							allClosed = false;
							break;
						}
					}
					if(allClosed)*/
						getRootViewer().focusViewer();
					repaint();
				}
			});
			
			frame.setVisible(true);
			viewers[0].focusViewer();
		}
	}
	
	public void minimizeSubflowFrames(){
		if(isRootViewer()){
			boolean icon = true;
			for(JInternalFrame f : desktopPane.getAllFrames()){
				if(f.isVisible() && !f.isIcon()){
					icon = false;
					break;
				}
			}
			for(JInternalFrame f : desktopPane.getAllFrames()){
				if(!f.isVisible())continue;
				if(icon)
					try {
						f.setIcon(false);
					} catch (PropertyVetoException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				else
					try {
						f.setIcon(true);
					} catch (PropertyVetoException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					//manager.iconifyFrame(f);
			}
			focusViewer();
		}else{
			getRootViewer().minimizeSubflowFrames();
		}
	}
	
	public void closeSubflowFrames(){
		if(isRootViewer()){
			for(JInternalFrame f : desktopPane.getAllFrames()){
				//manager.closeFrame(f);
				f.setVisible(false);
			}
		}else{
			getRootViewer().closeSubflowFrames();
		}
	}

	private class MouseHandler extends MouseAdapter {

		private final OnNodeActionManager onNodeActionManager = new OnNodeActionManager();

		private final OnGroundActionManager onGroundActionManager = new OnGroundActionManager(FlowViewer.this);

		@Override
		public void mousePressed(MouseEvent e) {
			if(SwingUtilities.isLeftMouseButton(e)){
				lastClickedPoint = e.getPoint();
			}
		}
		
		@Override
		public void mouseReleased(MouseEvent e) {
			final FlowViewer viewer = (FlowViewer) e.getSource();


			final Point2D p = viewer.inverseViewTransform(e.getPoint());
			final FlowNode v = (FlowNode) viewer.getPickSupport().getVertex(p.getX(), p.getY());
			final Edge edge = viewer.getPickSupport().getEdge(p.getX(),p.getY());

			/*
			 * ノード上
			 */
			if (v != null) {
				if (SwingUtilities.isLeftMouseButton(e)) {
					// SHIFT+左クリック(ノード上)
					if (e.getModifiers() == (MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK)) {
						viewer.getPickedState().selectInterval(v);
					}
					// 左ダブルクリック(ノード上)
					else if (e.getClickCount() == 2) {
						editNode(v);
					}
					// ALT+左クリック(ノード上)
					else if (e.getModifiers() == (MouseEvent.BUTTON1_MASK | MouseEvent.ALT_MASK)) {
						connectNode(v);
					}
				}
				// 中クリック(ノード上)
				else if (SwingUtilities.isMiddleMouseButton(e)) {
					connectNode(v);
				}
				// 右クリック(ノード上)
				else if (SwingUtilities.isRightMouseButton(e)) {
					//pick if not selected
					if(!getPickedState().isPicked(v)){
						getPickedState().clearPickedVertices();
						getPickedState().pick(v, true);
					}
					onNodeActionManager.showPopup(viewer, viewer, rflow, v,
							(int) p.getX(), (int) p.getY());
				}
			}
			/*
			 * ノード以外
			 */
			else {
				/*
				 * エッジ上
				 */
				if (edge != null) {
					if (SwingUtilities.isRightMouseButton(e)) {

						JMenuItem delete = new JMenuItem(Messages.getString("Action.remove_edge")); //$NON-NLS-1$
						delete.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								viewer.deleteEdge((FlowEdge)edge);
							}
						});

						JPopupMenu popup = new JPopupMenu();
						popup.add(delete);
						popup.show(viewer, (int) p.getX(), (int) p.getY());
					}
				}
				/*
				 * 何もないところ
				 */
				else if (SwingUtilities.isLeftMouseButton(e)) {
					// 地面ダブルクリック
					if (e.getClickCount() == 2) {
						createNode(SentenceNodeModel.class, e.getPoint(), true);
					}
				} else if (SwingUtilities.isRightMouseButton(e)) {
					// 何もないところを右クリック
					onGroundActionManager.showPopup((int) e.getPoint().getX(), (int) e.getPoint().getY());
				}

			}
		}
	}
	
	public static enum ViewerActionKey {
		UNDO,
		REDO,
		CUT,
		COPY,
		PASTE,
		COPY_SCRIPT,
		NEW_NODE,
		NEW_FREE_NODE,
		CREATE_SUBFLOW_NODE,
		CREATE_IF_NODE,
		CREATE_FOR_NODE,
		CREATE_WHILE_NODE,
		CREATE_REPEAT_NODE,
		CREATE_FUNCTION_NODE,
		EDIT_NODE,
		REMOVE_NODE,
		SELECT_ALL,
		RUN,
		RUN_CLEAR,
		RUN_SELECTED,
		//UPDATE_CACHE,
		CLEAN_CACHE,
		MINIMIZE_WINDOWS,
		CLOSE_ALL_WINDOWS
		;
	}
	
	public static class ViewerActionManager extends AbstractActionManager<ViewerActionKey>{
		
		private FlowViewer viewer;
		private OnGroundActionManager groundActionManager;
		
		public ViewerActionManager(FlowViewer viewer){
			this.viewer = viewer;
			groundActionManager = new OnGroundActionManager(viewer);
			
			putAction(ViewerActionKey.UNDO, new UndoAction());
			putAction(ViewerActionKey.REDO, new RedoAction());
			putAction(ViewerActionKey.CUT, new CutAction());
			putAction(ViewerActionKey.COPY, new CopyAction());
			putAction(ViewerActionKey.PASTE, groundActionManager.getAction(OnGroundActionKey.PASTE));
			putAction(ViewerActionKey.COPY_SCRIPT, new CopyScriptAction());
			
			putAction(ViewerActionKey.NEW_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_SENTENCE_NODE));
			putAction(ViewerActionKey.NEW_FREE_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_FREE_NODE));
			putAction(ViewerActionKey.CREATE_SUBFLOW_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_SUBFLOW_NODE));
			putAction(ViewerActionKey.CREATE_IF_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_IF_NODE));
			putAction(ViewerActionKey.CREATE_FOR_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_FOR_NODE));
			putAction(ViewerActionKey.CREATE_WHILE_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_WHILE_NODE));
			putAction(ViewerActionKey.CREATE_REPEAT_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_REPEAT_NODE));
			putAction(ViewerActionKey.CREATE_FUNCTION_NODE, groundActionManager.getAction(OnGroundActionKey.CREATE_FUNCTION_NODE));

			putAction(ViewerActionKey.EDIT_NODE, new EditNodeAction());
			putAction(ViewerActionKey.REMOVE_NODE, new RemoveNodeAction());
			putAction(ViewerActionKey.SELECT_ALL, new SelectAllAction());
			putAction(ViewerActionKey.RUN, new RunAction());
			putAction(ViewerActionKey.RUN_CLEAR, new RunClearAction());
			putAction(ViewerActionKey.RUN_SELECTED, new RunSelectedAction());
			//putAction(ViewerActionKey.UPDATE_CACHE, new UpdateCacheAction());
			putAction(ViewerActionKey.CLEAN_CACHE, new CleanCacheAction());
			putAction(ViewerActionKey.MINIMIZE_WINDOWS, new MinimizeWindowsAction());
			putAction(ViewerActionKey.CLOSE_ALL_WINDOWS, new CloseAllWindowsAction());
		}
		
		public void setViewer(FlowViewer viewer){
			this.viewer = viewer;
			groundActionManager.setViewer(viewer);
		}
		
		public InputMap getInputMap(){
			return getInputMap(ViewerActionKey.values());
		}
		
		
		private class UndoAction extends AbstractAction{
			UndoAction(){
				putValue(Action.NAME, Messages.getString("Action.undo")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Z, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.undo();
			}
		}
		
		private class RedoAction extends AbstractAction{
			RedoAction(){
				putValue(Action.NAME, Messages.getString("Action.redo")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Y, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.redo();
			}
		}
		
		private class CutAction extends AbstractAction{
			CutAction(){
				putValue(Action.NAME, Messages.getString("Action.cut")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_X, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.cut();
			}
		}
		
		private class CopyAction extends AbstractAction{
			CopyAction(){
				putValue(Action.NAME, Messages.getString("Action.copy")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.copy();
			}
		}
		
		/*private class PasteAction extends AbstractAction{
			PasteAction(){
				putValue(Action.NAME, Messages.getString("Action.paste"));
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.paste(null);
			}
		}*/

		private class CopyScriptAction extends AbstractAction{
			CopyScriptAction(){
				putValue(Action.NAME, Messages.getString("Action.copy_script")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.copyScript(viewer.getPickedState().getSelectedNode());
			}
		}
		
		/*private class NewNodeAction extends AbstractAction{
			NewNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.new_sentence_node"));
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createNode(SentenceNodeModel.class, null, true);
			}
		}*/
		
		/*private class NewFreeNodeAction extends AbstractAction{
			NewFreeNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.new_free_node"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createNode(FreeNodeModel.class, null, true);
			}
		}*/
		
		private class EditNodeAction extends AbstractAction{
			EditNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.edit_node")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.editNode();
			}
		}
		
		private class RemoveNodeAction extends AbstractAction{
			RemoveNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.remove_node")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.delete();
			}
		}

		private class SelectAllAction extends AbstractAction{
			SelectAllAction(){
				putValue(Action.NAME, Messages.getString("Action.select_all")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_A, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.selectAll();
			}
		}

		private class RunAction extends AbstractAction{
			RunAction(){
				putValue(Action.NAME, Messages.getString("Action.run")); //$NON-NLS-1$
				putValue(Action.SMALL_ICON, FileResource.ACTION_RUN);
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_R, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.getFlowExecuter().execute(viewer.getPickedState().getSelectedNode(), null, true, false, true);
			}
		}

		private class RunClearAction extends AbstractAction{
			RunClearAction(){
				putValue(Action.NAME, Messages.getString("Action.run_clear")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.ALT_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.getFlowExecuter().execute(viewer.getPickedState().getSelectedNode(), null, false, true, true);
			}
		}
		
		private class RunSelectedAction extends AbstractAction{
			RunSelectedAction(){
				putValue(Action.NAME, Messages.getString("Action.run_selected")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				NodeSelectionInfo info = FlowExecuter.getExecutableSelectionInfo(viewer.getPickedState().getPickedVertices());
				if(info != null){
					FlowNode invokeNode = info.getOut();
					FlowNode startNode = info.getSource();
					viewer.getFlowExecuter().execute(invokeNode, startNode, false, false, false);
				}
			}
		}
		
		
		/*private class UpdateCacheAction extends AbstractAction{
			UpdateCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_read_all")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.getCacheHandler().update();
			}
		}*/
		
		private class CleanCacheAction extends AbstractAction{
			CleanCacheAction(){
				putValue(Action.NAME, ""); 
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.getCacheHandler().cleanCaches();
			}
		}

		private class MinimizeWindowsAction extends AbstractAction{
			MinimizeWindowsAction(){
				putValue(Action.NAME, Messages.getString("Action.window_minimize_all")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_M, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.minimizeSubflowFrames();
			}
		}

		private class CloseAllWindowsAction extends AbstractAction{
			CloseAllWindowsAction(){
				putValue(Action.NAME, Messages.getString("Action.window_close_all")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.closeSubflowFrames();
			}
		}
	}

	private static enum OnGroundActionKey {
		CREATE_SENTENCE_NODE, 
		CREATE_FREE_NODE,
		CREATE_SUBFLOW_NODE,
		CREATE_IF_NODE,
		CREATE_FOR_NODE,
		CREATE_WHILE_NODE,
		CREATE_REPEAT_NODE,
		CREATE_FUNCTION_NODE,
		INSERT_SOURCE, 
		INSERT_CLIPBOARD,
		PASTE,
		SELECT_ALL,
		PROPERTY;
	}

	private static class OnGroundActionManager extends AbstractActionManager<OnGroundActionKey> {
		
		private FlowViewer viewer;

		private Point p;

		// private Action[] actions;

		OnGroundActionManager(FlowViewer viewer) {
			this.viewer = viewer;
			putAction(OnGroundActionKey.CREATE_SENTENCE_NODE, new CreateSentenceNodeAction());
			putAction(OnGroundActionKey.CREATE_FREE_NODE, new CreateFreeNodeAction());
			putAction(OnGroundActionKey.CREATE_SUBFLOW_NODE, new CreateSubflowNodeAction());
			putAction(OnGroundActionKey.CREATE_IF_NODE, new CreateIfNodeAction());
			putAction(OnGroundActionKey.CREATE_FOR_NODE, new CreateLoopNodeAction(LoopType.FOR, Messages.getString("Action.box_for"))); //$NON-NLS-1$
			putAction(OnGroundActionKey.CREATE_WHILE_NODE, new CreateLoopNodeAction(LoopType.WHILE, Messages.getString("Action.box_while"))); //$NON-NLS-1$
			putAction(OnGroundActionKey.CREATE_REPEAT_NODE, new CreateLoopNodeAction(LoopType.REPEAT, Messages.getString("Action.box_repeat"))); //$NON-NLS-1$
			putAction(OnGroundActionKey.CREATE_FUNCTION_NODE, new CreateFunctionNodeAction());
			putAction(OnGroundActionKey.INSERT_SOURCE, new InsertSourceAction());
			putAction(OnGroundActionKey.INSERT_CLIPBOARD, new InsertClipBoardAction());
			putAction(OnGroundActionKey.PASTE, new PasteAction());
			putAction(OnGroundActionKey.SELECT_ALL, new SelectAllAction());
			putAction(OnGroundActionKey.PROPERTY, new PropertyAction());
		}
		
		public void setViewer(FlowViewer viewer){
			this.viewer = viewer;
		}

		public void showPopup(int x, int y) {
			this.p = new Point(x, y);
			
			//set enabled
			getAction(OnGroundActionKey.INSERT_SOURCE).setEnabled(!Rflow.isBusy());
			getAction(OnGroundActionKey.PASTE).setEnabled(clipedLayout != null);

			//popup
			JPopupMenu popup = new JPopupMenu();
			popup.add(getAction(OnGroundActionKey.CREATE_SENTENCE_NODE));
			popup.add(getAction(OnGroundActionKey.CREATE_FREE_NODE));
			popup.addSeparator();
			popup.add(getAction(OnGroundActionKey.CREATE_SUBFLOW_NODE));
			JMenu special = new JMenu(Messages.getString("Menu.create_special_box")); //$NON-NLS-1$
			special.add(getAction(OnGroundActionKey.CREATE_FUNCTION_NODE));
			special.add(getAction(OnGroundActionKey.CREATE_IF_NODE));
			special.add(getAction(OnGroundActionKey.CREATE_FOR_NODE));
			special.add(getAction(OnGroundActionKey.CREATE_WHILE_NODE));
			special.add(getAction(OnGroundActionKey.CREATE_REPEAT_NODE));
			popup.add(special);
			popup.addSeparator();
			popup.add(getAction(OnGroundActionKey.PASTE));
			popup.add(getAction(OnGroundActionKey.SELECT_ALL));
			popup.addSeparator();
			popup.add(getAction(OnGroundActionKey.INSERT_SOURCE));
			popup.add(getAction(OnGroundActionKey.INSERT_CLIPBOARD));
			if(!viewer.isRootViewer()){
				popup.addSeparator();
				popup.add(getAction(OnGroundActionKey.PROPERTY));
			}

			popup.show(viewer, x, y);
		}


		private class CreateSentenceNodeAction extends AbstractAction{
			CreateSentenceNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.new_sentence_node")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createNode(SentenceNodeModel.class, p, true);
			}
		}

		private class CreateFreeNodeAction extends AbstractAction{
			CreateFreeNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.new_free_node")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createNode(FreeNodeModel.class, p, true);
			}
		}

		private class CreateSubflowNodeAction extends AbstractAction{
			CreateSubflowNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.create_box")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createSubflowNode(SubflowNodeModel.class, p);
			}
		}

		private class CreateIfNodeAction extends AbstractAction{
			CreateIfNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.box_if")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createSubflowNode(IfNodeModel.class, p);
			}
		}
		
		private class CreateLoopNodeAction extends AbstractAction{
			
			private LoopType type;
			
			CreateLoopNodeAction(LoopType type, String title){
				this.type = type;
				putValue(Action.NAME, title);
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createLoopNode(type, p);
			}
		}

		private class CreateFunctionNodeAction extends AbstractAction{
			CreateFunctionNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.box_function")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.createSubflowNode(FunctionNodeModel.class, p);
			}
		}
		
		
		private class InsertSourceAction extends AbstractAction{
			InsertSourceAction(){
				putValue(Action.NAME, Messages.getString("Action.insert_source")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.insertSource(p, false);
			}
		}

		private class InsertClipBoardAction extends AbstractAction{
			InsertClipBoardAction(){
				putValue(Action.NAME, Messages.getString("Action.insert_clipboard")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.insertSource(p, true);
			}
		}
		
		private class PasteAction extends AbstractAction{
			PasteAction(){
				putValue(Action.NAME, Messages.getString("Action.paste")); //$NON-NLS-1$
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, Constants.MENU_KEY_MASK));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.paste(p);
			}
		}

		private class SelectAllAction extends AbstractAction{
			SelectAllAction(){
				putValue(Action.NAME, Messages.getString("Action.select_all")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.selectAll();
			}
		}

		private class PropertyAction extends AbstractAction{
			PropertyAction(){
				putValue(Action.NAME, Messages.getString("Action.property")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.editNode(viewer.getGraph().getParentNode(), true);
			}
		}
		
	}

	private static enum OnNodeActionKey {
		//EXECUTE,
		//EXECUTE_CLEAR,
		// EXECUTE_LOCAL,
		// EXECUTE_FLAG,
		//COPY_SCRIPT,
		//EDIT_NODE,
		MARK_CACHE,
		CACHE,
		CLEAR_CACHE,
		//CHECK_CACHE,
		REMOVE_CACHE,
		//REMOVE_NODE, 
		TAKE_OUT, 
		BRANCH, 
		REMOVE_EDGE, 
		SELECT_DESCENDANT,
		// CREATE_FUNCTION,
		CREATE_SUBFLOW,
		UNFOLD_SUBFLOW,
		// SAVE_AS_PARTS,
		PROPERTY
		;

	}

	private static class OnNodeActionManager extends AbstractActionManager<OnNodeActionKey> {

		private FlowViewer viewer;

		private FlowNode node;

		// private Point p;

		OnNodeActionManager() {

			//putAction(OnNodeActionKey.EXECUTE, new ExecuteAction());
			//putAction(OnNodeActionKey.EXECUTE_CLEAR, new ExecuteClearAction());
			//putAction(OnNodeActionKey.COPY_SCRIPT, new CopyScriptAction());
			//putAction(OnNodeActionKey.EDIT_NODE, new EditNodeAction());
			putAction(OnNodeActionKey.CACHE, new CreateCacheAction());
			putAction(OnNodeActionKey.MARK_CACHE, new MarkCacheAction());
			putAction(OnNodeActionKey.CLEAR_CACHE, new ClearCacheAction());
			//putAction(OnNodeActionKey.CHECK_CACHE, new CheckCacheAction());
			putAction(OnNodeActionKey.REMOVE_CACHE, new RemoveCacheAction());
			//putAction(OnNodeActionKey.REMOVE_NODE, new RemoveNodeAction());
			putAction(OnNodeActionKey.TAKE_OUT, new TakeOutAction());
			putAction(OnNodeActionKey.BRANCH, new BranchAction());
			putAction(OnNodeActionKey.REMOVE_EDGE, new RemoveEdgeAction());
			putAction(OnNodeActionKey.SELECT_DESCENDANT, new SelectDescendantAction());
			putAction(OnNodeActionKey.CREATE_SUBFLOW, new CreateSubflowNodeAction());
			putAction(OnNodeActionKey.UNFOLD_SUBFLOW, new UnfoldSubflowAction());
			putAction(OnNodeActionKey.PROPERTY, new PropertyAction());
			
			/*
			 * putAction(OnNodeActionKey.EXECUTE_LOCAL, new
			 * AbstractAction("ローカルで実行"){ public void
			 * actionPerformed(ActionEvent e) { executer.excuteInLocal(node);
			 * viewer.repaint(); } });
			 */

			/*
			 * putAction(OnNodeActionKey.EXECUTE_FLAG, new
			 * AbstractAction("フラグから実行"){ public void
			 * actionPerformed(ActionEvent e) { executer.excuteRest(node);
			 * getPickedState().setFlagNode(node); viewer.repaint(); } });
			 */

			/*
			 * putAction(OnNodeActions.CREATE_FUNCTION, new
			 * AbstractAction("関数を作成"){ public void actionPerformed(ActionEvent
			 * e) { if(!viewer.getPickedState().isPinFlowSelected())return;
			 * 
			 * String command = ""; //$NON-NLS-1$ List<FlowNode> stream =
			 * viewer.getPickedState().getSelectionNodeFlow(); for(FlowNode n :
			 * stream) command += n.getModel().getCommand();
			 * //TODO整形にRのparseを使うことを検討 command = command.replaceAll("\n",
			 * "\n\t"); //$NON-NLS-1$ //$NON-NLS-2$ command = "\t" + command;
			 * //$NON-NLS-1$ if(command.endsWith("\t"))command =
			 * command.substring(0, command.length()-1); //$NON-NLS-1$
			 * 
			 * command = "function()"+Constants.NEWLINE+ //$NON-NLS-1$
			 * "{"+Constants.NEWLINE+ //$NON-NLS-1$ command+
			 * "}"+Constants.NEWLINE; //$NON-NLS-1$
			 * 
			 * REXP globals = rflow.R.eval("findGlobals("+command+",
			 * merge=FALSE)"); //$NON-NLS-1$ //$NON-NLS-2$ if(globals != null &&
			 * globals.asList() != null){ RList list = globals.asList();
			 * String[] variables = list.at("variables").asStringArray();
			 * //$NON-NLS-1$ if(variables != null && variables.length > 0){
			 * String args = ""; //$NON-NLS-1$ for(String s : variables) args +=
			 * s + ", "; //$NON-NLS-1$ args = args.substring(0,
			 * args.length()-2);
			 * 
			 * command = command.replace("function()", "function("+args+")");
			 * //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ } }
			 * 
			 * FreeNodeModel model = new FreeNodeModel();
			 * model.setTitle("new_function"); //$NON-NLS-1$ model.command =
			 * command; try { model =
			 * (FreeNodeModel)NodeModelManager.getModelController(model.getClass()).getNodeModel(model); }
			 * catch (CanceledException ex) { return; } Point pp =
			 * (Point)p.clone(); pp.translate(0, 50); addNode(new
			 * FlowNode(model), pp, false);
			 * //rflow.explorerPanel.addUserNode(model); } });
			 */


			/*
			 * putAction(OnNodeActions.SAVE_AS_PARTS, new
			 * AbstractAction("選択部分をパーツ登録"){ @SuppressWarnings("unchecked")
			 * //$NON-NLS-1$ public void actionPerformed(ActionEvent e) { Set<FlowNode>
			 * picked = viewer.getPickedState().getPickedVertices();
			 * if(picked.size() < 1)return;
			 * 
			 * ViewLayout subLayout = getSubLayout(picked);
			 * 
			 * rflow.explorer.getPartsExplorerPanel().addParts("NEW_PARTS",
			 * subLayout); viewer.getPickedState().clearPickedVertices(); } });
			 */
		}

		/**
		 * クリックされたノードが選択されていたら選択ノード全体 選択されていなかったらクリックされたノードのみを返す
		 * 
		 * @return
		 */
		private FlowNode[] getNodeSelected() {
			if (viewer.getPickedState().isPicked(node)) {
				Set<FlowNode> picked = viewer.getPickedState().getPickedVertices();
				return picked.toArray(new FlowNode[0]);
			} else {
				return new FlowNode[] { node };
			}
		}

		private static boolean TEST = false;
		private static void addIfEnabled(JPopupMenu c, Action action){
			if(TEST | action.isEnabled())
				c.add(action);
		}
		private static void addIfEnabled(JMenu c, Action action){
			if(TEST | action.isEnabled())
				c.add(action);
		}
		
		public void showPopup(Component invoker, FlowViewer viewer, Rflow rflow,
				FlowNode node, int x, int y) {
			this.viewer = viewer;
			this.node = node;
			
			boolean single = getNodeSelected().length == 1;
			boolean canExec = viewer.isExecutable(node);
			boolean isCache = (single && node.isCacheData());
			boolean isCacheFile = isCache && node.getCacheData().isValid();
			boolean notBusy = !Rflow.isBusy();
			//set enabled
			getAction(OnNodeActionKey.REMOVE_EDGE).setEnabled(single);
			getAction(OnNodeActionKey.PROPERTY).setEnabled(single);
			getAction(OnNodeActionKey.MARK_CACHE).setEnabled(single && !isCache);
			//getAction(OnNodeActionKey.CHECK_CACHE).setEnabled(isCache && !isCacheFile);
			getAction(OnNodeActionKey.CACHE).setEnabled(single && canExec && !isCache && notBusy);
			getAction(OnNodeActionKey.CLEAR_CACHE).setEnabled(isCacheFile);
			getAction(OnNodeActionKey.REMOVE_CACHE).setEnabled(isCache);
			getAction(OnNodeActionKey.UNFOLD_SUBFLOW).setEnabled(single && node.getModel().isOpenSubflow());

			
			JPopupMenu popup = new JPopupMenu();
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.RUN));
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.RUN_CLEAR));
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.RUN_SELECTED));
			//addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.COPY_SCRIPT));
			popup.addSeparator();
			Action markCache = getAction(OnNodeActionKey.MARK_CACHE);
			Action cache = getAction(OnNodeActionKey.CACHE);
			//Action checkCache = getAction(OnNodeActionKey.CHECK_CACHE);
			Action clearCache = getAction(OnNodeActionKey.CLEAR_CACHE);
			Action removeCache = getAction(OnNodeActionKey.REMOVE_CACHE);


			JMenu cacheMenu = new JMenu(Messages.getString("Menu.cache")); //$NON-NLS-1$
			cacheMenu.setIcon(FileResource.FLAG_IMAGE);

			addIfEnabled(cacheMenu, markCache);
			addIfEnabled(cacheMenu, cache);
			//addIfEnabled(cacheMenu, checkCache);
			addIfEnabled(cacheMenu, clearCache);
			addIfEnabled(cacheMenu, removeCache);

			if(cacheMenu.getMenuComponentCount() == 0)
				cacheMenu.setEnabled(false);

			if(cacheMenu.isEnabled()){
				popup.add(cacheMenu);
				popup.addSeparator();
			}
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.EDIT_NODE));
			popup.addSeparator();
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.CUT));
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.COPY));
			addIfEnabled(popup, rflow.actionManager.getAction(ActionKey.REMOVE_NODE));
			popup.addSeparator();
			addIfEnabled(popup, getAction(OnNodeActionKey.TAKE_OUT));
			addIfEnabled(popup, getAction(OnNodeActionKey.BRANCH));
			addIfEnabled(popup, getAction(OnNodeActionKey.SELECT_DESCENDANT));
			popup.addSeparator();
			//popup.addSeparator();
			//addIfEnabled(popup, getAction(OnNodeActionKey.REMOVE_EDGE));
			//if(Rflow.TEST){
			Action createSubflow = getAction(OnNodeActionKey.CREATE_SUBFLOW);
			Action unfoldSubflow = getAction(OnNodeActionKey.UNFOLD_SUBFLOW);
			addIfEnabled(popup, createSubflow);
			addIfEnabled(popup, unfoldSubflow);
			//}
			popup.addSeparator();
			addIfEnabled(popup, getAction(OnNodeActionKey.PROPERTY));
			
			SwingUtils.clearPopupSeparator(popup);

			popup.show(invoker, x, y);
		}

		/*private class ExecuteAction extends AbstractAction{
			ExecuteAction(){
				putValue(Action.NAME, Messages.getString("Action.run"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.execute(node, true, false, false);
			}
		}*/

		/*private class ExecuteClearAction extends AbstractAction{
			ExecuteClearAction(){
				putValue(Action.NAME, Messages.getString("Action.run_clear"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.execute(node, false, true, true);
			}
		}*/

		/*private class CopyScriptAction extends AbstractAction{
			CopyScriptAction(){
				putValue(Action.NAME, Messages.getString("Action.copy_script"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.copyScript(node);
			}
		}*/

		/*private class EditNodeAction extends AbstractAction{
			EditNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.edit_node"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.editNode(node);
			}
		}*/
		
		private class MarkCacheAction extends AbstractAction{
			MarkCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_mark")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.cache(node, false);
			}
		}

		private class CreateCacheAction extends AbstractAction{
			CreateCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_mark_and_run")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.cache(node, true);
			}
		}

		/*private class CheckCacheAction extends AbstractAction{
			CheckCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_read")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.getCacheHandler().update(new FlowNode[]{node});
			}
		}*/

		private class ClearCacheAction extends AbstractAction{
			ClearCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_clear")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
				viewer.removeCache(node, false, backup);
				if(backup){
					viewer.undoManager.removeOldClearCacheEdit(node);
					viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createClearCache(
							node, false, true));
				}
			}
		}

		private class RemoveCacheAction extends AbstractAction{
			RemoveCacheAction(){
				putValue(Action.NAME, Messages.getString("Action.cache_unmark")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				boolean backup = (Boolean)Rflow.setting.getProperty(Setting.Key.USE_CACHE_BACKUP);
				viewer.removeCache(node, true, backup);
				if(backup && node.isCacheData() && node.getCacheData().isValid()){
					viewer.undoManager.removeOldClearCacheEdit(node);
					viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createClearCache(
							node, true, true));
				}
			}
		}

		/*private class RemoveNodeAction extends AbstractAction{
			RemoveNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.remove_node"));
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.delete_internal(getNodeSelected());
			}
		}*/

		private class TakeOutAction extends AbstractAction{
			TakeOutAction(){
				putValue(Action.NAME, Messages.getString("Action.take_out")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				FlowNode[] nodes = getNodeSelected();
				CompoundEdit edit = new NamedCompoundEdit(getValue(
						Action.NAME).toString());
				/*if (nodes.length == 1 && node.outDegree() < 1) {
					if (!node.getInEdges().isEmpty()) {
						FlowEdge edge = (FlowEdge) node
								.getInEdges().iterator().next();
						viewer.getGraph().removeEdge(edge);
						edit.addEdit(viewer.undoableEditFactory
								.createDeleteEdit(null,
										new FlowEdge[] { edge }));
						edit.end();
						viewer.fireUndoableEditUpdate(edit);
					}
				} else {*/
					FlowUtil.takeOut(nodes, false,
							viewer.undoableEditFactory, edit);
					viewer.getPickedState().clearPickedVertices();
					edit.end();
					viewer.fireUndoableEditUpdate(edit);
				//}
			}
		}

		private class BranchAction extends AbstractAction{
			BranchAction(){
				putValue(Action.NAME, Messages.getString("Action.branch")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				CompoundEdit edit = new NamedCompoundEdit(getValue(Action.NAME).toString());
				FlowUtil.takeOut(getNodeSelected(), true, viewer.undoableEditFactory, edit);
				edit.end();
				viewer.fireUndoableEditUpdate(edit);
			}
		}

		private class RemoveEdgeAction extends AbstractAction{
			RemoveEdgeAction(){
				putValue(Action.NAME, Messages.getString("Action.remove_edge")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {				
				if (!node.getInEdges().isEmpty()) {
					FlowEdge edge = (FlowEdge) node.getInEdges().iterator().next();
					viewer.getGraph().removeEdge(edge);
					viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createDeleteEdit(
									null, new FlowEdge[] { edge }));
				}
			}
		}

		private class SelectDescendantAction extends AbstractAction{
			SelectDescendantAction(){
				putValue(Action.NAME, Messages.getString("Action.select_children")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				FlowNode[] selected = getNodeSelected();
				viewer.getPickedState().clearPickedVertices();
				for (FlowNode node : FlowUtil.getChildren(selected)) {
					viewer.getPickedState().pick(node, true);
				}
				viewer.repaint();
			}
		}

		private class CreateSubflowNodeAction extends AbstractAction{
			CreateSubflowNodeAction(){
				putValue(Action.NAME, Messages.getString("Action.box_selected")); //$NON-NLS-1$
			}
			
			private void showError(String s){
				JOptionPane.showMessageDialog(viewer, Messages.getString("Error.create_box") + " : " + s, "", JOptionPane.INFORMATION_MESSAGE);  //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
			}
			
			@SuppressWarnings("unchecked") //$NON-NLS-1$
			public void actionPerformed(ActionEvent e) {
				FlowNode[] selected = getNodeSelected();
				Set<FlowNode> nodeSet = new HashSet<FlowNode>(Arrays.asList(selected));
				Collection<FlowNode> unmods = viewer.getUnmodifiableNodes(nodeSet);
				if(!unmods.isEmpty()){
					//showError("追加できないノードが含まれています。");return;
					nodeSet.removeAll(unmods);
					selected = nodeSet.toArray(new FlowNode[0]);
				}
				
				NodeSelectionInfo[] infos = FlowUtil.getNodeSelectionInfos(selected, false);
				if(infos.length < 1){
					showError("");return; //$NON-NLS-1$
				}
				//INが一致するか
				List<FlowNode> srcList = new ArrayList<FlowNode>();
				FlowNode in0 = infos[0].getIn();
				//FlowNode out0 = infos[0].getOut();
				FlowNode last0 = infos[0].getLast();
				srcList.add(infos[0].getSource());
				for(int i = 1; i < infos.length; i++){
					if(!ObjectUtilities.equals(in0, infos[i].getIn())){
						showError(Messages.getString("Error.multistart"));return; //$NON-NLS-1$
					}
					srcList.add(infos[i].getSource());
					if(last0 != null && infos[i].getLast() != null && !last0.equals(infos[i].getLast())){
						showError(Messages.getString("Error.multiend")); //$NON-NLS-1$
						return;//outが二つ以上ある
					}
					if(last0 == null && infos[i].getLast() != null)
						last0 = infos[i].getLast();
				}
				
				FlowNode last = last0;
				if(last == null){
					List<Set<FlowNode>> clusters = FlowUtil.extractCluster(nodeSet);
					Set<FlowNode> cluster = clusters.get(0);
					FlowNode src = FlowUtil.getSource(cluster.iterator().next(), cluster);
					Set<FlowNode> lasts = FlowUtil.getLastChildren(src, cluster);
					last = lasts.iterator().next();
				}
				
				
				Map<FlowNode, FlowNode> nodeMap = new HashMap<FlowNode, FlowNode>();
				ViewLayout layout = viewer.getSubLayout(nodeSet, new Point(100, 50), true, nodeMap);
				FlowNode[] srcArray = new FlowNode[srcList.size()];
				for(int i = 0; i < srcArray.length; i++)
					srcArray[i] = nodeMap.get(srcList.get(i));
				
				
				//新ノード
				SubflowNodeModel model = new SubflowNodeModel(layout, srcArray, nodeMap.get(last));
				FlowNode newNode = new FlowNode(model);
				
				
				
				//undo
				CompoundEdit edit = new NamedCompoundEdit((String)getValue(Action.NAME));
				
				//lock cache
				viewer.getCacheHandler().setCacheStateLock(true);
				edit.addEdit(viewer.undoableEditFactory.createLockCacheStateEdit(true));
				
				//追加
				viewer.getGraphLayout().initLocation(newNode);
				viewer.getGraph().addVertex(newNode);
				//viewer.getGraphLayout().update();
				Set<FlowEdge> newEdges = new HashSet<FlowEdge>();
				if(in0 != null){
					newEdges.add((FlowEdge)viewer.getGraph().addEdge(new FlowEdge(in0, newNode)));
				}
				Set<FlowNode> outs = (Set<FlowNode>)last.getSuccessors();
				for(FlowNode out : outs){
					newEdges.add((FlowEdge)viewer.getGraph().addEdge(new FlowEdge(newNode, out)));
				}
				edit.addEdit(viewer.undoableEditFactory.createAddSublayoutEdit(
						new FlowNode[]{newNode}, 
						newEdges.toArray(new FlowEdge[0])));
				//inherits state
				FlowState state = viewer.getRootFlowState();
				for(FlowNode old : nodeMap.keySet()){
					FlowNode nNode = nodeMap.get(old);
					state.passState(old, nNode);
				}
				edit.addEdit(viewer.undoableEditFactory.createInheritStateEdit(nodeMap));
				//消す
				Set<FlowEdge> deletedEdges = new HashSet<FlowEdge>();
				for(FlowNode node : nodeSet){
					deletedEdges.addAll(node.getIncidentEdges());
					viewer.getGraph().removeVertex(node);
					viewer.getPickedState().pick(node, false);
				}
				edit.addEdit(viewer.undoableEditFactory.createDeleteEdit(
						nodeSet.toArray(new FlowNode[0]), deletedEdges.toArray(new FlowEdge[0])));
				//移動
				Point2D p = viewer.getGraphLayout().getCentroid(nodeSet);
				viewer.getGraphLayout().forceMove(newNode, p.getX(), p.getY());

				//unlock cache
				viewer.getCacheHandler().setCacheStateLock(false);
				edit.addEdit(viewer.undoableEditFactory.createLockCacheStateEdit(false));
				
				edit.end();
				viewer.fireUndoableEditUpdate(edit);

			}
		}

		private class UnfoldSubflowAction extends AbstractAction{
			UnfoldSubflowAction(){
				putValue(Action.NAME, Messages.getString("Action.unfold_box")); //$NON-NLS-1$
			}
			
			@SuppressWarnings("unchecked") //$NON-NLS-1$
			public void actionPerformed(ActionEvent e) {
				if(!node.getModel().isOpenSubflow())return;
				SubflowNodeModel model = (SubflowNodeModel)node.getModel();
				
				Set<FlowNode> startSet = new HashSet<FlowNode>(model.getIn().getSuccessors());
				startSet.remove(model.getOut());
				FlowNode end = null;
				if(model.getOut().inDegree() > 0)
					end = (FlowNode)model.getOut().getPredecessors().iterator().next();
				
				boolean direct = false;
				if(end == model.getIn()){
					direct = true;
					end = null;
				}
				
				FlowNode in = null;
				if(node.inDegree() > 0)
					in = (FlowNode)node.getPredecessors().iterator().next();
				Set<FlowNode> outs = new HashSet<FlowNode>(node.getSuccessors());
				
				Set<FlowNode> nodeSet = new HashSet<FlowNode>(model.getLayouts()[0].getGraph().getVertices());
				nodeSet.remove(model.getIn());
				nodeSet.remove(model.getOut());
				
				FlowViewer dummy = new FlowViewer(viewer.rflow, viewer);
				dummy.init(model.getLayouts()[0]);
				Map<FlowNode, FlowNode> nodeMap = new HashMap<FlowNode, FlowNode>();
				ViewLayout layout = dummy.getSubLayout(nodeSet, new Point(0,0), true, nodeMap);

				//start edit
				viewer.undoableEditSupport.beginUpdate(new NamedCompoundEdit(Messages.getString("Action.unfold_box"))); //$NON-NLS-1$
				
				//lock cache
				viewer.getCacheHandler().setCacheStateLock(true);
				viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createLockCacheStateEdit(true));
				
				//add layout
				Point2D p = viewer.transform(viewer.getGraphLayout().getLocation(node));
				if(layout != null)
					viewer.addSubLayout(layout, new Point((int)p.getX(), (int)p.getY()), false, nodeMap);
				
				//remove subflow node
				Set<FlowEdge> deletedEdges = new HashSet<FlowEdge>(node.getIncidentEdges());
				viewer.getGraph().removeVertex(node);
				viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createDeleteEdit(
						new FlowNode[]{node}, deletedEdges.toArray(new FlowEdge[0])));
				
				//add edge
				Set<FlowEdge> addedEdges = new HashSet<FlowEdge>();
				if(in != null)
					for(FlowNode fn : startSet)
						addedEdges.add((FlowEdge)viewer.getGraph().addEdge(new FlowEdge(in, nodeMap.get(nodeMap.get(fn)))));
				if(end != null)
					for(FlowNode out : outs)
						addedEdges.add((FlowEdge)viewer.getGraph().addEdge(new FlowEdge(nodeMap.get(nodeMap.get(end)), out)));
				if(direct && in != null)
					for(FlowNode out : outs)
						addedEdges.add((FlowEdge)viewer.getGraph().addEdge(new FlowEdge(in, out)));
				if(!addedEdges.isEmpty())
					viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createAddEdgeEdit(addedEdges.toArray(new FlowEdge[0])));
				
				//unlock cache
				viewer.getCacheHandler().setCacheStateLock(false);
				viewer.fireUndoableEditUpdate(viewer.undoableEditFactory.createLockCacheStateEdit(false));

				//end edit
				viewer.undoableEditSupport.endUpdate();
				
				//select added nodes
				for(FlowNode fn : nodeSet)
					viewer.getPickedState().pick(nodeMap.get(nodeMap.get(fn)), true);
			}
		}

		private class PropertyAction extends AbstractAction{
			PropertyAction(){
				putValue(Action.NAME, Messages.getString("Action.property")); //$NON-NLS-1$
			}
			
			public void actionPerformed(ActionEvent e) {
				viewer.editNode(node, true);
			}
		}
		
	}

	private class DropTargetHandler extends DropTargetAdapter {

		@SuppressWarnings("unchecked")//$NON-NLS-1$
		public void drop(DropTargetDropEvent dtde) {
			Transferable trans = dtde.getTransferable();
			// 関数ノード：関数名
			if (trans.isDataFlavorSupported(FunctionNodeFlavor.getInstance())) {
				try {
					// String function =
					// trans.getTransferData(DataFlavor.stringFlavor).toString();
					FunctionNode[] nodes = (FunctionNode[]) trans
							.getTransferData(FunctionNodeFlavor.getInstance());
					FunctionNode node = nodes[0];
					if (nodes.length > 1 | !node.hasFunction()) {
						dtde.rejectDrop();
						dtde.dropComplete(false);
						return;
					}

					NodeModel nodeModel = new SentenceNodeModel();

					try {
						nodeModel.setCommand(node.getFunction() + "()"); //$NON-NLS-1$
					} catch (IncompatibleCommandException e) {
						// TODO 再読み込み検討
						e.printStackTrace();
					}
					// nodeModel.function = function;

					Point p = dtde.getLocation();
					FlowNode newNode = new FlowNode(nodeModel);
					addNode(newNode, p, true);
					
					newNode.editNode(FlowViewer.this, false, true);

				} catch (UnsupportedFlavorException e) {
					dtde.dropComplete(false);
				} catch (IOException e) {
					dtde.dropComplete(false);
				} /*catch (CanceledException e) {
				}*/
			}
			// レイアウト：パーツ
			else if (trans
					.isDataFlavorSupported(LayoutDataFlavor.getInstance())) {
				try {
					ViewLayout layout = (ViewLayout) trans
							.getTransferData(LayoutDataFlavor.getInstance());
					// System.out.println(layout.getGraph());
					Point p = dtde.getLocation();
					p.translate(-layout.getSize().width / 2,
							-layout.getSize().height / 2);
					addSubLayout(layout, p);
					removeDragImage();
				} catch (UnsupportedFlavorException e) {
					dtde.dropComplete(false);
				} catch (IOException e) {
					dtde.dropComplete(false);
				}
			}
			// ファイル:読み込みノード作成
			else if (trans.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
				dtde.acceptDrop(DnDConstants.ACTION_LINK);
				List<File> list;
				try {
					list = (List<File>) trans
							.getTransferData(DataFlavor.javaFileListFlavor);
				} catch (Exception e) {
					dtde.rejectDrop();
					dtde.dropComplete(false);
					return;
				}

				if (list.size() > 1) {
					dtde.dropComplete(false);
					return;
				}

				File file = list.get(0);

				if (file.getName().endsWith(Constants.RFLOW_FILE_EXTENSION)) {
					// .rflowファイルの場合
					if (rflow.getWorkingFile() != null
							&& rflow.getWorkingFile().equals(file))
						return;
					if (!rflow.actionManager.askSave())
						return;
					rflow.actionManager.open(file, false);
				} else {
					NodeModel model = NodeModelFactory.createNodeModel(file,
							Prefs.WORKING_DIRECTORY);
					if (model == null) {
						dtde.dropComplete(false);
						return;
					}

					FlowNode newNode = new FlowNode(model);
					addNode(newNode, dtde.getLocation(), false);
					
					newNode.editNode(FlowViewer.this, false, true);
					
					dtde.dropComplete(true);
				}
			}
			dtde.dropComplete(false);
		}

		ImagePaintable partsDragImage;

		@SuppressWarnings("unchecked")//$NON-NLS-1$
		@Override
		public void dragEnter(DropTargetDragEvent dtde) {
			Transferable trans = dtde.getTransferable();
			if (trans.isDataFlavorSupported(LayoutDataFlavor.getInstance())
					&& trans.isDataFlavorSupported(DataFlavor.imageFlavor)) {
				try {
					Image image = (Image) trans
							.getTransferData(DataFlavor.imageFlavor);
					partsDragImage = new ImagePaintable(image);
					partsDragImage.setPoint(dtde.getLocation());
					FlowViewer.this.addPostRenderPaintable(partsDragImage);
					FlowViewer.this.repaint();
				} catch (UnsupportedFlavorException e) {
					dtde.rejectDrag();
				} catch (IOException e) {
					dtde.rejectDrag();
				}
			} else if (trans
					.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
				List<File> list;
				try {
					list = (List<File>) trans
							.getTransferData(DataFlavor.javaFileListFlavor);
				} catch (Exception e) {
					dtde.rejectDrag();
					return;
				}

				if (list.size() > 1) {
					dtde.rejectDrag();
					return;
				}

				File file = list.get(0);
				if (!file.getName().endsWith(Constants.RFLOW_FILE_EXTENSION)
						&& !NodeModelFactory.isModelCreationSupported(file))
					dtde.rejectDrag();
			}
		}

		@Override
		public void dragOver(DropTargetDragEvent dtde) {
			if(Rflow.isBusy()){
				dtde.rejectDrag();
				return;
			}
			if (partsDragImage != null) {
				partsDragImage.setPoint(dtde.getLocation());
				FlowViewer.this.repaint();
			}
		}

		@Override
		public void dragExit(DropTargetEvent dte) {
			removeDragImage();
			FlowViewer.this.repaint();
		}

		private void removeDragImage() {
			if (partsDragImage != null) {
				FlowViewer.this.removePostRenderPaintable(partsDragImage);
				partsDragImage.flushImage();
				partsDragImage = null;
			}
		}
	}

	private class GraphEventHandler implements GraphEventListener {

		public void edgeAdded(GraphEvent event) {
			update();
			
			for(GraphEventListener l : viewerEventListers)
				l.edgeAdded(event);

			fireSubflowNodeChanged((FlowEdge) event.getGraphElement());
		}

		public void edgeRemoved(GraphEvent event) {
			update();
			
			for(GraphEventListener l : viewerEventListers)
				l.edgeRemoved(event);

			fireSubflowNodeChanged((FlowEdge) event.getGraphElement());
		}
		
		public void vertexAdded(GraphEvent event) {
			FlowNode node = (FlowNode) event.getGraphElement();
			updateGroup(node.getModel());
			update();
			
			for(GraphEventListener l : viewerEventListers)
				l.vertexAdded(event);
			
		}

		public void vertexRemoved(GraphEvent event) {
			FlowNode node = (FlowNode) event.getGraphElement();
			//close edit window
			node.clearEditWindow();
			//close subflow window
			if(node.getModel().isSubflow()){
				clearFrame((SubflowNodeModel)node.getModel());
			}
			update();
			
			for(GraphEventListener l : viewerEventListers)
				l.vertexRemoved(event);
		}
		
		private void fireSubflowNodeChanged(FlowEdge e){
			FlowNode node = (FlowNode)e.getSource();
			if(!node.getGraph().isRootGraph()){
				//SubflowNodeModel model = (SubflowNodeModel)node.getGraph().getParentNode().getModel();
				FlowNode.fireSubflowNodeModelChanged(node.getGraph().getParentNode());
			}
		}
		
		private void clearFrame(SubflowNodeModel model){
			FlowGraph[] graphs = model.getGraphs();
			for(FlowGraph graph : graphs){
				for(Object o : graph.getVertices()){
					FlowNode node = (FlowNode)o;
					node.clearEditWindow();
					NodeModel nm = node.getModel();
					if (nm.isSubflow()) {
						clearFrame((SubflowNodeModel) nm);
					}
				}
			}
			model.clearFrame();
		}

		private void update() {
			rflow.setDirty(true);
			repaint();
		}
	}

	private static class ToolTipHandler implements ToolTipFunction {

		private FlowState state;
		
		ToolTipHandler(FlowState state){
			this.state = state;
		}
		
		public String getToolTipText(Vertex v) {
			FlowNode node = (FlowNode) v;
			Set<FlowError> errors = state.getError(node);
			if(errors != null){
				StringBuffer msg = new StringBuffer();
				for(FlowError err : errors){
					msg.append(err.getMessage());
					msg.append(Constants.NEWLINE);
				}
				return convertToHTML(msg.toString());
			}
			String s = node.getProperty().getComment();
			if (s == null)
				return null;
			return convertToHTML(s);
		}

		public String getToolTipText(Edge e) {
			return null;
		}

		public String getToolTipText(MouseEvent event) {
			return null;
		}

		private String convertToHTML(String s) {
			s = SwingUtils.replaceCharForHtml(s);
			String html = "<html>" + s + "</html>"; //$NON-NLS-1$ //$NON-NLS-2$
			html = html.replaceAll("\\n", "<br>"); //$NON-NLS-1$ //$NON-NLS-2$
			return html;
		}
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public ViewLayout getViewLayout() {
		if(getGraph().getVertices().size() == 0){
			return new ViewLayout(new FlowGraph(), new HashMap<Vertex, Point2D>(), 
					getSize(), new Point());
		}
		return getSubLayout(getGraph().getVertices(), null, false, null);
	}

	public void paintToGraphics(Graphics2D g) {
		renderGraph(g);
	}

	private class ImagePaintable implements Paintable {

		private Image image;

		// private ImageIcon icon;
		private Point p = new Point();

		public ImagePaintable(Image image) {
			super();
			this.image = image;

			// icon = new ImageIcon(image);
		}

		public void paint(Graphics g) {

			// icon.paintIcon(StreamViewer.this, g, p.x-icon.getIconWidth()/2,
			// p.y-icon.getIconHeight()/2);
			ImageObserver observer = FlowViewer.this;
			g.drawImage(image, p.x - image.getWidth(observer) / 2, p.y
					- image.getHeight(observer) / 2, observer);
		}

		public void setPoint(Point p) {
			this.p = p;
		}

		public void flushImage() {
			image.flush();// 必要？
		}

		public boolean useTransform() {
			return false;
		}

	}

	protected void renderGraph(Graphics2D g2d) {
		
		if(!repaintEnabled)return;
		
		FlowLayout layout = getGraphLayout();

		g2d.setRenderingHints(renderingHints);

		// the size of the VisualizationViewer
		Dimension d = getSize();

		// clear the offscreen image
		g2d.setColor(getBackground());
		g2d.fillRect(0, 0, d.width, d.height);

		AffineTransform oldXform = g2d.getTransform();
		AffineTransform newXform = new AffineTransform(oldXform);
		newXform.concatenate(viewTransformer.getTransform());

		g2d.setTransform(newXform);

		// if there are preRenderers set, paint them
		for (Iterator iterator = preRenderers.iterator(); iterator.hasNext();) {
			Paintable paintable = (Paintable) iterator.next();
			if (paintable.useTransform()) {
				paintable.paint(g2d);
			} else {
				g2d.setTransform(oldXform);
				paintable.paint(g2d);
				g2d.setTransform(newXform);
			}
		}

		locationMap.clear();

		// paint all the edges
		//
		getRenderer().clearArrowOnlyEdges();
		try {
			for (Iterator iter = layout.getGraph().getEdges().iterator(); iter
					.hasNext();) {
				Edge e = (Edge) iter.next();
				Vertex v1 = (Vertex) e.getEndpoints().getFirst();
				Vertex v2 = (Vertex) e.getEndpoints().getSecond();

				Point2D p = getLocation(v1);
				Point2D q = getLocation(v2);

				if (p != null && q != null) {
					renderer.paintEdge(g2d, e, (int) p.getX(), (int) p.getY(),
							(int) q.getX(), (int) q.getY());
				}
			}
		} catch (ConcurrentModificationException cme) {
			repaint();
		}

		// paint all the vertices label space
		try {
			for (Iterator iter = layout.getGraph().getVertexIterator(); iter
					.hasNext();) {

				Vertex v = (Vertex) iter.next();
				Point2D p = getLocation(v);
				if (p != null) {
					getRenderer().paintVertexSpaces(g2d, v, (int) p.getX(),
							(int) p.getY());
				}
			}
		} catch (ConcurrentModificationException cme) {
			repaint();
		}

		// paint all the vertices
		try {
			for (Iterator iter = layout.getGraph().getVertexIterator(); iter
					.hasNext();) {

				Vertex v = (Vertex) iter.next();
				Point2D p = getLocation(v);
				if (p != null) {
					renderer.paintVertex(g2d, v, (int) p.getX(), (int) p.getY());
				}
			}
		} catch (ConcurrentModificationException cme) {
			repaint();
		}

		// arrow only edge
		try {
			for (Edge e : getRenderer().getArrowOnlyEdges()) {
				Vertex v1 = (Vertex) e.getEndpoints().getFirst();
				Vertex v2 = (Vertex) e.getEndpoints().getSecond();

				Point2D p = getLocation(v1);
				Point2D q = getLocation(v2);

				if (p != null && q != null) {
					getRenderer().paintArrowOnlyEdge(g2d, e, (int) p.getX(),
							(int) p.getY(), (int) q.getX(), (int) q.getY());
				}
			}
		} catch (ConcurrentModificationException cme) {
			repaint();
		}

		// if there are postRenderers set, do it
		for (Iterator iterator = postRenderers.iterator(); iterator.hasNext();) {
			Paintable paintable = (Paintable) iterator.next();
			if (paintable.useTransform()) {
				paintable.paint(g2d);
			} else {
				g2d.setTransform(oldXform);
				paintable.paint(g2d);
				g2d.setTransform(newXform);
			}
		}
		g2d.setTransform(oldXform);
	}

	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private Point2D getLocation(Vertex v){
		Point2D p = (Point2D) locationMap.get(v);
		if (p == null) {
			p = getGraphLayout().getLocation(v);
			p = layoutTransformer.transform(p);
			locationMap.put(v, p);
		}
		return p;
	}
	
	public void repaintFamiliy(){
		repaintChildren(getRootViewer());
	}
	
	private void repaintChildren(FlowViewer viewer){
		for(FlowViewer child : viewer.getChildren()){
			repaintChildren(child);
		}
		viewer.repaint();
	}
	
	
	/*
	private static class DragHandler implements MouseListener, MouseMotionListener{

		private Point down;
		private Rectangle bounds;
		
		private boolean checkModifier(MouseEvent e){
			return e.getModifiers() == (MouseEvent.BUTTON1_MASK | MouseEvent.ALT_MASK);
		}

		public void mousePressed(MouseEvent e) {
			if(e.isConsumed())return;
			if(checkModifier(e)){
		        FlowViewer fv = (FlowViewer)e.getSource();
				down = e.getPoint();
				SwingUtilities.convertPointToScreen(down, fv);
				bounds = fv.getBounds();
			}
		}

		public void mouseReleased(MouseEvent e) {
			down = null;
			bounds = null;
	        FlowViewer fv = (FlowViewer)e.getSource();
	        fv.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}

		public void mouseDragged(MouseEvent e) {
			if(e.isConsumed())return;
	        FlowViewer fv = (FlowViewer)e.getSource();
			if(checkModifier(e) && down != null){
				fv.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
				Point up = e.getPoint();
				SwingUtilities.convertPointToScreen(up, fv);
				//System.out.println("down" + down + " up" + up);
				
				double dx = up.getX() - down.getX();
				double dy = up.getY() - down.getY();

				Rectangle rect = (Rectangle)bounds.clone();
				//System.out.println(dx + ":" + dy);
				rect.translate((int)dx, (int)dy);
				fv.setBounds(rect);
				
				//down.setLocation(up.getX(), up.getY());
			}
		}

		public void mouseMoved(MouseEvent e) {
		}

		public void mouseClicked(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}
	}*/
}
