package com.ef_prime.rflow.flow;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.BoundedRangeModel;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class FlowViewerScrollPane extends JPanel{

	private FlowViewer viewer;
    private JScrollBar horizontalScrollBar;
    private JScrollBar verticalScrollBar;
    private JComponent corner;
    private boolean scrollBarsMayControlAdjusting = true;
    private JPanel south;
	
	public FlowViewerScrollPane(FlowViewer viewer) {
        super(new BorderLayout());
		this.viewer = viewer;
		setOpaque(false);
		
		
        addComponentListener(new ResizeListener());        
        Dimension d = viewer.getGraphLayout().getCurrentSize();
        verticalScrollBar = new JScrollBar(JScrollBar.VERTICAL, 0, d.height, 0, d.height);
        horizontalScrollBar = new JScrollBar(JScrollBar.HORIZONTAL, 0, d.width, 0, d.width);
        verticalScrollBar.addAdjustmentListener(new VerticalAdjustmentListenerImpl());
        horizontalScrollBar.addAdjustmentListener(new HorizontalAdjustmentListenerImpl());
        verticalScrollBar.setUnitIncrement(20);
        horizontalScrollBar.setUnitIncrement(20);
        // respond to changes in the VisualizationViewer's transform
        // and set the scroll bar parameters appropriately
        viewer.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent evt) {
            	FlowViewer viewer = (FlowViewer)evt.getSource();
                setScrollBars(viewer);
            }
        });
        add(viewer);
        add(verticalScrollBar, BorderLayout.EAST);
        south = new JPanel(new BorderLayout());
        south.add(horizontalScrollBar);
        setCorner(new JPanel());
        add(south, BorderLayout.SOUTH);
	}
	class HorizontalAdjustmentListenerImpl implements AdjustmentListener {
		int previous = 0;
		public void adjustmentValueChanged(AdjustmentEvent e) {
			int hval = e.getValue();
			float dh = previous - hval;
			previous = hval;
			if(dh != 0 && scrollBarsMayControlAdjusting) {
				//need if viewtransform
				/*float layoutScale = (float) viewer.getViewTransformer().getScale();
                dh /= layoutScale;*/

				AffineTransform at = AffineTransform.getTranslateInstance(dh, 0);
				viewer.getLayoutTransformer().preConcatenate(at);
			}
		}
	}
    
    class VerticalAdjustmentListenerImpl implements AdjustmentListener {
        int previous = 0;
        public void adjustmentValueChanged(AdjustmentEvent e) {
            JScrollBar sb = (JScrollBar)e.getSource();
            BoundedRangeModel m = sb.getModel();
            int vval = m.getValue();
            float dv = previous - vval;
            previous = vval;
            if(dv != 0 && scrollBarsMayControlAdjusting) {
            	//need if viewtransform
                /*float layoutScale = (float) viewer.getViewTransformer().getScale();
                dv /= layoutScale;*/
                AffineTransform at = AffineTransform.getTranslateInstance(0, dv);
                viewer.getLayoutTransformer().preConcatenate(at);
            }
        }
    }
	
	private void setScrollBars(FlowViewer viewer) {
		
		Rectangle rect = viewer.getGraphLayout().getLayoutSize();
		Point temp = rect.getLocation();
		Point2D upperLeft = viewer.transform(new Point2D.Float(temp.x, temp.y));
		temp.translate(rect.width, rect.height);
		Point2D lowerRight = viewer.transform(new Point2D.Float(temp.x, temp.y));
		Rectangle layoutRect = new Rectangle();
		layoutRect.setFrameFromDiagonal(upperLeft, lowerRight);
		if(rect.width > 0)//don't grow if graph is emtpy
			layoutRect.grow(50, 50);
		
		Rectangle viewRect = viewer.getBounds();
		
		Rectangle2D union = layoutRect.createUnion(viewRect);
		
		scrollBarsMayControlAdjusting = false;
		
		horizontalScrollBar.setValues((int)viewRect.getMinX(), (int)viewRect.getWidth(), (int)union.getMinX(), (int)union.getMaxX());
		verticalScrollBar.setValues((int)viewRect.getMinY(), (int)viewRect.getHeight(), (int)union.getMinY(), (int)union.getMaxY());
		
		//horizontalScrollBar.setVisible((viewRect.getWidth() < union.getWidth()));
		boolean hVisible = (viewRect.getWidth() < union.getWidth());
		boolean vVisible = (viewRect.getHeight() < union.getHeight());
		south.setVisible(hVisible);
		corner.setVisible(hVisible && vVisible);
		verticalScrollBar.setVisible(vVisible);
		
		scrollBarsMayControlAdjusting = true;
	}

	protected class ResizeListener extends ComponentAdapter {

		public void componentHidden(ComponentEvent e) {
		}

		public void componentResized(ComponentEvent e) {
		    setScrollBars(viewer);
		}	
		public void componentShown(ComponentEvent e) {
		}
	}

    public JComponent getCorner() {
        return corner;
    }

    public void setCorner(JComponent corner) {
        this.corner = corner;
        corner.setPreferredSize(new Dimension(verticalScrollBar.getPreferredSize().width,
                horizontalScrollBar.getPreferredSize().height));
        south.add(this.corner, BorderLayout.EAST);
    }
    
    @Override
    public Dimension getPreferredSize() {
    	return viewer.getPreferredSize();
    }
    
}
