package com.ef_prime.rflow.flow.model;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.SparseGraph;

public class FlowGraph extends SparseGraph implements ItemListener{
	
	/*
	 * parentNodeは表示される時に設定される。
	 * ->SubflowNodeModel.getSubflowFrame
	 * 
	 */
	private FlowNode parentNode = null;
	
	/**
	 * ノードに順番を与えるためのリストです。
	 * ノードの描画の順番を決めるためのものです。
	 */
	private LinkedList<Vertex> vertexList = new LinkedList<Vertex>();
	
	List<GraphWillChangeListener> graphWillChangeListener = new ArrayList<GraphWillChangeListener>();
	
	public boolean isRootGraph(){
		return parentNode == null;
	}
	
	public void setParentNode(FlowNode parentNode) {
		this.parentNode = parentNode;
	}
	
	public FlowNode getParentNode() {
		return parentNode;
	}
	
	/**
	 * 描画のための（順序のある）リストのIteratorを返します
	 * @return
	 */
	public Iterator<Vertex> getVertexIterator() {
		return vertexList.iterator();
	}
	
	/**
	 * 描画用リストのコピーを返します
	 * 必要がない限りgetVertexIterator()推奨
	 * @return
	 */
	public LinkedList<Vertex> getVertexList(){
		return new LinkedList<Vertex>(vertexList);
	}
	
	/**
	 * 指定されたノードをリストの最後に移動します。
	 * このリストは描画に使用されるので、最後に移動されたノードは
	 * 一番上に描画されることになります。
	 * @param v
	 */
	public void toTop(Vertex v){
		if(vertexList.remove(v)){
			vertexList.addLast(v);
		}
	}
	
	@Override
	public Vertex addVertex(Vertex v) {
		fireGraphWillChange(v, GraphChangeType.ADD, this);
		vertexList.add(v);
		return super.addVertex(v);
	}
	
	@Override
	public Edge addEdge(Edge e) {
		fireGraphWillChange(e, GraphChangeType.ADD, this);
		return super.addEdge(e);
	}

	@Override
	public void removeEdge(Edge e) {
		fireGraphWillChange(e, GraphChangeType.REMOVE, this);
		super.removeEdge(e);
	}
	
	@Override
	public void removeEdges(Set edges) {
		super.removeEdges(edges);
	}
	
	@Override
	public void removeAllEdges() {
		super.removeAllEdges();
	}
	
	@Override
	public void removeVertex(Vertex v) {
		fireGraphWillChange(v, GraphChangeType.REMOVE, this);
		vertexList.remove(v);
		super.removeVertex(v);
	}
	
	@Override
	public void removeVertices(Set vertices) {
		super.removeVertices(vertices);
	}
	
	@Override
	public void removeAllVertices() {
		for(Object o : getVertices()){
			fireGraphWillChange((FlowNode)o, GraphChangeType.REMOVE, this);
		}
		super.removeAllVertices();
	}
	
	public void addGraphWillChangeListener(GraphWillChangeListener l){
		graphWillChangeListener.add(l);
	}
	
	private void fireGraphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph){
		for(GraphWillChangeListener l : graphWillChangeListener){
			l.graphWillChange(graphElement, type, graph);
		}
	}
	
	/**
	 * グラフが変更される直前に投げられるイベントに対するリスナー
	 *
	 */
	public static interface GraphWillChangeListener{
		//TODO not used
		void graphWillChange(Object graphElement, GraphChangeType type, FlowGraph graph);
	}
	
	public static enum GraphChangeType{
		ADD,REMOVE;
	}

	/*-----------------------ItemListener----------------------------*/
	/*
	 * flow state に登録
	 * ノード選択の変更時の処理
	 */
	public void itemStateChanged(ItemEvent e) {
		if(e.getStateChange() == ItemEvent.SELECTED && e.getItem() instanceof Vertex){
			toTop((Vertex)e.getItem());
		}
	}
}
