package com.ef_prime.rflow.flow.model;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.Set;


import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.visualization.AbstractLayout;
import edu.uci.ics.jung.visualization.Coordinates;
import edu.uci.ics.jung.visualization.LayoutMutable;

public class FlowLayout extends AbstractLayout implements LayoutMutable {

	private int gridDist = 10;
	
	public FlowLayout(FlowGraph g) {
		super(g);
	}
	
	public Point2D getCentroid(Iterable<FlowNode> nodes){
		double x = 0;
		double y = 0;
		
		int size = 0;
		for(FlowNode node : nodes){
			Point2D p = getLocation(node);
			x += p.getX();
			y += p.getY();
			size++;
		}
		
		x /= size;
		y /= size;
		
		return new Point2D.Float((float)x, (float)y);
	}
	
	
	@Override
	public FlowGraph getGraph() {
		return (FlowGraph)super.getGraph();
	}

	@Override
	public void advancePositions() {
	}
	/*
	@Override
	protected void initialize_local() {
		updateCurrentSize();
	}*/

	@Override
	protected void initialize_local_vertex(Vertex v) {
		
	}

	public boolean incrementsAreDone() {
		return true;
	}

	public boolean isIncremental() {
		return false;
	}
	
	@Override
	public void restart() {
		//do nothing
	}
	
	public void initLocation(Vertex v){
        Coordinates coord = getCoordinates(v);
        if (coord == null) {
        	coord = new Coordinates();
            v.addUserDatum(getBaseKey(), coord, UserData.REMOVE);
        }
	}

	public void update() {
        try {
            for (Iterator iter = getGraph().getVertices().iterator(); iter
            .hasNext();) {
                Vertex v = (Vertex) iter.next();
                Coordinates coord = getCoordinates(v);
                if (coord == null) {
                    coord = new Coordinates();
                    v.addUserDatum(getBaseKey(), coord, UserData.REMOVE);
                    initializeLocation(v, coord, getCurrentSize());
                    initialize_local_vertex(v);
                }
                
            } 
        } catch(ConcurrentModificationException cme) {
            update();
        }
        initialize_local();
	}
	
	/**
	 * 
	 * @param picked
	 * @param x
	 * @param y
	 * @return whether node moved or not; true if moved
	 */
	public boolean forceMoveOnGrid(Vertex picked, double x, double y) {
		Point p = getPointOnGrid(x, y);
		Coordinates coord = getCoordinates(picked);
		if(coord.x == (float)p.x && coord.y == (float)p.y)
			return false;
		super.forceMove(picked, p.x, p.y);
		return true;
	}
	
	public Point getPointOnGrid(double x, double y){
        int mx = (int)x % gridDist;
        int my = (int)y % gridDist;
		x = (mx < gridDist/2)?x - mx:x - mx + gridDist;
		y = (my < gridDist/2)?y - my:y - my + gridDist;
		return new Point((int)x, (int)y);
	}
		
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public Rectangle getLayoutSize(){
		Set<FlowNode> nodes = getGraph().getVertices();
		if(nodes.size() > 0){
			Rectangle rect = getVertexRect(nodes);
			return rect;
		}else
			return new Rectangle();
	}
	
	public Rectangle getVertexRect(Iterable<FlowNode> nodes){
		if(!nodes.iterator().hasNext())
			return new Rectangle();
		
		double minX = Double.MAX_VALUE, minY = Double.MAX_VALUE;
		double maxX = 0, maxY = 0;
		for(FlowNode node : nodes){
			Point2D p = getLocation(node);
			if(p.getX() < minX)minX = p.getX();
			if(p.getY() < minY)minY = p.getY();
			if(p.getX() > maxX)maxX = p.getX();
			if(p.getY() > maxY)maxY = p.getY();
		}
		
		return new Rectangle((int)minX, (int)minY, (int)maxX-(int)minX, (int)maxY-(int)minY);
	}
	
	public int getGridDistance() {
		return gridDist;
	}
	
	public void setGridDistance(int gridDist) {
		this.gridDist = gridDist;
	}
}
