package com.ef_prime.rflow.flow.rendering;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;

import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.flow.FlowError;
import com.ef_prime.rflow.flow.FlowState;
import com.ef_prime.rflow.flow.model.FlowNode;

import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.utils.Pair;
import edu.uci.ics.jung.visualization.PluggableRenderer;

public class FlowRenderer extends PluggableRenderer{
	
	private FlowState rootState;
	
	private NodeRendering nodeRendering;
	private EdgeRendering edgeRendering;
	
	private Map<Edge, Shape> edgeShapes = new HashMap<Edge, Shape>();  
	
	private Set<Edge> arrowOnlyEdges = new HashSet<Edge>();
	
	/*private static final ImageIcon RED;
	static{
		ImageIcon icone = (ImageIcon)UIManager.getIcon("OptionPane.errorIcon");
		if(icone != null)
			RED = new ImageIcon(icone.getImage().getScaledInstance(16, 16, Image.SCALE_SMOOTH));
		else
			RED = null;
	}*/
	
	public FlowRenderer(FlowState state){
		super();
		
		this.rootState = state;
		
		nodeRendering = new NodeRendering(state);
		setVertexShapeFunction(nodeRendering);
		setVertexStrokeFunction(nodeRendering);
		setVertexPaintFunction(nodeRendering);
		setVertexStringer(nodeRendering);
		setVertexFontFunction(nodeRendering);
		setVertexIconFunction(nodeRendering);
		
		edgeRendering = new EdgeRendering(state);
		setEdgePaintFunction(edgeRendering);
		setEdgeShapeFunction(edgeRendering);
		setEdgeStrokeFunction(edgeRendering);
		
		setGraphLabelRenderer(new FlowGraphLabelRenderer(rootState));
	}
	
	public void clearArrowOnlyEdges(){
		arrowOnlyEdges.clear();
	}
	
	public Set<Edge> getArrowOnlyEdges() {
		return arrowOnlyEdges;
	}
	
	public Map<Edge, Shape> getEdgeShapes() {
		return edgeShapes;
	}
	
	public NodeRendering getNodeRendering() {
		return nodeRendering;
	}
	
	public EdgeRendering getEdgeRendering() {
		return edgeRendering;
	}
	
	@Override
	protected void drawSimpleEdge(Graphics2D g, Edge e, int x1, int y1, int x2, int y2) {

        Pair endpoints = e.getEndpoints();
        Vertex v1 = (Vertex)endpoints.getFirst();
        Vertex v2 = (Vertex)endpoints.getSecond();
        boolean isLoop = v1.equals(v2);
        Shape s2 = vertexShapeFunction.getShape(v2);
        Shape edgeShape = edgeShapeFunction.getShape(e);
        
        boolean edgeHit = true;
        boolean arrowHit = true;
        Rectangle deviceRectangle = null;
        if(screenDevice != null) {
            Dimension d = screenDevice.getSize();
            if(d.width <= 0 || d.height <= 0) {
                d = screenDevice.getPreferredSize();
            }
            deviceRectangle = new Rectangle(0,0,d.width,d.height);
        }

        AffineTransform xform = AffineTransform.getTranslateInstance(x1, y1);
        
        if(isLoop) {
            // this is a self-loop. scale it is larger than the vertex
            // it decorates and translate it so that its nadir is
            // at the center of the vertex.
            Rectangle2D s2Bounds = s2.getBounds2D();
            xform.scale(s2Bounds.getWidth(),s2Bounds.getHeight());
            xform.translate(0, -edgeShape.getBounds2D().getWidth()/2);
        } else {
            // this is a normal edge. Rotate it to the angle between
            // vertex endpoints, then scale it to the distance between
            // the vertices
            float dx = x2-x1;
            float dy = y2-y1;
            float thetaRadians = (float) Math.atan2(dy, dx);
            xform.rotate(thetaRadians);
            float dist = (float) Math.sqrt(dx*dx + dy*dy);
            xform.scale(dist, 1.0);
        }
        
        edgeShape = xform.createTransformedShape(edgeShape);
        
        edgeHit = viewTransformer.transform(edgeShape).intersects(deviceRectangle);

        if(edgeHit == true) {
            
            Paint oldPaint = g.getPaint();
            
            //source edge point
            Shape srcVertexShape = nodeRendering.getGlassShape(v1);
            AffineTransform srcXf = AffineTransform.getTranslateInstance(x1, y1);
            srcVertexShape = srcXf.createTransformedShape(srcVertexShape);

            AffineTransform srcAt;
            if (edgeShape instanceof GeneralPath)
            	srcAt = getReverseArrowTransform((GeneralPath)edgeShape, srcVertexShape, !isLoop);
            else
            	srcAt = getReverseArrowTransform(new GeneralPath(edgeShape), srcVertexShape, !isLoop);
            if(srcAt == null)return;
            if(srcAt.isIdentity()) {
            	arrowOnlyEdges.add(e);
            	edgeShapes.put(e, null);
            	return;
            }

            int xx1 = (int)(srcAt.getTranslateX());
            int yy1 = (int)(srcAt.getTranslateY());
            
            
            //dest edge point
            Shape destVertexShape = nodeRendering.getGlassShape(v2);
            AffineTransform xf = AffineTransform.getTranslateInstance(x2, y2);
            destVertexShape = xf.createTransformedShape(destVertexShape);

            AffineTransform at;
            if (edgeShape instanceof GeneralPath)
                at = getArrowTransform((GeneralPath)edgeShape, destVertexShape);
            else
                at = getArrowTransform(new GeneralPath(edgeShape), destVertexShape);
            if(at == null)return;
            
            int xx2 = (int)(at.getTranslateX());
            int yy2 = (int)(at.getTranslateY());
            
            //don't paint if direction is reverse
            int dx = x2-x1;
            int dy = y2-y1;
            int dxx = xx2-xx1;
            int dyy = yy2-yy1;
            if(((dx > 0 ^ dxx > 0)&&(dx != 0 && dxx != 0)) 
            		|| ((dy > 0 ^ dyy > 0)&& (dy != 0 && dyy != 0))){
            	arrowOnlyEdges.add(e);
            	edgeShapes.put(e, null);
            	return;
            }
            
            AffineTransform spacedTrans = AffineTransform.getTranslateInstance(xx1, yy1);
            float thetaRadians = (float) Math.atan2(dyy, dxx);
            spacedTrans.rotate(thetaRadians);
            float dist = (float) Math.sqrt(dxx*dxx + dyy*dyy);
            spacedTrans.scale(dist, 1.0);
            edgeShape = spacedTrans.createTransformedShape(edgeShapeFunction.getShape(e));
            
            edgeShapes.put(e, edgeShape);
            
            // get Paints for filling and drawing
            // (filling is done first so that drawing and label use same Paint)
            Paint fill_paint = edgePaintFunction.getFillPaint(e); 
            if (fill_paint != null)
            {
                g.setPaint(fill_paint);
                g.fill(edgeShape);
            }
            Paint draw_paint = edgePaintFunction.getDrawPaint(e);
            if (draw_paint != null)
            {
                g.setPaint(draw_paint);
                g.draw(edgeShape);
            }
            
            float scalex = (float)g.getTransform().getScaleX();
            float scaley = (float)g.getTransform().getScaleY();
            // see if arrows are too small to bother drawing
            if(scalex < .3 || scaley < .3) return;
            
            if (edgeArrowPredicate.evaluate(e)) {
                /*
                Shape destVertexShape = 
                    nodeRendering.getGlassShape((Vertex)e.getEndpoints().getSecond());
                AffineTransform xf = AffineTransform.getTranslateInstance(x2, y2);
                destVertexShape = xf.createTransformedShape(destVertexShape);
                */
                
                arrowHit = viewTransformer.transform(destVertexShape).intersects(deviceRectangle);
                if(arrowHit) {
                    /*
                    AffineTransform at;
                    if (edgeShape instanceof GeneralPath)
                        at = getArrowTransform((GeneralPath)edgeShape, destVertexShape);
                    else
                        at = getArrowTransform(new GeneralPath(edgeShape), destVertexShape);
                    if(at == null) return;
                    */
                    Shape arrow = edgeArrowFunction.getArrow(e);
                    arrow = at.createTransformedShape(arrow);
                    // note that arrows implicitly use the edge's draw paint
                    g.fill(arrow);
                }
                
                /*flow has direction
                if (e instanceof UndirectedEdge) {
                }
                */
            }
            // use existing paint for text if no draw paint specified
            if (draw_paint == null)
                g.setPaint(oldPaint);
            String label = edgeStringer.getLabel(e);
            if (label != null) {
                labelEdge(g, e, label, x1, x2, y1, y2);
            }
            
            
            // restore old paint
            g.setPaint(oldPaint);
        }
	}
	
	public void paintVertexSpaces(Graphics2D g, Vertex v, int x, int y){

        paintVertexLabelSpace(g, v, x, y);
	}

	private void paintVertexLabelSpace(Graphics2D g, Vertex v, int x, int y) {
		/*
	     * ラベル
	     */
		
		String label = vertexStringer.getLabel(v);
		if (label != null) {
			Component component = prepareRenderer(graphLabelRenderer, label, getPickedKey().isPicked(v), v);
			Dimension d = component.getPreferredSize();
	
	        int h_offset;
	        int v_offset;
	        if (centerVertexLabel)
	        {
	            h_offset = -d.width / 2;
	            v_offset = -d.height / 2;
	        }
	        else
	        {
	            Rectangle2D bounds = vertexShapeFunction.getShape(v).getBounds2D();
	            h_offset = -(int)d.getWidth()/2;
	            v_offset = (int)(bounds.getHeight() / 2);
	        }
	        
	        g.setColor(new Color(1.0f, 1.0f, 1.0f, 0.5f));
	        g.fillRect(x+h_offset, y+v_offset, d.width, d.height);
		}
	}
	
	@Override
    protected void labelVertex(Graphics g, Vertex v, String label, int x, int y)
    {
        Component component = prepareRenderer(graphLabelRenderer, label, getPickedKey().isPicked(v), v);

        Dimension d = component.getPreferredSize();
        
        int h_offset;
        int v_offset;
        if (centerVertexLabel)
        {
            h_offset = -d.width / 2;
            v_offset = -d.height / 2;
        }
        else
        {
            Rectangle2D bounds = nodeRendering.getGlassShape(v).getBounds2D();
            h_offset = -(int)d.getWidth()/2;
            v_offset = (int)((bounds.getHeight()-10-10) / 2);
        }
        
        rendererPane.paintComponent(g, component, screenDevice, x+h_offset, y+v_offset,
                d.width, d.height, true);
        
        //キャッシュ
        if(((FlowNode)v).isCacheData()){
        	Rectangle rect = vertexShapeFunction.getShape(v).getBounds();
        	
        	ImageIcon cacheIcon = (((FlowNode)v).getCacheData().isValid())?
        			//new ColorMaskedIcon(NodeRendering.FLAG_IMAGE_INVALID, new Color(255, 102, 0), 0.6f)
        			NodeRendering.FLAG_IMAGE_VALID
        			:NodeRendering.FLAG_IMAGE_INVALID;
        	
        	cacheIcon.paintIcon(screenDevice, g, 
        			x+rect.width/2-5, 
        			y-rect.height/2-cacheIcon.getIconHeight()+5);
        }
        
        //error icon
        Set<FlowError> errors = rootState.getError((FlowNode)v);
		if(errors != null)
			paintError(g, v, x, y);
        
        //ロックオン
        if(v.equals(rootState.getFlagNode())){
        	paintLockOn(g, v, x, y, Color.BLACK);
        }        
        //weak lock on
        else if(rootState.getFlagBoxSet().contains(v)){
        	paintLockOn(g, v, x, y, Color.LIGHT_GRAY);
        }
        
        //executing node
        if(rootState.isExecutingNode((FlowNode)v)){
        	paintLockOn(g, v, x, y, Color.ORANGE);
        }
    }

	private void paintError(Graphics g, Vertex v, int x, int y) {
		Rectangle rect = vertexShapeFunction.getShape(v).getBounds();
		//if(RED != null)
		ImageIcon icon = FileResource.ERROR;
		icon.paintIcon(screenDevice, g, 
				x+(rect.width/2)-(icon.getIconWidth()/2), 
				y+(rect.height/2)-(icon.getIconHeight())-5);
	}
	
	private static final Stroke LOCK_ON_STROKE = new BasicStroke(3.0f);
	private void paintLockOn(Graphics g, Vertex v, int x, int y, Paint paint) {
		Rectangle rect = nodeRendering.getGlassShape(v).getBounds();
		boolean isW = rect.width > rect.height;
		int len = Math.abs(rect.width - rect.height)/2;
		rect.grow((isW)?0:len, (isW)?len:0);
		rect.grow(5, 5);
		rect.translate(x, y);
		
		Graphics2D g2d = (Graphics2D)g;
		Paint old_paint = g2d.getPaint();
		Stroke old_stroke = g2d.getStroke();
		g2d.setPaint(paint);
		g2d.setStroke(LOCK_ON_STROKE);
		
		paintLockOnPart(g2d, (int)rect.getMinX(), (int)rect.getMinY(), true, true);
		paintLockOnPart(g2d, (int)rect.getMinX(), (int)rect.getMaxY(), true, false);
		paintLockOnPart(g2d, (int)rect.getMaxX(), (int)rect.getMaxY(), false, false);
		paintLockOnPart(g2d, (int)rect.getMaxX(), (int)rect.getMinY(), false, true);
		
		
		g2d.setPaint(old_paint);
		g2d.setStroke(old_stroke);
	}
	
	private void paintLockOnPart(Graphics2D g, int x1, int y1, boolean h, boolean v){
		int hOffset = 5;
		int vOffset = 5;
		if(!h)hOffset *= -1;
		if(!v)vOffset *= -1;
		
		GeneralPath path = new GeneralPath();
		path.moveTo(x1+hOffset, y1);
		path.lineTo(x1, y1);
		path.lineTo(x1, y1+vOffset);
		
		g.draw(path);
	}
	
	public void paintArrowOnlyEdge(Graphics2D g, Edge e, int x1, int y1, int x2, int y2){

        float dx = x2-x1;
        float dy = y2-y1;
        float thetaRadians = (float) Math.atan2(dy, dx);
        AffineTransform xform = AffineTransform.getTranslateInstance(x1+dx/2, y1+dy/2);
        xform.rotate(thetaRadians);
        

        Shape arrow = edgeArrowFunction.getArrow(e);
        arrow = xform.createTransformedShape(arrow);

        Paint oldPaint = g.getPaint();
        
        Paint draw_paint = edgePaintFunction.getDrawPaint(e);
        g.setPaint(draw_paint);
        g.fill(arrow);
        
        g.setPaint(oldPaint);
        
        
	}
	
	@Override
	protected void subdivide(Line2D src, Line2D left, Line2D right) {
        double x1 = src.getX1();
        double y1 = src.getY1();
        double x2 = src.getX2();
        double y2 = src.getY2();
        
        double mx = x1 + (x2-x1)/2.0;
        double my = y1 + (y2-y1)/2.0;
        if(Math.abs(mx-x1) < 1E-20 && Math.abs(my-y1) < 1E-20){
            if (left != null) {
                left.setLine(x1, y1, x2, y2);
            }
            if (right != null) {
                right.setLine(x2, y2, x2, y2);
            }
        }else{
        	if (left != null) {
        		left.setLine(x1, y1, mx, my);
        	}
        	if (right != null) {
        		right.setLine(mx, my, x2, y2);
        	}
        }
	}
}
