package com.ef_prime.rflow.flow.rendering;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.flow.FlowState;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.node.base.TunnelNodeModel;
import com.ef_prime.rflow.util.ColorMaskedIcon;

import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.decorators.VertexFontFunction;
import edu.uci.ics.jung.graph.decorators.VertexIconFunction;
import edu.uci.ics.jung.graph.decorators.VertexPaintFunction;
import edu.uci.ics.jung.graph.decorators.VertexShapeFunction;
import edu.uci.ics.jung.graph.decorators.VertexStringer;
import edu.uci.ics.jung.graph.decorators.VertexStrokeFunction;

public class NodeRendering implements VertexShapeFunction, VertexPaintFunction, 
	VertexStrokeFunction, VertexStringer, VertexFontFunction, VertexIconFunction
{
	public static final ImageIcon FLAG_IMAGE_INVALID = FileResource.FLAG_IMAGE;
	public static final ImageIcon FLAG_IMAGE_VALID = new ColorMaskedIcon(FLAG_IMAGE_INVALID, Color.RED, 0.5f);
	//new Color(255, 102, 0)
	
	public static final Color FILLPAINT = new Color(0,120,120);
	//private static final Color FILLPAINT_LIGHT = new Color(0,120,120, 150);
	public static final Color PICKED_FILLPAINT = new Color(100,200,100);//new Color(0,200,185);
	public static final Color PICKED_FILLPAINT_LIGHT = new Color(0,180,160);
	
	public static final Color TRANSPARENT = new Color(0,0,0,0);
//	private static final Color PICKED_ICON_COLOR = new Color(235, 242, 254);
	//private static final Color PICKED_ICON_COLOR = new Color(127, 187, 110);
	public static final Stroke DEFAULT_STROKE = new BasicStroke(1);
	public static final Stroke BOLDSTROKE = new BasicStroke(2.5f);
	//private static final Stroke BOLDSTROKE2 = new BasicStroke(3.0f);
	
	
	private FlowState state;
	
	public NodeRendering(FlowState state){
		this.state = state;
	}
	
	public Shape getShape(Vertex v) {
		FlowNode node = (FlowNode)v;
		AppearanceGroup group = node.getProperty().getAppearanceGroup();
		if(group == null)
			return node.getModel().getShape();
		else
			return group.getShape();
	}
	
	public Shape getGlassShape(Vertex v){
		FlowNode node = (FlowNode)v;
		AppearanceGroup group = node.getProperty().getAppearanceGroup();
		if(group == null)
			return node.getModel().getGlassShape();
		else
			return group.getGlassShape();
	}
	
	public Icon getIcon(ArchetypeVertex v) {
		return colorIcon(((FlowNode)v).getIcon(), v);
	}
	
	public Icon getSmallIcon(Vertex v){
		return colorIcon(((FlowNode)v).getSmallIcon(), v);
	}
	
	private Icon colorIcon(Icon icon, ArchetypeVertex v){
		if(icon == null)
			return null;
		if(state.isPicked(v)){
			return new ColorMaskedIcon(icon, Color.YELLOW, 0.3f);
		}
		else if(state.isHighlightNode((FlowNode)v)){
			return new ColorMaskedIcon(icon, Color.BLUE, 0.3f);
			//return new ColorMaskedIcon(icon, Color.RED, 0.3f);
		}
		//
//		else if(state.isFlowSelected() && !state.getSelectionNodeFlow().contains(v)){
//			return new ColorMaskedIcon(icon, Color.WHITE, 0.5f);
//		}
		else{
			return icon;
		}
	}

	public Paint getDrawPaint(Vertex v) {
		if(((FlowNode)v).getModel() instanceof TunnelNodeModel){
			if(state.isPicked(v))
				return PICKED_FILLPAINT;
			else
				return FILLPAINT;
		}
		
		/*if(pickedState.isFlagNode((FlowNode)v)){
			return Color.BLACK;
		}else*/ 
		/*if(pickedState.isPinNode((FlowNode)v)){
			return Color.LIGHT_GRAY;
		}*/
		return TRANSPARENT;
	}
	
	public Stroke getStroke(Vertex v) {
		if(((FlowNode)v).getModel() instanceof TunnelNodeModel){
			return BOLDSTROKE;
		}
		/*if(pickedState.isFlagNode((FlowNode)v)){
			return BOLDSTROKE;
		}else*/ 
		/*if(pickedState.isPinNode((FlowNode)v)){
			return BOLDSTROKE2;
		}*/
		return DEFAULT_STROKE;
	}

	public Paint getFillPaint(Vertex v) {
		if(((FlowNode)v).getModel() instanceof TunnelNodeModel){
			return TRANSPARENT;
		}
		
		if(state.isPicked(v)){
			return PICKED_FILLPAINT;
		}
		else if(state.isHighlightNode((FlowNode)v)){
			return PICKED_FILLPAINT_LIGHT;
		}
		//
//		else if(state.isFlowSelected() && !state.getSelectionNodeFlow().contains(v)){
//			return FILLPAINT_LIGHT;
//		}
		else{
			return FILLPAINT;
		}
	}

	public String getLabel(ArchetypeVertex v) {
		//return ((FlowNode)v).getModel().getTitle();
		return ((FlowNode)v).getTitle();
	}
	
	
	public Font getFont(Vertex v) {
		return Prefs.CODE_FONT;
		//return new Font("Courier New", Font.PLAIN, 12);
	}
}
