package com.ef_prime.rflow.flow.rendering;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.ef_prime.rflow.resource.Messages;

public enum NodeShape{

	CIRCLE(Messages.getString("NodeShape.circle")), //$NON-NLS-1$
	TRIANGLE(Messages.getString("NodeShape.triangle")), //$NON-NLS-1$
	RECTANGLE(Messages.getString("NodeShape.rectangle")), //$NON-NLS-1$
	STAR(Messages.getString("NodeShape.star")); //$NON-NLS-1$
	
	private String name;
	private Shape shape;
	private Icon smallIcon;
	
	private NodeShape(String name){
		this.name = name;
	}

	public Shape getShape() {
		return shape;
	}
	
	public Icon getSmallIcon() {
		if(smallIcon == null){
			Shape s = getShape();
			Rectangle rect = s.getBounds();
			BufferedImage image = new BufferedImage(rect.width, rect.height, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = (Graphics2D)image.getGraphics();
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			s = AffineTransform.getTranslateInstance(-rect.x, -rect.y).createTransformedShape(s);
			g.setPaint(NodeRendering.FILLPAINT);
			g.fill(s);
			
			smallIcon = new ImageIcon(image.getScaledInstance(16, 16, Image.SCALE_AREA_AVERAGING));
			
			image.flush();
		}
		return smallIcon;
	}
	
	public String getName() {
		return name;
	}
	
	public static final int DEFAULT_SIZE = 22;
	public static final int DEFAULT_GLASS_SIZE = 30;
	public static final int DEFAULT_SPACE = 5;
	private static final Shape GLASS_SHAPE;
	static{		
		CIRCLE.shape = new Ellipse2D.Double(-DEFAULT_SIZE/2,-DEFAULT_SIZE/2,DEFAULT_SIZE,DEFAULT_SIZE);
		TRIANGLE.shape = getTriangle(DEFAULT_SIZE);
		RECTANGLE.shape = new Rectangle(-DEFAULT_SIZE/2,-DEFAULT_SIZE/2,DEFAULT_SIZE,DEFAULT_SIZE);
		STAR.shape = getRegularStar(5, DEFAULT_SIZE);
		
		/*
		GLASS_SHAPE = new Rectangle(
				-DEFAULT_SIZE/2-DEFAULT_SPACE,
				-DEFAULT_SIZE/2-DEFAULT_SPACE,
				DEFAULT_SIZE+DEFAULT_SPACE*2,
				DEFAULT_SIZE+DEFAULT_SPACE*2+10);
		*/
		GLASS_SHAPE = new Rectangle(
				-DEFAULT_GLASS_SIZE/2-DEFAULT_SPACE,
				-DEFAULT_GLASS_SIZE/2-DEFAULT_SPACE,
				DEFAULT_GLASS_SIZE+DEFAULT_SPACE*2,
				DEFAULT_GLASS_SIZE+DEFAULT_SPACE*2+10);
	}
	
	public static Shape getGlassShape() {
		return GLASS_SHAPE;
	}
	
	private static Shape getTriangle(int size){
		int num = 3;
		int w = size;
        double theta = (2 * Math.PI) / num; 
        double angle = 0;
		
		GeneralPath path = new GeneralPath();
		path.moveTo(0, 0);
        for (int i = 0; i < num; i++)
        {
            angle -= theta; 
            float delta_x = (float) (w * Math.cos(angle));
            float delta_y = (float) (w * Math.sin(angle));
            Point2D prev = path.getCurrentPoint();
            path.lineTo((float)prev.getX() + delta_x, (float)prev.getY() + delta_y);
        }
        path.closePath();
        
        double h = path.getBounds().getHeight()/2;
        Shape tri = AffineTransform.getTranslateInstance((double)w/2, h).createTransformedShape(path);
        return tri;
	}
	
	private static Shape getRegularStar(int num_points, int size)
    {
        if (num_points < 5)
            throw new IllegalArgumentException("Number of sides must be >= 5"); //$NON-NLS-1$
        GeneralPath thePolygon = new GeneralPath();
        float width = size;
        float height = size;
        
        // generate coordinates
        double theta = (2 * Math.PI) / num_points;
        double angle = -theta/2;
        thePolygon.reset();
        thePolygon.moveTo(0,0);
        float delta_x = width * (float)Math.cos(angle);
        float delta_y = width * (float)Math.sin(angle);
        Point2D prev = thePolygon.getCurrentPoint();
        thePolygon.lineTo((float)prev.getX() + delta_x, (float)prev.getY() + delta_y);
        for (int i = 1; i < num_points; i++)
        {
            angle += theta; 
            delta_x = width * (float)Math.cos(angle);
            delta_y = width * (float)Math.sin(angle);
            prev = thePolygon.getCurrentPoint();
            thePolygon.lineTo((float)prev.getX() + delta_x, (float)prev.getY() + delta_y);
            angle -= theta*2; 
            delta_x = width * (float)Math.cos(angle);
            delta_y = width * (float)Math.sin(angle);
            prev = thePolygon.getCurrentPoint();
            thePolygon.lineTo((float)prev.getX() + delta_x, (float)prev.getY() + delta_y);
        }
        thePolygon.closePath();
        
        // scale polygon to be right size, translate to center at (0,0)
        Rectangle2D r = thePolygon.getBounds2D();
        double scale_x = width / r.getWidth();
        double scale_y = height / r.getHeight();

        float translationX = (float) (r.getMinX() + r.getWidth()/2);
        float translationY = (float) (r.getMinY() + r.getHeight()/2);
        
        AffineTransform at = AffineTransform.getScaleInstance(scale_x, scale_y);
        at.translate(-translationX, -translationY);

        Shape shape = at.createTransformedShape(thePolygon);
        return shape;
    }

}
