package com.ef_prime.rflow.group;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import com.ef_prime.rflow.flow.rendering.NodeShape;

public class AppearanceGroupManager {
	
	private final Map<String, AppearanceGroup> group_map = new LinkedHashMap<String, AppearanceGroup>();
	private final Map<String, AppearanceGroup> functionToGroupMap = new HashMap<String, AppearanceGroup>();
	//private AppearanceGroup[] availableGroups;
	
	public AppearanceGroup addFunctionGroup(String title, String iconPath){
		AppearanceGroup group = new IconAppearanceGroup(title, iconPath);
		group_map.put(title, group);
		return group;
	}
	
	public AppearanceGroup addFunctionGroup(String title, NodeShape nodeShape){
		AppearanceGroup group = new ShapeAppearanceGroup(title, nodeShape);
		group_map.put(title, group);
		return group;
	}
	
	public Collection<AppearanceGroup> getGroups(){
		return group_map.values();
	}
	
	public AppearanceGroup getAppearanceGroupFromGroupTitle(String groupTitle){
		/*for(FunctionGroup group : group_map.values()){
			if(group.getTitle().equals(groupTitle)){
				return group;
			}
		}
		return null;*/
		return group_map.get(groupTitle);
	}
	
	public AppearanceGroup addFunction(String function, AppearanceGroup group, boolean replace){
		if(replace){
			return functionToGroupMap.put(function, group);
		}else{
			AppearanceGroup previous = functionToGroupMap.get(function);
			if(previous != null)
				return previous;
			functionToGroupMap.put(function, group);
			return null;
		}
	}
	
	public AppearanceGroup removeFunction(String function){
		return functionToGroupMap.remove(function);
	}
	
	public AppearanceGroup getAppearanceGroup(String function){
		return functionToGroupMap.get(function);
	}
	
	public boolean hasFunction(String function){
		return functionToGroupMap.containsKey(function);
	}
	
	/**
	 * clear all groups and functions
	 *
	 */
	public void clear(){
		clearFunctions();
		clearGroups();
	}
	
	/**
	 * clear registered functions
	 *
	 */
	public void clearFunctions(){
		functionToGroupMap.clear();
	}
	
	public void clearGroups(){
		group_map.clear();
	}
	
	/*
	public AppearanceGroup[] getAvailableGroups() {
		if(availableGroups == null)
			availableGroups = createAvailableGroups();
		return availableGroups;
	}
	
	private AppearanceGroup[] createAvailableGroups(){
		LinkedHashSet<AppearanceGroup> set = new LinkedHashSet<AppearanceGroup>();
		for(NodeShape nodeShape : NodeShape.values()){
			set.add(new ShapeAppearanceGroup(nodeShape.getName(), nodeShape));
		}
		for(AppearanceGroup group : Rflow.GROUP_MANAGER.getGroups()){
			if(group instanceof IconAppearanceGroup)
				set.add(group);
		}
		
		return set.toArray(new AppearanceGroup[set.size()]);
	}*/
}
