package com.ef_prime.rflow.io;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.ext.DefaultHandler2;

import com.ef_prime.rflow.exception.ReadException;
import com.ef_prime.rflow.explorer.PartsTreeElements;

public class FlowDataReader implements PartsTreeElements{

	public void read(File file, ElementReader[] readerManagers) throws ReadException, IOException{
		read(new InputSource(file.toURI().toASCIIString()), readerManagers);
	}
	
	public void read(String data, ElementReader[] readerManagers) throws ReadException, IOException{
		read(new InputSource(new StringReader(data)), readerManagers);
	}
	
	private void read(InputSource inputSource, ElementReader[] readerManagers) throws ReadException, IOException{
		try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = factory.newSAXParser();
            
			saxParser.parse(inputSource, new Handler(readerManagers));
		} catch (SAXException e) {
			e.printStackTrace();
			throw new ReadException();
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new ReadException();
		}
		
	}
	
	private static class Handler extends DefaultHandler2{
		
		private ElementReader readerManager = new ElementReader();
		private StringBuffer sb;
		
		Handler(ElementReader[] readerManagers){
			for(ElementReader manager : readerManagers){
				readerManager.addElementReader(manager);
			}
		}

		public void startElement(
	            String namespaceURI,
	            String lName,
	            String qName,
	            Attributes attrs) throws SAXException 
	    {
			sb = new StringBuffer();
			
			SingleElementReader reader = readerManager.getSingleElementReader(qName);
			if(reader != null)
				reader.startElement(attrs);
	    }
		

		public void endElement(
	            String namespaceURI,
	            String lName,
	            String qName) throws SAXException 
	    {	
			SingleElementReader reader = readerManager.getSingleElementReader(qName);
			if(reader != null)
				reader.endElement(sb.toString());
	    }
		
		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
			sb.append(new String(ch, start, length));
		}
	}
	
	

	

	
}
