package com.ef_prime.rflow.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter;

import com.ef_prime.rflow.explorer.PartsTreeElements;
import com.ef_prime.rflow.xml.DefaultXMLWriter;
import com.ef_prime.rflow.xml.XMLException;
import com.ef_prime.rflow.xml.XMLWriter;
import com.ef_prime.rflow.xml.XMLWriterFactory;

public class FlowDataWriter implements PartsTreeElements{

	
	public void write(File file, ElementWriter[] elementWriters) throws IOException, XMLException{
		write(XMLWriterFactory.createUTF8XMLWriter(new FileOutputStream(file)),
				elementWriters, true);
	}
	
	public String writeToString(ElementWriter[] elementWriters) throws XMLException{
		StringWriter stringWriter = new StringWriter();
		write(new DefaultXMLWriter(stringWriter), elementWriters, false);
		return stringWriter.getBuffer().toString();
	}

	public void write(XMLWriter out, ElementWriter[] elementWriters, boolean header) 
		throws XMLException{
		write(out, elementWriters, header, true);
	}
	
	public void write(XMLWriter out, ElementWriter[] elementWriters,
			boolean header, boolean close) throws XMLException 
	{
		try {
			writeInternal(out, elementWriters, header);
		} finally {
			if (close && out != null) {
				out.close();
			}
		}
	}
	
	private void writeInternal(XMLWriter out, ElementWriter[] elementWriters,
			boolean header) throws XMLException 
	{
		if(header){
			out.writeStartDocument("UTF-8", "1.0"); //$NON-NLS-1$ //$NON-NLS-2$
			out.writeDTD(getDTD());
			out.writeStartElement(RFLOW);
		}
		
		for(ElementWriter writer : elementWriters){
			writer.write(out);
		}
		
		if(header){
			out.writeEndElement();//rflow
			out.writeEndDocument();
		}
	}

	private String getDTD(){
		return "<!DOCTYPE "+RFLOW+" [\n"+ //$NON-NLS-1$ //$NON-NLS-2$
				"<!ENTITY lt \"&#38;#60;\">\n"+ //$NON-NLS-1$
				"<!ENTITY gt \"&#62;\">\n"+ //$NON-NLS-1$
				"<!ENTITY amp \"&#38;#38;\">\n"+ //$NON-NLS-1$
				"<!ENTITY apos \"&#39;\">\n"+ //$NON-NLS-1$
				"<!ENTITY quot \"&#34;\">\n"+ //$NON-NLS-1$
				"]>"; //$NON-NLS-1$
	}
}
