package com.ef_prime.rflow.node;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.exception.InvalidOptionException;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.base.SubflowNodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.ButtonPanel;


public abstract class DefaultNodeModelController extends AbstractNodeModelController{
	
	protected Rflow rflow;
	private Window parent;
	
	public NodeModel createNewModel() {
		return createDefaultModelInstance();
	}
	
	public void install(Rflow rflow){
		this.rflow = rflow;
		setParentWindow(rflow.frame);
	}
	
	public Window getParentWindow() {
		return parent;
	}
	
	public void setParentWindow(Window parent) {
		this.parent = parent;
	}
	
	/**
	 * 初期値が設定されたモデルの新しいインスタンスを返すメソッドです。
	 * 引数なしのコンストラクタを使ってインスタンスを作成します。
	 * @return
	 */
	public NodeModel createDefaultModelInstance(){
		try {
			return getModelClass().newInstance();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException("Failed to create default model Instance."); //$NON-NLS-1$
		}
	}
	
	public boolean isNodeOptionPanel(){
		return true;
	}

	protected NodeOptionPanelHandler createNodeOptionPanelHandler(){
		return new AbstractNodeOptionPanelHandler(){
			@Override
			protected NodeOptionPanel[] createNodeOptionPanels(NodeModel orginalModel) {
				NodeOptionPanel panel = createNodeOptionPanel(orginalModel);
				return new NodeOptionPanel[]{panel};
			}
		};
	}
	
	protected NodeOptionPanel createNodeOptionPanel(NodeModel model){
		final NodeModel model0 = model;
		return new NodeOptionPanel(){
			@Override
			public NodeModel getModel() throws InvalidOptionException {
				return model0;
			}
		};
	}
	
	protected final void requestOkButtonFocus(){
		if(ok != null && ok.isVisible())
			ok.requestFocus();
	}
	
	public Window createEditDialog(FlowViewer viewer, FlowNode node, boolean selectProperty, boolean isNewNode) {
		NodeOptionDialog dialog;
		Window window = getParentWindow();
		if (window instanceof Frame) 
			dialog= new NodeOptionDialog((Frame)window, viewer, node, selectProperty, isNewNode);
		else
			dialog= new NodeOptionDialog((Dialog)window, viewer, node, selectProperty, isNewNode);
		return dialog;
	}	
	
	private JButton ok;//OK Button only for requestFocus
	private class NodeOptionDialog extends JDialog{

		private NodeOptionPanelHandler panelHandler;
		private NodePropertyPanel propertyPanel;
		
		private boolean canceled = true;
		
		public NodeOptionDialog(Dialog parent, FlowViewer viewer, FlowNode node, boolean selectProperty, boolean isNewNode){
			super(parent, "", false); //$NON-NLS-1$
			init(viewer, node, selectProperty, isNewNode);
		}
		
		public NodeOptionDialog(Frame parent, FlowViewer viewer, FlowNode node, boolean selectProperty, boolean isNewNode){
			super(parent, "", false); //$NON-NLS-1$

			init(viewer, node, selectProperty, isNewNode);
		}
		
		private void setTitle(FlowNode node){
			setTitle(Messages.getString("DefaultNodeModelController.title") + " - " + node.getTitle()); //$NON-NLS-1$ //$NON-NLS-2$
			if(node.getModel().isSubflow())
				((SubflowNodeModel)node.getModel()).update();
		}

		private void init( 
				final FlowViewer viewer,
				final FlowNode node, 
				boolean selectProperty, 
				final boolean isNewNode) 
		{
			setTitle(node);
			setDefaultCloseOperation(NodeOptionDialog.DISPOSE_ON_CLOSE);
			

			//button panel
			ActionListener okAction = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					if(setModel(node)){
						canceled = false;
						NodeOptionDialog.this.dispose();
					}
				}
			};
			ActionListener applyAction = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					setModel(node);
					canceled = false;
				}
			};
			ActionListener cancelAction = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					NodeOptionDialog.this.dispose();
				}
			};
			ButtonPanel buttonPanel = new ButtonPanel(
					new ActionListener[]{okAction, cancelAction, applyAction},
					new String[]{Messages.getString("Button.ok"),  //$NON-NLS-1$
							Messages.getString("Button.cancel"), //$NON-NLS-1$
							Messages.getString("Button.apply")}, //$NON-NLS-1$
					Prefs.BUTTON_SIZE,	10);
			
			propertyPanel = new NodePropertyPanel(
					node.getProperty().copy(), node.getCacheData(), 
					rflow.viewer.getCacheHandler(), buttonPanel);
			propertyPanel.setPreferredSize(new Dimension(400, 400));
			
			JTabbedPane tabbedPane;
			if(isNodeOptionPanel()){
				panelHandler = createNodeOptionPanelHandler();
				tabbedPane = panelHandler.getNodeOptionTabbedPane(node.getModel().copy());
			}else{
				//usually subflow node has no nodeoption
				tabbedPane = new JTabbedPane();
			}
			tabbedPane.add(propertyPanel, Messages.getString("DefaultNodeModelController.property")); //$NON-NLS-1$

			
			if(selectProperty){
				tabbedPane.setSelectedComponent(propertyPanel);
			}
			
			addWindowListener(new WindowAdapter(){
				@Override
				public void windowClosed(WindowEvent e) {
					if(canceled && isNewNode){
						canceled = false;//just to avoid calling this method twice
						viewer.getUndoManager().undoNewNodeCreation(node);
					}
				}
			});
			
			Box btnBox = Box.createHorizontalBox();
			btnBox.add(Box.createHorizontalGlue());
			btnBox.add(buttonPanel);
			//btnBox.add(Box.createHorizontalGlue());
			btnBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 100));
			ok = buttonPanel.getFirstButton();
			
			Box box = Box.createVerticalBox();
			int sp = 10;
			box.setBorder(new EmptyBorder(sp,sp,sp,sp));
			box.add(tabbedPane);
			box.add(Box.createVerticalStrut(10));
			box.add(btnBox);
			
			getContentPane().setLayout(new BorderLayout());
			getContentPane().add(box);
			
			pack();
			setLocationRelativeTo(getOwner());

			//focus (do after component is added)
			if(panelHandler != null)
				panelHandler.initFocus();
		}
		
		private boolean setModel(FlowNode node){
			if(isNodeOptionPanel()){
				//model
				NodeModel model;
				try {
					model = NodeOptionDialog.this.panelHandler.getModel().copy();
				} catch (InvalidOptionException e1) {
					JOptionPane.showMessageDialog(getParentWindow(), e1.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
					return false;
				}
				node.setModel(model);
			}
			//property
			NodeProperty property = propertyPanel.getProperty();
			node.setProperty(property);
			
			setTitle(node);
			
			return true;
		}
		
	}
	
}
