package com.ef_prime.rflow.node;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import com.ef_prime.rflow.flow.CacheData;
import com.ef_prime.rflow.flow.CacheHandler;
import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.group.DefaultAppearanceGroup;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.ButtonPanel;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.util.SwingUtils;

public class NodePropertyPanel extends JPanel{
	
	private NodeProperty property;
	private List<ModelUpdater> modelUpdaters = new ArrayList<ModelUpdater>();
	private ButtonPanel buttonPanel;
	
	public NodePropertyPanel(
			final NodeProperty property, 
			CacheData cacheData, 
			CacheHandler cacheHandler,
			ButtonPanel buttonPanel) 
	{
		super();
		this.property = property;
		this.buttonPanel = buttonPanel;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		int space = 10;
		setBorder(new EmptyBorder(space,space,space,space));
		
		addPropertyPanel(new TitlePanel());
		addPropertyPanel(new AppearancePanel());
		addPropertyPanel(new CommentPanel());
		add(new CachePanel(cacheData, cacheHandler));
	}
	
	private void requestOKButtonFocus(){
		buttonPanel.getFirstButton().requestFocus();
	}
	
	private void addPropertyPanel(AbstractPropertyPanel propertyPanel){
		modelUpdaters.add(propertyPanel);
		propertyPanel.setModel(property);
		add(propertyPanel);
	}
	
	public NodeProperty getProperty(){
		NodeProperty property = NodeProperty.createDefaultProperty();
		for(ModelUpdater updater : modelUpdaters)
			updater.updateModel(property);
		return property;
	}
	
	private interface ModelUpdater{
		void setModel(NodeProperty property);
		void updateModel(NodeProperty property);
	}
	
	private abstract class AbstractPropertyPanel extends JPanel implements ModelUpdater{
	}
	
	private class TitlePanel extends AbstractPropertyPanel{

		private JRadioButton custom;
		private JTextField titleInput = new RflowTextField();
		
		public TitlePanel(){
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.title"))); //$NON-NLS-1$
			
			
			JRadioButton def = new JRadioButton(Messages.getString("NodePropertyPanel.default_title"), true); //$NON-NLS-1$
			custom = new JRadioButton(""); //$NON-NLS-1$
			custom.addItemListener(new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					titleInput.setEnabled(custom.isSelected());
				}
			});
			Box defBox = Box.createHorizontalBox();
			defBox.add(def);
			defBox.add(Box.createHorizontalGlue());
			
			ButtonGroup group = new ButtonGroup();
			group.add(def);
			group.add(custom);
			
			titleInput.setEnabled(false);
			titleInput.addKeyListener(new KeyAdapter(){
				@Override
				public void keyPressed(KeyEvent e) {
					if(e.getKeyCode() == KeyEvent.VK_ENTER){
						requestOKButtonFocus();
					}
				}
			});
			Box customBox = Box.createHorizontalBox();
			customBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
			customBox.add(custom);
			customBox.add(titleInput);
			
			add(defBox);
			add(customBox);
		}
		
		public void setModel(NodeProperty property) {
			String title = property.getTitle();
			boolean isTitle = (title != null);
			custom.setSelected(isTitle);
			if(isTitle)
				titleInput.setText(title);
		}

		public void updateModel(NodeProperty property) {
			if(custom.isSelected())
				property.setTitle(titleInput.getText());
			else
				property.setTitle(null);
		}
		
	}
	
	private class AppearancePanel extends AbstractPropertyPanel{
		
		private JComboBox comboBox;
		
		public AppearancePanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.node_shape"))); //$NON-NLS-1$
			
			Vector<AppearanceGroup> groupVector = new Vector<AppearanceGroup>();
			groupVector.add(null);
			groupVector.addAll(Arrays.asList(DefaultAppearanceGroup.getAppearanceGroups()));
			comboBox = new JComboBox(groupVector);
			comboBox.setRenderer(new DefaultListCellRenderer(){
				@Override
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
					super.getListCellRendererComponent(list, value, index, isSelected,
							cellHasFocus);
					
					
					AppearanceGroup group = (AppearanceGroup)value;
					if(group == null){
						setText(Messages.getString("NodePropertyPanel.default")); //$NON-NLS-1$
					}else{
						setIcon(group.getSmallIcon());
					}
					
					return this;
				}
			});
			comboBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 25));
			comboBox.addKeyListener(new KeyAdapter(){
				@Override
				public void keyPressed(KeyEvent e) {
					if(e.getKeyCode() == KeyEvent.VK_ENTER){
						requestOKButtonFocus();
					}
				}
			});
			add(comboBox);
		}
		
		public void setModel(NodeProperty property) {
			comboBox.setSelectedItem(property.getAppearanceGroup());
		}
		
		public void updateModel(NodeProperty property){
			property.setAppearanceGroup((AppearanceGroup)comboBox.getSelectedItem());
		}
	}
	
	private class CommentPanel extends AbstractPropertyPanel{
		
		private JTextArea textArea;
		
		public CommentPanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.comment"))); //$NON-NLS-1$
			
			textArea = new JTextArea();
			SwingUtils.setDefaultTextComponentOption(textArea);
			add(new JScrollPane(textArea));
		}
		
		public void setModel(NodeProperty property) {
			if(property.getComment() != null)
				textArea.setText(property.getComment());
		}
		
		public void updateModel(NodeProperty property){
			String comment = textArea.getText();
			if(comment != null && comment.length() < 1)
				comment = null;
			property.setComment(comment);
		}
	}
	
	private class CachePanel extends JPanel{
		public CachePanel(CacheData data, CacheHandler cacheHandler){
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setBorder(new TitledBorder(Messages.getString("NodePropertyPanel.cache"))); //$NON-NLS-1$
			
			Dimension size = new Dimension(80, 20);
			
			boolean isCache = (data != null && data.isValid());
			
			//file
			JLabel fileLabel = new JLabel(Messages.getString("NodePropertyPanel.file")); //$NON-NLS-1$
			//fileLabel.setHorizontalTextPosition(JLabel.RIGHT);
			fileLabel.setPreferredSize(size);
			fileLabel.setMaximumSize(size);
			Box fileBox = Box.createHorizontalBox();
			fileBox.add(fileLabel);
			if(isCache){
				JLabel file = new JLabel(data.getFile().getName());
				fileBox.add(file);
			}
			fileBox.add(Box.createHorizontalGlue());
			
			//location
			JLabel locLabel = new JLabel(Messages.getString("NodePropertyPanel.address")); //$NON-NLS-1$
			//locLabel.setHorizontalTextPosition(JLabel.RIGHT);
			locLabel.setPreferredSize(size);
			locLabel.setMaximumSize(size);
			locLabel.setMinimumSize(size);
			Box locBox = Box.createHorizontalBox();
			locBox.add(locLabel);
			if(isCache){
					//File file = new File(cacheHandler.getCacheDir(), data.getFile() + Constants.RDATA_EXTENSION);
					JLabel loc = new JLabel(data.getFile().getPath());
					loc.setToolTipText(loc.getText());
					locBox.add(loc);
			}
			locBox.add(Box.createHorizontalGlue());
			
			//date
			JLabel dateLabel = new JLabel(Messages.getString("NodePropertyPanel.created_datetime")); //$NON-NLS-1$
			//dateLabel.setHorizontalTextPosition(JLabel.RIGHT);
			dateLabel.setPreferredSize(size);
			dateLabel.setMaximumSize(size);
			Box dateBox = Box.createHorizontalBox();
			dateBox.add(dateLabel);
			if(isCache && data.isDate()){
				JLabel date = new JLabel(DateFormat.getInstance().format(data.getDate().getTime()));
				dateBox.add(date);
			}
			dateBox.add(Box.createHorizontalGlue());
			
			Box mainBox = Box.createVerticalBox();
			mainBox.add(fileBox);
			mainBox.add(locBox);
			mainBox.add(dateBox);
			
			add(mainBox);
			add(Box.createHorizontalGlue());
		}
	}
}
