package com.ef_prime.rflow.node.base;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import com.ef_prime.rflow.RCommand;

/**
 * 
 * null element is prohibited.
 *
 */
public class ArgListMod extends Vector<ArgEntry>{
	
	private ArgListMod defaultList;
	
	public ArgListMod(){
		super();
	}
	
	public ArgListMod(ArgListMod defaultList){
		super();
		setDefaultList(defaultList);
	}
	
	public boolean isDefaultValue(String param){
		int index = getIndexOfParam(param);
		if(index < 0)
			throw new IllegalArgumentException("param doesn't exist"); //$NON-NLS-1$
		return isDefaultValue(index);
	}
	
	public boolean isDefaultValue(int index){
		if(!isDefaultList())return false;
		return getDefaultList().contains(elementData[index]);
	}
	
	public void setDefaultList(String function){
		ArgListMod argList = RCommand.argList(function);
		if(argList == null || argList.size() < 1){
			setDefaultList((ArgListMod)null);
			return;
		}
		setDefaultList(argList);
	}
	
	public void setDefaultList(ArgListMod defaultList) {
		if(defaultList == null){
			clearDefaultList();return;
		}
		this.defaultList = (ArgListMod)defaultList.clone();
	}
	
	public ArgListMod getDefaultList() {
		if(defaultList == null)
			defaultList = new ArgListMod();
		return defaultList;
	}
	
	public boolean isDefaultList(){
		return !getDefaultList().isEmpty();
	}
	
	public void clearDefaultList(){
		getDefaultList().clear();
	}

	@Override
	public synchronized int indexOf(Object elem, int index) {
		if(elem == null)throw new IllegalArgumentException("elem cannot be null"); //$NON-NLS-1$
		for (int i = index ; i < elementCount ; i++)
			if (hashFirstEquals(elem, elementData[i]))
				return i;
		return -1;
	}
	
	public synchronized int getIndexOfParam(String param){
		for(int i = 0; i < elementCount; i++){
			ArgEntry entry = (ArgEntry)elementData[i];
			if(hashFirstEquals(entry.getParam(), param)){
				return i;
			}
		}
		return -1;
	}
	
	public synchronized String getValue(String param, boolean searchDefault) {
		for(int i = 0; i < elementCount; i++){
			ArgEntry entry = (ArgEntry)elementData[i];
			if(hashFirstEquals(entry.getParam(), param)){
				return entry.getValue();
			}
		}
		if(searchDefault && isDefaultList()){
			return getDefaultList().getValue(param, searchDefault);
		}
		return null;
	}
	
	public synchronized Set<String> paramSet(){
		Set<String> set = new LinkedHashSet<String>();
		for(int i = 0; i < elementCount; i++){
			ArgEntry entry = (ArgEntry)elementData[i];
			set.add(entry.getParam());
		}
		return set;
	}
	
	
	public synchronized ArgListMod list(boolean defaultOrder, boolean withDefault){
		ArgListMod list = new ArgListMod();
		if(defaultOrder){
			Set<ArgEntry> rmSet = new HashSet<ArgEntry>();
			for(int i = 0; i < getDefaultList().elementCount; i++){
				ArgEntry entry = (ArgEntry)getDefaultList().elementData[i];
				int index = getIndexOfParam(entry.getParam());
				if(index < 0){
					if(withDefault)
						list.add(entry);
				}else{
					ArgEntry argEntry = get(index);
					rmSet.add(argEntry);
					if(withDefault || !isDefaultValue(index)){
						list.add(argEntry);
					}
				}
			}
			ArgListMod clone = (ArgListMod)this.clone();
			for(ArgEntry entry : rmSet)
				clone.remove(entry);
			for(int i = 0; i < clone.elementCount; i++){
				list.add((ArgEntry)clone.elementData[i]);
			}
		}else{
			Set<ArgEntry> rmSet = new HashSet<ArgEntry>();
			for(int i = 0; i < elementCount; i++){
				if(withDefault){
					ArgEntry entry = (ArgEntry)elementData[i];
					int index = getDefaultList().getIndexOfParam(entry.getParam());
					if(index >= 0){
						rmSet.add(getDefaultList().get(index));
					}
					list.add(entry);
				}else{
					if(!isDefaultValue(i)){
						list.add((ArgEntry)elementData[i]);
					}
				}
			}
			if(withDefault){
				ArgListMod defClone = (ArgListMod)getDefaultList().clone();
				for(ArgEntry entry : rmSet)
					defClone.remove(entry);
				for(int i = 0; i < defClone.elementCount; i++){
					list.add((ArgEntry)defClone.elementData[i]);
				}
			}
		}
		return list;
	}
	
	public synchronized String toArgString(){
		String s = "("; //$NON-NLS-1$
		if(elementCount > 0){
			for(int i = 0; i < elementCount; i++){
				s += get(i).toString()+", "; //$NON-NLS-1$
			}
			s = s.substring(0, s.length()-2);
		}
		s += ")"; //$NON-NLS-1$
		return s;
	}
	
	public static ArgListMod toArgList(String args){
		String s = "function" + args + "{}"; //$NON-NLS-1$ //$NON-NLS-2$
		return RCommand.argList(s);
	}
	
	private static boolean hashFirstEquals(Object a, Object b){
		if(a.hashCode() == b.hashCode()){
			return a.equals(b);
		}
		return false;
	}
	
	@Override
	public synchronized Object clone() {
		return super.clone();
	}
}
