package com.ef_prime.rflow.node.base;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.HashSet;
import java.util.Set;

import javax.swing.DefaultCellEditor;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.text.JTextComponent;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.tool.SupportControl;
import com.ef_prime.rflow.util.FileChooserUtility;
import com.ef_prime.rflow.util.FileUtility;


public class ArgsTable extends JTable{
	
	private String function;
	
	
	public ArgsTable(){
		super();

		setModel(new ArgsTableModel());
		setDefaultRenderer(String.class, new ArgsTableRenderer());
		setDefaultEditor(String.class, new ArgsTableCellEditor(new RflowTextField()));
		//setDefaultEditor(String.class, new ArgsTableCellEditor2());
		addKeyListener(new KeyHandler());
		addMouseListener(new MouseHandler());
		setSurrendersFocusOnKeystroke(true);
	}
	
	public TableCellEditor getTableCellEditor(){
		return getDefaultEditor(String.class);
	}
	
	public void setArgsList(ArgListMod arglist){
		((ArgsTableModel)getModel()).setEntrys(arglist);
	}
	
	public ArgListMod getArgsList(){
		return (ArgListMod)((ArgsTableModel)getModel()).entrys.clone();
	}
	
	public void setDefaultList(String function){
		ArgsTableModel model = (ArgsTableModel)getModel();
		model.entrys.setDefaultList(function);
	}	

	public void setDefaultMap(ArgListMod defaultList) {
		ArgsTableModel model = (ArgsTableModel)getModel();
		model.entrys.setDefaultList(defaultList);
	}
	
	
	public void clearDefaultArgs(){
		ArgsTableModel model = (ArgsTableModel)getModel();
		for(int i = model.getRowCount()-1; i >= 0; i--){
			if(model.isRowDefaultValue(i))
				model.removeRow(i);
		}
	}
	
	public ArgListMod getDefaultList(){
		ArgsTableModel model = (ArgsTableModel)getModel();
		return (ArgListMod)model.entrys.getDefaultList().clone();
	}
	
	public void addRow(ArgEntry entry){
		ArgsTableModel model = (ArgsTableModel)getModel();
		model.addRow(entry);
	}
	
	public void setFunction(String function) {
		this.function = function;
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		((Component)getTableHeader().getDefaultRenderer()).setEnabled(enabled);
		((Component)getDefaultRenderer(String.class)).setEnabled(enabled);
	}
	
	
	private static class KeyHandler extends KeyAdapter{
		@Override
		public void keyPressed(KeyEvent e) {
			ArgsTable table = (ArgsTable)e.getSource();
			ArgsTableModel model = (ArgsTableModel)table.getModel();
			
			switch(e.getKeyCode()){
			case KeyEvent.VK_DELETE:
				int[] rows = table.getSelectedRows();
				table.clearSelection();
				for(int i = rows.length-1; i >= 0; i--){
					model.removeRow(rows[i]);
				}
				e.consume();
				break;
			}
		}
	}
	
	private static class MouseHandler extends MouseAdapter{
		@Override
		public void mousePressed(MouseEvent e) {
			
			
			final ArgsTable table = (ArgsTable)e.getSource();
			//final ArgsTableModel model = (ArgsTableModel)table.getModel();

			//右クリック
			if(SwingUtilities.isRightMouseButton(e)){
				final int col = table.columnAtPoint(e.getPoint());
				if(col != ArgsTableModel.COLUMN_VALUE)return;
				
				final int row = table.rowAtPoint(e.getPoint());
				if(row < 0)return;
				
				//ファイル選択
				JPopupMenu popup = new JPopupMenu();
				JMenuItem file = new JMenuItem(Messages.getString("ArgsTable.choose_file")); //$NON-NLS-1$
				file.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						File file = FileChooserUtility.chooseOpenFile(Messages.getString("ArgsTable.choose_file"), Prefs.WORKING_DIRECTORY, new FileFilter[0], table); //$NON-NLS-1$
						if(file == null)return;
						File abstractFile = FileUtility.makeAbstractPath(Prefs.WORKING_DIRECTORY, file);
						String path = Constants.replaceFileSeparator(abstractFile.getPath());
						
						table.setValueAt(Constants.quote(path), row, col);
						table.editCellAt(row, col);
						table.getCellEditor(row, col).stopCellEditing();
					}
				});
				popup.add(file);
				popup.show(table, e.getPoint().x, e.getPoint().y);
			}
		}
	}
	
	private class ArgsTableModel extends AbstractTableModel implements TableModelListener{
		
		static final int COLUMN_PARAM = 0;
		static final int COLUMN_VALUE = 1;

		private final String[] colNames = {Messages.getString("ArgsTable.parameter"),Messages.getString("ArgsTable.value")}; //$NON-NLS-1$ //$NON-NLS-2$
		
		private ArgListMod entrys;
		
		public ArgsTableModel(){
			this(new ArgListMod()); 
		}
		
		public ArgsTableModel(ArgListMod entrys){
			this.entrys = entrys;
			addTableModelListener(this);
		}
		
		public void setEntrys(ArgListMod entrys){
			this.entrys = entrys;
			fireTableDataChanged();
		}
		
		public void addRow(ArgEntry entry){
			entrys.add(entry);
			fireTableRowsInserted(entrys.size()-1, entrys.size()-1);
		}
		
		public boolean isRowDefaultValue(int row){
			/*String arg = (String)getValueAt(row, COLUMN_ARG);
			String defaultValue = getDefaultMap().get(arg);
			
			if(defaultValue == null)
				return false;
			
			String value = (String)getValueAt(row, COLUMN_VALUE);
			
			return value.equals(defaultValue);*/
			if(row < 0 || row > entrys.size()-1)
				return false;
			return entrys.isDefaultValue(row);
		}

	    public String getColumnName(int column) {
	    	return colNames[column];
	    }
	    
	    @Override
	    public Class<?> getColumnClass(int columnIndex) {
	    	return String.class;
	    }
	    

	    public boolean isCellEditable(int rowIndex, int columnIndex) {
	    	return true;
	    }
		
		public int getColumnCount() {
			return colNames.length;
		}

		public int getRowCount() {
			return entrys.size() + 1;
		}
		
		

		public Object getValueAt(int rowIndex, int columnIndex){
	    	if(entrys.size() <= rowIndex)
	    		return null;
			switch(columnIndex){
			case COLUMN_PARAM:
				return entrys.get(rowIndex).getParam();
			case COLUMN_VALUE:
				return entrys.get(rowIndex).getValue();
			}
			return null;
		}

	    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	    	if(entrys.size() <= rowIndex){
	    		entrys.add(new ArgEntry("", "")); //$NON-NLS-1$ //$NON-NLS-2$
	    		fireTableRowsInserted(entrys.size()-1, entrys.size()-1);
	    	}
	    	ArgEntry entry = entrys.get(rowIndex);
			switch(columnIndex){
			case COLUMN_PARAM:
				entrys.set(rowIndex, new ArgEntry(aValue.toString(), entry.getValue()));
				break;
			case COLUMN_VALUE:
				entrys.set(rowIndex, new ArgEntry(entry.getParam(), aValue.toString()));
				break;
			}
			fireTableCellUpdated(rowIndex, columnIndex);
	    }
	    
	    public void removeRow(int row) {
	    	if(entrys.size() <= row)return;
	        entrys.remove(row);
	        fireTableRowsDeleted(row, row);
	    }

		public void tableChanged(TableModelEvent e) {
			switch(e.getType()){
			case TableModelEvent.UPDATE:
			case TableModelEvent.INSERT:
				if(entrys.size() < 1)return;
				Set<Integer> removeRows = new HashSet<Integer>();
				int lastRow = (e.getLastRow() > entrys.size()-1)?entrys.size()-1:e.getLastRow();
				for(int i = e.getFirstRow(); i <= lastRow; i++){
					ArgEntry updated = entrys.get(i);
					boolean isDefault = isRowDefaultValue(i);
					for(int j = 0; j < entrys.size()-1; j++){
						if(i == j)continue;
						ArgEntry entry = entrys.get(j);
						if(updated.getParam() == null)continue;
						if(updated.getParam().equals(entry.getParam()) &&	(isDefault != isRowDefaultValue(j)))
						{
							removeRows.add((isDefault)?i:j);
						}
					}
				}
				for(Integer row : removeRows){
					removeRow(row);
				}
				ArgsTable.this.repaint();
			}
		}
	}

	private static class ArgsTableRenderer extends DefaultTableCellRenderer{
		
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			
			ArgsTable argsTable = (ArgsTable)table;
			ArgsTableModel model = (ArgsTableModel)argsTable.getModel();
			if (model.isRowDefaultValue(row)) {
				this.setForeground(Color.LIGHT_GRAY);
			} else {
				this.setForeground(Color.BLACK);
			}
			
			setFont(Prefs.CODE_FONT_12);
			
			return this;
		}
	}
	
	/*private class ArgsTableCellEditor2 extends DefaultCellEditor{
		public ArgsTableCellEditor2() {
			super(new JTextField());//dummy
			final CodingTextPane textPane = new CodingTextPane();
			editorComponent = textPane;
			delegate = new EditorDelegate(){
				@Override
				public void setValue(Object value) {
					textPane.setText((value != null) ? value.toString() : "");
				}
				@Override
				public Object getCellEditorValue() {
					return textPane.getRawText();
				}
			};
		}
	}*/
	
	private class ArgsTableCellEditor extends DefaultCellEditor{

		public boolean isValue;
		
		public ArgsTableCellEditor(final JTextField textField) {
			super(textField);

			final SupportControl supportControl = new SupportControl(textField){
				@Override
				protected String[] getSupportWords(String input, int index) {
					if(isValue){
						switch(index){
						case 1: return RCommand.supportListGlobal(input);
						default: return RCommand.supportList(input);
						}
					}else{
						if(function == null)return null;
						ArgListMod map = RCommand.argList(function);
						if(map == null)return null;
						//System.out.println(map);
						return map.paramSet().toArray(new String[0]);
					}
				}
				
				@Override
				public void itemAccepted(JTextComponent source, String selected) {
					if(isValue)
						super.itemAccepted(source, selected);
					else{
						source.setText(selected);
					}
				}
			};
			addCellEditorListener(new CellEditorListener(){
				public void editingCanceled(ChangeEvent e) {
					supportControl.removeSupportPopup();
				}

				public void editingStopped(ChangeEvent e) {
					supportControl.removeSupportPopup();
				}
			});
		}
		
		@Override
		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
			JComponent editorComponent = (JComponent)getComponent();
			editorComponent.setBorder(new LineBorder(Color.black));
			editorComponent.setFont(Prefs.CODE_FONT_12);
			isValue = (column == 1);
			return super.getTableCellEditorComponent(table, value, isSelected, row, column);
		}
	}
}
