package com.ef_prime.rflow.node.base;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.util.DocumentAdapter;

public class FunctionNodeModel extends SubflowNodeModel implements ControlNodeModel{
	
	String function = ""; //$NON-NLS-1$
	String args = "()"; //$NON-NLS-1$
	
	public FunctionNodeModel(){
		super();
	}
	
	FunctionNodeModel(String function, String args, ViewLayout subflow){
		super(subflow);
		this.function = function;
		this.args = args;
	}
	
	private FunctionNodeModel(FunctionNodeModel model){
		super(model);
		function = model.function;
		args = model.args;
	}
	
	@Override
	public String getCommand() {
		String command = ""; //$NON-NLS-1$
		if(function.length() > 0)
			command = function + " <- "; //$NON-NLS-1$
		command += "function" + args + "{" + Constants.NEWLINE; //$NON-NLS-1$ //$NON-NLS-2$
		command += super.getCommand() + "}"; //$NON-NLS-1$
		return command;
	}
	
	@Override
	public Command getCommandObject() {
		return new BasicCommand(getCommand(), false);
	}
	
	@Override
	public String getAllCommand() {
		return super.getAllCommand() + Constants.NEWLINE + args + Constants.NEWLINE + "function"; //$NON-NLS-1$
	}
	
	@Override
	public boolean isOpenSubflow() {
		return false;
	}

	@Override
	JComponent createMainComponent(FlowViewer[] viewer) {
		
		ArgsPanel argsPanel = new ArgsPanel();
		int tableWidth = 200;
		argsPanel.setPreferredSize(new Dimension(tableWidth, viewer[0].getPreferredSize().height));

		JPanel panel = new JPanel(new BorderLayout());
		FunctionPanel funcPanel = new FunctionPanel();
		panel.add(funcPanel, BorderLayout.NORTH);
		panel.add(argsPanel, BorderLayout.CENTER);
		
		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel, createFlowPanel(viewer[0]));
		sp.setDividerLocation(tableWidth);

		return sp;
	}
	
	private class FunctionPanel extends JPanel{
		FunctionPanel(){
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
			
			JLabel label = new JLabel(Messages.getString("Message.function_name")); //$NON-NLS-1$
			JTextField textField = new RflowTextField(function);
			textField.getDocument().addDocumentListener(new DocumentAdapter(){
				@Override
				public void documentUpdate(DocumentEvent e) {
					Document d = e.getDocument();
					try {
						function = d.getText(0, d.getLength());
						FunctionNodeModel.this.update();
						FlowNode.fireSubflowNodeModelChanged(getParentNode());
					} catch (BadLocationException e1) {
						e1.printStackTrace();
					}
				}
			});
			
			add(label);
			add(textField);
		}
	}
	
	private class ArgsPanel extends JPanel{
		ArgsPanel(){
			super();
			setLayout(new BorderLayout());

			final ArgsTable table = new ArgsTable();
			
			ArgListMod list = ArgListMod.toArgList(args);
			if(list == null)
				list = new ArgListMod();
			table.setArgsList(list);
			table.getModel().addTableModelListener(new TableModelListener(){
				public void tableChanged(TableModelEvent e) {
					args = toArgString(table.getArgsList());
					FlowNode.fireSubflowNodeModelChanged(getParentNode());
				}
			});
			
			JScrollPane pane = new JScrollPane(table);
			//pane.setBorder(BorderFactory.createEmptyBorder());
			add(pane);
		}
		
		private String toArgString(ArgListMod list){
			String s = "("; //$NON-NLS-1$
			for(ArgEntry entry : list){
				if(!entry.isParam())continue;
				s += entry.getParam();
				if(entry.isValue())
					s += " = " + entry.getValue(); //$NON-NLS-1$
				s += ", "; //$NON-NLS-1$
			}
			if(s.length() > 1)
				s = s.substring(0, s.length()-2);
			s += ")"; //$NON-NLS-1$
			return s;
		}
	}
	
	@Override
	public String getDefaultTitle() {
		return function + "()"; //$NON-NLS-1$
	}
	
	

	@Override
	public NodeModel copy() {
		return new FunctionNodeModel(this);
	}
}
