package com.ef_prime.rflow.node.base;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.util.DocumentAdapter;

public class IfNodeModel extends SubflowNodeModel implements ControlNodeModel{
	
	ArrayList<String> conditions;
	
	public IfNodeModel(){
		super(2);
		conditions = new ArrayList<String>(1);
		conditions.add(""); //$NON-NLS-1$
	}
	
	IfNodeModel(ArrayList<String> conditions, ViewLayout[] subflows){
		super(subflows);
		this.conditions = conditions;
	}
	
	IfNodeModel(String[] conditions, ViewLayout[] subflows){
		super(subflows);
		this.conditions = new ArrayList<String>();
		for(String s : conditions)
			this.conditions.add(s);
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private IfNodeModel(IfNodeModel model){
		super(model);
		this.conditions = (ArrayList<String>)model.conditions.clone();
	}
	
	@Override
	public String getDefaultTitle() {
		String title = "if"; //$NON-NLS-1$
		if(getElseSubflowSet().isInOutConnected())
			title += " else"; //$NON-NLS-1$
		return title;
	}
	
	@Override
	public boolean isOpenSubflow() {
		return false;
	}
	
	@Override
	public String getCommand() {
		List<SubflowSet> validSets = new ArrayList<SubflowSet>();
		for(int i = 0; i < subflowSets.length; i++){
			SubflowSet set = subflowSets[i];
			if(i == 0 || set.isInOutConnected())
				validSets.add(set);
		}
		String command = ""; //$NON-NLS-1$
		if(validSets.size() > 0){
			for(int i = 0; i < validSets.size(); i++){
				if(i != validSets.size()-1 || validSets.size() == 1){
					command +=  " if(" + conditions.get(i) + "){" + Constants.NEWLINE; //$NON-NLS-1$ //$NON-NLS-2$
					command += subflowSets[i].getCommand() + "}"; //$NON-NLS-1$
					if(validSets.size() != 1)
						command += "else"; //$NON-NLS-1$
				}else{
					command += "{" + Constants.NEWLINE; //$NON-NLS-1$
					command += subflowSets[i].getCommand() + "}"; //$NON-NLS-1$
				}
			}
		}else{
			command += " if("+Constants.FALSE+"){}"; //$NON-NLS-1$ //$NON-NLS-2$
		}
		command = command.substring(1);
		return command;
	}
	
	@Override
	public Command getCommandObject() {
		return new BasicCommand(getCommand(), false);
	}

	@Override
	public String getAllCommand() {
		String condition = ""; //$NON-NLS-1$
		for(String c : conditions)
			condition += c + Constants.NEWLINE;
		//TODO if and else and ()...
		return "if" + super.getAllCommand() + Constants.NEWLINE + condition; //$NON-NLS-1$
	}
	/*
	private SubflowSet getFirstSubflowSet(){
		return subflowSets[0];
	}
	*/
	private SubflowSet getElseSubflowSet(){
		return subflowSets[subflowSets.length-1];
	}
	

	private JTabbedPane tab;
	@Override
	JComponent createMainComponent(FlowViewer[] viewers) {
		tab = new JTabbedPane();
		for(int i = 0; i < subflowSets.length; i++){
			String tabName;
			if(i == 0){
				tabName = Messages.getString("Node.IfNodeModel.if"); //$NON-NLS-1$
			}else if(i == (subflowSets.length-1)){
				tabName = Messages.getString("Node.IfNodeModel.else"); //$NON-NLS-1$
			}else{
				tabName = "ELSEIF";
			}
			tab.add(createFlowPanel(viewers[i]), tabName);
		}
		tab.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JTabbedPane tab = (JTabbedPane)e.getSource();
				int index = tab.getSelectedIndex();
				subflowSets[index].smallViewer.focusViewer();
				activeViewer = subflowSets[index].smallViewer;
			}
		});
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(new ConditionPanel());
		panel.add(tab);
		
		return panel;
	}

	private class ConditionPanel extends JPanel{
		
		ConditionPanel(){
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
			
			JLabel label = new JLabel(Messages.getString("Message.condition")); //$NON-NLS-1$
			JTextField textField = new RflowTextField(conditions.get(0));
			textField.getDocument().addDocumentListener(new DocumentAdapter(){
				@Override
				public void documentUpdate(DocumentEvent e) {
					Document d = e.getDocument();
					try {
						conditions.set(0, d.getText(0, d.getLength()));
						FlowNode.fireSubflowNodeModelChanged(getParentNode());
					} catch (BadLocationException e1) {
						e1.printStackTrace();
					}
				}
			});
			
			add(label);
			add(textField);
		}
	}
	
	@Override
	public NodeModel copy() {
		return new IfNodeModel(this);
	}
}
