package com.ef_prime.rflow.node.base;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.util.DocumentAdapter;

public class LoopNodeModel extends SubflowNodeModel implements ControlNodeModel{
	//memo:フィールドを増やす場合、copy系のメソッドの変更が必要

	LoopType loopType;
	String condition = ""; //$NON-NLS-1$
	
	public LoopNodeModel(LoopType loopType) {
		super();
		this.loopType = loopType;
	}
	
	LoopNodeModel(LoopType loopType, String condition, ViewLayout layout){
		super(layout);
		this.loopType = loopType;
		if(loopType.isCondition())
			this.condition = condition;
	}
	
	private LoopNodeModel(LoopNodeModel model){
		super(model);
		this.loopType = model.loopType;
		if(loopType.isCondition())
			this.condition = model.condition;
	}
	
	LoopType getLoopType(){
		return this.loopType;
	}
	
	String getCondition(){
		return this.condition;
	}
	
	@Override
	public String getCommand() {
		String command = ""; //$NON-NLS-1$
		if(getLoopType().isCondition()){
			command = getLoopType().getCode() + "(" + getCondition() + "){" + Constants.NEWLINE; //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			command = getLoopType().getCode() + "{" + Constants.NEWLINE; //$NON-NLS-1$
		}
		command += super.getCommand() + "}"; //$NON-NLS-1$
		
		return command;
	}
	
	@Override
	public Command getCommandObject() {
		return new BasicCommand(getCommand(), false);
	}
	
	@Override
	public String getAllCommand() {
		return super.getAllCommand() + Constants.NEWLINE + getCondition() + Constants.NEWLINE + getLoopType().getCode();
	}
	
	@Override
	public String getDefaultTitle() {
		return getLoopType().getCode();
	}
	
	@Override
	public boolean isOpenSubflow() {
		return false;
	}
	
	@Override
	JComponent createMainComponent(FlowViewer[] viewers) {
		JPanel panel = new JPanel(new BorderLayout());
		if(getLoopType().isCondition()){
			ConditionPanel conditionPanel = new ConditionPanel();
			panel.add(conditionPanel, BorderLayout.NORTH);
		}
		panel.add(createFlowPanel(viewers[0]), BorderLayout.CENTER);
		return panel;
	}
	
	private class ConditionPanel extends JPanel{
		
		ConditionPanel(){
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, 25));
			//setBorder(new EmptyBorder(2,2,2,2));
			//setBorder(new LineBorder(Color.GRAY));
			//setBorder(new SideBorder(SideBorder.SIDE_BOTTOM, Color.LIGHT_GRAY));
			
			JLabel label = new JLabel(Messages.getString("Message.condition")); //$NON-NLS-1$
			JTextField textField = new RflowTextField(getCondition());
			//textField.setBorder(BorderFactory.createEmptyBorder());
			//textField.setBorder(new LineBorder(Color.LIGHT_GRAY));
			Insets m = textField.getMargin();
			textField.setMargin(new Insets(m.top,m.left+5,m.bottom,m.right));

			textField.getDocument().addDocumentListener(new DocumentAdapter(){
				@Override
				public void documentUpdate(DocumentEvent e) {
					Document d = e.getDocument();
					try {
						LoopNodeModel.this.condition = d.getText(0, d.getLength());
						FlowNode.fireSubflowNodeModelChanged(getParentNode());
					} catch (BadLocationException e1) {
						e1.printStackTrace();
					}
				}
			});
			
			
			//add(Box.createHorizontalStrut(5));
			add(label);
			add(Box.createHorizontalStrut(2));
			add(Box.createVerticalStrut(20));
			add(textField);
		}
	}
	
	@Override
	public NodeModel copy() {
		return new LoopNodeModel(this);
	}
}
