package com.ef_prime.rflow.node.base;

import java.awt.Shape;

import javax.swing.Icon;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.RflowUtil;
import com.ef_prime.rflow.console.BasicCommand;
import com.ef_prime.rflow.console.Command;
import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.flow.rendering.NodeShape;
import com.ef_prime.rflow.group.AppearanceGroup;
import com.ef_prime.rflow.group.AppearanceGroupManager;
import com.ef_prime.rflow.node.AbstractNodeModel;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.SentenceInfo;

public class SentenceNodeModel extends AbstractNodeModel{
	
	private String command = ""; //$NON-NLS-1$
	private String symbol = ""; //$NON-NLS-1$
	private String function = ""; //$NON-NLS-1$
	
	private AppearanceGroup group;
	
	public SentenceNodeModel() {
		super();
	}
	
	private SentenceNodeModel(SentenceNodeModel model) {
		command = model.command;
		symbol = model.symbol;
		function = model.function;
		group = model.group;
	}

	@Override
	public boolean isEditable() {
		return !getCommand().contains(Constants.NEWLINE_STR);
	}
	

	private static final int MAX_TRY = 1;
	public void setCommand(String s)throws IncompatibleCommandException{
		if(s.equals(getCommand()))
			return;
		/*
		 * MAX_TRY回コマンド
		 */
		int count = MAX_TRY;
		while(count > 0 && !extractFunction(s)){
			count--;
			if(Rflow.DEBUG > 0)
				System.out.println("IncompatibleCommandException:"+(MAX_TRY-count)); //$NON-NLS-1$
			if(count == 0)
				throw new IncompatibleCommandException();
		}
	}
	
	public void setCommand(SentenceInfo info){
		command = info.getLine();
		symbol = info.getSymbol();
		function = info.getFunction();
	}
	
	/** SentenceInfo with no argList*/
	public SentenceInfo getSentenceInfo(){
		return new SentenceInfo(command, symbol, function, false, new ArgListMod());
	}
	
	private boolean extractFunction(String command){
		SentenceInfo info = RflowUtil.retrieveSentenceInfo(command);
		if(info == null)return false;
		
		setCommand(info);
		
		return true;
	}

	public String getSymbol() {
		return symbol;
	}
	
	public String getFunction() {
		return function;
	}
	
	public boolean isFunction(){
		return function.length() > 0;
	}

	public boolean isAssign(){
		return symbol.length() > 0;
	}
	
	@Override
	public String getDefaultTitle() {
		if(isFunction()){
			String title = ""; //$NON-NLS-1$
			if(isAssign()){
				title += symbol + "<-"; //$NON-NLS-1$
			}
			title += function;
			return title;
		}else{
			String c = getCommand();
			if(c.length() > 10)
				return getCommand().substring(0, 10)+"..."; //$NON-NLS-1$
			else
				return c;
		}
	}
	
	public NodeModel copy() {		
		return new SentenceNodeModel(this);
	}

	public String getCommand() {
		return command;
	}
	
	@Override
	public Command getCommandObject() {
		return new BasicCommand(getCommand(), false);
	}
	

	//-----------------------------------------------------------------
	//shape
	//-----------------------------------------------------------------
	
	@Override
	public Icon getIcon() {
		if(group == null)
			return null;
		return group.getIcon();
	}
	
	@Override
	public Icon getSmallIcon() {
		if(group == null)
			return super.getSmallIcon();
		return group.getSmallIcon();
	}
	
	@Override
	public Shape getShape() {
		if(group != null)
			return group.getShape();
		
		return getNodeShape().getShape();
	}
	
	@Override
	protected NodeShape getNodeShape() {
		if(isAssign())
			return NodeShape.CIRCLE;
		else
			return NodeShape.TRIANGLE;
	}
	
	@Override
	public Shape getGlassShape() {
		if(group != null)
			return group.getGlassShape();
		return super.getGlassShape();
	}
	
	@Override
	public void update(AppearanceGroupManager groupManager) {
		group = groupManager.getAppearanceGroup(function);
	}
	
	public void setGroup(AppearanceGroup group){
		this.group = group;
	}
	

	@Override
	public int hashCode() {
		return super.hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof SentenceNodeModel) {
			return super.equals(obj);
		}
		return false;		
	}
}
