package com.ef_prime.rflow.node.base;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.RflowUtil;
import com.ef_prime.rflow.console.BusyListener;
import com.ef_prime.rflow.exception.BuildFailedException;
import com.ef_prime.rflow.exception.IncompatibleCommandException;
import com.ef_prime.rflow.exception.InvalidOptionException;
import com.ef_prime.rflow.exception.ModelCreationFailedException;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.node.AbstractNodeOptionPanelHandler;
import com.ef_prime.rflow.node.DefaultNodeModelController;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeOptionPanel;
import com.ef_prime.rflow.node.NodeOptionPanelHandler;
import com.ef_prime.rflow.node.SentenceInfo;
import com.ef_prime.rflow.node.pack.UIPack;
import com.ef_prime.rflow.node.pack.UIPackManager;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.CodingTextPane;
import com.ef_prime.rflow.tool.ItemAcceptListener;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.tool.SupportControl;
import com.ef_prime.rflow.tool.SupportedInputFactory;
import com.ef_prime.rflow.util.BackgroundLayout;
import com.ef_prime.rflow.util.DocumentAdapter;

public class SentenceNodeModelController extends DefaultNodeModelController{

	public String getModelName() {
		return Messages.getString("SentenceNodeModelController.model_name"); //$NON-NLS-1$
	}
	
	public Class<? extends NodeModel> getModelClass() {
		return SentenceNodeModel.class;
	}
	
	public Properties getOptionProperties(NodeModel model) {
		return null;
	}
	
	public NodeModel createModel(Properties properties, String command, ViewLayout[] subflows) throws ModelCreationFailedException {
		SentenceNodeModel model = new SentenceNodeModel();
		try {
			model.setCommand(command);
		} catch (IncompatibleCommandException e) {
			throw new ModelCreationFailedException();
		}
		return model;
	}	

	@Override
	protected NodeOptionPanelHandler createNodeOptionPanelHandler() {
		return new SentenceNodeOptionPanelHandler();
	}
	
	@Override
	protected NodeOptionPanel createNodeOptionPanel(NodeModel model) {
		//unreachable
		return null;
	}
	
	private class SentenceNodeOptionPanelHandler extends AbstractNodeOptionPanelHandler{

		private boolean TEST = false;
		
		private static final int TAB_UI = 0;
		private static final int TAB_BASIC = 1;
		
		private String function = ""; //$NON-NLS-1$
		private SentenceNodeModel model;
		
		@Override
		protected NodeOptionPanel[] createNodeOptionPanels(NodeModel orginalModel) {
			model = (SentenceNodeModel)orginalModel;
			if(TEST){
				return new NodeOptionPanel[]{
						new UINodeOptionPanel(orginalModel),
						new SentenceNodeOptionPanel(orginalModel)
				};
			}else{
				return new NodeOptionPanel[]{
						new SentenceNodeOptionPanel(orginalModel)
				};
			}
		}
		
		@Override
		protected void initTabbedPane() {
			if(TEST){
				tabPane.addChangeListener(new TabChangedListener());
				updateUITab(model.getFunction());
				if(tabPane.isEnabledAt(TAB_UI)){
					updateUIPanel();
				}else{
					tabPane.setSelectedIndex(TAB_BASIC);
				}
			}
		}
		
		@Override
		public void initFocus() {
			if (tabPane.getSelectedComponent() instanceof NodeOptionPanel) {
				NodeOptionPanel panel = (NodeOptionPanel) tabPane.getSelectedComponent();
				panel.initFocus();
			}
		}
		
		private class TabChangedListener implements ChangeListener{
			private int lastTab = tabPane.getSelectedIndex();
			public void stateChanged(ChangeEvent e) {
				JTabbedPane tab = (JTabbedPane)e.getSource();
				
				UINodeOptionPanel uiPanel = (UINodeOptionPanel)tab.getComponentAt(TAB_UI);
				SentenceNodeOptionPanel basicPanel = (SentenceNodeOptionPanel)tab.getComponentAt(TAB_BASIC);

				try {
					//TODO model dont update when property panel is selected
					// this is first aid
					int tabIndex = tab.getSelectedIndex();
					if(tabIndex > TAB_BASIC){
						tabIndex = (lastTab > TAB_UI)?TAB_UI:TAB_BASIC; 
					}
					switch(tabIndex){
					case TAB_UI:
						model = (SentenceNodeModel)basicPanel.getModel();
						updateUIPanel();
						break;
					case TAB_BASIC:
						if(!uiPanel.isEmpty()){
							model = (SentenceNodeModel)uiPanel.getModel();
						}
						basicPanel.setCommand(model.getCommand());
						break;
					}
				} catch (InvalidOptionException e1) {
					JOptionPane.showMessageDialog(getParentWindow(), e1.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
					//tab.setSelectedIndex((tab.getSelectedIndex() == TAB_UI)?TAB_BASIC:TAB_UI);
					return;
				} finally {
					lastTab = tab.getSelectedIndex();
				}
				
			}
		}
		
		@Override
		public NodeModel getModel() throws InvalidOptionException {
			if(TEST){
				switch(tabPane.getSelectedIndex()){
				case TAB_UI:
					UINodeOptionPanel uiPanel = (UINodeOptionPanel)tabPane.getSelectedComponent();
					if(!uiPanel.isEmpty()){
						return uiPanel.getModel();
					}else{
						return model;
					}
				case TAB_BASIC:
					SentenceNodeOptionPanel basicPanel = (SentenceNodeOptionPanel)tabPane.getSelectedComponent();
					return basicPanel.getModel();
				default:
					return model;
				}
			}else{
				SentenceNodeOptionPanel basicPanel = (SentenceNodeOptionPanel)tabPane.getComponentAt(0);
				return basicPanel.getModel();
			}
		}
		
		void updateUITab(String function){
			if(TEST){
				this.function = function;
				UIPack pack = UIPackManager.getUIPack(function);
				if(tabPane.getTabCount() > 0){
					tabPane.setEnabledAt(TAB_UI, pack != null);
				}
			}
		}

		private void updateUIPanel() {
			if(TEST){
				UIPack pack = UIPackManager.getUIPack(function);
				if(pack != null){
					UINodeOptionPanel uiPanel = (UINodeOptionPanel)tabPane.getComponentAt(TAB_UI);
					NodeOptionPanel panel;
					try {
						panel = pack.getBuilder().build(model);
					} catch (BuildFailedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return;
					}
					uiPanel.setPanel(panel);
				}
			}
		}
		
		private class UINodeOptionPanel extends NodeOptionPanel{
			
			NodeOptionPanel panel;
			
			UINodeOptionPanel(NodeModel orginalModel){
				setLayout(new BorderLayout());
			}
			
			void setPanel(NodeOptionPanel panel){
				this.panel = panel;
				removeAll();
				add(panel);
				validate();
			}
			
			boolean isEmpty(){
				return panel == null;
			}
			
			@Override
			public NodeModel getModel() throws InvalidOptionException {
				return panel.getModel();
			}
		}

		private class SentenceNodeOptionPanel extends NodeOptionPanel{
		
			private FunctionCodePanel functionCodePanel;
			private AssignOptionPanel assignOption;
			private FunctionOptionPanel functionOption;
		
			private boolean lockCode2Options = false;
			private boolean lockOption2Code = false;
			
			public SentenceNodeOptionPanel(NodeModel model){
				super();
				SentenceNodeModel mdl = (SentenceNodeModel)model;
				
				functionCodePanel = new FunctionCodePanel(mdl);
				assignOption = new AssignOptionPanel(mdl);
				functionOption = new FunctionOptionPanel(mdl);
		
				
				final Box box = Box.createVerticalBox();
				box.setBorder(DEFAULT_BORDER);
				box.add(functionCodePanel);
				box.add(assignOption);
				box.add(functionOption);
				
				box.addMouseListener(new MouseAdapter(){
					@Override
					public void mousePressed(MouseEvent e) {
						requestFocus();
					}
				});
				this.setLayout(new BorderLayout());
				//this.add(box, BorderLayout.CENTER);
				

				JPanel panel = new JPanel();
				final JPanel glassPane = new JPanel(){
					@Override
					protected void paintComponent(Graphics g) {
						super.paintComponent(g);
						box.print(g);
					}
				};
				glassPane.setBackground(box.getBackground());
				panel.setLayout(new BackgroundLayout(new Component[]{box, glassPane}));
				glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				glassPane.setFocusable(true);
				panel.add(box, -1);
				panel.add(glassPane, 0);

				boolean busy = Rflow.isBusy();
				box.setVisible(!busy);
				glassPane.setVisible(busy);
				this.add(panel, BorderLayout.CENTER);
				
				final BusyListener listener = new BusyListener(){
					public void busyStateChanged(int which) {
						boolean busy = (which == BusyListener.BUSY);
						box.setVisible(!busy);
						glassPane.setVisible(busy);
					}
				};
				rflow.console.addBusyListener(listener);
				addAncestorListener(new AncestorListener(){
					public void ancestorAdded(AncestorEvent event) {
					}
					public void ancestorMoved(AncestorEvent event) {
					}
					public void ancestorRemoved(AncestorEvent event) {
						rflow.console.removeBusyListener(listener);
					}
				});
				
				
				
				if(functionCodePanel.textPane.getText().length() > 0)
					functionCodePanel.editFinished();
			}
			
			@Override
			public void initFocus() {
				functionCodePanel.textPane.requestFocus();
			}
			
			public void setCommand(String command){
				functionCodePanel.setCommand(command);
			}
			
			private void setFunctionOptionEnable(boolean b){
				//setEnable(assignOption, b);
				setEnable(functionOption, b);
			}
			
			private void setEnable(Container c, boolean b){
				for(Component child : c.getComponents()){
					if(child instanceof Container){
						child.setEnabled(b);
						setEnable((Container)child, b);
					}else{
						child.setEnabled(b);
					}
				}
			}
			
			@Override
			public NodeModel getModel() throws InvalidOptionException{
				//SentenceInfo info = getSentenceInfoFromOption();
				/*SentenceInfo infoNew = new SentenceInfo(
						functionCodePanel.getCommand(), info.getSymbol(), 
						info.getFunction(), false, info.getArgList());*/
				SentenceNodeModel model = new SentenceNodeModel();
				try {
					model.setCommand(functionCodePanel.getCommand());
				} catch (IncompatibleCommandException e) {
					if(Rflow.isBusy())
						throw new InvalidOptionException(Messages.getString("Error.rbusy_extract_command")); //$NON-NLS-1$
					throw new InvalidOptionException(Messages.getString("SentenceNodeModelController.extract_command_failed")); //$NON-NLS-1$
				}
				return model;
			}		
		
			private void code2Options(SentenceInfo info) {
				if(lockCode2Options)return;
		
				lockOption2Code = true;
				
				assignOption.assignButton.setSelected(info.isAssign());
				assignOption.assignTarget.setText(info.getSymbol());
				
				if(info.isFunction()){
					functionOption.control.setAutoSupport(false);
					functionOption.functionText.setText(info.getFunction());
					functionOption.control.setAutoSupport(true);
					functionOption.table.setArgsList(info.getArgList());
					
					functionOption.argsComplete(info.getFunction());
				}
				lockOption2Code = false;
				
				updateUITab(info.getFunction());
			}
			
			
			private void option2Code(){
				if(lockOption2Code)return;
				
				lockCode2Options = true;
		
				SentenceInfo info = getSentenceInfoFromOption();
				if(info.isFunction()){
					String command = info.getLine();
					if(!command.equals("")) //$NON-NLS-1$
						functionCodePanel.setCommand(command);
				}else{
					String c = ""; //$NON-NLS-1$
					if(info.isAssign())
						c += info.getSymbol() + " <- "; //$NON-NLS-1$
					c += RflowUtil.getSentence(functionCodePanel.getCommand());
					functionCodePanel.setCommand(c);
				}
				
				lockCode2Options = false;
				

				updateUITab(info.getFunction());
			}
			
			private SentenceInfo getSentenceInfoFromOption(){
				String symbol = ""; //$NON-NLS-1$
				String function = ""; //$NON-NLS-1$
				ArgListMod argList = new ArgListMod();
				
				if(assignOption.assignButton.isSelected())
					symbol = assignOption.assignTarget.getText();
				
				if(functionOption.functionText.isEnabled()){
					function = functionOption.functionText.getText();
					argList = functionOption.table.getArgsList().list(false, false);
				}
				
				return new SentenceInfo(symbol, function, false, argList);
			}
		
			private KeyListener keylistener = new KeyAdapter(){
				@Override
				public void keyPressed(KeyEvent e) {
					if(e.isConsumed())return;
					Component c = (Component)e.getSource();
					switch(e.getKeyCode()){
					case KeyEvent.VK_ENTER:
						c.transferFocus();
						e.consume();
						break;
					}
				}
			};
			
			private class FunctionCodePanel extends JPanel implements FocusListener{
				/*
				private Color editing_background = Color.WHITE;
				private Color non_editing_background = new Color(240,240,240); 
				*/
				private JLabel msgLabel;
				private CodingTextPane textPane;
				
				FunctionCodePanel(SentenceNodeModel model){
					super();
					setLayout(new BorderLayout());
					setBorder(new TitledBorder(Messages.getString("SentenceNodeModelController.code"))); //$NON-NLS-1$
					
					msgLabel = new JLabel(" "); //$NON-NLS-1$
					
					final JPanel borderPanel = new JPanel(new BorderLayout());
					
					textPane = new CodingTextPane();
					textPane.setText(model.getCommand());
					textPane.setParenthesisSupport(true);
					textPane.addFocusListener(this);
					//textPane.addKeyListener(keylistener);
					textPane.addKeyListener(new KeyAdapter(){
						@Override
						public void keyPressed(KeyEvent e) {
							if(e.isConsumed())return;
							switch(e.getKeyCode()){
							case KeyEvent.VK_ENTER:
								requestOkButtonFocus();
								e.consume();
								break;
							}
						}
					});
					textPane.getDocument().addDocumentListener(new DocumentAdapter(){
						@Override
						public void documentUpdate(DocumentEvent e) {
							if(e.getDocument().getLength() == 0){
								updateUITab(""); //$NON-NLS-1$
								return;
							}
							editFinished();
						}
					});
					final Border focusBorder = new LineBorder(Color.GRAY, 1); 
					//final Border focusBorder = new LineBorder(new Color(0,0.5f,1.0f,0.3f), 1); 
					final Border noFocusBorder = new LineBorder(Color.WHITE, 1);
					borderPanel.setBorder(noFocusBorder);
					textPane.addFocusListener(new FocusListener(){
						public void focusGained(FocusEvent e) {
							borderPanel.setBorder(focusBorder);
						}
		
						public void focusLost(FocusEvent e) {
							borderPanel.setBorder(noFocusBorder);
						}					
					});
					
					borderPanel.add(textPane);
					
					add(borderPanel);
					add(msgLabel, BorderLayout.NORTH);
				}
				
				public void setCommand(String s){
					textPane.setText(s);
				}
				
				public String getCommand(){
					return textPane.getRawText();
				}
				
				public void editFinished(){
					String c = getCommand();
					if(c.equals("")){ //$NON-NLS-1$
						//空の場合
						functionOption.functionText.setText(""); //$NON-NLS-1$
						setFunctionOptionEnable(true);						
						return;
					}
					try {
						SentenceInfo[] infos = RCommand.parseToNodes(c);
						if(infos == null || infos.length > 1)
							throw new IncompatibleCommandException();
						SentenceInfo info = infos[0];
						
						//model.setCommand(c);
						
						code2Options(info);
						if(info.isFunction()){
							msgLabel.setText(" "); //$NON-NLS-1$
							setFunctionOptionEnable(true);
						}else{
							msgLabel.setText(Messages.getString("SentenceNodeModelController.extract_function_failed")); //$NON-NLS-1$
							setFunctionOptionEnable(false);
						}
						//textPane.setBackground(non_editing_background);
					} catch (IncompatibleCommandException e) {
						//textPane.setBackground(editing_background);
						msgLabel.setText(Messages.getString("SentenceNodeModelController.syntax_error_or_multiline")); //$NON-NLS-1$
						setFunctionOptionEnable(false);
						//textPane.setText(getModel().getCommand());
					}
				}
		
				public void focusGained(FocusEvent e) {
					//textPane.setBackground(editing_background);
				}
		
				public void focusLost(FocusEvent e) {
					editFinished();
				}
			}
			
			private class AssignOptionPanel extends JPanel{
				
				JRadioButton evalOnlyButton;
				JRadioButton assignButton;
				JTextField assignTarget;
				
				AssignOptionPanel(SentenceNodeModel model){
					super();
					setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
					setBorder(new TitledBorder(Messages.getString("SentenceNodeModelController.assign_option"))); //$NON-NLS-1$
					
					evalOnlyButton = new JRadioButton(Messages.getString("SentenceNodeModelController.eval_only"), !model.isAssign()); //$NON-NLS-1$
					assignButton = new JRadioButton(Messages.getString("SentenceNodeModelController.assign"), model.isAssign()); //$NON-NLS-1$
					assignButton.addItemListener(new ItemListener(){
						public void itemStateChanged(ItemEvent e) {
							assignTarget.setEnabled(assignButton.isSelected());
							option2Code();
						}
					});
					assignTarget = new RflowTextField();
					assignTarget.setFont(Prefs.CODE_FONT_12);
					assignTarget.setEnabled(assignButton.isSelected());
					assignTarget.setMaximumSize(HEIGHTLIMIT_LABEL_SIZE);
					assignTarget.addKeyListener(keylistener);
					assignTarget.addFocusListener(new FocusAdapter(){
						@Override
						public void focusLost(FocusEvent e) {
							option2Code();
						}
					});
					//assignTarget.setText(model.getSymbol());
					
					ButtonGroup group = new ButtonGroup();
		
					group.add(evalOnlyButton);
					group.add(assignButton);
					
					Box evalBox = Box.createHorizontalBox();
					evalBox.add(evalOnlyButton);
					evalBox.add(Box.createHorizontalGlue());
					
					Box symbolBox = Box.createHorizontalBox();
					symbolBox.add(assignButton);
					symbolBox.add(Box.createHorizontalStrut(30));
					symbolBox.add(assignTarget);
					
					add(evalBox);
					add(symbolBox);
				}
				
			}
			
			private class FunctionOptionPanel extends JPanel {
				
				
				JTextField functionText;
				SupportControl control;
				ArgsTable table;
				
				protected FunctionOptionPanel(SentenceNodeModel model){
					super();
					setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));	
					setBorder(new TitledBorder(Messages.getString("SentenceNodeModelController.function"))); //$NON-NLS-1$
					
//					JLabel functionLabel = new JLabel(Messages.getString("SentenceNodeModelController.function")); //$NON-NLS-1$
//					functionLabel.setMaximumSize(DEFAULT_LABEL_SIZE);
//					functionLabel.setPreferredSize(DEFAULT_LABEL_SIZE);
					
					functionText = new RflowTextField();
					functionText.setFont(Prefs.CODE_FONT_12);
					functionText.setMaximumSize(HEIGHTLIMIT_LABEL_SIZE);
					//functionText.setText(model.getFunction());
					functionText.addFocusListener(new FocusAdapter(){
						@Override
						public void focusLost(FocusEvent e) {
							option2Code();
						}
					});
					functionText.addKeyListener(new KeyAdapter(){
						public void keyPressed(KeyEvent e) {
							if(control.isSupportPopup())return;
							if(e.isConsumed())return;
							Component c = (Component)e.getSource();
							switch(e.getKeyCode()){
							case KeyEvent.VK_ENTER:
								c.transferFocus();
								argsComplete(functionText.getText());
								e.consume();
								break;
							}
						};
					});
					
					control = SupportedInputFactory.setFunctionSupport(functionText);
					control.setAutoSupport(true);
					control.addItemAcceptListener(new ItemAcceptListener(){
						public void itemAccepted(JTextComponent source, String accepted) {
							//source.requestFocus();
							option2Code();
						}
					});
					functionText.getDocument().addDocumentListener(new DocumentAdapter(){
						@Override
						public void documentUpdate(DocumentEvent e) {
							try {
								Document document = e.getDocument();
								String function = document.getText(0, document.getLength());
								table.setFunction(function);
								argsComplete(function);
							} catch (BadLocationException e1) {e1.printStackTrace();}
						}
					});
					
					//Dimension littelButtonSize = new Dimension(45,20);
					
					JButton sourceButton = new JButton(Messages.getString("SentenceNodeModelController.src")); //$NON-NLS-1$
					//sourceButton.setMaximumSize(littelButtonSize);
					//sourceButton.setPreferredSize(littelButtonSize);
					sourceButton.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e) {
							RCommand.getReceiver().passCommand(functionText.getText());
						}
					});
					
					JButton helpButton = new JButton(Messages.getString("SentenceNodeModelController.help")); //$NON-NLS-1$
					//helpButton.setMaximumSize(littelButtonSize);
					//helpButton.setPreferredSize(littelButtonSize);
					helpButton.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e) {
							RCommand.getReceiver().passCommand("help("+functionText.getText()+")"); //$NON-NLS-1$ //$NON-NLS-2$
						}
					});
					
					Box functionBox = Box.createHorizontalBox();
					//functionBox.add(functionLabel);
					functionBox.add(functionText);
					functionBox.add(sourceButton);
					functionBox.add(helpButton);
					
					
//					JLabel argsLabel = new JLabel(Messages.getString("SentenceNodeModelController.parameter")); //$NON-NLS-1$
//					argsLabel.setMaximumSize(DEFAULT_LABEL_SIZE);
//					argsLabel.setPreferredSize(DEFAULT_LABEL_SIZE);
					
					table = new ArgsTable();
					//if(model.argList != null)
					//	table.setArgsList(model.argList);
					table.getTableCellEditor().addCellEditorListener(new CellEditorListener(){
						public void editingCanceled(ChangeEvent e) {}
						public void editingStopped(ChangeEvent e) {
							option2Code();
							//System.out.println("edit changed");
						}
					});
					table.getModel().addTableModelListener(new TableModelListener(){
						public void tableChanged(TableModelEvent e) {
							if(e.getType() == TableModelEvent.DELETE){
								option2Code();
								//System.out.println("model changed");
							}
						}
					});
					JScrollPane pane = new JScrollPane(table);
					pane.setPreferredSize(new Dimension(300, 200));
					
					//Box argsBox = Box.createHorizontalBox();
					//argsBox.add(argsLabel);
					//argsBox.add(pane);
					/*
					JButton argsSupportButton = new JButton(Messages.getString("SentenceNodeModelController.parameter_complete")); //$NON-NLS-1$
					//argsSupportButton.setMaximumSize(DEFAULT_LABEL_SIZE);
					//argsSupportButton.setPreferredSize(DEFAULT_LABEL_SIZE);
					argsSupportButton.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e) {
							String function = functionText.getText();
							argsComplete(function);
						}
					});
					
					Box argsSupportBox = Box.createHorizontalBox();
					argsSupportBox.setFocusable(true);
					argsSupportBox.add(Box.createHorizontalGlue());
					argsSupportBox.add(argsSupportButton);
					*/
					
					add(functionBox);
					add(pane);
					//add(argsSupportBox);//引数補完ボタン
					
					//
					table.setFunction(functionText.getText());
				}
		
				private void argsComplete(String function){
					/*ArgListMod map = RCommand.argList(function);
					if(map == null || map.size() < 1){
						table.clearDefaultMap();
						return;
					}
					table.setDefaultMap(map);*/
					lockOption2Code = true;
					table.clearDefaultArgs();
					table.setDefaultList(function);
					
					for(ArgEntry entry : table.getDefaultList()){
						table.addRow(new ArgEntry(entry.getParam(), entry.getValue()));
					}
					lockOption2Code = false;
				}
				
			}
			
		}
	}
}
