package com.ef_prime.rflow.node.base;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.geom.Point2D;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import com.ef_prime.rflow.CodeBar;
import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.FlowUtil;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.FlowViewerScrollPane;
import com.ef_prime.rflow.flow.ViewLayout;
import com.ef_prime.rflow.flow.model.FlowEdge;
import com.ef_prime.rflow.flow.model.FlowGraph;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.node.AbstractNodeModel;
import com.ef_prime.rflow.node.NodeModel;

import edu.uci.ics.jung.graph.Vertex;

public class SubflowNodeModel extends AbstractNodeModel{
	
	private static final Icon SMALLICON = new ImageIcon(FileResource.BOX.getImage().getScaledInstance(16, 16, Image.SCALE_SMOOTH));
	
	protected SubflowSet[] subflowSets;
	protected FlowViewer activeViewer;
	
	protected SubflowFrame frame = null;
	
	private final UpdateListener updateListener = new UpdateListener();
	
	public SubflowNodeModel(){
		this(1);
	}
	
	protected SubflowNodeModel(int n){
		subflowSets = new SubflowSet[n];
		for(int i = 0; i < n; i++){
			SubflowSet set = new SubflowSet();

			set.in = new FlowNode(TunnelNodeModel.inModel);
			set.out = new FlowNode(TunnelNodeModel.outModel);
			
			FlowGraph graph = new FlowGraph();
			graph.addVertex(set.in);
			graph.addVertex(set.out);
			
			Map<Vertex, Point2D> map = new HashMap<Vertex, Point2D>();
			map.put(set.in, new Point2D.Double(50, 100));
			map.put(set.out, new Point2D.Double(250, 100));
			
			set.layout = new ViewLayout(graph, map, new Dimension(300, 200), new Point(0, 0));
			
			subflowSets[i] = set;
		}
	}
	
	public SubflowNodeModel(ViewLayout layout, FlowNode[] ins, FlowNode out) {
		subflowSets = new SubflowSet[]{new SubflowSet()};
		SubflowSet pack = getSubflowPack();
		
		pack.layout = layout;
		
		pack.in = (FlowNode)layout.getGraph().addVertex(new FlowNode(TunnelNodeModel.inModel));
		pack.out = (FlowNode)layout.getGraph().addVertex(new FlowNode(TunnelNodeModel.outModel));

		int offset = 50;
		
		FlowNode in0 = ins[0];
		Point2D inPoint = layout.getLocationMap().get(in0);
		layout.getLocationMap().put(pack.in, new Point2D.Double(inPoint.getX()-offset, inPoint.getY()));
		for(FlowNode in : ins)
			layout.getGraph().addEdge(new FlowEdge(pack.in, in));
		
		
		layout.getGraph().addEdge(new FlowEdge(out, pack.out));		
		Point2D outPoint = layout.getLocationMap().get(out);
		layout.getLocationMap().put(pack.out, new Point2D.Double(outPoint.getX()+offset, outPoint.getY()));
	}
	
	public SubflowNodeModel(ViewLayout layout){
		this(new ViewLayout[]{layout});
	}
	
	public SubflowNodeModel(ViewLayout[] layouts){
		subflowSets = new SubflowSet[layouts.length];
		for(int i = 0; i < layouts.length; i++){
			subflowSets[i] = new SubflowSet();
			subflowSets[i].layout = layouts[i];
		}
		setInOutNode();
	}
	
	SubflowNodeModel(SubflowNodeModel model){
		this.subflowSets = new SubflowSet[model.subflowSets.length];
		
		for(int i = 0; i < subflowSets.length; i++){
			subflowSets[i] = new SubflowSet();
			subflowSets[i].layout = model.subflowSets[i].getLayout().copy();
		}
		setInOutNode();
	}
	
	private SubflowSet getSubflowPack(){
		return subflowSets[0];
	}
	
	/**
	 * グラフからin/outノードを見つけ出し、設定する
	 *
	 */
	private void setInOutNode(){
		for(SubflowSet pack : subflowSets){
			boolean inHit = false;
			boolean outHit = false;
			for(Object o : pack.getGraph().getVertices()){
				FlowNode node = (FlowNode)o;
				if(!inHit && node.getModel() == TunnelNodeModel.inModel){
					pack.in = node;
					inHit = true;
				}
				if(!outHit && node.getModel() == TunnelNodeModel.outModel){
					pack.out = node;
					outHit = true;
				}
				if(inHit && outHit)break;
			}
		}
	}
	
	@Override
	public String getDefaultTitle() {
		return "BOX"; //$NON-NLS-1$
	}

	public String getCommand() {
		return getSubflowPack().getCommand();
	}
	
	@Override
	public String getAllCommand() {
		String command = ""; //$NON-NLS-1$
		for(SubflowSet set : subflowSets){
			for(Object o : set.getGraph().getVertices()){
				FlowNode node = (FlowNode)o;
				command += node.getModel().getAllCommand() + Constants.NEWLINE;
			}
		}
		return command;
	}
	
	
	public List<List<FlowNode>> getFlows(){
		List<List<FlowNode>> list = new ArrayList<List<FlowNode>>();
		for(SubflowSet set : subflowSets){
			list.add(set.getFlow());
		}
		return list;
	}
	
	public FlowNode getIn(){
		return getSubflowPack().in;
	}
	
	public FlowNode getOut(){
		return getSubflowPack().out;
	}
	
	public FlowNode getParentNode(){
		return subflowSets[0].getGraph().getParentNode();
	}
	
	public void setParentNode(FlowNode node){
		for(SubflowSet set : subflowSets){
			set.getGraph().setParentNode(node);
		}
	}
	
	public void showSubflowFrame(){
		if(frame != null){
			frame.setVisible(true);
			try {
				frame.setSelected(true);
			} catch (PropertyVetoException e) {
			}
		}
	}
	
	public SubflowFrame getSubflowFrame(Rflow rflow, FlowViewer parentViewer, 
			FlowNode parentNode)
	{

		if(frame != null){
			// frame is already made
			frame.setVisible(true);
			try {
				frame.setIcon(false);
				frame.setSelected(true);
			} catch (PropertyVetoException e) {
			}
			return null;
		}
		
		//create frame
		for(SubflowSet set : subflowSets){
			//if(set.smallViewer == null) {
				set.smallViewer = new FlowViewer(rflow, parentViewer);
				set.layout.getGraph().setParentNode(parentNode);
				set.smallViewer.init(set.layout);
				set.smallViewer.addFocusListener(updateListener);
				set.smallViewer.setPreferredSize(set.layout.getSize());
			//}
		}
		
		FlowViewer[] viewers = new FlowViewer[subflowSets.length];
		for(int i = 0; i < viewers.length; i++){
			viewers[i] = subflowSets[i].smallViewer;
		}

		frame = new SubflowFrame(viewers, createMainComponent(viewers));
		frame.addInternalFrameListener(updateListener);
		update();
		
		return frame;
	}

	protected JPanel createFlowPanel(final FlowViewer viewer){
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());

		final CodeBar codeBar = new CodeBar(viewer);
		codeBar.clear();
		codeBar.setPreferredSize(null);
		codeBar.addAncestorListener(new AncestorListener(){
			public void ancestorRemoved(AncestorEvent event) {
				//System.out.println("removed");
				codeBar.clear();
				viewer.getRootFlowState().removeItemListener(codeBar);
				FlowNode.removeNodeModelChangeListener(codeBar);
			}

			public void ancestorAdded(AncestorEvent event) {
				//System.out.println("added");
				viewer.getRootFlowState().addItemListener(codeBar);
				FlowNode.addNodeModelChangeListener(codeBar);
			}

			public void ancestorMoved(AncestorEvent event) {}
		});
		FlowViewerScrollPane scrollPane = new FlowViewerScrollPane(viewer);
		
		panel.add(codeBar, BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);
		
		return panel;
	}
	
	JComponent createMainComponent(FlowViewer[] viewers){
		return createFlowPanel(viewers[0]);
	}
	
	public boolean isInOutConnected(){
		return getSubflowPack().isInOutConnected();
	}
	
	public boolean isInConnected(){
		return getSubflowPack().isInConnected();
	}
	
	public boolean isOutConnected(){
		return getSubflowPack().isOutConnected();
	}
	
	public void setUpdateListener(){
		for(SubflowSet pack : subflowSets)
			if(pack.smallViewer != null)pack.smallViewer.addFocusListener(updateListener);
		if(frame != null)frame.addInternalFrameListener(updateListener);
	}
	
	public void removeUpdateListener(){
		for(SubflowSet pack : subflowSets)
			if(pack.smallViewer != null)pack.smallViewer.removeFocusListener(updateListener);
		if(frame != null)frame.removeInternalFrameListener(updateListener);
	}
	
	private class UpdateListener extends InternalFrameAdapter implements FocusListener{

		@Override
		public void internalFrameDeactivated(InternalFrameEvent e) {
			//update();
		}
		@Override
		public void internalFrameClosed(InternalFrameEvent e) {
			//update();
		}
		
		public void focusGained(FocusEvent e) {
			FlowViewer viewer = (FlowViewer)e.getComponent();
			viewer.getRootViewer().getRootFlowState().clearOutsidePickedVertices(viewer);
		}

		public void focusLost(FocusEvent e) {
			//update();
		}
	}
	
	public NodeModel copy() {		
		return new SubflowNodeModel(this);
	}
	
	public FlowViewer getActiveViewer(){
		if(activeViewer == null)
			return subflowSets[0].smallViewer;
		return activeViewer;
	}
	
	public FlowGraph[] getGraphs(){
		FlowGraph[] graphs = new FlowGraph[subflowSets.length];
		for(int i = 0; i < graphs.length; i++){
			graphs[i] = subflowSets[i].getGraph();
		}
		
		return graphs;
	}

	public ViewLayout[] getLayouts() {
		ViewLayout[] layouts = new ViewLayout[subflowSets.length];
		for(int i = 0; i < layouts.length; i++){
			layouts[i] = subflowSets[i].getLayout();
		}
		return layouts;
	}
	
	//private static final String PATH_SEP = "/";
	
	public void update(){
		if(frame != null){
			String title = getParentNode().getTitle();
			/*FlowGraph g = getSubflowPack().layout.getGraph().getParentNode().getGraph();
			//System.out.println(g == null);
			while(g != null && !g.isRootGraph()){
				title = g.getParentNode().getTitle() + PATH_SEP + title;
				g = g.getParentNode().getGraph();
			}*/
			frame.setTitle(title);
		}

		/*for(SubflowSet pack : subflowSets){
			if(pack.smallViewer != null)
				pack.layout = pack.smallViewer.getViewLayout();
		}*/
	}
	
	/*public void updateDescendants(){
		updateDescendants(this);
	}
	
	private static void updateDescendants(SubflowNodeModel model){
		FlowGraph[] graphs = model.getGraphs();
		for(FlowGraph graph : graphs){
			for(Object o : graph.getVertices()){
				NodeModel nm = ((FlowNode)o).getModel();
				if (nm.isSubflow()) {
					updateDescendants((SubflowNodeModel) nm);
				}
			}
		}
		model.update();
	}*/
	
	public void clearFrame(){
		if(frame != null){
			frame.setVisible(false);
			frame = null;
		}
		for(SubflowSet pack : subflowSets)
			pack.smallViewer = null;
	}

	@Override
	public Icon getIcon() {
		return FileResource.BOX;
	}
	
	
	@Override
	public Icon getSmallIcon() {
		return SMALLICON;
	}
	
	@Override
	public boolean isSubflow() {
		return true;
	}
	
	public boolean isOpenSubflow(){
		return true;
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		//TODO BoxNodeModel equals実装
		/*if (obj instanceof BoxNodeModel) {
			BoxNodeModel model = (BoxNodeModel) obj;

			return 
				super.equals(model)
				&&
				ObjectUtilities.equals(layout, model.layout);
		}*/
		return false;
	}
	
	protected static class SubflowSet {
		private ViewLayout layout;
		FlowNode in;
		FlowNode out;

		FlowViewer smallViewer;
		
		public FlowGraph getGraph(){
			if(smallViewer != null)
				return smallViewer.getGraph();
			else
				return layout.getGraph();
		}
		
		public ViewLayout getLayout(){
			if(smallViewer != null){
				layout = smallViewer.getViewLayout();
			}
			return layout;
		}

		public List<FlowNode> getFlow(){
			return FlowUtil.getStreamSimple(out);
		}
		
		public String getCommand() {
			FlowNode out = this.out;
			if(!isInOutConnected())
				return ""; //$NON-NLS-1$
			FlowNode node = (FlowNode)out.getPredecessors().iterator().next();
			List<FlowNode> stream = FlowUtil.getStreamSimple(node);
			//if(stream.get(0).getModel() == TunnelNodeModel.inModel)
			//	stream.remove(0);
			return FlowUtil.getCommandFromStream(stream, false);
		}
		

		public boolean isInOutConnected(){
			for(FlowNode node : getFlow()){
				if(node.getModel().isSubflow()){
					SubflowNodeModel model = (SubflowNodeModel)node.getModel();
					if(!model.isInOutConnected())
						return false;
				}
			}
			return FlowUtil.isHeadInNode(out);
		}

		public boolean isInConnected(){
			return this.in.outDegree() > 0;
		}
		
		public boolean isOutConnected(){
			return this.out.inDegree() > 0;
		}
	}
}
