package com.ef_prime.rflow.node.pack;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.HashSet;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

import org.rosuda.JRI.REXP;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.exception.InvalidOptionException;
import com.ef_prime.rflow.tool.RflowTextField;
import com.ef_prime.rflow.tool.SupportedInputFactory;
import com.ef_prime.rflow.util.FileChooserUtility;
import com.ef_prime.rflow.util.FileUtility;
import com.ef_prime.rflow.util.RObjFilter;
import com.ef_prime.rflow.util.RTypeFilter;

public class DefaultArgComponents {
	
	private static final int DEFAULT_INPUT_WIDTH = 150;
	private static final int DEFAULT_HEIGHT = 20;
	private static final Dimension DEFAULT_INPUT_SIZE = new Dimension(DEFAULT_INPUT_WIDTH, DEFAULT_HEIGHT);
	
	public static ACFreeInput createSymbolInput(String label){
		ACFreeInput fi = new ACFreeInput("", label, false); //$NON-NLS-1$
		fi.field.setPreferredSize(DEFAULT_INPUT_SIZE);
		fi.field.setMaximumSize(DEFAULT_INPUT_SIZE);
		return fi;
	}
	
	public static ACFreeInput createFreeInput(String param, String label, boolean quote){
		return new ACFreeInput(param, label, quote);
	}
	
	public static ACFileInput createFileInput(String param, String label, boolean quote){
		return new ACFileInput(param, label, quote);
	}
	
	public static ACFreeInput createSupportedFreeInput(String param, String label, 
			RObjFilter filter)
	{
		ACFreeInput input = new ACFreeInput(param, label, false);
		SupportedInputFactory.setFunctionSupport(input.field, filter);
		
		return input;
	}
	
	public static ACRadioButtons createRadioButton(String param, String title,
			String[] labels, String[] values, boolean quote, boolean x_axis)
	{
		return new ACRadioButtons(param, title, labels, values, quote, x_axis);
	}

	public static ACComboBox createComboBox(String param, String title,
			String[] labels, String[] values, boolean editable)
	{
		return new ACComboBox(param, title, labels, values, editable);
	}
	
	public static ACComboBox createObjectComboBox(String param, String title, RObjFilter filter, boolean editable){
		String[] values = getFilteredObject(filter);
		return new ACComboBox(param, title, values, values, editable);
	}

	public static ACComboBox createDataFrameChooser(String param){
		return createObjectComboBox(param, "データ", RTypeFilter.DataFrameFilter, false);
	}
	
	public static ACList createList(String param, String title,
			String[] labels, String[] values, boolean singleSelection)
	{
		return new ACList(param, title, labels, values, singleSelection);
	}

	public static ACList createObjectList(String param, String title,
			RObjFilter filter, boolean singleSelection)
	{
		String[] values = getFilteredObject(filter);
		return new ACList(param, title, values, values, singleSelection);
	}	
	
	public static ArgComponent createFormulaVariableChooser(String paramFormula, String paramData, String title, boolean singleSelection){
		ArgComponent ac = ArgComponent.createNoParamArgumentComponent();
		ac.setLayout(new BoxLayout(ac, BoxLayout.Y_AXIS));
		ac.setBorder(new TitledBorder(title));

		ACComboBox data = createDataFrameChooser(paramData);
		ACFormulaVariableChooser chooser = new ACFormulaVariableChooser(paramFormula, singleSelection, data.comboBox);
		data.setAlignmentX(1.0f);
		chooser.setAlignmentX(1.0f);
		
		ac.add(data);
		ac.add(chooser);
		
		ac.addArgumentComponent(data);
		ac.addArgumentComponent(chooser);

		return ac;
	}
	
	public static ArgComponent createSimpleFormulaEditor(String paramFormula, String paramData, 
			String title, String title1, String title2)
	{
		ArgComponent ac = ArgComponent.createNoParamArgumentComponent();
		ac.setLayout(new BoxLayout(ac, BoxLayout.Y_AXIS));
		ac.setBorder(new TitledBorder(title));
		
		ACComboBox data = createDataFrameChooser(paramData);
		ACSimpleFormula formula = new ACSimpleFormula(paramFormula, title1, title2, data.comboBox);
		
		ac.add(data);
		ac.add(formula);
		
		ac.addArgumentComponent(data);
		ac.addArgumentComponent(formula);
		
		ac.setMaximumSize(ac.getPreferredSize());
		
		return ac;
	}
	
	public static ACCheckBox createCheckBox(String param, String label){
		return new ACCheckBox(param, label);
	}
	
	public static ACNumberSpinner createNumberSpinner(String param, String label,
			Number min, Number max, Number step, boolean isInt)
	{
		return new ACNumberSpinner(param, label, min, max, step, isInt);
	}
	
	private static String[] getFilteredObject(RObjFilter filter){
		String[] objects = RCommand.getObjectsFiltered(filter.getFilterFunction());
		if(objects == null)
			return new String[]{""}; //$NON-NLS-1$
		String[] values = new String[objects.length+1];
		values[0] = ""; //$NON-NLS-1$
		System.arraycopy(objects, 0, values, 1, objects.length);
		return values;
	}
	
	private static boolean isQuoted(String value){
		if(value.length() < 2)
			return false;
		return (value.charAt(0) == '"') && (value.charAt(value.length()-1) == '"');
	}
	
	
	public static class ACFreeInput extends ArgComponent{
		
		public final JTextField field;
		private boolean quote;
		
		public ACFreeInput(String param, String label, boolean quote) {
			super(param);
			this.quote = quote;
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_HEIGHT));
			
			JLabel l = new JLabel(label);
			field = new RflowTextField();
			
			add(l);
			add(field);
		}

		@Override
		public String getValue() throws InvalidOptionException {
			if(quote)
				return Constants.quote(field.getText());
			return field.getText();
		}

		@Override
		public void setValueToComponent(String value) {
			if(quote)
				value = Constants.unquote(value);
			field.setText(value);
		}
		
		@Override
		protected boolean isValidValue(String value) {
			if(quote){
				return isQuoted(value);
			}
			return true;
		}
	}
	
	public static class ACFileInput extends ACFreeInput{

		private FileFilter[] filters = new FileFilter[0];
		
		public ACFileInput(String param, String label, boolean quote) {
			super(param, label, quote);
			
			JButton ref = new JButton("..."); //$NON-NLS-1$
			//ref.setMaximumSize(new Dimension(20, DEFAULT_HEIGHT));
			ref.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					File file = FileChooserUtility.chooseOpenFile("ファイル選択", Prefs.WORKING_DIRECTORY, filters, ACFileInput.this); //$NON-NLS-1$
					if(file == null)return;
					File abstractFile = FileUtility.makeAbstractPath(Prefs.WORKING_DIRECTORY, file);
					
					field.setText(abstractFile.getPath());
				}
			});
			
			add(ref);
		}
		
		public void setFileFilter(FileFilter[] filters) {
			this.filters = filters.clone();
		}

		@Override
		public String getValue() throws InvalidOptionException {
			return Constants.replaceFileSeparator(super.getValue());
		}
	}

	public static class ACRadioButtons extends ArgComponent{
		
		public final JRadioButton[] buttons;
		private String[] values;
		public JTextField field;
		private boolean quote;

		public ACRadioButtons(String param, String title, String[] labels, String[] values,
				boolean quote, boolean x_axis) 
		{
			super(param);
			//setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			setLayout(new BoxLayout(this, (x_axis)?BoxLayout.X_AXIS:BoxLayout.Y_AXIS));
			setBorder(new TitledBorder(title));

			this.quote = quote;
			this.values = values.clone();
				
			buttons = new JRadioButton[labels.length];
			
			ButtonGroup group = new ButtonGroup();
			for(int i = 0; i < labels.length; i++){
				buttons[i] = new JRadioButton(labels[i]);
				group.add(buttons[i]);
				if(values[i] == null){//自由入力
					if(field != null)
						throw new RuntimeException("cannot make free input > 1"); //$NON-NLS-1$
					Box box = Box.createHorizontalBox();
					field = new RflowTextField(""); //$NON-NLS-1$
					//field.setHorizontalAlignment(JTextField.RIGHT);
					field.setPreferredSize(new Dimension(50, DEFAULT_HEIGHT));
					field.setMaximumSize(DEFAULT_INPUT_SIZE);
					box.add(buttons[i]);
					box.add(field);
					if(!x_axis)
						box.setAlignmentX(0.0f);
					add(box);
				}else{
					if(!x_axis)
						buttons[i].setAlignmentX(0.0f);
					add(buttons[i]);
				}
			}
		}

		@Override
		public String getValue() throws InvalidOptionException {
			for(int i = 0; i < buttons.length; i++){
				if(buttons[i].isSelected()){
					if(values[i] == null){
						if(quote)
							return Constants.quote(field.getText());
						return field.getText();
					}else{
						return values[i];
					}
				}
			}
			throw new RuntimeException();
		}

		@Override
		public void setValueToComponent(String value) {
			int freeInput = -1;
			for(int i = 0; i < values.length; i++){
				if(values[i] == null)
					freeInput = i;
				else if(values[i].equals(value)){
					buttons[i].setSelected(true);
					return;
				}
			}
			buttons[freeInput].setSelected(true);
			if(quote)
				value = Constants.unquote(value);
			field.setText(value);
			return;
		}
		

		@Override
		protected boolean isValidValue(String value) {
			if(field != null){
				if(quote)
					return isQuoted(value);
				return true;
			}else{
				if(quote && !isQuoted(value))
					return false;
				for(int i = 0; i < values.length; i++){
					if(values[i].equals(value))
						return true;
				}
				return false;
			}
		}
	}
	
	public static class ACComboBox extends ArgComponent{
		
		public JComboBox comboBox;
		private String[] values;
		
		public ACComboBox(String param, String title, String[] labels, String[] values, boolean editable)
		{
			super(param);
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			//setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_HEIGHT));
			
			this.values = values.clone();
			
			JLabel label = new JLabel(title);
			comboBox = new JComboBox(labels);
			comboBox.setEditable(editable);
			//if(editable){
				comboBox.setPreferredSize(DEFAULT_INPUT_SIZE);
				comboBox.setMaximumSize(DEFAULT_INPUT_SIZE);
			//}
			
			add(label);
			add(comboBox);
		}
		
		@Override
		public String getValue() throws InvalidOptionException {
			int index = comboBox.getSelectedIndex();
			if(index < 0)
				return (String)comboBox.getSelectedItem();
			return values[comboBox.getSelectedIndex()];
		}
		@Override
		public void setValueToComponent(String value) {
			for(int i = 0; i < values.length; i++){
				if(values[i].equals(value)){
					comboBox.setSelectedIndex(i);
					return;
				}
			}
			if(comboBox.isEditable())
				comboBox.setSelectedItem(value);
		}
		
		@Override
		protected boolean isValidValue(String value) {
			if(comboBox.isEditable())return true;
			for(int i = 0; i < values.length; i++){
				if(values[i].equals(value))
					return true;
			}
			return false;
		}
	}
	
	public static class ACList extends ArgComponent{

		public JList list;
		protected String[] values;
		private boolean sigleSelection;
		
		public ACList(String param, String title, String[] labels, String[] values,
				boolean sigleSelection) 
		{
			super(param);
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			
			this.values = values.clone();
			this.sigleSelection = sigleSelection;
			
			if(title != null){
				JLabel label = new JLabel(title);
				label.setPreferredSize(DEFAULT_INPUT_SIZE);
				label.setMaximumSize(DEFAULT_INPUT_SIZE);
				label.setAlignmentX(0.0f);
				add(label);
			}
			
			
			list = new JList(labels);
			list.setSelectionMode((sigleSelection)?
					ListSelectionModel.SINGLE_SELECTION:ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			
			JScrollPane sp = new JScrollPane(list);
			sp.setPreferredSize(new Dimension(150, 100));
			sp.setMaximumSize(sp.getPreferredSize());
			sp.setAlignmentX(0.0f);
			
			add(sp);
		}
		
		@Override
		public String getValue() throws InvalidOptionException {
			if(sigleSelection){
				int index = list.getSelectedIndex();
				if(index < 0)return ""; //$NON-NLS-1$
				return values[list.getSelectedIndex()];
			}else{
				int[] selectedIndices = list.getSelectedIndices();
				String[] selected = new String[selectedIndices.length];
				for(int i = 0; i < selected.length; i++){
					selected[i] = values[selectedIndices[i]];
				}
				return Constants.combine(selected);
			}
		}
		@Override
		public void setValueToComponent(String value) {
			if(sigleSelection){
				for(int i = 0; i < values.length; i++){
					if(values[i].equals(value)){
						list.setSelectedIndex(i);
						return;
					}
				}
			}else{
				String[] vars = RCommand.getAllVars(value);
				if(vars == null)return;
				HashSet<String> varSet = new HashSet<String>();
				for(String var : vars)varSet.add(var);
				for(int i = 0; i < values.length; i++){
					if(varSet.contains(values[i])){
						list.addSelectionInterval(i, i);
					}
				}
			}
		}
	}
	
	public static class ACVariableChooser extends ACList{

		public JComboBox dataComboBox;
		
		public ACVariableChooser(String param, String title, 
				JComboBox dataComboBox, boolean singleSelection) 
		{
			super(param, title, new String[]{""}, new String[]{""}, singleSelection); //$NON-NLS-1$ //$NON-NLS-2$
			list.setModel(new DefaultListModel());
			
			this.dataComboBox = dataComboBox;
			
			dataComboBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					updateList();
				}
			});
			
			updateList();
		}
		
		public void updateList(){
			JComboBox comboBox = dataComboBox;
			String dataframe = (String)comboBox.getSelectedItem();
        	DefaultListModel listModel = (DefaultListModel)list.getModel();
			if(dataframe.equals("")){ //$NON-NLS-1$
	        	listModel.clear();
			}else{
				REXP names = RCommand.getR().eval("names("+dataframe+")"); //$NON-NLS-1$ //$NON-NLS-2$
				String[] c = null;
		        if (names != null && (c=names.asStringArray()) != null){
		        	listModel.clear();
		        	for(String var : c){
		        		listModel.addElement(var);
		        	}
		        	values = c;
	        }	
			}
		}
	}
	
	public static class ACFormulaVariableChooser extends ArgComponent{

		public ACVariableChooser variables;
		
		public ACFormulaVariableChooser(String param, boolean singleSelection, 
				JComboBox dataComboBox) 
		{
			super(param);
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			dataComboBox.setPreferredSize(DEFAULT_INPUT_SIZE);
			dataComboBox.setMaximumSize(DEFAULT_INPUT_SIZE);
			variables = new ACVariableChooser(param, null, dataComboBox, singleSelection);
			add(variables);
		}
		
		@Override
		public String getValue() throws InvalidOptionException {
			String exp = variables.getValue();
			String[] exps = RCommand.getAllVars(exp);
			if(exps == null || exps.length < 1)return ""; //$NON-NLS-1$
			String expFormula = ""; //$NON-NLS-1$
			for(String e : exps)
				expFormula += e + " + "; //$NON-NLS-1$
			expFormula = expFormula.substring(0, expFormula.length()-3);
			return "~ " + expFormula; //$NON-NLS-1$
		}
		
		@Override
		public void setValueToComponent(String value) {
			String[] vars = RCommand.getAllVars(value);
			if(vars == null || vars.length < 1)return;
			//TODO .の変換等
			variables.setValueToComponent(Constants.combine(vars));
		}
		
	}
	
	public static class ACSimpleFormula extends ArgComponent{

		public ACVariableChooser response;
		public ACVariableChooser explain;
		
		public ACSimpleFormula(String param, String title1, String title2,
				JComboBox dataComboBox) 
		{
			super(param);
			
			response = new ACVariableChooser(param, title1, dataComboBox, true);
			explain = new ACVariableChooser(param, title2, dataComboBox, false);
			
			add(response);
			add(explain);
		}
		
		@Override
		public String getValue() throws InvalidOptionException {
			String res = response.getValue();
			String exp = explain.getValue();
			String[] exps = RCommand.getAllVars(exp);
			if(exps == null || exps.length < 1)return ""; //$NON-NLS-1$
			String expFormula = ""; //$NON-NLS-1$
			for(String e : exps)
				expFormula += e + " + "; //$NON-NLS-1$
			expFormula = expFormula.substring(0, expFormula.length()-3);
			return res + " ~ " + expFormula; //$NON-NLS-1$
		}
		
		@Override
		public void setValueToComponent(String value) {
			String[] vars = RCommand.getAllVars(value);
			if(vars == null)return;
			response.setValueToComponent(vars[0]);
			//TODO .の変換等
			String[] exps = new String[vars.length-1];
			System.arraycopy(vars, 1, exps, 0, exps.length);
			explain.setValueToComponent(Constants.combine(exps));
		}
	}

	public static class ACCheckBox extends ArgComponent{
		
		public final JCheckBox checkBox;
		
		public ACCheckBox(String param, String label) {
			super(param);
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_HEIGHT));
			
			checkBox = new JCheckBox(label);
			
			add(checkBox);
		}

		@Override
		public String getValue() throws InvalidOptionException {
			return Constants.toRBoolean(checkBox.isSelected());
		}

		@Override
		public void setValueToComponent(String value) {
			checkBox.setSelected(Constants.toBoolean(value));
		}
		
		@Override
		protected boolean isValidValue(String value) {
			try{
				Constants.toBoolean(value);
				return true;
			}catch(IllegalArgumentException e){
				return false;
			}
		}
	}


	public static class ACNumberSpinner extends ArgComponent{
		
		public final JSpinner spinner;
		private boolean isInt;
		
		public ACNumberSpinner(String param, String label,
				Number min, Number max, Number step, boolean isInt) 
		{
			super(param);
			this.isInt = isInt;
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_HEIGHT));
			
			SpinnerNumberModel model;
			if(isInt){
				model = new SpinnerNumberModel((Integer)min,(Integer)min,(Integer)max,(Integer)step);
			}else{
				model = new SpinnerNumberModel((Double)min,(Double)min,(Double)max,(Double)step);
			}
			spinner = new JSpinner(model);
			spinner.setPreferredSize(DEFAULT_INPUT_SIZE);
			spinner.setMaximumSize(DEFAULT_INPUT_SIZE);
			
			add(new JLabel(label));
			add(spinner);
		}

		@Override
		public String getValue() throws InvalidOptionException {
			return spinner.getValue().toString();//TODO Ｒとの互換の確認
		}

		@Override
		public void setValueToComponent(String value) {
			Number num;
			if(isInt)
				num = Integer.parseInt(value);
			else
				num = Double.parseDouble(value);
			spinner.setValue(num);
		}
		
		@SuppressWarnings("unchecked") //$NON-NLS-1$
		@Override
		protected boolean isValidValue(String value) {
			try{
				Number num;
				if(isInt)
					num = Integer.parseInt(value);
				else
					num = Double.parseDouble(value);
				SpinnerNumberModel model = (SpinnerNumberModel)spinner.getModel();
				return model.getMinimum().compareTo(num) <= 0 
					&& model.getMaximum().compareTo(num) >= 0;
			}catch(NumberFormatException e){
				return false;
			}
		}
	}
}
