package com.ef_prime.rflow.node.pack;

import java.awt.BorderLayout;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.RflowUtil;
import com.ef_prime.rflow.exception.BuildFailedException;
import com.ef_prime.rflow.exception.InvalidOptionException;
import com.ef_prime.rflow.node.NodeModel;
import com.ef_prime.rflow.node.NodeOptionPanel;
import com.ef_prime.rflow.node.SentenceInfo;
import com.ef_prime.rflow.node.base.ArgEntry;
import com.ef_prime.rflow.node.base.ArgListMod;
import com.ef_prime.rflow.node.base.SentenceNodeModel;

public abstract class UIPackPanelBuilder{

	public NodeOptionPanel build(SentenceNodeModel model) throws BuildFailedException{
		return new UINodeOptionPanel(model,
				getSetting(),
				getArgComponents(),
				getAssignCompIndex(),
				getPanelIndices(),
				getPanelTitles());
	}
	
	protected UIPackPanelSetting getSetting() {
		return new UIPackPanelSetting();
	}
	
	protected abstract ArgComponent[] getArgComponents() throws BuildFailedException;
	
	protected abstract int getAssignCompIndex();
	
	protected abstract int[] getPanelIndices();
	
	protected abstract String[] getPanelTitles();
}

class UINodeOptionPanel extends NodeOptionPanel{
	
	private SentenceInfo info;
	private ArgListMod orgArgList;
	private ArgComponent symbolComp;
	private int assignCompIndex;
	private ArgComponent[] ac;
	
	public UINodeOptionPanel(SentenceNodeModel model, 
			UIPackPanelSetting setting,
			ArgComponent[] argComponents,
			int assignCompIndex,
			int[] panelIndices,
			String[] panelTitles)
	{
		info = RflowUtil.retrieveSentenceInfo(model.getCommand());
		orgArgList = info.getArgList();
		orgArgList.setDefaultList(info.getFunction());
		this.ac = argComponents;
		this.assignCompIndex = assignCompIndex;
		boolean isAssignComponent = (assignCompIndex >= 0);
		if(isAssignComponent)
			this.symbolComp = argComponents[assignCompIndex];
		if(panelIndices.length == 0){
			panelIndices = new int[]{0};
		}
		
		//help 
		if(false){
			Map<String, String> map = RflowUtil.getArgHelp(info.getFunction());
			for(ArgComponent ac : argComponents){
				String des = map.get(ac.getParameter());
				if(des != null){
					des = des.replaceAll("\n", "<br>"); //$NON-NLS-1$ //$NON-NLS-2$
					des = "<html>"+des+"</html>"; //$NON-NLS-1$ //$NON-NLS-2$
					ac.setToolTipText(des);
				}
			}
		}
		
		/*
		 * 
		 */
		JPanel[] panels = new JPanel[panelIndices.length];
		for(int i = 0; i < panels.length; i++){
			JPanel mainPanel = new JPanel();
			mainPanel.setBorder(new EmptyBorder(10,10,10,10+16));
			mainPanel.setPreferredSize(setting.size);
			mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
			panels[i] = mainPanel;
		}
		int vgap = 5;
		
		//Set<String> paramSet = new HashSet<String>();

		for(int i = 0, panelIndex = 0; i < ac.length; i++){
			ArgComponent c = ac[i];
			JPanel panel = c.getComponentPanel();
			panel.setAlignmentX(0.0f);
			if(panelIndex+1 < panelIndices.length &&  i == panelIndices[panelIndex+1])
				panelIndex++;
			JPanel mainPanel = panels[panelIndex];
			mainPanel.add(panel);
			mainPanel.add(Box.createVerticalStrut(vgap));
//			if(i != assignCompIndex){
//				paramSet.add(c.getParameter());
//			}
		}
		
		//set initial value
		for(int i = 0; i < ac.length; i++){
			ArgComponent c = ac[i];
			if(i != assignCompIndex){
				setValue(c, orgArgList);
			}else{
				c.setValue(model.getSymbol());
			}
		}
		
		
		setLayout(new BorderLayout());
		if(panels.length > 1){
			JTabbedPane tabpane = new JTabbedPane();
			for(int i = 0; i < panels.length; i++){
				tabpane.add(panels[i], panelTitles[i]);
			}
			add(tabpane);
		}else{
			add(panels[0]);
		}
	}
	
	private void setValue(ArgComponent c, ArgListMod argList){
		if(c.getParameter() != null){
			String value = argList.getValue(c.getParameter(), true);
			c.setValue((value == null)?"":value); //$NON-NLS-1$
		}
		if(c.hasChild()){
			Iterator<ArgComponent> ite = c.children();
			while(ite.hasNext())
				setValue(ite.next(), argList);
		}
	}
	
	@Override
	public NodeModel getModel() throws InvalidOptionException {
		//UINodeModel model = new UINodeModel(info);
		String symbol = ""; //$NON-NLS-1$
		ArgListMod argList = new ArgListMod();
		if(symbolComp != null){
			if(symbolComp.isOperative())
				symbol = symbolComp.getValue();
			else
				symbol = info.getSymbol();
		}
		for(int i = 0; i < ac.length; i++){
			ArgComponent c = ac[i];
			if(i != assignCompIndex){
				getValue(c, argList);
			}
		}
		//argList.setDefaultList(info.getFunction());
		argList.setDefaultList(orgArgList.getDefaultList());
		SentenceInfo newInfo = new SentenceInfo(symbol, info.getFunction(), false, argList.list(true, false));

		SentenceNodeModel model = new SentenceNodeModel();
		model.setCommand(newInfo);
		return model;
	}
	
	private void getValue(ArgComponent c, ArgListMod argList) throws InvalidOptionException{
		if(c.getParameter() != null){
			if(c.isOperative())
				argList.add(new ArgEntry(c.getParameter(), c.getValue()));
			else
				argList.add(new ArgEntry(c.getParameter(), orgArgList.getValue(c.getParameter(), true)));
		}
		if(c.hasChild()){
			Iterator<ArgComponent> ite = c.children();
			while(ite.hasNext())
				getValue(ite.next(), argList);
		}
	}
}
