package com.ef_prime.rflow.node.pack;

import java.awt.Dimension;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.JOptionPane;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.exception.BuildFailedException;
import com.ef_prime.rflow.io.ElementReader;
import com.ef_prime.rflow.io.SingleElementReader;
import com.ef_prime.rflow.resource.Messages;

public class UIPackReader extends ElementReader{
	/*--------------------------element----------------------------*/
	private static final String UIPLUGIN = "uiplugin"; //$NON-NLS-1$
	
	private static final String PACK = "pack"; //$NON-NLS-1$
	private static final String ATTR_FUNCTION = "function"; //$NON-NLS-1$
	
	private static final String PANEL = "panel"; //$NON-NLS-1$
	
	private static final String ASSIGNMENT_COMPONENT = "assigncomponent"; //$NON-NLS-1$
	private static final String COMPONENT = "component"; //$NON-NLS-1$
	private static final String ATTR_ARG = "arg"; //$NON-NLS-1$
	private static final String ATTR_TITLE = "title"; //$NON-NLS-1$
	private static final String ATTR_SIZE = "size"; //$NON-NLS-1$
	private static final String ATTR_TYPE = "type"; //$NON-NLS-1$
	
	private static final String ENTRY = "entry"; //$NON-NLS-1$
	private static final String ENTRYLIST = "entrylist"; //$NON-NLS-1$
	private static final String ATTR_KEY = "key"; //$NON-NLS-1$
	
	/*--------------------------data----------------------------*/
	private List<UIPack> packs = new ArrayList<UIPack>();
	
	private UIPackData data;
	
	private ArrayList<ArgComponentBuilder> builderList = new ArrayList<ArgComponentBuilder>();
	private int assignIndex = -1;
	private List<Integer> panelIndex = new ArrayList<Integer>();
	private List<String> panelTitles = new ArrayList<String>();
	
	private Properties entrys = new Properties();

	/*--------------------------control----------------------------*/
	private List<String> entryList = null;//non null value means currently in entrylist

	/*--------------------------other----------------------------*/
	private ArgComponentManager manager = new ArgComponentManager();
	private StringWriter errorLog = new StringWriter();
	private PrintWriter errorLogWriter = new PrintWriter(errorLog);
	
	
	public UIPackReader(){
		addSingleElementReader(UIPLUGIN, new UIPluginReader());
		addSingleElementReader(PACK, new PackReader());
		addSingleElementReader(PANEL, new PanelReader());
		addSingleElementReader(ASSIGNMENT_COMPONENT, new AssignComponentReader());
		addSingleElementReader(COMPONENT, new ComponentReader());
		addSingleElementReader(ENTRY, new EntryReader());
		addSingleElementReader(ENTRYLIST, new EntryListReader());
	}
	
	public UIPack[] getUIPacks(){
		return packs.toArray(new UIPack[0]);
	}
	
	private void init(){
		builderList = new ArrayList<ArgComponentBuilder>();
		assignIndex = -1;
		panelIndex = new ArrayList<Integer>();
		
		entrys = new Properties();
		entryList = null;//non null value means currently in entrylist
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private UIPack getUIPack(){
		final int[] panelIndexArray = new int[panelIndex.size()];
		for(int i = 0; i < panelIndexArray.length; i++)
			panelIndexArray[i] = panelIndex.get(i);
		final String[] panelTitleArray = panelTitles.toArray(new String[0]);
		
		final ArrayList<ArgComponentBuilder> builderList = (ArrayList<ArgComponentBuilder>)this.builderList.clone();
		final int assignIndex = this.assignIndex;
		
		UIPackPanelBuilder builder = new UIPackPanelBuilder(){
			
			@Override
			protected ArgComponent[] getArgComponents() throws BuildFailedException{
				ArgComponent[] ac = new ArgComponent[builderList.size()];
				for(int i = 0; i < builderList.size(); i++){
					ac[i] = builderList.get(i).buildArgComponent();
				}
				
				return ac;
			}
			
			@Override
			protected int getAssignCompIndex() {
				return assignIndex;
			}
			
			@Override
			protected int[] getPanelIndices() {
				return panelIndexArray;
			}
			
			@Override
			protected String[] getPanelTitles() {
				return panelTitleArray;
			}
		};
		
		return new UIPack((UIPackData)data.clone(), builder);
	}
	
	private class UIPluginReader extends SingleElementReader{
		@Override
		public void endElement(String text) throws SAXException {
			if(errorLog.getBuffer().length() > 0)
				JOptionPane.showMessageDialog(null, errorLog.getBuffer().toString(), Messages.getString("Error.title.warning"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
		}
	}
	
	private class PackReader extends SingleElementReader{
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			init();
			
			String function = attrs.getValue(ATTR_FUNCTION);
			data = new UIPackData(function, ""); //$NON-NLS-1$
		}
		
		@Override
		public void endElement(String text) throws SAXException {
			
			packs.add(getUIPack());
			
		}
	}
	
	private class PanelReader extends SingleElementReader{
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			panelIndex.add(builderList.size());
			panelTitles.add(attrs.getValue(ATTR_TITLE));
		}
	}
	
	private class AssignComponentReader extends SingleElementReader{
		
		private ComponentReader compReader = new ComponentReader();
		
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			compReader.startElement(attrs);
		}
		
		@Override
		public void endElement(String text) throws SAXException {
			if(compReader.type == null)compReader.type = "AssignInput"; //$NON-NLS-1$
			ArgComponentBuilder builder = manager.getArgComponentBuilder(
					compReader.type, compReader.param, compReader.title, compReader.size, entrys);
			if(builder != null){
				assignIndex = builderList.size();
			}
			compReader.endElement(text);
		}
	}
	
	private class ComponentReader extends SingleElementReader{
		
		private String param;
		private String title;
		private Dimension size;
		private String type;
		
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			entrys.clear();
			
			param = attrs.getValue(ATTR_ARG);
			title = attrs.getValue(ATTR_TITLE);
			String sizeStr = attrs.getValue(ATTR_SIZE);
			if(sizeStr == null){
				size = null;
			}else{
				String[] sizes = sizeStr.split(","); //$NON-NLS-1$
				size = new Dimension(Integer.valueOf(sizes[0]), Integer.valueOf(sizes[1]));
			}
			type = attrs.getValue(ATTR_TYPE);
		}
		
		@Override
		public void endElement(String text) throws SAXException {
			ArgComponentBuilder builder = manager.getArgComponentBuilder(
					type, param, title, size, (Properties)entrys.clone());
			if(builder != null){
				builderList.add(builder);
			}else{
				errorLogWriter.println("識別されないタイプです"+": "+type); //$NON-NLS-2$
			}
		}
	}
	
	private class EntryReader extends SingleElementReader{
		
		private String key;
		
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			key = attrs.getValue(ATTR_KEY);
		}
		
		@Override
		public void endElement(String text) throws SAXException {
			if(entryList == null)
				entrys.put(key, text);
			else
				entryList.add(text);
		}
	}
	
	private class EntryListReader extends SingleElementReader{
		
		private String key;
		
		@Override
		public void startElement(Attributes attrs) throws SAXException {
			key = attrs.getValue(ATTR_KEY);
			entryList = new ArrayList<String>();
		}
		
		@Override
		public void endElement(String text) throws SAXException {
			String value = ""; //$NON-NLS-1$
			if(entryList.size() > 0){
				for(String entry : entryList)
					value += entry + Constants.DELIM;
				value = value.substring(0, value.length()-1);
			}
			entrys.put(key, value);
			
			//clear list
			entryList = null;
		}
	}
}
