package com.ef_prime.rflow.object;

import java.awt.Window;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.resource.Messages;

public class DefaultObjectActions {
	
	public static final ObjectAction NULL_ACTION = new NullAction();
	
	public static class NullAction extends ObjectAction{

		@Override
		public void action(RObject[] robj) {
		}

		@Override
		public String getActionName() {
			return ""; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return false;
		}
		
	}
	
	public static class PrintAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "print"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getType().length() > 0;
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand("print("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class SummaryAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "summary"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getType().length() > 0;
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand("summary("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class PageAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "page"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getType().length() > 0;
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand("page("+robj[0].getFullName()+", method=\"print\")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class PlotAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "plot"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			if(robj.length == 1)
				return true;
			int len = robj[0].getLength()[0];
			for(RObject ro : robj){
				if(!(ro.getLength().length == 1 && ro.getLength()[0] == len))
					return false;
			}
			return true;
		}

		@Override
		public void action(RObject[] robj) {
			if(robj.length == 1){
				RCommand.getReceiver().passCommand("plot("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
			}else{
				//データフレーム内のオブジェクトか調べる
				boolean dataframe = false;
				RObject parent = robj[0].getParent();
				if(parent != null && parent.getClas().equals("data.frame")){ //$NON-NLS-1$
					dataframe = true;
					for(RObject ro : robj){
						if(ro.getParent() != null && !parent.equals(ro.getParent())){
							dataframe = false;
							break;
						}
					}
				}
				if(dataframe){
					String c = "~"; //$NON-NLS-1$
					for(RObject ro : robj){
						c += ro.getName()+"+"; //$NON-NLS-1$
					}
					c = c.substring(0, c.length()-1);
					c += ", data="+parent.getFullName(); //$NON-NLS-1$
					RCommand.getReceiver().passCommand("plot("+c+")"); //$NON-NLS-1$ //$NON-NLS-2$
				}else if(robj.length == 2){
					RCommand.getReceiver().passCommand("plot(" //$NON-NLS-1$
							+(robj[0].getFullName()+", "+robj[1].getFullName())+")"); //$NON-NLS-1$ //$NON-NLS-2$
				}else{
					String c = "~"; //$NON-NLS-1$
					for(RObject ro : robj){
						c += ro.getFullName()+"+"; //$NON-NLS-1$
					}
					c = c.substring(0, c.length()-1);
					RCommand.getReceiver().passCommand("plot("+c+")"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
	}
	
	public static class HistAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "hist"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && (robj[0].getClas().equals(RClass.NUMERIC) || robj[0].getClas().equals(RClass.INTEGER));
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand("hist("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class HistAction0 extends ObjectAction{

		@Override
		public String getActionName() {
			return "hist"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && (robj[0].getClas().equals(RClass.NUMERIC) || robj[0].getClas().equals(RClass.INTEGER));
		}

		@Override
		public void action(RObject[] robj) {
			String obj = robj[0].getFullName();
			RCommand.getReceiver().passCommand("hist("+ obj+", col=\"wheat\", sub=ifelse(any(!is.finite("+obj+")), paste(sum(!is.finite("+obj+")), \"value(s) unavailable\"), \"\"), font.sub=2, col.sub=2)"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
	}
	
	public static class PieAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "pie"; //$NON-NLS-1$
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && 
				(robj[0].getClas().equals(RClass.NUMERIC) || 
				robj[0].getClas().equals(RClass.INTEGER) || 
				robj[0].getClas().equals(RClass.CHARACTER) || 
				robj[0].getClas().equals(RClass.FACTOR));
		}

		@Override
		public void action(RObject[] robj) {
			String obj = robj[0].getFullName();
			RCommand.getReceiver().passCommand("pie(table("+ obj+"))"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class LogicalBarplotAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "barplot for logical";
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getClas().equals(RClass.LOGICAL);
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand(
					"(function(x){\n"+ //$NON-NLS-1$
					"xname <- paste(deparse(substitute(x), 100), collapse = \"\\n\")\n" + //$NON-NLS-1$
					"barplot(summary(factor(x), maxsum=6),  col=terrain.colors(3), main=xname)\n" + //$NON-NLS-1$
					"})("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class FactorBarplotAction extends ObjectAction{

		@Override
		public String getActionName() {
			return "barplot for factor";
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getClas().equals(RClass.FACTOR);
		}

		@Override
		public void action(RObject[] robj) {
			RCommand.getReceiver().passCommand(
					"(function(x){\n"+ //$NON-NLS-1$
					"xname <- paste(deparse(substitute(x), 100), collapse = \"\\n\")\n"+ //$NON-NLS-1$
					"barplot(summary(x, maxsum=6),  col=terrain.colors(6)[c(1,3,5,2,4,6)], main=xname)\n"+ //$NON-NLS-1$
					"})("+robj[0].getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static class DataframePlotAction extends ObjectAction{

		@Override
		public void action(RObject[] robj) {

			RObject[] vars = RCommand.getObjects(robj[0], false, 100);
			if(vars == null)return;
			RCommand.getReceiver().passCommand("split.screen(c(2,2))"); //$NON-NLS-1$
			int len = (vars.length > 4)?4:vars.length;
			for(int i = 0; i < len; i++){
				RCommand.getReceiver().passCommand("screen("+(i+1)+")"); //$NON-NLS-1$ //$NON-NLS-2$
				RObject obj = vars[i];
				if(obj.getClas().equals(RClass.NUMERIC) || obj.getClas().equals(RClass.INTEGER)){
					new HistAction0().action(new RObject[]{obj});
				}else if(obj.getClas().equals(RClass.LOGICAL)){
					new LogicalBarplotAction().action(new RObject[]{obj});
				}else if(obj.getClas().equals(RClass.FACTOR)){
					new FactorBarplotAction().action(new RObject[]{obj});
				}
			}
			RCommand.getReceiver().passCommand("close.screen(all.screens=TRUE)"); //$NON-NLS-1$
		}

		@Override
		public String getActionName() {
			return "dataframe plot";
		}

		@Override
		public boolean isActionEnable(RObject[] robj) {
			return robj.length == 1 && robj[0].getClas().equals(RClass.DATAFRAME) && robj[0].getLength()[1] < 26;
		}
		
	}
	
	public static abstract class AbstractDataTableAction extends ObjectAction{

		@Override
		public boolean isActionEnable(RObject[] robj) {
			if(robj.length == 1){
			return robj[0].getClas().equals("data.frame") //$NON-NLS-1$
				|| robj[0].getClas().equals("matrix") //$NON-NLS-1$
				|| robj[0].getType().equals(RType.STRSXP)
				|| robj[0].getType().equals(RType.REALSXP)
				|| robj[0].getType().equals(RType.REALSXPa)
				|| robj[0].getType().equals(RType.LGLSXP)
				|| robj[0].getType().equals(RType.INTSXP)
				//|| robj[0].getType().equals(RType.CPLXSXP)//TODO complexにテーブル対応
				;
			}
			return false;
		}
	}
	
	public static class DataTableAction extends AbstractDataTableAction{

		@Override
		public void action(RObject[] robj) {
			//Window window = SwingUtilities.windowForComponent((Component)e.getSource());
			Window window = null;
			ObjectDataTable table = new ObjectDataTable();
			table.setOnDemand(true);
			table.update(robj[0], 0);
			
			JFrame frame = new JFrame(table.getTitle());
			frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			
			frame.getContentPane().add(table);
			frame.pack();
			frame.setLocationRelativeTo(window);
			frame.setVisible(true);
			
		}

		@Override
		public String getActionName() {
			return Messages.getString("Object.action.datatable"); //$NON-NLS-1$
		}
	}
	
	/**
	 * 固定行数まで読み込んでテーブルを表示します。
	 *
	 */
	public static class HeadDataTableAction extends AbstractDataTableAction{

        private int nrow = 1000;
        
		@Override
		public void action(RObject[] robj) {
			
			//Window window = SwingUtilities.windowForComponent((Component)e.getSource());
			Window window = null;
			ObjectDataTable table = new ObjectDataTable();
			table.update(robj[0], nrow);

			JFrame frame = new JFrame(table.getTitle());
			frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			
			frame.getContentPane().add(table);
			frame.pack();
			frame.setLocationRelativeTo(window);
			frame.setVisible(true);
			
		}

		@Override
		public String getActionName() {
			return "HeadDataTable("+nrow+")";
		}

	}

	/**
	 * 何行読み込みかGUIで問い合わせてテーブルを作成します
	 *
	 */
	public static class HeadDataTableAction2 extends AbstractDataTableAction{

		@Override
		public void action(RObject[] robj) {
			
			JSpinner spinner = new JSpinner(new SpinnerNumberModel(100, 1, Integer.MAX_VALUE, 1));
			JOptionPane optionPane = new JOptionPane(spinner, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
			JDialog opDialog = optionPane.createDialog(null, "表示する行数");
			opDialog.setVisible(true);
			opDialog.dispose();

	        Object selectedValue = optionPane.getValue();
	        if(selectedValue == null || !(selectedValue instanceof Integer))
	        	return;
	        if(((Integer)selectedValue).intValue() != JOptionPane.YES_OPTION)
	        	return;
	        int nrow = ((Integer)spinner.getValue()).intValue();
			
			//Window window = SwingUtilities.windowForComponent((Component)e.getSource());
			Window window = null;
			ObjectDataTable table = new ObjectDataTable();
			table.update(robj[0], nrow);

			JFrame frame = new JFrame(table.getTitle());
			frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			
			frame.getContentPane().add(table);
			frame.pack();
			frame.setLocationRelativeTo(window);
			frame.setVisible(true);
			
		}

		@Override
		public String getActionName() {
			return "HeadDataTable";
		}

	}
}
