package com.ef_prime.rflow.object;

import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.tree.DefaultMutableTreeNode;

public class ObjectActionManager {
	
	private DefaultMutableTreeNode root = new DefaultMutableTreeNode();
	private Hashtable<String, ObjectAction> defaultActions = new Hashtable<String, ObjectAction>();
	private ObjectAction defaultAction;
	
	private static final String SEPARATOR = "<separator>"; //$NON-NLS-1$
	
	public ObjectActionManager(){
		
		defaultActions.put(RClass.DATAFRAME, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.CHARACTER, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.NUMERIC, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.INTEGER, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.LOGICAL, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.FACTOR, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.MATRIX, new DefaultObjectActions.DataTableAction());
		defaultActions.put(RClass.FUNCTION, new DefaultObjectActions.PageAction());
		defaultActions.put(RClass.LIST, DefaultObjectActions.NULL_ACTION);
		defaultActions.put(RClass.ENVIRONMENT, DefaultObjectActions.NULL_ACTION);
		defaultAction = new DefaultObjectActions.SummaryAction();

		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.DataTableAction()));
		root.add(new DefaultMutableTreeNode(SEPARATOR));
		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.PrintAction()));
		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.SummaryAction()));
//		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.PageAction()));
//		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.HeadDataTableAction()));
//		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.HeadDataTableAction2()));
//		DefaultMutableTreeNode plot = new DefaultMutableTreeNode("Plot");
//		plot.add(new DefaultMutableTreeNode(new DefaultObjectActions.PlotAction()));
//		plot.add(new DefaultMutableTreeNode(new DefaultObjectActions.HistAction()));
//		plot.add(new DefaultMutableTreeNode(new DefaultObjectActions.PieAction()));
		//plot.add(new DefaultMutableTreeNode(new DefaultObjectActions.DataframePlotAction()));
		root.add(new DefaultMutableTreeNode(SEPARATOR));
		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.PlotAction()));
		root.add(new DefaultMutableTreeNode(new DefaultObjectActions.HistAction()));
//		root.add(plot);

	}
	
	public JPopupMenu getPopupMenu(RObject[] robj){
		JPopupMenu popup = new JPopupMenu();
		for(Enumeration en = root.children(); en.hasMoreElements(); ){
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)en.nextElement();
			if(n.getUserObject() instanceof ObjectAction){
				ObjectAction action = (ObjectAction)n.getUserObject();
				if(action.isActionEnable(robj))
					popup.add(new ObjectActionImpl(robj, action));
			}if(n.getUserObject().toString().equals(SEPARATOR)){
				if(popup.getComponentCount() > 0 && 
						!(popup.getComponent(popup.getComponentCount()-1) instanceof JSeparator))
					popup.addSeparator();
			}else{
				JMenu m = new JMenu(n.getUserObject().toString());
				if(!n.isLeaf())
					createPopupMenu(robj, m, n);
				if(m.getItemCount() > 0)
					popup.add(m);
			}
		}
		
//		if(popup.getComponentCount() > 0)
			return popup;
//		else 
//			return null;
	}
	
	
	private void createPopupMenu(RObject[] robj, JMenu menu, DefaultMutableTreeNode node){
		for(Enumeration en = node.children(); en.hasMoreElements(); ){
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)en.nextElement();
			if(n.getUserObject() instanceof ObjectAction){
				ObjectAction action = (ObjectAction)n.getUserObject();
				if(action.isActionEnable(robj))
					menu.add(new ObjectActionImpl(robj, action));
			}else if(n.getUserObject().toString().equals(SEPARATOR)){
				if(menu.getItemCount() > 0 && 
					(menu.getMenuComponent(menu.getItemCount()-1) instanceof JSeparator))
					menu.addSeparator();
			}else{
				JMenu m = new JMenu(n.getUserObject().toString());
				if(!n.isLeaf())
					createPopupMenu(robj, m, n);
				if(m.getItemCount() > 0)
					menu.add(m);
			}
		}
	}
	
	public Action getDefaultAction(RObject robj){
		ObjectAction a = defaultActions.get(robj.getClas());
		if(a instanceof DefaultObjectActions.NullAction)
			return null;
		if(a == null)
			a = defaultAction;
		return new ObjectActionImpl(new RObject[]{robj}, a);
	}
	
	class ObjectActionImpl extends AbstractAction{
		
		private RObject[] robj;
		private ObjectAction objAction;
		
		ObjectActionImpl(RObject[] robj, ObjectAction objAction){
			super(objAction.getActionName());
			this.robj = robj;
			this.objAction = objAction;
		}

		public void actionPerformed(ActionEvent e) {
			objAction.action(robj);
		}
	}
}
