package com.ef_prime.rflow.object;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;

import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.resource.Messages;

public class ObjectBrowser extends JPanel{
	
	private ObjectActionManager objActionManager;
	private JTree tree;
	
	public ObjectBrowser(){
		this.setLayout(new BorderLayout());
		
		objActionManager = new ObjectActionManager();
		
//		JButton updateButton = new JButton("update");
//		updateButton.addActionListener(new ActionListener(){
//			public void actionPerformed(ActionEvent e) {
//				update();
//			}
//		});
		
		tree = new JTree(new DefaultTreeModel(new ObjectTreeRoot(new RObject[0])));
		tree.addTreeWillExpandListener(new ObjectTreeWillExpand());
		tree.addKeyListener(new KeyHandler());
		tree.addMouseListener(new MouseHandler());
		tree.setToggleClickCount(3);
		
//		add(updateButton, BorderLayout.NORTH);
		add(new JScrollPane(tree));
		
		
		//create actions
		Action update = new AbstractAction(Messages.getString("Explorer.objects.update")) { //$NON-NLS-1$
			@Override
			public void actionPerformed(ActionEvent e) {
				update();
			}
		};
		update.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		getActionMap().put("Update", update); //$NON-NLS-1$
		getInputMap().put((KeyStroke)update.getValue(Action.ACCELERATOR_KEY), "Update"); //$NON-NLS-1$
	}
	
	/**
	 * グローバル環境のオブジェクト一覧を取得し、ツリーに設定します
	 *
	 */
	public void update(){
		RObject[] objs = RCommand.getObjects(null, false, 200);
		if(objs!=null)
			update(objs);
	}
	
	/**
	 * 与えられたのオブジェクト一覧を、ツリーに設定します
	 * @param objects
	 */
	public void update(RObject[] objects){
		//save expanded and selection
		LinkedList<TreePath> expanded = getExpandedPaths();
		TreePath[] selection = tree.getSelectionPaths();

		//update
		((DefaultTreeModel)tree.getModel()).setRoot(new ObjectTreeRoot(objects));

		//restore expanded and selection
		for(TreePath path : expanded){
			TreePath path2 = getTreePath(path);
			if(path2 != null)
				tree.expandPath(path2);
		}
		if(selection != null)
			tree.setSelectionPaths(selection);
	}
	
	
	/**
	 * 開いているTreePathを長さについて昇順に入ったLinkedListで返します。
	 * @return
	 */
	private LinkedList<TreePath> getExpandedPaths(){
		LinkedList<TreePath> paths = new LinkedList<TreePath>();
		LinkedList<DefaultMutableTreeNode> nodes = new LinkedList<DefaultMutableTreeNode>();
		nodes.add((DefaultMutableTreeNode)tree.getModel().getRoot());
		
		while(nodes.size() > 0){
			Enumeration children = nodes.poll().children();
			while(children.hasMoreElements()){
				DefaultMutableTreeNode n =(DefaultMutableTreeNode)children.nextElement();
				if(!n.isLeaf()){
					TreePath path = new TreePath(n.getPath());
					if(tree.isExpanded(path)){
						nodes.addLast(n);
						paths.add(path);
					}
				}
			}
		}
		
		return paths;
	}
	
	/**
	 * 更新前のモデルのTreePathから現在のモデルのノードのインスタンスが入ったTreePathを取得する
	 * ための関数です。一致するものがない場合nullを返します。
	 * @param path
	 * @return
	 */
	private TreePath getTreePath(TreePath path){
		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode)tree.getModel().getRoot();
		if(!treeNode.equals(path.getPathComponent(0)))return null;
		for(int i = 1; i < path.getPathCount(); i++){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getPathComponent(i);
			int index = 0;
			Enumeration children = treeNode.children();
			while(children.hasMoreElements()){
				if(node.equals(children.nextElement()))
					break;
				index++;
			}
			if(index >= treeNode.getChildCount())return null;
			treeNode = (DefaultMutableTreeNode)treeNode.getChildAt(index);
		}
		
		return new TreePath(treeNode.getPath());
	}
	
	/**
	 * オブジェクトのルートノード
	 *
	 */
	private class ObjectTreeRoot extends DefaultMutableTreeNode{
		ObjectTreeRoot(RObject[] objects){
			super(Messages.getString("Explorer.objects.root")); //$NON-NLS-1$
			for(RObject ro : objects){
				this.add(new ObjectNode(ro));
			}
		}
		
		@Override
		public int hashCode() {
			return getUserObject().hashCode();
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || !(obj instanceof ObjectTreeRoot))
				return false;
			return true;
		}
	}
	
	/**
	 * オブジェクトのツリーノード
	 *
	 */
	private class ObjectNode extends DefaultMutableTreeNode{
		ObjectNode(RObject robj){
			super(robj, !robj.isAtomic());
		}
		@Override
		public boolean isLeaf() {
			return ((RObject)getUserObject()).isAtomic();
		}
		@Override
		public int hashCode() {
			return getUserObject().hashCode();
		}
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || !(obj instanceof ObjectNode))
				return false;
			return getUserObject().equals(((ObjectNode)obj).getUserObject());
		}
	}
	
	/**
	 * ツリー展開リスナー
	 * 展開時に子ノードを作成する処理をします
	 *
	 */
	private class ObjectTreeWillExpand implements TreeWillExpandListener{
		public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
		}
		public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
			JTree tre = (JTree)event.getSource();
	        if (!tre.hasBeenExpanded(event.getPath())) {
	            DefaultMutableTreeNode node = (DefaultMutableTreeNode)event.getPath().getLastPathComponent();
	            if(node.isRoot())return;
	            RObject robj = (RObject)node.getUserObject();
	            RObject[] robjs = RCommand.getObjects(robj, false, 200);
	            if(robjs != null)
					for(RObject ro : robjs){
						node.add(new ObjectNode(ro));
					}
	        }
		}
	}
	
	/**
	 * キーリスナー
	 *
	 */
	private class KeyHandler extends KeyAdapter{
		@Override
		public void keyPressed(KeyEvent e) {
			switch(e.getKeyCode()){
			case KeyEvent.VK_F5:
				update();
				break;
			}
		}
	}
	
	/**
	 * マウスリスナー
	 *
	 */
	private class MouseHandler extends MouseAdapter{
		
		@Override
		public void mouseClicked(MouseEvent e) {
			if(e.getClickCount() == 2 && SwingUtilities.isLeftMouseButton(e)){
				//左ダブルクリック
				JTree tree = (JTree)e.getSource();
				Point p = e.getPoint();
				TreePath path = tree.getPathForLocation((int)p.getX(), (int)p.getY());
				if(path == null)return;
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
				if(node.isRoot())return;
				Action a = objActionManager.getDefaultAction((RObject)node.getUserObject());
				if(a != null){
					a.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_FIRST, "")); //$NON-NLS-1$
				}else{
					if(tree.isExpanded(path))
						tree.collapsePath(path);
					else
						tree.expandPath(path);
				}
			}
		}
		
		@Override
		public void mouseReleased(MouseEvent e) {
			if(SwingUtilities.isRightMouseButton(e)){
				//右クリック
				JTree tree = (JTree)e.getSource();
				Point p = e.getPoint();
				TreePath path0 = tree.getPathForLocation((int)p.getX(), (int)p.getY());
				if(path0 == null)return;

				//選択されていなかったら選択する
				if(!tree.getSelectionModel().isPathSelected(path0))
					tree.setSelectionPath(path0);
				
				TreePath[] paths = tree.getSelectionPaths();
				List<RObject> robjs = new ArrayList<RObject>();
				for(TreePath path : paths){
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
					if(node.isRoot())return;
					RObject robj = (RObject)node.getUserObject();
					robjs.add(robj);
				}
				
				JPopupMenu popup = objActionManager.getPopupMenu(robjs.toArray(new RObject[0]));
				if(popup.getComponentCount() > 0)
					popup.add(new JPopupMenu.Separator());
				popup.add(new JMenuItem(getActionMap().get("Update"))); //$NON-NLS-1$
				popup.show(tree, p.x, p.y);
			}
		}
	}
}
