package com.ef_prime.rflow.object;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.RFactor;
import org.rosuda.JRI.RVector;

import com.ef_prime.rflow.RCommand;
import com.ef_prime.rflow.resource.Messages;

public class ObjectDataTable extends JPanel{
	
	private JTable table;
	private JScrollPane sp;
	
	private RObject robj;
	private int nrow;
	private boolean isOnDemand = false;
	
	public ObjectDataTable(){
		setLayout(new BorderLayout());
		
		JButton updateButton = new JButton(Messages.getString("Object.ObjectDataTable.update")); //$NON-NLS-1$
		updateButton.setPreferredSize(new Dimension(100, 20));
		updateButton.setMaximumSize(updateButton.getPreferredSize());
		updateButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				update();
			}
		});
		
		Box boxNorth = Box.createHorizontalBox();
		boxNorth.add(updateButton);
		boxNorth.add(Box.createHorizontalGlue());
		
		table = new JTable();
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.setRowSelectionAllowed(false);
		table.setColumnSelectionAllowed(false);
		table.addMouseListener(new MouseHandler());
		table.getTableHeader().addMouseListener(new HeaderMouseHandler());
		table.setDefaultRenderer(String.class, new StringCellRenderer());
		table.addKeyListener(new KeyHandler());
		sp = new JScrollPane(table);
		
		add(boxNorth, BorderLayout.NORTH);
		add(sp);
	}
	
	public void setOnDemand(boolean isOnDemand) {
		this.isOnDemand = isOnDemand;
	}
	
	public boolean update(){
		return update(robj, nrow);
	}
	
	public boolean update(RObject robj, int nrow){
		if(robj == null)return false;
		this.robj = robj;
		this.nrow = nrow;
		
		if(isOnDemand){
			try{
				Integer rowCount = RCommand.getLength(robj.getFullName());
				if(rowCount == null) return false;
				table.setModel(new OnDemandTableModel(robj, rowCount.intValue()));
				updateTable();
				return true;
			}catch(Exception e){
				return false;
			}
		}
		
		ObjectDataTableModel model = createTableModel(robj, nrow);
		if(model == null)
			return false;
		else{
			table.setModel(model);
			updateTable();
			return true;
		}
	}

	@SuppressWarnings("unchecked")
	private ObjectDataTableModel createTableModel(RObject robj, int nrow){
		if(robj.getClas().equals(RClass.DATAFRAME)){
			REXP rexp = null;
			if(nrow > 0)
				rexp = RCommand.silentIdleEval("head("+robj.getFullName()+", n="+nrow+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			else
				rexp = RCommand.silentIdleEval("format.data.frame("+robj.getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
				//rexp = RCommand.silentIdleEval(robj.getFullName());
			if(rexp == null)return null;
			RVector v = rexp.asVector();
			if(v == null)return null;
			REXP rownames = rexp.getAttribute("row.names"); //$NON-NLS-1$
			REXP colClass = RCommand.silentIdleEval("sapply("+robj.getFullName()+", class)"); //$NON-NLS-1$ //$NON-NLS-2$
			if(colClass == null)return null;
			String[] colClassA = colClass.asStringArray();
			if(colClassA == null)return null;
			
			return new ObjectDataTableModel(v, v.getNames(), new Vector<String>(Arrays.asList(colClassA)), rownames);
		}else if(robj.getClas().equals(RClass.MATRIX)){
			REXP rexp = null;
			if(nrow > 0)
				rexp = RCommand.silentIdleEval("head("+robj.getFullName()+", n="+nrow+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			else
				rexp = RCommand.silentIdleEval("format("+robj.getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
			REXP dimExp = RCommand.silentIdleEval("dim("+ robj.getFullName() +")"); //$NON-NLS-1$ //$NON-NLS-2$
			REXP dimnames = RCommand.silentIdleEval("dimnames("+ robj.getFullName() +")"); //$NON-NLS-1$ //$NON-NLS-2$
			if(rexp == null || dimExp == null || dimExp.asIntArray() == null)return null;
			int[] dim = dimExp.asIntArray();

			Vector<REXP> v = new Vector<REXP>();
			if(rexp.asFactor() != null){//impossible?
				RFactor rf = rexp.asFactor();
				for(int i = 0; i < dim[1]; i++){
					String[] s = new String[dim[0]];
					for(int j = 0; j < dim[0]; j++){
						s[j] = rf.at(i*dim[0]+j);
					}
					v.add(new REXP(s));
				}
			}else if(rexp.asIntArray() != null){
				int[] array = rexp.asIntArray();
				for(int i = 0; i < dim[1]; i++){
					int[] s = new int[dim[0]];
					for(int j = 0; j < dim[0]; j++){
						s[j] = array[i*dim[0]+j];
					}
					v.add(new REXP(s));
				}
			}else if(rexp.asDoubleArray() != null){
				double[] array = rexp.asDoubleArray();
				for(int i = 0; i < dim[1]; i++){
					double[] s = new double[dim[0]];
					for(int j = 0; j < dim[0]; j++){
						s[j] = array[i*dim[0]+j];
					}
					v.add(new REXP(s));
				}
			}else if(rexp.asStringArray() != null){
				String[] array = rexp.asStringArray();
				for(int i = 0; i < dim[1]; i++){
					String[] s = new String[dim[0]];
					for(int j = 0; j < dim[0]; j++){
						s[j] = array[i*dim[0]+j];
					}
					v.add(new REXP(s));
				}
			}
			
			//dim names
			REXP rownames = null;
			Vector<String> colnames = null;
			if(dimnames != null && dimnames.rtype != REXP.NILSXP){
				RVector dimnamesV = dimnames.asVector();
				if(dimnamesV == null)return null;
				rownames = dimnamesV.at(0);
				REXP colnamesExp = dimnamesV.at(1);
				if(colnamesExp != null){
					if(colnamesExp.asStringArray() != null){
						colnames = new Vector<String>();
						for(String name : colnamesExp.asStringArray())
							colnames.add(name);
					}
				}
			}
			
			REXP colClass = RCommand.silentIdleEval("class(as.vector("+robj.getFullName()+"))"); //$NON-NLS-1$ //$NON-NLS-2$
			String s;
			if(colClass == null || (s = colClass.asString()) == null)return null;
			Vector<String> colClassV = new Vector<String>();
			for(int i = 0; i < robj.getLength()[1]; i++)
				colClassV.add(s);
			
			return new ObjectDataTableModel(v, colnames, colClassV, rownames);
		}else{
			if(robj.getType().equals(RType.STRSXP)//charexp?
				|| robj.getType().equals(RType.REALSXP)
				|| robj.getType().equals(RType.REALSXPa)
				|| robj.getType().equals(RType.LGLSXP)
				|| robj.getType().equals(RType.INTSXP)
				//|| robj.getType().equals(RType.CPLXSXP)//TODO complexにテーブル対応
				)
			{
				REXP rexp = null;
				if(nrow > 0)
					rexp = RCommand.silentIdleEval("head("+robj.getFullName()+", n="+nrow+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				else
					rexp = RCommand.silentIdleEval("format("+robj.getFullName()+")"); //$NON-NLS-1$ //$NON-NLS-2$
				if(rexp == null)return null;
				Vector<REXP> v = new Vector<REXP>();
				v.add(rexp);
				Vector<String> name = new Vector<String>();
				name.add(robj.getName());
				REXP rownames = rexp.getAttribute("names"); //$NON-NLS-1$
				if(rownames == null)
					rownames = rexp.getAttribute("row.names"); //$NON-NLS-1$
				
				Vector<String> colClass = new Vector<String>();
				colClass.add(robj.getClas());
				
				return new ObjectDataTableModel(v, name, colClass, rownames);
			}
		}
		return null;
	}
	
	private void updateTable(){
		/*JTableHeader header = table.getTableHeader();
		TableCellRenderer renderer = header.getDefaultRenderer();
		if (renderer instanceof DefaultTableCellRenderer) {
			DefaultTableCellRenderer defRenderer = (DefaultTableCellRenderer) renderer;
			defRenderer.setHorizontalAlignment(SwingConstants.CENTER);
		}*/
		
		int w = 30;
		table.getColumnModel().getColumn(0).setCellRenderer(new RowHeaderRenderer());
		table.getColumnModel().getColumn(0).setPreferredWidth(30);
		
		for(int i = 1; i < table.getColumnCount(); i++){	
			table.getColumnModel().getColumn(i).setPreferredWidth(70);
			w += table.getColumnModel().getColumn(i).getPreferredWidth();
		}
		w += sp.getVerticalScrollBar().getPreferredSize().width;
		w += 2;
		if(w > 800)w = 800;
		
		this.setPreferredSize(new Dimension(w, 400));
		
	}
	
	public String getTitle(){
		if(robj == null)return ""; //$NON-NLS-1$
		int row = robj.getLength()[0];
		if(nrow > 0 && row > nrow)
			return robj.getName()+" "+ robj.getTip()+" : head "+nrow+" row"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		else
			return robj.getName()+" "+ robj.getTip(); //$NON-NLS-1$
	}
	
	/**
	 * 
	 * @param col zero start
	 * @return
	 */
	public RObject getRObjectAt(int col){
		if(robj == null)return null;
		if(robj.getClas().equals(RClass.DATAFRAME)){
			RObject[] objects = RCommand.getObjects(robj, false, 0);
			return objects[col];
		}else if(robj.getClas().equals(RClass.MATRIX)){
			return new RObject(robj, "[,"+(col+1)+"]", robj.getType(), //$NON-NLS-1$ //$NON-NLS-2$
					"<unknown>", String.valueOf(robj.getLength()[0]), true, true, true); //$NON-NLS-1$
		}else{
			return robj;
		}
	}
	
	/**
	 * 
	 * @param row zero base
	 * @return
	 */
	public RObject getRowRObject(int row, int length){
		if(row < 0 || robj == null)return null;
		if(robj.getLength()[0] <= row)return null;
		int len = (robj.getLength()[0] >= row+length)?length:robj.getLength()[0]-row;
		
		String rowIndex = (len == 1)?""+(row+1):(row+1)+":"+(row+len); //$NON-NLS-1$ //$NON-NLS-2$
		if(robj.getClas().equals(RClass.DATAFRAME)){
			return new RObject(robj, "["+rowIndex+",,drop=FALSE]", robj.getType(),  //$NON-NLS-1$ //$NON-NLS-2$
					robj.getClas(), "["+len+","+robj.getLength()[1]+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}else if(robj.getClas().equals(RClass.MATRIX)){
			return new RObject(robj, "["+rowIndex+",,drop=FALSE]", robj.getType(), //$NON-NLS-1$ //$NON-NLS-2$
					RClass.MATRIX, "["+len+","+robj.getLength()[1]+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}else{
			return new RObject(robj, "["+rowIndex+"]", robj.getType(), //$NON-NLS-1$ //$NON-NLS-2$
					robj.getClas(), "["+len+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * 
	 * @param rows zero base
	 * @return
	 */
	public RObject getRowRObjects(int[] rows){
		if(rows.length < 1)return null;
		if(rows.length == 1)
			return getRowRObject(rows[0], 1);
		if(robj == null)return null;
		//int len = (robj.getLength()[0] >= row+length)?length:robj.getLength()[0]-row;
		rows = Arrays.copyOf(rows, rows.length);
		Arrays.sort(rows);

		ArrayList<String> rowIndices = new ArrayList<String>();
		int last = rows[0];
		int seqStart = -1;
		for(int i = 1; i < rows.length; i++){
			int row = rows[i];
			if(row < 0 || robj.getLength()[0] <= row)return null;
			
			if(row == last + 1){
				if(seqStart < 0){
					seqStart = last;
				}
			}else{
				if(seqStart < 0 || seqStart == last){
					rowIndices.add(String.valueOf(last+1));
				}else{
					rowIndices.add((seqStart+1)+":"+(last+1));
				}
				seqStart = -1;
			}
			
			last = row;
		}
		if(seqStart < 0 || seqStart == last){
			rowIndices.add(String.valueOf(last+1));
		}else{
			rowIndices.add((seqStart+1)+":"+(last+1));
		}
		
		String rowIndex;
		if(rowIndices.size() > 1){
			rowIndex = "c(";
			for(String s : rowIndices){
				rowIndex += s + ", ";
			}
			rowIndex = rowIndex.substring(0, rowIndex.length()-2);
			rowIndex += ")";
		}else{
			rowIndex = rowIndices.get(0);
		}
		
		int len = rows.length;
		
		if(robj.getClas().equals(RClass.DATAFRAME)){
			return new RObject(robj, "["+rowIndex+",]", robj.getType(),  //$NON-NLS-1$ //$NON-NLS-2$
					robj.getClas(), "["+len+","+robj.getLength()[1]+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}else if(robj.getClas().equals(RClass.MATRIX)){
			return new RObject(robj, "["+rowIndex+",]", robj.getType(), //$NON-NLS-1$ //$NON-NLS-2$
					RClass.MATRIX, "["+len+","+robj.getLength()[1]+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}else{
			return new RObject(robj, "["+rowIndex+"]", robj.getType(), //$NON-NLS-1$ //$NON-NLS-2$
					robj.getClas(), "["+len+"]", true, true, true); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * 選択されているセルをシステムのクリップボードにコピーします。
	 *
	 */
	public void copy(){
		int[] cols = table.getSelectedColumns();
		int[] rows = table.getSelectedRows();
		String sep = "\t"; //$NON-NLS-1$
		String nl = "\n"; //$NON-NLS-1$
		String s = ""; //$NON-NLS-1$
		
		if(cols.length == 0 && rows.length == 0)
			return;
		if(cols.length > 0 && rows.length == 0){
			//列コピー
			for(int row = 0; row < table.getRowCount(); row++){
				for(int col = 0; col < cols.length; col++){
					s += table.getValueAt(row, cols[col]);
					s += (col == cols.length-1)?nl:sep;
				}
			}
		}else if(cols.length == 0 && rows.length > 0){
			//行コピー
			for(int row = 0; row < rows.length; row++){
				for(int col = 0; col < table.getColumnCount(); col++){
					s += table.getValueAt(rows[row], col);
					s += (col == table.getColumnCount()-1)?nl:sep;
				}
			}
		}else if(cols.length > 0 && rows.length > 0){
			//セル選択範囲コピー
			for(int row = 0; row < rows.length; row++){
				for(int col = 0; col < cols.length; col++){
					s += table.getValueAt(rows[row], cols[col]);
					s += (col == cols.length-1)?nl:sep;
				}
			}
		}

		Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
		StringSelection selection = new StringSelection(s);
		clip.setContents(selection, selection);
	}
	
	private interface ColClass {
		String getColClass(int col);
	}

	private class ObjectDataTableModel extends AbstractTableModel implements ColClass{
		
		private Vector<REXP> vector;
		private Vector<String> colNames;
		private Vector<String> colClass;
		private REXP rownames;
		
		ObjectDataTableModel(Vector<REXP> vector, Vector<String> colNames, Vector<String> colClass, REXP rownames){
			this.vector = vector;
			this.colNames = colNames;
			this.colClass = colClass;
			this.rownames = rownames;
		}
		
		public String getColClass(int col) {
			if(col == 0)
				return RClass.CHARACTER;
			
			return colClass.get(col -1);
		}
		
		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if(columnIndex == 0)
				return Object.class;
			
			
			REXP col = vector.get(columnIndex-1);
			if(col.asFactor() != null)
				return String.class;
			else if(col.asIntArray() != null)
				return Integer.class;
			else if(col.asDoubleArray() != null)
				return Double.class;
			else if(col.asStringArray() != null)
				return String.class;
			return String.class;
		}
		
		@Override
		public String getColumnName(int column) {
			if(column == 0)
				return " "; //$NON-NLS-1$
			if(colNames == null)
				return String.valueOf(column);
			return colNames.get(column-1);
		}
		
		public int getColumnCount() {
			return vector.size()+1;
		}

		public int getRowCount() {
			REXP col = vector.get(0);
			if(col.asFactor() != null)
				return col.asFactor().size();
			else if(col.asIntArray() != null)
				return col.asIntArray().length;
			else if(col.asDoubleArray() != null)
				return col.asDoubleArray().length;
			else if(col.asStringArray() != null)
				return col.asStringArray().length;
			return 0;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if(columnIndex == 0){
				//row names
				if(rownames == null)
					return rowIndex+1;
				else if(rownames.asIntArray() != null)
					return rownames.asIntArray()[rowIndex];
				else if(rownames.asStringArray() != null)
					return rownames.asStringArray()[rowIndex];
				else
					return rowIndex+1;
			}
				
			REXP col = vector.get(columnIndex-1);
			if(col == null)return null;
			if(col.asFactor() != null)
				return col.asFactor().at(rowIndex);
			else if(col.asIntArray() != null)
				return col.asIntArray()[rowIndex];
			else if(col.asDoubleArray() != null)
				return col.asDoubleArray()[rowIndex];
			else if(col.asStringArray() != null)
				return col.asStringArray()[rowIndex];
			return null;
		}
		
	}
	
	private class OnDemandTableModel extends AbstractTableModel implements ColClass{

		private static final int UNIT = 20;
		private static final int CACHE_LIMIT = 15;

		
		
		HashMap<Integer, ObjectDataTableModel> map = new HashMap<Integer, ObjectDataTableModel>(CACHE_LIMIT+1);
		LinkedList<Integer> indexList = new LinkedList<Integer>();
		
		private Vector<REXP> vector;//最初のデータ
		int rowCount;//行数
		private Vector<String> colNames;
		private Vector<String> colClass;
		
		OnDemandTableModel(RObject robj, int rowCount){
			//rowCount = robj.getLength()[0];
			this.rowCount = rowCount;
			ObjectDataTableModel model = createTableModel(getRowRObject(0, UNIT), 0);
			if(model == null){
				throw new RuntimeException();
			}else{
				map.put(0, model);
				indexList.add(0);
				
				this.vector = model.vector;
				//ベクターの場合、列名はオブジェクト名にする
				if(robj.getLength().length == 1){
					Vector<String> cnames = new Vector<String>();
					cnames.add(robj.getName());
					this.colNames = cnames;
				}else{
					this.colNames = model.colNames;
				}
				this.colClass = model.colClass;
			}
		}

		public String getColClass(int col) {
			if(col == 0)
				return RClass.CHARACTER;
			
			return colClass.get(col -1);
		}

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if(columnIndex == 0)
				return Object.class;

			REXP col = vector.get(columnIndex-1);
			if(col.asFactor() != null)
				return String.class;
			else if(col.asIntArray() != null)
				return Integer.class;
			else if(col.asDoubleArray() != null)
				return Double.class;
			else if(col.asStringArray() != null)
				return String.class;
			return String.class;
		}
		
		@Override
		public String getColumnName(int column) {
			if(column == 0)
				return " "; //$NON-NLS-1$
			if(colNames == null)
				return String.valueOf(column);
			return colNames.get(column-1);
		}
		
		public int getColumnCount() {
			return colNames.size() + 1;
		}

		public int getRowCount() {
			if(robj == null)
				return 0;
			
			return rowCount;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			int unitRowIndex = rowIndex / UNIT;
			ObjectDataTableModel model = map.get(unitRowIndex);
			if(model == null){
				model = createTableModel(getRowRObject(unitRowIndex*UNIT, UNIT), 0);
				if(model == null)return null;
				map.put(unitRowIndex, model);
				indexList.addLast(unitRowIndex);
				//キャッシュサイズの調整
				if(indexList.size() > CACHE_LIMIT)
					map.remove(indexList.poll());
			}
			
			//行名がない場合
			if(columnIndex == 0 && model.rownames == null)
				return rowIndex + 1;
			
			return model.getValueAt(rowIndex-(unitRowIndex * UNIT), columnIndex);
		}
		
	}
	
	private class RowHeaderRenderer extends DefaultTableCellRenderer{
		
		RowHeaderRenderer(){
			setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
		}
		
		@Override
		public Color getBackground() {
			return table.getTableHeader().getBackground();
		}
		
		@Override
		public Color getForeground() {
			return table.getTableHeader().getForeground();
		}
	}
	
	private class MouseHandler extends MouseAdapter{
		private ObjectActionManager objActionManager = new ObjectActionManager();
		@Override
		public void mousePressed(MouseEvent e) {
			if(SwingUtilities.isLeftMouseButton(e)){
				//左クリック
				JTable table = (JTable)e.getSource();
				int col = table.convertColumnIndexToModel(table.columnAtPoint(e.getPoint()));
				int row = table.rowAtPoint(e.getPoint());
				if(col >= 0 && row >= 0){
					if(col == 0){
						//row header
						table.setColumnSelectionAllowed(false);
						table.setRowSelectionAllowed(true);
					}else{
						table.setColumnSelectionAllowed(true);
						table.setRowSelectionAllowed(true);
					}
				}
			}
		}
		@Override
		public void mouseReleased(MouseEvent e) {
			if(SwingUtilities.isRightMouseButton(e)){
				//右クリック
				JTable table = (JTable)e.getSource();
				int col = table.convertColumnIndexToModel(table.columnAtPoint(e.getPoint()));
				int row = table.rowAtPoint(e.getPoint());
				if(col == 0 && row >= 0){
					//row header
					table.setColumnSelectionAllowed(false);
					table.setRowSelectionAllowed(true);
					if(!table.getSelectionModel().isSelectedIndex(row))
						table.changeSelection(row, col, false, false);

					List<RObject> robjs = new ArrayList<RObject>();
//					for(int selected : table.getSelectedRows())
//						if(getRowRObject(selected, 1) != null)
//							robjs.add(getRowRObject(selected, 1));
					robjs.add(getRowRObjects(table.getSelectedRows()));
					if(robjs.size() == 0)return;

					JPopupMenu popup = objActionManager.getPopupMenu(robjs.toArray(new RObject[0]));
					if(popup != null)
						popup.show(table, e.getPoint().x, e.getPoint().y);
				}
			}
		}
	}
	
	private class HeaderMouseHandler extends MouseAdapter{

		private ObjectActionManager objActionManager = new ObjectActionManager();
		
		@Override
		public void mouseReleased(MouseEvent e) {
			if(SwingUtilities.isLeftMouseButton(e)){
				//左クリック
				JTableHeader tableHeader = (JTableHeader)e.getSource();
				int col = table.convertColumnIndexToModel(tableHeader.columnAtPoint(e.getPoint()));
				if(col > 0){
					tableHeader.getTable().setColumnSelectionAllowed(true);
					tableHeader.getTable().setRowSelectionAllowed(false);
					int selectedRow = table.getSelectedRow();
					if(selectedRow < 0)selectedRow = 0;
					tableHeader.getTable().changeSelection(selectedRow, col, e.isControlDown(), e.isShiftDown());
				}
			}else if(SwingUtilities.isRightMouseButton(e)){
				//右クリック
				JTableHeader tableHeader = (JTableHeader)e.getSource();
				int col = table.convertColumnIndexToModel(tableHeader.columnAtPoint(e.getPoint()));
				
				if(col > 0){
					tableHeader.getTable().setColumnSelectionAllowed(true);
					tableHeader.getTable().setRowSelectionAllowed(false);
					if(!(table.getColumnModel().getSelectionModel().isSelectedIndex(col)))
						tableHeader.getTable().changeSelection(0, col, false, false);
					
					//note: col at 0 is row header
					List<RObject> robjs = new ArrayList<RObject>();
					for(int selected : tableHeader.getColumnModel().getSelectedColumns())
						if(selected != 0 && getRObjectAt(selected-1) != null)
							robjs.add(getRObjectAt(selected-1));
					if(robjs.size() == 0)return;

					
					JPopupMenu popup = objActionManager.getPopupMenu(robjs.toArray(new RObject[0]));
					if(popup != null)
						popup.show(tableHeader, e.getPoint().x, e.getPoint().y);
				}
			}
		}
	}
	
	private class KeyHandler extends KeyAdapter{
		@Override
		public void keyTyped(KeyEvent e) {
			if(e.isControlDown()){
				switch(e.getKeyCode()){
				case KeyEvent.VK_C:
					copy();
					break;
				}
			}
		}
	}
	
	private class StringCellRenderer extends DefaultTableCellRenderer{
		
		StringCellRenderer(){
			
		}
		
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			
			if(table.getModel() instanceof ColClass){
				ColClass c = (ColClass)table.getModel();
				String clas = c.getColClass(column);
				if(clas.equals(RClass.NUMERIC) || clas.equals(RClass.INTEGER)){
					this.setHorizontalAlignment(JLabel.RIGHT);
				}else{
					this.setHorizontalAlignment(JLabel.LEFT);
				}
			}
			
			return this;
		}
	}
}
