package com.ef_prime.rflow.object;

import com.ef_prime.rflow.util.ObjectUtilities;

/**
 * Ｒのオブジェクトをあらわすクラスです
 *
 */
public class RObject {
	
	/** 
	 * parent of this object 
	 * null if no parent
	 */
	private RObject parent;
	
	/** name of object */
	private String name;
	
	/** value of typeof() */
	private String type;
	
	/** value of class() */
	private String clas;
	
	private String tip;
	
	/** is name syntactically valid*/
	private boolean validName;
	
	/** is name index or not */
	private boolean indexName;
	
	/** 
	 * if true, this object doesn't exists but is representing 
	 * a return value of '[' indexer.
	 * if true, name must be like "[,12]" or "[3,]" or "[3,12]"
	 */
	private boolean indexObject;
	

	public RObject(RObject parent, String name, String type, String clas, String tip, boolean validName, boolean indexName) {
		this(parent, name, type, clas, tip, validName, indexName, false);
	}

	public RObject(RObject parent, String name, String type, String clas, String tip, boolean validName, boolean indexName, boolean indexObject) {
		super();
		this.parent = parent;
		this.name = name;
		this.type = type;
		this.clas = clas;
		this.tip = tip;
		this.validName = validName;
		this.indexName = indexName;
		this.indexObject = indexObject;
	}

	public String getClas() {
		return clas;
	}

	public boolean isIndexName() {
		return indexName;
	}
	
	public boolean isIndexObject() {
		return indexObject;
	}

	public String getName() {
		if(indexObject && parent != null)
			return parent.getName()+name;
		if(validName)
			return name;
		else
			return '`'+name+'`';
	}

	public RObject getParent() {
		return parent;
	}

	public String getType() {
		return type;
	}
	
	public String getTip() {
		return tip;
	}
	
	/**
	 * extrace length from tip
	 * @return  
	 */
	public int[] getLength(){
		if(tip.startsWith("[")){ //$NON-NLS-1$
			String[] s = tip.substring(1, tip.length() -1).split(","); //$NON-NLS-1$
			int[] len = new int[s.length];
			for(int i = 0; i < s.length; i++)
				len[i] = Integer.valueOf(s[i]);
			return len;
		}else{
//			return new int[1];
			return new int[]{1};
		}
	}
	
	/**
	 * グローバル環境から参照できる完全なオブジェクト名を返します
	 * @return
	 */
	public String getFullName(){
		if(parent == null)
			return getName();
		
		String index = getName();
		if(indexObject){
			index = index.substring(parent.getName().length(), index.length());
		}else if(!isIndexName()){
			//index = "\""+index+"\"";
			index = "$"+index; //$NON-NLS-1$
		}else{
			index = "[["+index+"]]"; //$NON-NLS-1$ //$NON-NLS-2$
		}

		return parent.getFullName()+index;
	}
	
	public boolean isAtomic(){
		return !(type.equals(RType.LISTSXP) || type.equals(RType.VECSXP) || type.equals(RType.ENVSXP));
	}


	@Override
	public String toString() {
		return name+" : "+clas+" "+tip; //$NON-NLS-1$ //$NON-NLS-2$
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((clas == null) ? 0 : clas.hashCode());
		result = PRIME * result + (indexName ? 1231 : 1237);
		result = PRIME * result + (indexObject ? 1231 : 1237);
		result = PRIME * result + ((name == null) ? 0 : name.hashCode());
		result = PRIME * result + ((parent == null) ? 0 : parent.hashCode());
		result = PRIME * result + ((tip == null) ? 0 : tip.hashCode());
		result = PRIME * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null || !(obj instanceof RObject))
			return false;
		final RObject o = (RObject) obj;
		return ObjectUtilities.equals(parent, o.parent)
			&& ObjectUtilities.equals(name, o.name)
			&& ObjectUtilities.equals(type, o.type)
			&& ObjectUtilities.equals(clas, o.clas)
			&& ObjectUtilities.equals(tip, o.tip)
			&& indexObject == o.indexObject
			&& indexName == o.indexName;
	}
	

}
