package com.ef_prime.rflow.tool;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.regex.PatternSyntaxException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.text.Caret;
import javax.swing.text.Element;
import javax.swing.text.View;

import org.fife.ui.rsyntaxtextarea.AbstractTokenMakerFactory;
import org.fife.ui.rsyntaxtextarea.RSyntaxTextArea;
import org.fife.ui.rsyntaxtextarea.RSyntaxTextAreaUI;
import org.fife.ui.rsyntaxtextarea.Token;
import org.fife.ui.rsyntaxtextarea.TokenMakerFactory;
import org.fife.ui.rsyntaxtextarea.WrappedSyntaxView;
import org.fife.ui.rtextarea.ConfigurableCaret;
import org.fife.ui.rtextarea.RTextArea;
import org.fife.ui.rtextarea.RTextAreaUI;
import org.fife.ui.rtextarea.SearchEngine;

import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.SwingUtils;

public class CodingTextArea extends RSyntaxTextArea{

	private JDialog findAndReplaceDialog = null;
	
	static{
		AbstractTokenMakerFactory atmf = (AbstractTokenMakerFactory) TokenMakerFactory.getDefaultInstance();
		atmf.putMapping("RSYNTAX", RTokenMaker.class.getCanonicalName()); //$NON-NLS-1$
		TokenMakerFactory.setDefaultInstance(atmf);
	}
	
	public CodingTextArea(){
		this(""); //$NON-NLS-1$
	}

	public CodingTextArea(String code){
		super();
		setText(code);
		setCaretPosition(0);
		
		//sytle
		setSyntaxEditingStyle("RSYNTAX"); //$NON-NLS-1$
		//use preprocessor for composed text
		getSyntaxScheme().styles[Token.PREPROCESSOR].foreground = null;
		//getSyntaxScheme().styles[Token.PREPROCESSOR].underline = true;

		//setLineWrap(true);
		setMarkOccurrences(true);
		
		SwingUtils.setTextComponentPopupMenu(this);
		SupportedInputFactory.setFunctionSupport(this);
		
		//create actions
		int mod = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		Action findAndReplace = new AbstractAction(Messages.getString("CodingTextArea.find_and_replace")){ //$NON-NLS-1$
			@Override
			public void actionPerformed(ActionEvent e) {
				showFindAndReplaceDialog();
			}
		};
		findAndReplace.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F, mod));
		getActionMap().put("FindAndReplace", findAndReplace); //$NON-NLS-1$
		getInputMap().put((KeyStroke)findAndReplace.getValue(Action.ACCELERATOR_KEY), "FindAndReplace"); //$NON-NLS-1$
	}
	
	@Override
	protected RTextAreaUI createRTextAreaUI() {
		return new RSyntaxTextAreaUI(this){
			@Override
			public View create(Element elem) {
				RTextArea c = getRTextArea();
				if (c instanceof RSyntaxTextArea) {
					RSyntaxTextArea area = (RSyntaxTextArea) c;
					View v;
					if (area.getLineWrap())
						v = new WrappedSyntaxView(elem);
					else
						v = new RSyntaxView(elem);
					return v;
				}
				return null;
			}
		};
	}
	
	@Override
	protected JPopupMenu createPopupMenu() {
		JPopupMenu pop = super.createPopupMenu();
		JMenuItem find = new JMenuItem(getActionMap().get("FindAndReplace")); //$NON-NLS-1$
		pop.add(find, 0);
		pop.add(new JPopupMenu.Separator(), 1);
		
		return pop;
	}
	
	@Override
	public void setCaret(Caret caret) {
		//ConfigurableCaretでない場合,super.setCaret(caret)にてExceptionが投げられるのを回避する
		//文字列変換時にJTextComponentにより、変換中専用のCaretを設定するために呼び出される
		if (caret instanceof ConfigurableCaret) {
			super.setCaret(caret);
		}
	}
	
	/**
	 * 検索ダイアログを表示する
	 */
	public void showFindAndReplaceDialog(){
		if(findAndReplaceDialog == null){
			findAndReplaceDialog = new JDialog(SwingUtilities.windowForComponent(this), Messages.getString("CodingTextArea.find_and_replace")); //$NON-NLS-1$
			findAndReplaceDialog.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
			findAndReplaceDialog.getContentPane().add(new FindAndReplacePanel());
			findAndReplaceDialog.pack();
			findAndReplaceDialog.setLocationRelativeTo(this);
		}
		findAndReplaceDialog.setVisible(true);
	}
	
	class FindAndReplacePanel extends JPanel implements ActionListener{
		
		//Input
		JComboBox cbFind = new JComboBox(new DefaultComboBoxModel());
		JComboBox cbReplace = new JComboBox(new DefaultComboBoxModel());

		//Direction
		JRadioButton rbForward = new JRadioButton(Messages.getString("CodingTextArea.forward"), true); //$NON-NLS-1$
		JRadioButton rbBackward = new JRadioButton(Messages.getString("CodingTextArea.backward")); //$NON-NLS-1$
		
		//Options
		JCheckBox chkCaseSensitive = new JCheckBox(Messages.getString("CodingTextArea.case_sensitive")); //$NON-NLS-1$
		//JCheckBox chkWholeWord = new JCheckBox("Whole word");
		JCheckBox chkRegExp = new JCheckBox(Messages.getString("CodingTextArea.regular_expressions")); //$NON-NLS-1$

		//Status
		JLabel lStatus = new JLabel(""); //$NON-NLS-1$
		
		public FindAndReplacePanel() {
			super();
			
			//Find Input
			JLabel lFind = new JLabel(Messages.getString("CodingTextArea.find_word")+":"); //$NON-NLS-1$ //$NON-NLS-2$
			lFind.setPreferredSize(new Dimension(80, 20));
			lFind.setMaximumSize(lFind.getPreferredSize());
			cbFind.setEditable(true);
			Box bFind = Box.createHorizontalBox();
			bFind.add(lFind);
			bFind.add(cbFind);
			
			//Replace Input
			JLabel lReplace = new JLabel(Messages.getString("CodingTextArea.replace_with")+":"); //$NON-NLS-1$ //$NON-NLS-2$
			lReplace.setPreferredSize(new Dimension(80, 20));
			lReplace.setMaximumSize(lFind.getPreferredSize());
			cbReplace.setEditable(true);
			Box bReplace = Box.createHorizontalBox();
			bReplace.add(lReplace);
			bReplace.add(cbReplace);
			
			//Direction
			ButtonGroup bg = new ButtonGroup();
			bg.add(rbForward);
			bg.add(rbBackward);
			Box bDirection = Box.createHorizontalBox();
			bDirection.add(rbForward);
			bDirection.add(rbBackward);
			bDirection.add(Box.createHorizontalGlue());
			bDirection.setBorder(new TitledBorder(Messages.getString("CodingTextArea.direction"))); //$NON-NLS-1$
			
			//Options
			Box bOptions0 = Box.createVerticalBox();
			bOptions0.add(chkCaseSensitive);
			//bOptions0.add(chkWholeWord);
			bOptions0.add(chkRegExp);
			Box bOptions = Box.createHorizontalBox();
			bOptions.add(bOptions0);
			bOptions.add(Box.createHorizontalGlue());
			bOptions.setBorder(new TitledBorder(Messages.getString("CodingTextArea.options"))); //$NON-NLS-1$
			
			//Button and StatusLabel
			JButton btnFind = new JButton(Messages.getString("CodingTextArea.find")); //$NON-NLS-1$
			JButton btnRepAndFind = new JButton(Messages.getString("CodingTextArea.replace_and_find")); //$NON-NLS-1$
			JButton btnReplace = new JButton(Messages.getString("CodingTextArea.replace")); //$NON-NLS-1$
			JButton btnRepAll = new JButton(Messages.getString("CodingTextArea.replace_all")); //$NON-NLS-1$
			JButton btnClose = new JButton(Messages.getString("Button.close")); //$NON-NLS-1$
			lStatus.setPreferredSize(new Dimension(120, 20));
			lStatus.setMaximumSize(lStatus.getPreferredSize());
			Dimension btnSize = new Dimension(110, 20);
			btnFind.setPreferredSize(btnSize);
			btnFind.setMaximumSize(btnSize);
			btnRepAndFind.setPreferredSize(btnSize);
			btnRepAndFind.setMaximumSize(btnSize);
			btnReplace.setPreferredSize(btnSize);
			btnReplace.setMaximumSize(btnSize);
			btnRepAll.setPreferredSize(btnSize);
			btnRepAll.setMaximumSize(btnSize);
			btnClose.setPreferredSize(new Dimension(80, 20));
			btnClose.setMaximumSize(btnClose.getPreferredSize());
			btnFind.addActionListener(this);
			btnFind.setActionCommand("Find"); //$NON-NLS-1$
			btnRepAndFind.addActionListener(this);
			btnRepAndFind.setActionCommand("ReplaceAndFind"); //$NON-NLS-1$
			btnReplace.addActionListener(this);
			btnReplace.setActionCommand("Replace"); //$NON-NLS-1$
			btnRepAll.addActionListener(this);
			btnRepAll.setActionCommand("ReplaceAll"); //$NON-NLS-1$
			btnClose.addActionListener(this);
			btnClose.setActionCommand("Close"); //$NON-NLS-1$
			Box bButtonFirst = Box.createHorizontalBox();
			bButtonFirst.add(Box.createHorizontalGlue());
			bButtonFirst.add(btnFind);
			bButtonFirst.add(Box.createHorizontalStrut(3));
			bButtonFirst.add(btnRepAndFind);
			Box bButtonSecond = Box.createHorizontalBox();
			bButtonSecond.add(Box.createHorizontalGlue());
			bButtonSecond.add(btnReplace);
			bButtonSecond.add(Box.createHorizontalStrut(3));
			bButtonSecond.add(btnRepAll);
			Box bButtonThird = Box.createHorizontalBox();
			bButtonThird.add(lStatus);
			bButtonThird.add(Box.createHorizontalGlue());
			bButtonThird.add(btnClose);
			
			//panel
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new EmptyBorder(10, 10, 10, 10));
			add(bFind);
			add(Box.createVerticalStrut(5));
			add(bReplace);
			add(bDirection);
			add(bOptions);
			add(Box.createVerticalStrut(5));
			add(bButtonFirst);
			add(Box.createVerticalStrut(5));
			add(bButtonSecond);
			add(Box.createVerticalStrut(5));
			add(bButtonThird);
		}
		
		private void updateSearchBox(JComboBox box, String s){
			DefaultComboBoxModel model = (DefaultComboBoxModel) box.getModel();
			int index = model.getIndexOf(s);
			if(index > 0){
				model.removeElementAt(index);
			}
			if(index != 0){
				model.insertElementAt(s, 0);
				if(model.getSize() > 20)
					model.removeElementAt(20);
				box.setSelectedIndex(0);
			}
		}
		
		
		@Override
		public void actionPerformed(ActionEvent e) {
			String s = e.getActionCommand();
			try{
				if(s.equals("Find")){ //$NON-NLS-1$
					String text = cbFind.getSelectedItem().toString();
					if(text.length() == 0)return;
					updateSearchBox(cbFind, text);
					boolean b = SearchEngine.find(CodingTextArea.this, text, rbForward.isSelected(),
							chkCaseSensitive.isSelected(), false, chkRegExp.isSelected());
					lStatus.setText((b)?"":Messages.getString("CodingTextArea.string_not_found")); //$NON-NLS-1$ //$NON-NLS-2$
				}else if(s.equals("ReplaceAndFind")){ //$NON-NLS-1$
					String toFind = cbFind.getSelectedItem().toString();
					String replaceWith = cbReplace.getSelectedItem().toString();
					if(toFind.length() == 0)return;
					updateSearchBox(cbFind, toFind);
					updateSearchBox(cbReplace, replaceWith);
					boolean b = SearchEngine.replace(CodingTextArea.this, toFind, replaceWith, rbForward.isSelected(),
							chkCaseSensitive.isSelected(), false, chkRegExp.isSelected());
					lStatus.setText((b)?"":Messages.getString("CodingTextArea.string_not_found")); //$NON-NLS-1$ //$NON-NLS-2$
					if(b){
						b = SearchEngine.find(CodingTextArea.this, toFind, rbForward.isSelected(),
								chkCaseSensitive.isSelected(), false, chkRegExp.isSelected());
						lStatus.setText((b)?"":Messages.getString("CodingTextArea.string_not_found")); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}else if(s.equals("Replace")){ //$NON-NLS-1$
					String toFind = cbFind.getSelectedItem().toString();
					String replaceWith = cbReplace.getSelectedItem().toString();
					if(toFind.length() == 0)return;
					updateSearchBox(cbFind, toFind);
					updateSearchBox(cbReplace, replaceWith);
					boolean b = SearchEngine.replace(CodingTextArea.this, toFind, replaceWith, rbForward.isSelected(),
							chkCaseSensitive.isSelected(), false, chkRegExp.isSelected());
					lStatus.setText((b)?"":Messages.getString("CodingTextArea.string_not_found")); //$NON-NLS-1$ //$NON-NLS-2$
				}else if(s.equals("ReplaceAll")){ //$NON-NLS-1$
					String toFind = cbFind.getSelectedItem().toString();
					String replaceWith = cbReplace.getSelectedItem().toString();
					if(toFind.length() == 0)return;
					updateSearchBox(cbFind, toFind);
					updateSearchBox(cbReplace, replaceWith);
					int b = SearchEngine.replaceAll(CodingTextArea.this, toFind, replaceWith, 
							chkCaseSensitive.isSelected(), false, chkRegExp.isSelected());
					lStatus.setText((b > 0)?"":Messages.getString("CodingTextArea.string_not_found")); //$NON-NLS-1$ //$NON-NLS-2$
				}else if(s.equals("Close")){ //$NON-NLS-1$
					Window w = SwingUtilities.windowForComponent(this);
					if(w != null)
						w.setVisible(false);
				}
			}catch(PatternSyntaxException e1){
				lStatus.setText(Messages.getString("CodingTextArea.invalid_pattern")); //$NON-NLS-1$
			}catch(IndexOutOfBoundsException e1){
				lStatus.setText(Messages.getString("CodingTextArea.search_failed")); //$NON-NLS-1$
			}
		}
	}
}
