package com.ef_prime.rflow.tool;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JTextPane;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;
import javax.swing.undo.UndoManager;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.util.SwingUtils;

public class CodingTextPane extends JTextPane{
	
	private UndoManager undoManager;
	private boolean intend = true;
	private boolean parenthesisSupport = false;
	
	public CodingTextPane(){
		this(""); //$NON-NLS-1$
	}
	
	/**
	 * 
	 * @param text 初期値
	 */
	public CodingTextPane(String text){
		super();
		
		setFont(Prefs.CODE_FONT);
		//setCharacterAttributes(Prefs.CODE_ATTR, false);
		//setEditorKit(new NoWrapEditorKit());
		setParagraphAttributes(Prefs.CODE_ATTR, false);
		SwingUtils.setTabs(this, Prefs.CHAR_PER_TAB);
		
		setText(text);
		undoManager = new UndoManager();
		getDocument().addUndoableEditListener(undoManager);
		
		setBorder(new EmptyBorder(3,3,3,3));
		//setMargin(new Insets(3,3,3,3));
		
		addKeyListener(new KeyHandler());
		//addMouseListener(new MouseHandler());
		SwingUtils.setTextComponentPopupMenu(this);
		
		SupportedInputFactory.setFunctionSupport(this);
	}
	
	/**
	 * getText()で\nが\r\nになってしまうのを回避するために作成しました。
	 * StringWriterをかいさずに直接Documentからテキストを取得する。
	 * @return
	 */
	public String getRawText() {
		try {
			return getDocument().getText(0, getDocument().getLength());
		} catch (BadLocationException e) {
			e.printStackTrace();throw new RuntimeException();
		}
	}
	
	public UndoManager getUndoManager() {
		return undoManager;
	}
	
	public boolean isIntend() {
		return intend;
	}

	public void setIntend(boolean intend) {
		this.intend = intend;
	}

	public boolean isParenthesisSupport() {
		return parenthesisSupport;
	}

	public void setParenthesisSupport(boolean parenthesisSupport) {
		this.parenthesisSupport = parenthesisSupport;
	}
	
	private class KeyHandler extends KeyAdapter{
		@Override
		public void keyPressed(KeyEvent e) {
			if(e.isControlDown()){
				switch(e.getKeyCode()){
				/*case KeyEvent.VK_SPACE:
					String parts = getCaretPosPart();
					supportControl.showSupport(parts);
					break;*/
				case KeyEvent.VK_Z:
					if(undoManager.canUndo())undoManager.undo();
					break;
				case KeyEvent.VK_Y:
					if(undoManager.canRedo())undoManager.redo();
					break;
				}
				return;
			}
			
			JTextPane pane = (JTextPane)e.getSource();
			switch(e.getKeyCode()){
			case KeyEvent.VK_ENTER:
				if(isIntend()){
					//改行前一行を取得
					String beforeInsert;
					try {
						beforeInsert = pane.getDocument().getText(0, pane.getCaretPosition());
					} catch (BadLocationException e2) {
						// TODO Auto-generated catch block
						e2.printStackTrace();return;
					}
					int lastNewLine = beforeInsert.lastIndexOf(Constants.NEWLINE);
					if(lastNewLine > -1){
						beforeInsert = beforeInsert.substring(lastNewLine+1, beforeInsert.length());
					}
					//インテンドの階層
					int tabCount = 0;
					for(int i = 0; i < beforeInsert.length(); i++){
						if(beforeInsert.charAt(i) == '\t'){
							tabCount++;
						}else{
							break;
						}
					}
					if(beforeInsert.endsWith("{")) //$NON-NLS-1$
						tabCount++;
					//インテンド
					if(tabCount > 0){
						try {
							int pos = pane.getCaretPosition();
							pane.getDocument().insertString(pos++, Constants.NEWLINE_STR, Prefs.SIMPLE_ATTR);
							while(tabCount-- > 0)
								pane.getDocument().insertString(pos++, "\t", Prefs.SIMPLE_ATTR); //$NON-NLS-1$
								pane.setCaretPosition(pos);
						} catch (BadLocationException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
						e.consume();
					}
				}
				break;
			}
			
			switch(e.getKeyChar()){
			case '(':
				if(isParenthesisSupport()){
					int pos = pane.getCaretPosition();
					try {
						pane.getDocument().insertString(pos, ")", Prefs.SIMPLE_ATTR); //$NON-NLS-1$
					} catch (BadLocationException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					pane.setCaretPosition(pos);
				}
				break;
			case '}':
				if(isIntend()){
					try {
						if(pane.getText(getCaretPosition()-1, 1).equals("\t")) //$NON-NLS-1$
								pane.getDocument().remove(getCaretPosition()-1, 1);
					} catch (BadLocationException e1) {
						//do nothing
					}
					e.consume();
				}
				break;
			}
		}
	}
}
