package com.ef_prime.rflow.tool;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.FileChooserUtility;

public class DirectoryChooser {
	
	private File directory = null;
	private JDialog dialog;
	
	private DirectoryChooser(final Window parent, String title, File defaultValue){
		//dialog = new JDialog(parent, title, ModalityType.APPLICATION_MODAL);
		if(parent instanceof Dialog)
			dialog = new JDialog((Dialog)parent, title, true);
		else
			dialog = new JDialog((Frame)parent, title, true);
				
		dialog.getContentPane().setLayout(new BorderLayout());
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		int sp = 10;
		panel.setBorder(new EmptyBorder(sp, sp, sp, sp));
		
		String defaultText = ""; //$NON-NLS-1$
		if(defaultValue != null)
			defaultText = defaultValue.getAbsolutePath();
		final JTextField directoryInput = new RflowTextField(Constants.replaceFileSepR2System(defaultText));
		JButton ref = new JButton("..."); //$NON-NLS-1$
		ref.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				File file = FileChooserUtility.chooseDir(dialog, 
						new File(directoryInput.getText()));
				if(file == null)return;
				directoryInput.setText(file.getAbsolutePath());
			}
		});
		
		
		Box inputBox = Box.createHorizontalBox();
		
		inputBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
		inputBox.setPreferredSize(new Dimension(400, 20));
		
		inputBox.add(directoryInput);
		inputBox.add(ref);
		

		ActionListener okAction = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				File d = new File(directoryInput.getText());
				if(!d.exists() || !d.isDirectory()){
					JOptionPane.showMessageDialog(parent, Messages.getString("Error.invalid_workingdir"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE);  //$NON-NLS-1$//$NON-NLS-2$
					return;
				}
				directory = d;
				dialog.dispose();
			}
		};
		ActionListener cancelAction = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				directory = null;
				dialog.dispose();
			}
		};
		final ButtonPanel buttonPanel = new ButtonPanel(okAction, cancelAction); 
		
		panel.add(inputBox);
		panel.add(Box.createVerticalStrut(10));
		panel.add(buttonPanel);
		
		//フォーカスの移動
		directoryInput.addKeyListener(new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_ENTER){
					buttonPanel.getFirstButton().requestFocus();
				}
			}
		});
		
		dialog.getContentPane().add(panel);
		
		dialog.pack();
		dialog.setLocationRelativeTo(parent);
	}
	
	
	public static File chooseDirectory(Window parent, String title, File defaultValue){
		DirectoryChooser chooser = new DirectoryChooser(parent, title, defaultValue);
		chooser.dialog.setVisible(true);
		chooser.dialog.dispose();
		return chooser.directory;
	}
}
