package com.ef_prime.rflow.tool;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import com.ef_prime.rflow.flow.FlowSetting;
import com.ef_prime.rflow.resource.Messages;

/**
 * TODO SettingDialogと構造が同じなので共通部分の抜き出しを検討
 *
 */
public class FlowSettingDialog {

	private JDialog dialog;
	private FlowSetting setting = null;
	private List<UpdateSetting> updateSettings = new ArrayList<UpdateSetting>();

	private FlowSettingDialog(Window parent, final FlowSetting setting) {
		if(parent instanceof Dialog)
			dialog = new JDialog((Dialog)parent, true);
		else
			dialog = new JDialog((Frame)parent, true);
		
		dialog.setTitle(Messages.getString("Action.file_option")); //$NON-NLS-1$
		
		LayoutOptionPanel layoutOptionPanel = new LayoutOptionPanel(setting);
		updateSettings.add(layoutOptionPanel);
		
		//ボタン
		ActionListener ok = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				FlowSettingDialog.this.setting = (FlowSetting)setting.clone();
				update(FlowSettingDialog.this.setting);
				dialog.dispose();
			}
		};
		ActionListener cancel = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				FlowSettingDialog.this.setting = null;
				dialog.dispose();
			}
		};
		ButtonPanel buttonPanel = new ButtonPanel(ok, cancel);
		
		
		Box mainBox = Box.createVerticalBox();
		mainBox.add(layoutOptionPanel);
		mainBox.add(Box.createVerticalStrut(10));
		mainBox.add(buttonPanel);
		mainBox.setBorder(new EmptyBorder(10,10,10,10));
		
		dialog.getContentPane().add(mainBox);
		dialog.pack();
		dialog.setLocationRelativeTo(parent);
		dialog.setResizable(false);
	}

	private boolean update(FlowSetting setting){
		try {
			for(UpdateSetting us : updateSettings)
					us.update(setting);
		} catch (InvalidOptionException e) {
			JOptionPane.showMessageDialog(dialog.getParent(), e.getMessage(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			return false;
		}
		return true;
	}
	
	public FlowSetting getSetting() {
		return setting;
	}

	private static class InvalidOptionException extends Exception{
		InvalidOptionException(String message){
			super(message);
		}
	}
	
	private static interface UpdateSetting{
		void update(FlowSetting settings) throws InvalidOptionException;
	}

	private static class LayoutOptionPanel extends JPanel implements UpdateSetting{
		
		private JCheckBox useGrid;
		private JSpinner gridDistance;
		
		LayoutOptionPanel(FlowSetting setting){
			super();
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(new TitledBorder(Messages.getString("Setting.grid"))); //$NON-NLS-1$
			
			//グリッドのon/off
			useGrid = new JCheckBox(Messages.getString("Setting.use_grid"),  //$NON-NLS-1$
					(Boolean)setting.getProperty(FlowSetting.Key.USE_GRID));
			Box useGridBox = Box.createHorizontalBox();
			useGridBox.add(useGrid);
			useGridBox.add(Box.createHorizontalGlue());
			//グリッドの幅
			JLabel gridDistLabel = new JLabel(Messages.getString("Setting.grid_distance"));  //$NON-NLS-1$
			int value = (Integer)setting.getProperty(FlowSetting.Key.GRID_DISTANCE);
			gridDistance = new JSpinner(new SpinnerNumberModel(value, 0, 500, 1));
			gridDistance.setMaximumSize(new Dimension(50, Integer.MAX_VALUE));
			Box gridDistBox = Box.createHorizontalBox();
			gridDistBox.add(gridDistLabel);
			gridDistBox.add(Box.createHorizontalStrut(10));
			gridDistBox.add(gridDistance);
			gridDistBox.add(Box.createHorizontalGlue());
			
			
			add(useGridBox);
			add(Box.createVerticalStrut(5));
			add(gridDistBox);
		}

		public void update(FlowSetting settings) throws InvalidOptionException{
			settings.setProperty(FlowSetting.Key.USE_GRID, useGrid.isSelected());
			settings.setProperty(FlowSetting.Key.GRID_DISTANCE, (Integer)gridDistance.getValue());
		}
	}
	
	public static FlowSetting showSettingDialog(Window parent, FlowSetting orginal){
		FlowSettingDialog sd = new FlowSettingDialog(parent, orginal);
		sd.dialog.setVisible(true);
		sd.dialog.dispose();
		return sd.getSetting();
	}
}
