package com.ef_prime.rflow.tool;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.util.SwingUtils;

public class OptionDialog {

	private JDialog dialog;
	private JRadioButton[] buttons;
	private Integer selected = null;
	
	private OptionDialog(final Window parent, String title, String message, String[] options){
		if(parent instanceof Dialog)
			dialog = new JDialog((Dialog)parent, title, true);
		else
			dialog = new JDialog((Frame)parent, title, true);
		
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.getContentPane().setLayout(new BorderLayout());

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		int sp = 10;
		panel.setBorder(new EmptyBorder(sp, sp, sp, sp));
		
		JTextArea messageLabel = new JTextArea(message);
		messageLabel.setEditable(false);
		messageLabel.setOpaque(false);
		Box buttonBox = Box.createVerticalBox();
		buttonBox.setBorder(new EmptyBorder(sp,sp,sp,sp));
		buttons = SwingUtils.createRadioButtons(options);
		for(JRadioButton button : buttons)
			buttonBox.add(button);
		buttons[0].setSelected(true);
		
		ActionListener okAction = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				for(int i = 0; i < buttons.length; i++){
					JRadioButton button = buttons[i];
					if(button.isSelected())
						selected = i;
				}
				dialog.dispose();
			}
		};
		ActionListener cancelAction = new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				selected = null;
				dialog.dispose();
			}
		};
		ButtonPanel buttonPanel = new ButtonPanel(okAction, cancelAction);
		
		panel.add(messageLabel);
		panel.add(SwingUtils.createGluedHorizontalBox(buttonBox));
		panel.add(buttonPanel);
		
		dialog.getContentPane().add(panel);

		dialog.pack();
		dialog.setLocationRelativeTo(parent);
	}
	
	/**
	 * 
	 * @param parent
	 * @param title
	 * @param message
	 * @param options
	 * @return index of selected option of options. null if canceled.
	 */
	public static Integer showOptionDialog(final Window parent, String title, String message, String[] options){
		OptionDialog optionDialog = new OptionDialog(parent, title, message, options);
		optionDialog.dialog.setVisible(true);
		optionDialog.dialog.dispose();
		return optionDialog.selected;
	}
}
