package com.ef_prime.rflow.tool;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.resource.Messages;

public class ProgressMonitor {
	
	private ProgressDialog dialog;
	//private long openDelay = 500;//msec
	private boolean canceled;

	public ProgressMonitor(Dialog owner, String title, boolean modal) throws HeadlessException {
		dialog = new ProgressDialog(owner, title, modal);
	}

	public ProgressMonitor(Frame owner, String title, boolean modal) throws HeadlessException {
		dialog = new ProgressDialog(owner, title, modal);
	}
	
	public void open(){
		if(dialog != null){
			dialog.setVisible(true);
		}
	}
	
	public void close(){
		dialog.dispose();
		dialog = null;
	}
	
	public boolean isCanceled() {
		return canceled;
	}

	public void setLabelText(String s) {
		dialog.setLabelText(s);
	}

	private class ProgressDialog extends JDialog {
	
		private JLabel label;
		private JProgressBar progressBar;
	
	
		public ProgressDialog(Dialog owner, String title, boolean modal) throws HeadlessException {
			super(owner, title, modal);
			init();
		}
	
		public ProgressDialog(Frame owner, String title, boolean modal) throws HeadlessException {
			super(owner, title, modal);
			init();
		}
	
		private void init(){
			setDefaultCloseOperation(ProgressDialog.DO_NOTHING_ON_CLOSE);
			getContentPane().setLayout(new BorderLayout());
	
			//label
			label = new JLabel(" "); //$NON-NLS-1$
			Box labelBox = Box.createHorizontalBox();
			labelBox.add(label);
			labelBox.add(Box.createHorizontalGlue());
			//progress bar
			progressBar = new JProgressBar();
			progressBar.setIndeterminate(true);
			progressBar.setPreferredSize(new Dimension(200, 20));
			progressBar.setMaximumSize(progressBar.getPreferredSize());
			//cancel button
			JButton cancelButton = new JButton(Messages.getString("Button.cancel")); //$NON-NLS-1$
			cancelButton.setPreferredSize(new Dimension(100, 20));
			cancelButton.setMaximumSize(cancelButton.getPreferredSize());
			cancelButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					canceled = true;
				}
			});
			Box buttonBox = Box.createHorizontalBox();
			buttonBox.add(Box.createHorizontalGlue());
			buttonBox.add(cancelButton);
			buttonBox.add(Box.createHorizontalGlue());
	
	
			Box mainBox = Box.createVerticalBox();
			mainBox.setBorder(new EmptyBorder(10,10,10,10));
			mainBox.add(labelBox);
			mainBox.add(Box.createVerticalStrut(5));
			mainBox.add(progressBar);
			mainBox.add(Box.createVerticalStrut(10));
			mainBox.add(buttonBox);
	
			getContentPane().add(mainBox);
	
			pack();
			setLocationRelativeTo(getOwner());
			setResizable(false);
		}
	
		public void setLabelText(String s){
			label.setText(s);
		}
	}
}
