package com.ef_prime.rflow.tool;

import javax.swing.text.Segment;

import org.fife.ui.rsyntaxtextarea.AbstractTokenMaker;
import org.fife.ui.rsyntaxtextarea.RSyntaxUtilities;
import org.fife.ui.rsyntaxtextarea.Token;
import org.fife.ui.rsyntaxtextarea.TokenMap;

public class RTokenMaker extends AbstractTokenMaker{
	
	
	@Override
	public void addToken(Segment segment, int start, int end, int tokenType, int startOffset) {

		switch (tokenType) {
			case Token.IDENTIFIER:
				int value = wordsToHighlight.get(segment, start,end);
				if (value!=-1)
					tokenType = value;
				break;
		}

		super.addToken(segment, start, end, tokenType, startOffset);

	}
	
	@Override
	public boolean getCurlyBracesDenoteCodeBlocks() {
		return true;
	}

	@Override
	public boolean getShouldIndentNextLineAfter(Token t) {
		if (t!=null && t.textCount==1) {
			char ch = t.text[t.textOffset];
			return ch=='{' || ch=='(';
		}
		return false;
	}
	
	@Override
	public TokenMap getWordsToHighlight() {
		TokenMap map = new TokenMap();
		map.put("if", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("else", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("while", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("repeat", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("for", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("break", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("next", Token.RESERVED_WORD); //$NON-NLS-1$
		map.put("function", Token.RESERVED_WORD); //$NON-NLS-1$
		
		map.put("TRUE", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("FALSE", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("T", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("F", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		
		//not boolean really
		map.put("NULL", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NaN", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NA", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("Inf", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NA_integer_", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NA_real_", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NA_character_", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		map.put("NA_complex_", Token.LITERAL_BOOLEAN); //$NON-NLS-1$
		
		return map;
	}
	
	@Override
	public String[] getLineCommentStartAndEnd() {
		return new String[]{"#", null}; //$NON-NLS-1$
	}
	
	@Override
	public Token getTokenList(Segment text, int initialTokenType,
			int startOffset) {
		
		resetTokenList();

		int newStartOffset = startOffset - text.offset;

		int currentTokenStart = text.offset;
		int currentTokenType  = initialTokenType;

		int end = text.offset + text.count;
		for (int i = text.offset; i < end; ) {

			currentTokenStart = i;
			char c = text.array[i];

			a:switch (currentTokenType) {
			default:
				//SkipSpace
				currentTokenStart = i;
				while(i < end && RSyntaxUtilities.isWhitespace((text.array[i]))){
					i++;
				};
				if(i > currentTokenStart)
					addToken(text, currentTokenStart, i-1, Token.WHITESPACE, newStartOffset+currentTokenStart);
				//end
				if(i == end)break;
				//SkipComment
				if(text.array[i] == '#'){
					addToken(text, i, end-1, Token.COMMENT_EOL, newStartOffset+i);
					i = end;
					break;
				}
				//. digit or symbol
				boolean gotosymbol = (text.array[i] == '.' && (i >= end-1 || !RSyntaxUtilities.isDigit(text.array[i+1])));
				if(!gotosymbol){
					//literal numbers (simple eval)
					if((c = text.array[i]) == '.' || RSyntaxUtilities.isDigit(c)){
						currentTokenStart = i;
						i++;
						while(i < end && (RSyntaxUtilities.isDigit((c = text.array[i])) ||
								c == '.' || c == 'e' || c == 'E' ||
								c == 'x' || c == 'X' || c == 'L')){
							i++;
						}
						addToken(text, currentTokenStart, i-1, Token.LITERAL_NUMBER_DECIMAL_INT, newStartOffset+currentTokenStart);
						break;
					}
					
					//literal strings or symbol
					if((c = text.array[i]) == '"' || c == '\'' || c == '`'){
						char quote = c;
						currentTokenStart = i;
						i++;
						while(i < end && (c = text.array[i]) != quote){
							if(c == '\\'){
								if(i >= end-1)break;
								c = text.array[++i];
								if(c >= '0' && c <= '8'){
									
								}else if(c == 'x'){
									
								}else if(c == 'u'){
									
								}else if(c == 'U'){
									
								}else{
									
								}
							}
							i++;
						}
						if(i < end){
							i++;//close quote
						}
						
						addToken(text, currentTokenStart, i-1, (quote == '`')?Token.IDENTIFIER:Token.LITERAL_CHAR, newStartOffset+currentTokenStart);
						break;
					}
					
					//special value
					if(c == '%'){
						currentTokenStart = i;
						i++;
						while(i < end && (c = text.array[i]) != '%'){
							i++;
						}
						if(c == '%'){
							i++;
							addToken(text, currentTokenStart, i-1, Token.OPERATOR, newStartOffset+currentTokenStart);
						}else{
							addToken(text, currentTokenStart, i-1, Token.ERROR_IDENTIFIER, newStartOffset+currentTokenStart);
						}
						break;
					}
				}
				
				//symbol
				if((c = text.array[i]) == '.' || Character.isUnicodeIdentifierStart(c) || c == '_'){
					currentTokenStart = i;
					i++;
					while(i < end && 
							(Character.isUnicodeIdentifierPart((c = text.array[i])) 
									|| c == '.' || c == '_')){
						i++;
					}
					addToken(text, currentTokenStart, i-1, Token.IDENTIFIER, newStartOffset+currentTokenStart);
					break;
				}
				
				//operator
				currentTokenStart = i;
				i++;
				char next = ' ';
				boolean noNext = (i >= end);
				if(!noNext)
					next = text.array[i];
				int type = Token.OPERATOR;
				switch(c){
			    case '<':
			    	if(!noNext && (next == '=' || next == '-' || next == '<')){
			    		i++;
			    		if(next == '<'){
			    			addToken(text, currentTokenStart, i-1, 
			    					(i < end && text.array[i] == '-')?Token.OPERATOR:Token.ERROR_IDENTIFIER, 
			    							newStartOffset+currentTokenStart);
			    			break a;
			    		}
			    	}
			    	break;
			    case '-':
			    	if(!noNext && (next == '>')){
			    		i++;
			    		if(i < end && text.array[i] == '>')
			    			i++;
			    	}
			    	break;
			    case '>':
			    	if(!noNext && (next == '=')) i++;
			    	break;
			    case '!':
			    	if(!noNext && (next == '=')) i++;
			    	break;
			    case '=':
			    	if(!noNext && (next == '=')) i++;
			    	break;
			    case ':':
			    	if(!noNext && (next == '=')){
			    		i++;
			    		if(i < end && text.array[i] == ':'){
			    			i++;
				    		if(i < end && text.array[i] == ':'){
				    			i++;
				    		}
			    		}
			    	}
			    	else if(!noNext && (next == '=')) i++;
			    	break;
			    case '&':
			    	if(!noNext && (next == '&')) i++;
			    	break;
			    case '|':
			    	if(!noNext && (next == '|')) i++;
			    	break;
			    case '{':
			    case '}':
			    case '(':
			    case ')':
			    	type = Token.SEPARATOR;
			    	break;
			    case '[':
			    	if(!noNext && (next == '[')) i++;
			    	type = Token.SEPARATOR;
			    	break;
			    case ']':
			    	type = Token.SEPARATOR;
			    case '?':
			    	break;
			    case '*':
			    	if(!noNext && (next == '*')) i++;
			    	break;
			    case '+':
			    case '/':
			    case '^':
			    case '~':
			    case '$':
			    case '@':
			    	break;
			    default:
			    	break;
				}
		    	addToken(text, currentTokenStart, i-1, type, newStartOffset+currentTokenStart);
		    	
				break;
			}
		}
		
		addNullToken();
		
		/*
		 * 文法解析
		 */

		//tokenlistから関数になるidentifierを抜き出して変更する
		Token t = firstToken;
		while(t != null){
			if(t.type == Token.IDENTIFIER){
				Token target = t;
				t = t.getNextToken();
				if(t == null)break;
				if(t.type == Token.WHITESPACE)t = t.getNextToken();
				if(t == null)break;
				if(t.type == Token.SEPARATOR && t.textCount == 1
						&& t.text[t.textOffset] == '('){
					target.type = Token.FUNCTION;
				}
			}
			
			t = t.getNextToken();
		}
		
		return firstToken;
	}
}
