package com.ef_prime.rflow.tool;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.ef_prime.rflow.ActiveViewerListener;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.flow.FlowViewer;
import com.ef_prime.rflow.flow.model.FlowNode;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.DocumentAdapter;

public class SearchDialog extends JDialog implements ActiveViewerListener{

	private Rflow rflow;
	
	private InputPanel input = new InputPanel();
	private OutputPanel output = new OutputPanel();
	private OptionPanel option = new OptionPanel();
	
	private static final int DEFAULT_WIDTH = 170;//+20(space)
	private static final int DEFAULT_LINE_HEIGHT = 20;
	
	public SearchDialog(Dialog owner, Rflow rflow) throws HeadlessException {
		super(owner);
		init(rflow);
	}

	public SearchDialog(Frame owner, Rflow rflow) throws HeadlessException {
		super(owner);
		init(rflow);
	}
	
	private void init(Rflow rflow){
		this.rflow = rflow;
		rflow.addActiveViewerListener(this);
		addWindowListener(new WindowAdapter(){
			@Override
			public void windowOpened(WindowEvent e) {
				update();
			}
		});
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		setTitle(Messages.getString("Action.find")); //$NON-NLS-1$
		
		Box box = Box.createVerticalBox();
		box.setBorder(new EmptyBorder(10,10,10,10));
		
		box.add(input);
		box.add(option);
		box.add(Box.createVerticalStrut(5));
		box.add(output);
		box.add(Box.createVerticalStrut(10));
		box.add(new ButtonPanel());
		
		getContentPane().add(box);
		
		pack();
	}
	
	private void setTitle(FlowViewer viewer){
		String title = Messages.getString("Action.find"); //$NON-NLS-1$
		if(!viewer.isRootViewer())
			title += " - " + viewer.getGraph().getParentNode().getTitle(); //$NON-NLS-1$
		setTitle(title);
	}
	
	private void update(){
		//set title
		setTitle(rflow.getActiveViewer());
		
		output.model.clear();
		
		String word = input.getText();
		
		if(word.length() < 1)
			return;
		
		if(!option.isCaseSensitive())
			word = word.toLowerCase();
		
		
		for(Object o : rflow.getActiveViewer().getGraph().getVertices()){
			FlowNode node = (FlowNode)o;
			String command = node.getModel().getAllCommand();

			if(!option.isCaseSensitive())
				command = command.toLowerCase();
				
			if(command.contains(word))
				output.model.addElement(node);
		}
	}
	
	private class InputPanel extends JPanel{
		
		private JTextField field;
		
		InputPanel(){
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_LINE_HEIGHT));
			setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_LINE_HEIGHT));
			
			field = new RflowTextField();
			field.getDocument().addDocumentListener(new DocumentAdapter(){
				@Override
				public void documentUpdate(DocumentEvent e) {
					SearchDialog.this.update();
				}
			});
			
			add(field);
		}
		
		String getText(){
			return field.getText();
		}
	}
	
	private class OutputPanel extends JPanel{
		
		private JList list;
		private DefaultListModel model;
		
		OutputPanel(){
			super();
			setLayout(new BorderLayout());
			//setMaximumSize(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
			setPreferredSize(new Dimension(DEFAULT_WIDTH, 200));
			
			model = new DefaultListModel();
			list = new JList(model);
			list.setCellRenderer(new ListRenderer());
			list.addListSelectionListener(new ListSelectionListener(){
				public void valueChanged(ListSelectionEvent e) {
					rflow.getActiveViewer().getPickedState().clearPickedVertices();
					JList list = (JList)e.getSource();
					for(Object o : list.getSelectedValues()){
						FlowNode node = (FlowNode)o;
						rflow.getActiveViewer().getPickedState().pick(node, true);
					}
				}
			});
			list.addMouseListener(new MouseAdapter(){
				@Override
				public void mousePressed(MouseEvent e) {
					if(e.getClickCount() == 2 && SwingUtilities.isLeftMouseButton(e)){
						int index = list.locationToIndex(e.getPoint());
						if(index < 0)return;
						showConfigDialog(index);
					}
				}
			});
			list.addKeyListener(new KeyAdapter(){
				@Override
				public void keyPressed(KeyEvent e) {
					if(e.getKeyCode() == KeyEvent.VK_ENTER){
						int[] rows = list.getSelectedIndices();
						if(rows.length > 1 || rows.length < 0)return;
						int row = rows[0];
						showConfigDialog(row);
					}
				}
			});
			
			add(new JScrollPane(list));
		}
		
		private void showConfigDialog(int index){
			FlowNode node = (FlowNode)model.getElementAt(index);
			rflow.getActiveViewer().editNode(node);
		}
		
		private class ListRenderer extends DefaultListCellRenderer{
			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				
				FlowNode node = (FlowNode)value;
				setText(node.getTitle());
				Icon icon = rflow.viewer.getRenderer().getNodeRendering().getSmallIcon(node);
				if(icon != null)
					setIcon(icon);
				
				return this;
			}
		}
	}
	
	private class OptionPanel extends JPanel{
		JCheckBox caseSensitive;
		
		public OptionPanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			//setBorder(new TitledBorder("オプション"));
			setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
			setPreferredSize(new Dimension(DEFAULT_WIDTH, 20));
			
			
			caseSensitive = new JCheckBox(Messages.getString("SearchDialog.case_sensitive"), false); //$NON-NLS-1$
			caseSensitive.setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_LINE_HEIGHT));
			caseSensitive.setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_LINE_HEIGHT));
			caseSensitive.addItemListener(new ItemListener(){
				public void itemStateChanged(ItemEvent e) {
					SearchDialog.this.update();
				}
			});
			
			
			Box box = Box.createVerticalBox();
			box.add(caseSensitive);
			
			add(Box.createHorizontalStrut(10));
			add(box);
			add(Box.createHorizontalGlue());
		}
		
		boolean isCaseSensitive(){
			return caseSensitive.isSelected();
		}
	}
	
	private class ButtonPanel extends JPanel{
		public ButtonPanel() {
			super();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			
			setMaximumSize(new Dimension(Integer.MAX_VALUE, DEFAULT_LINE_HEIGHT));
			setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_LINE_HEIGHT));
			
			
			JButton close = new JButton(Messages.getString("Button.close")); //$NON-NLS-1$
			close.setMaximumSize(new Dimension(80, DEFAULT_LINE_HEIGHT));
			close.setPreferredSize(new Dimension(80, DEFAULT_LINE_HEIGHT));
			close.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					SearchDialog.this.dispose();
				}
			});
			
			add(Box.createHorizontalGlue());
			add(close);
			add(Box.createHorizontalGlue());
		}
	}
	

	
	public void activeViewerChanged(FlowViewer activeViewer) {
		update();
	}
}
