package com.ef_prime.rflow.tool;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

import com.ef_prime.rflow.Prefs;
import com.ef_prime.rflow.Rflow;

public abstract class SupportControl  implements ChangeListener, ItemAcceptListener{
	
	protected JTextComponent input;
	//protected Document document;
	
	private boolean autoSupport = false;
	private boolean allowLeftRightInput = true;
	
	private int supportListIndex;
	private boolean supportFlag = false;
	
	private List<ItemAcceptListener> itemAcceptListeners = new ArrayList<ItemAcceptListener>();
	
	public SupportControl(JTextComponent input){
		this.input = input;
		//this.document = input.getDocument();
		input.addKeyListener(new InputKeyListener(this));
		//document.addDocumentListener(this);
		input.getCaret().addChangeListener(this);
		addItemAcceptListener(this);
		input.addFocusListener(new FocusAdapter(){
			@Override
			public void focusLost(FocusEvent e) {
				if(popup != null && e.getOppositeComponent() != popup.getList()){
					removeSupportPopup();
				}
			}
		});
	}
	
	/**
	 * サポート用のワードの一覧を返す。
	 * ない場合はnullを返す
	 * @param input
	 * @return ない場合はnull
	 */
	abstract protected String[] getSupportWords(String input, int index);
	
	protected Point getPopupPoint(){
		Point p;
		try {
			int pos = partsPos;//input.getCaretPosition();
			Rectangle r = input.modelToView(pos);
			p = new Point(r.x, r.y+r.height);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		SwingUtilities.convertPointToScreen(p, input);
		return p;
		/*
		return new Point(input.getLocationOnScreen().x,
				input.getLocationOnScreen().y+input.getHeight());
		*/
	}
	
	protected int getPopupWidth(){
		//return input.getWidth();
		return 200;
	}
	
	public void addItemAcceptListener(ItemAcceptListener listener){
		itemAcceptListeners.add(listener);
	}
	
	public void removeItemAcceptListener(ItemAcceptListener listener){
		itemAcceptListeners.remove(listener);
	}
	
	/*
	public void setDocumentSupport(boolean b){
		document.removeDocumentListener(this);
		if(b){
			document.addDocumentListener(this);
		}
	}
	*/
	
	public void setCaretSupport(boolean b){
		input.getCaret().removeChangeListener(this);
		if(b)
			input.getCaret().addChangeListener(this);
	}
	
	/*-------------------------Document Listener--------------------------*/
	/*
	public void changedUpdate(DocumentEvent e) {
		if(supportFlag)
			showSupport(getText(e));
	}
	public void insertUpdate(DocumentEvent e) {
		if(supportFlag)
			showSupport(getText(e));
	}
	public void removeUpdate(DocumentEvent e) {
		if(supportFlag)
			showSupport(getText(e));
	}
	*/
	/*-------------------------ChangeListener--------------------------*/
	public void stateChanged(ChangeEvent e) {
		if(supportFlag && input.isEnabled())
			showSupport(getText(e));
	}
	/*-------------------------ItemAcceptedListener--------------------------*/
	public void itemAccepted(JTextComponent source, String selected) {
		Document d = input.getDocument();
		String s = getCaretPosPart();
		//if(s == null)return;
		if(s == null)
			s = ""; //$NON-NLS-1$
		int pos = input.getCaretPosition()-s.length();
		//int pos = partsPos;
		int dPos = s.indexOf('$');
		dPos++;
		pos += dPos;
		try {
			d.remove(pos, s.length()-dPos);
			d.insertString(pos,selected, Prefs.SIMPLE_ATTR);
		} catch (BadLocationException e) {
			e.printStackTrace();throw new RuntimeException(e);
		}
	}	
	
	SupportPopup popup;

	public void showSupport(String text){
		String[] supportList = getSupportWords(text, supportListIndex);
		if(supportList == null || supportList.length < 1){
			removeSupportPopup();
			return;
		}
		
		if(popup == null){
			if(!input.isShowing())return;
			Point p = getPopupPoint();
			if(p == null)return;
			popup = new SupportPopup(input, p.x, p.y, new Dimension(getPopupWidth(), 100));
			popup.getList().addKeyListener(popupListKeyListener);
			popup.getList().addMouseListener(popupListMouseListener);
			showSupportPopup();
		}
		
		popup.setListModel(supportList);
	}
	
	protected String getText(DocumentEvent e){
		String s = null;
		try {
			s = e.getDocument().getText(0, e.getDocument().getLength());
		} catch (BadLocationException e1) {	}
		return s;
	}

	
	protected String getText(ChangeEvent e) {
		return getCaretPosPart();
	}
	
	public void showSupportPopup(){
		if(popup != null){
			popup.show();
			supportFlag = true;
		}
	}
	
	public void removeSupportPopup(){
		if(popup != null){
			popup.hide();
			popup = null;
			if(!autoSupport)
				supportFlag = false;
		}
	}
	
	public boolean isSupportPopup(){
		return popup != null;
	}
	
	public synchronized void acceptItem(){
		supportFlag = false;

		String s = popup.getSelected();
		
		for(ItemAcceptListener listener : itemAcceptListeners){
			listener.itemAccepted(input, s);
		}
		supportFlag = true;
		
		removeSupportPopup();
	}
	
	KeyListener popupListKeyListener = new KeyAdapter(){
		@Override
		public void keyPressed(KeyEvent e) {
			switch(e.getKeyCode()){
			case KeyEvent.VK_ENTER:
				acceptItem();
				break;
			case KeyEvent.VK_ESCAPE:
				removeSupportPopup();break;
			}
		}
	};
	
	MouseListener popupListMouseListener = new MouseAdapter(){
		@Override
		public void mousePressed(MouseEvent e) {
			if(SwingUtilities.isLeftMouseButton(e) && (e.getClickCount() == 2)){
				acceptItem();
			}
		}
	};
	

	public String getRawText() {
		try {
			Document document = input.getDocument();
			return document.getText(0, document.getLength());
		} catch (BadLocationException e) {
			e.printStackTrace();throw new RuntimeException();
		}
	}
	
	private int partsPos;
	private static final char[] non = new char[]{
			'#',
			'\"',
			'\'',
			'%',
			'`',
			'<',
			'-',
			'>',
			'!',
			'=',
			':',
			'&',
			'|',
			'{',
			'}',
			'(',
			')',
			'[',
			']',
			'?',
			'*',
			'+',
			'/',
			'^',
			'~',
			//'$',
			'@',
			' ',
			'\t',
			'\f',
			'\n',
			',',
			'\\'
	};
	
	private static final TreeSet<Character> nonSet;
	static{
		nonSet  = new TreeSet<Character>();
		for(char c : non)
			nonSet.add(c);
	}
	
	private String getCaretPosPart(){
		String text = getRawText();
		int caretPos = input.getCaretPosition();
		partsPos = caretPos;
		if(caretPos > text.length())caretPos = text.length();
		int pos = caretPos;
		pos--;
		while(pos > -1){
			char c = text.charAt(pos);
			if(!nonSet.contains(c))
				pos--;
			else{
				pos++;
				break;				
			}
		}
		if(pos < 0)pos = 0;
		if(caretPos <= pos)return null;
		partsPos = pos;
		return text.substring(pos, caretPos);
	}
	
	private class InputKeyListener extends KeyAdapter{
		
		private SupportControl supportControl;
		
		public InputKeyListener(SupportControl supportControl) {
			this.supportControl = supportControl;
		}

		@Override
		public void keyPressed(KeyEvent e) {
			SupportPopup popup = supportControl.popup;
				
			if(popup == null){
				switch(e.getKeyCode()){
				case KeyEvent.VK_SPACE:
					if(e.isControlDown()){
						String parts = getCaretPosPart();
						supportListIndex = (parts == null || parts.length() == 0)?1:0;
						supportControl.showSupport(parts);
						e.consume();
					}
					break;
				}
				return;
			}
			
			switch(e.getKeyCode()){
    		case KeyEvent.VK_P:
    			if(!Rflow.TEST || !e.isControlDown())break;
			case KeyEvent.VK_UP:
				popup.selectPrevious();e.consume();break;
    		case KeyEvent.VK_N:
    			if(!Rflow.TEST || !e.isControlDown())break;
			case KeyEvent.VK_DOWN:
				popup.selectNext();e.consume();break;
			case KeyEvent.VK_LEFT:
			case KeyEvent.VK_RIGHT:
				if(!supportControl.allowLeftRightInput)
					supportControl.removeSupportPopup();break;
			case KeyEvent.VK_ENTER:
				if(popup.isSelectionEmpty()){
					supportControl.removeSupportPopup();e.consume();break;
				}else{
					supportControl.acceptItem();e.consume();break;
				}
			case KeyEvent.VK_ESCAPE:
				supportControl.removeSupportPopup();e.consume();break;
			}
			
		}
	}

	public boolean isAutoSupport() {
		return autoSupport;
	}

	public void setAutoSupport(boolean autoSupport) {
		this.autoSupport = autoSupport;
		supportFlag = autoSupport;
	}

	public void setAllowLeftRightInput(boolean allowLeftRightInput) {
		this.allowLeftRightInput = allowLeftRightInput;
	}
}
