package com.ef_prime.rflow.tool;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

public abstract class TreeDnDMoveHandler extends DropTargetAdapter{

	private JTree tree;
	private MutableTreeNode targetNode;
	private boolean useUpperSide;
	private boolean upperSide;
	
	public TreeDnDMoveHandler(JTree tree, boolean useUpperSide){
		this.tree = tree;
		this.useUpperSide = useUpperSide;
	}
	
	private DefaultTreeModel getTreeModel(){
		try{
			return (DefaultTreeModel)tree.getModel();
		}catch(ClassCastException e){
			throw new ClassCastException("treemodel should be DefaultTreeModel"); //$NON-NLS-1$
		}
	}
	
	/**
	 * return target node
	 * target node is the node mouse is on
	 * @return
	 */
	public MutableTreeNode getTargetNode() {
		return targetNode;
	}
	
	public boolean isUpperSide() {
		return useUpperSide && upperSide;
	}
	
	public boolean useUpperSide() {
		return useUpperSide;
	}
	
	
	@Override
	public void dragExit(DropTargetEvent dte) {
        targetNode = null;
        tree.repaint();
	}
	
	@Override
	public void dropActionChanged(DropTargetDragEvent dtde) {
//		System.out.println("drop action changed");
//		System.out.println("d"+dtde.getDropAction());
//		System.out.println("s"+dtde.getSourceActions());
		dragOver(dtde);
	}
	
	@Override
	public void dragOver(DropTargetDragEvent dtde) {        
		if(!dtde.getTransferable().isDataFlavorSupported(getNodeFlavor())){
			rejectDrag(dtde);
			return;
		}
        Point p = dtde.getLocation();
        TreePath path = tree.getPathForLocation(p.x, p.y);
        if(path == null){
        	rejectDrag(dtde);
			return;
        }
        Rectangle rect = tree.getPathBounds(path);
        upperSide = p.y < rect.y+rect.height/3;
        
        MutableTreeNode[] nodes;
        try {
        	nodes = (MutableTreeNode[])dtde.getTransferable().getTransferData(getNodeFlavor());
		} catch (Exception e) {
			e.printStackTrace();rejectDrag(dtde);return;
		}

		MutableTreeNode parentNode = (MutableTreeNode)((TreeNode)path.getLastPathComponent()).getParent();
		//cannot move or copy to descendant
		while(parentNode!=null) {
            if(nodes[0] == parentNode) {
                rejectDrag(dtde);
                return;
            }
            parentNode = (MutableTreeNode)parentNode.getParent();
        }
		
		
		targetNode = (MutableTreeNode)path.getLastPathComponent();

		if(!checkDragAccept(dtde, nodes)){
            rejectDrag(dtde);
            return;
		}
		
        //tree.setSelectionPath(path);
        dtde.acceptDrag(dtde.getDropAction());
        tree.repaint();
	}
	
	public void drop(DropTargetDropEvent dtde) {
		if(targetNode == null){
			dropFailed(dtde);
			return;
		}

		MutableTreeNode[] nodes;
        try {
        	nodes = (MutableTreeNode[])dtde.getTransferable().getTransferData(getNodeFlavor());
		} catch (Exception e) {
			e.printStackTrace();
			dropFailed(dtde);return;
		}
		
		/*
		for(MutableTreeNode node : nodes)
			if(targetNode == node){
				dropFailed(dtde);
				return;
			}
		*/
		
		//remove if move
		//if(dtde.getDropAction() == DnDConstants.ACTION_MOVE)
		//	removeDragNode(dtde, nodes);
		switch(dtde.getDropAction()){
		case DnDConstants.ACTION_MOVE:
			moveDragNode(dtde, nodes, getCurrentTargetParent(), getCurrentTargetIndex());
			break;
		case DnDConstants.ACTION_COPY:
			copyDragNode(dtde, nodes, getCurrentTargetParent(), getCurrentTargetIndex());
			break;
		default:
			dropFailed(dtde);
		}
			
        onDropComplete(nodes);
		
        dtde.dropComplete(true);
        targetNode = null;
        tree.repaint();
	}

    protected void rejectDrag(DropTargetDragEvent dtde) {
        dtde.rejectDrag();
        targetNode = null;
        //tree.clearSelection();
        tree.repaint();
    }
    
    private void dropFailed(DropTargetDropEvent dtde){
		dtde.dropComplete(false);
        targetNode = null;
        tree.repaint();
    }
    
    /**
     * 
     * @return drop target's parent node
     */
    public MutableTreeNode getCurrentTargetParent(){
		MutableTreeNode parentNode = (MutableTreeNode)targetNode.getParent();
        if(targetNode.isLeaf() && parentNode!=null){
        	return parentNode;
        }else{
        	if(isUpperSide())
        		return parentNode;
        	else
        		return targetNode;
        }
    }
    
    /**
     * @return drop target index of target parent
     */
    public int getCurrentTargetIndex(){
		MutableTreeNode parentNode = (MutableTreeNode)targetNode.getParent();
        if(targetNode.isLeaf() && parentNode!=null){
        	if(isUpperSide())
        		return parentNode.getIndex(targetNode);
        	else
        		return parentNode.getIndex(targetNode)+1;
        }else{
        	if(isUpperSide())
        		return parentNode.getIndex(targetNode);
        	else
        		return targetNode.getChildCount();
        }
    }
    
    /**
     * returns accept drag or not
     * @param dtde
     * @return
     */
    protected boolean checkDragAccept(DropTargetDragEvent dtde, MutableTreeNode[] nodes){
    	return true;
    }
    
    /*
    protected void removeDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes){
    	for(MutableTreeNode node : nodes)
    		getTreeModel().removeNodeFromParent(node);
    }*/
    
    protected void moveDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes, MutableTreeNode parent, int index){
    	for(MutableTreeNode node : nodes)
    		getTreeModel().removeNodeFromParent(node);
    	
    	int indexNew = getCurrentTargetIndex();
    	
    	for(MutableTreeNode node : nodes)
    		getTreeModel().insertNodeInto(node, parent, indexNew);
    }
    
    protected abstract void copyDragNode(DropTargetDropEvent dtde, MutableTreeNode[] nodes, MutableTreeNode parent, int index);
    
    
    protected void onDropComplete(MutableTreeNode[] dragNodes){
    }
    
    protected DataFlavor getNodeFlavor(){
    	return MutableTreeNodeFlavor.getInstance();
    }

}
