package com.ef_prime.rflow.tool;

import java.awt.Cursor;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.Setting.Key;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.PropertiesUtility;

public class UpdateChecker {
	
	/*
	 * ファイルフォーマット
1.0.0,http://ef-prime.com/20081221/
0.0.5,http://ef-prime.com/20080121/
1.0.0,http://ef-prime.com/20081221/
	 * 
	 * 
	 * 
	 * 
	 */
	
	
	private static final String UPDATE_SITE = "http://www.ef-prime.com/ranalyticflow/update_check"; //$NON-NLS-1$
	//private static final String UPDATE_SITE = "file:///c:/test.txt";

	//update check and show dialog
	public static void updateCheck()
	{
		try {
			//final Setting setting = SerializeUtility.load(Rflow.settingFile);
			//final Setting setting = new Setting();
			//PropertiesUtility.load(setting, Rflow.settingFile);
			final Setting setting = Rflow.setting;
			List<String> lines = getTextFromServer();
			
			// for testing
			//lines = Collections.singletonList("10.15.3,http://www.ef-prime.com/");
			
			boolean result = existsUpdate(lines, setting);

			//System.out.println(result);
			if(result)
			{
				//show dialog and update setting				
				final JFrame frame = new JFrame(Messages.getString("UpdateChecker.title")); //$NON-NLS-1$

				final String version = getVersion(lines.get(0));
				final UpdateConfirmPanel updateConfirmPanel = new UpdateConfirmPanel(
						version,
						getURL(lines.get(0)));
				frame.getContentPane().add(
						updateConfirmPanel);
				
				updateConfirmPanel.jButton1.addActionListener(
						new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						
						boolean selected = updateConfirmPanel.jCheckBox1.isSelected();
						if(selected)
							setting.setProperty(Key.CHECKED_VERSION, version);
						try
						{
							//SerializeUtility.save(setting, Rflow.settingFile);
							PropertiesUtility.store(setting, FileResource.SETTING_FILE);
						}
						catch(Exception ex)
						{
							ex.printStackTrace();
						}
						
						frame.setVisible(false);
					}});
				frame.setIconImage(FileResource.DESKTOP_16.getImage());
				frame.pack();
				int centerY = GraphicsEnvironment.getLocalGraphicsEnvironment().getCenterPoint().y;
				frame.setLocation(600-200, centerY-300);

				
				SwingUtilities.invokeLater(new Runnable(){
					public void run() {
						frame.setVisible(true);
					}});
			}
		}
		catch (Exception ex)
		{
			//do nothing
			ex.printStackTrace();
		}
	}

	private static String getVersion(String line)throws RuntimeException
	{
		return line.split(",")[0]; //$NON-NLS-1$
	}
	
	private static String getURL(String line)throws RuntimeException
	{
		return line.split(",")[1];		 //$NON-NLS-1$
	}
	
	private static List<String> getTextFromServer()
	{
		URL url;
		try {
			url = new URL(UPDATE_SITE);
		} catch (MalformedURLException e) {
			//unreachable
			e.printStackTrace();
			return Collections.emptyList();
		}
		
		ArrayList<String> ret = new ArrayList<String>();
		
		try {
			InputStream in = url.openStream();
			InputStreamReader isr = new InputStreamReader(in);
			BufferedReader reader = new BufferedReader(isr);
			
			String line;
			while((line = reader.readLine()) != null)
				ret.add(line);
			
			reader.close();
			isr.close();
			in.close();
			
		} catch (IOException e) {
			e.printStackTrace();
			return Collections.emptyList();
		}
		
		return ret;
	}
	
	
	
	private static boolean existsUpdate(List<String> lines,Setting setting)
	{
		try
		{
			String latest = lines.get(0);
			String version = latest.split(",")[0];		 //$NON-NLS-1$

			return existsUpdate((String)setting.getProperty(Key.CHECKED_VERSION), version);
		}
		catch(RuntimeException ex){
			ex.printStackTrace();

			return false;
		}
	}	

	//nowよりupdateの方が新しかったらtrue;
	private static boolean existsUpdate(String now,String update)
	{
		String[] s1 = now.split("\\."); //$NON-NLS-1$
		String[] s2 = update.split("\\."); //$NON-NLS-1$
		try
		{
			for(int i = 0;i < s1.length && i < s2.length;i++)
			{
				int t1 = Integer.valueOf(s1[i]);
				int t2 = Integer.valueOf(s2[i]);
				if(t1 < t2)return true;
				if(t1 > t2)return false;
			}
		}
		catch(RuntimeException ex)
		{
			//ex.printStackTrace();
		}
		return false;
	}

	static class URILabel extends JLabel {
		//private final String href;

		public URILabel(final String href) {
			super("<html><a href='" + href + "'>" + href + "</a>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			//this.href = href;
			setCursor(new Cursor(Cursor.HAND_CURSOR));
			addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					/*
					if (!Desktop.isDesktopSupported())
						return;
					try {
						Desktop.getDesktop().browse(new URI(href));
					} catch (IOException ioe) {
						ioe.printStackTrace();
					} catch (URISyntaxException use) {
						use.printStackTrace();
					}
					*/
					BrowserControl.displayURL(href);
				}
			});
		}
	}
	
	public static class UpdateConfirmPanel extends javax.swing.JPanel {
	    
	    public UpdateConfirmPanel(String version,String url) {
	        initComponents(version,url);
	    }
	    
	    private void initComponents(String version,String url) {

	        jLabel1 = new URILabel(url);
	        jCheckBox1 = new javax.swing.JCheckBox();
	        jButton1 = new javax.swing.JButton();
	        jLabel2 = new javax.swing.JLabel();
	        jLabel3 = new javax.swing.JLabel();
	        jLabel4 = new javax.swing.JLabel();
	        
	        setName("Form"); //$NON-NLS-1$

	        //jLabel1.setText("<html><a href=\"" + url + "\">" + url + "</a></html>");
	        jLabel1.setName("jLabel1"); //$NON-NLS-1$

	        jCheckBox1.setText(Messages.getString("UpdateChecker.not_check")); //$NON-NLS-1$
	        jCheckBox1.setName("jCheckBox1"); //$NON-NLS-1$
	        jButton1.setText(Messages.getString("Button.close")); //$NON-NLS-1$
	        jButton1.setName("jButton1"); //$NON-NLS-1$
	        jLabel2.setFont(jLabel2.getFont().deriveFont(jLabel2.getFont().getStyle() | java.awt.Font.BOLD, 16));
	        jLabel2.setText(Messages.getString("UpdateChecker.update_available")); //$NON-NLS-1$
	        jLabel2.setName("jLabel2"); //$NON-NLS-1$
	        jLabel3.setText(Messages.getString("UpdateChecker.version") + version + Messages.getString("UpdateChecker.released")); //$NON-NLS-1$ //$NON-NLS-2$
	        jLabel3.setName("jLabel3"); //$NON-NLS-1$
	        jLabel4.setText(Messages.getString("UpdateChecker.can_download_from_website_below")); //$NON-NLS-1$
	        jLabel4.setName("jLabel4"); //$NON-NLS-1$

	        
	        setMaximumSize(new java.awt.Dimension(288, 147));
	        setMinimumSize(new java.awt.Dimension(288, 147));
	        setPreferredSize(new java.awt.Dimension(288, 147));	        
	        setLayout(null);

	        add(jLabel1);
	        jLabel1.setBounds(10, 90, 203, 13);
	        add(jLabel4);
	        jLabel4.setBounds(10, 60, 203, 13);
	        jLabel2.setFont(jLabel2.getFont().deriveFont(jLabel2.getFont().getStyle() | java.awt.Font.BOLD, 16));
	        add(jLabel2);
	        jLabel2.setBounds(10, 10, 137, 17);
	        add(jLabel3);
	        jLabel3.setBounds(10, 40, 163, 13);
	        add(jCheckBox1);
	        jCheckBox1.setBounds(10, 120, 197, 21);
	        add(jButton1);
	        jButton1.setBounds(220, 120, 61, 21);
	        }// </editor-fold>
	    
	    
	    // Variables declaration - do not modify
	    public javax.swing.JButton jButton1;
	    public javax.swing.JCheckBox jCheckBox1;
	    private javax.swing.JLabel jLabel1;
	    private javax.swing.JLabel jLabel2;
	    private javax.swing.JLabel jLabel3;
	    private javax.swing.JLabel jLabel4;
	    // End of variables declaration
	    
	}

	
	static class BrowserControl
	{
	    public static void displayURL(String url)
	    {
	        boolean windows = isWindowsPlatform();
	        String cmd = null;
	        try
	        {
	            if (windows)
	            {
	                cmd = WIN_PATH + " " + WIN_FLAG + " " + url; //$NON-NLS-1$ //$NON-NLS-2$
	                Runtime.getRuntime().exec(cmd);
	            }
	            else
	            {
	                cmd = UNIX_PATH + " " + UNIX_FLAG + "(" + url + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	                Process p = Runtime.getRuntime().exec(cmd);
	                try
	                {
	                    int exitCode = p.waitFor();
	                    if (exitCode != 0)
	                    {
	                        cmd = UNIX_PATH + " "  + url; //$NON-NLS-1$
	                        p = Runtime.getRuntime().exec(cmd);
	                    }
	                }
	                catch(InterruptedException x)
	                {
	                    System.err.println("Error bringing up browser, cmd='" + //$NON-NLS-1$
	                                       cmd + "'"); //$NON-NLS-1$
	                    System.err.println("Caught: " + x); //$NON-NLS-1$
	                }
	            }
	        }
	        catch(IOException x)
	        {
	            // couldn't exec browser
	            System.err.println("Could not invoke browser, command=" + cmd); //$NON-NLS-1$
	            System.err.println("Caught: " + x); //$NON-NLS-1$
	        }
	    }
	    
	    public static boolean isWindowsPlatform()
	    {
	        String os = System.getProperty("os.name"); //$NON-NLS-1$
	        return ( os != null && os.startsWith(WIN_ID));
	    }

	    private static final String WIN_ID = "Windows"; //$NON-NLS-1$
	    private static final String WIN_PATH = "rundll32"; //$NON-NLS-1$
	    private static final String WIN_FLAG = "url.dll,FileProtocolHandler"; //$NON-NLS-1$
	    private static final String UNIX_PATH = "netscape"; //$NON-NLS-1$
	    private static final String UNIX_FLAG = "-remote openURL"; //$NON-NLS-1$
	}

}
