package com.ef_prime.rflow.tool;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import com.ef_prime.rflow.FileResource;
import com.ef_prime.rflow.Platform;
import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.Setting;
import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.util.FileChooserUtility;
import com.ef_prime.rflow.util.FileControl;
import com.ef_prime.rflow.util.FileUtility;

public class WorkingDirSetter {
	
	public static boolean setWorkingDir(FileControl fileControl, Frame parent, File dir){
		File workDir;
		boolean copySample = false;
		if(dir != null){
			if(dir.exists())
				return true;
			JOptionPane.showMessageDialog(parent, Messages.getString("WorkingDirSetter.set_def_dir_notexist"),  //$NON-NLS-1$
					Messages.getString("Error.title.error"), JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
			workDir = dir;
		}else{
			workDir = Platform.getUserWorkingDir();
		}

		AskMakeDirDialog dialog = new AskMakeDirDialog(parent, workDir);
		dialog.setVisible(true);
		dialog.dispose();

		if(dialog.workDir == null)
			return false;

		workDir = dialog.workDir;
		copySample = dialog.copySample;
		
		
		if(!workDir.exists()){
			int ret = JOptionPane.showConfirmDialog(parent, Messages.getString("WorkingDirSetter.dir_notexist_ask_create"), //$NON-NLS-1$
					Messages.getString("Message.confirm"), JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE); //$NON-NLS-1$
			if(ret != JOptionPane.YES_OPTION)
				return setWorkingDir(fileControl, parent, null);

			if(!fileControl.makeDir(workDir))
				return setWorkingDir(fileControl, parent, null);
		}


		if(copySample){//サンプルのコピー
//			File sampleFolder = new File(workDir, "SampleAnalysis"); //$NON-NLS-1$
			//File sampleFile = new File(sampleFolder, FileResource.SAMPLE_FILE.getName());

			try{
//				if(!sampleFolder.exists())
//					if(!fileControl.makeDir(sampleFolder))throw new Exception();
//
				copySamples(fileControl, workDir);
//				LinkedList<File> copyFiles = new LinkedList<File>();
//				for(File f : FileResource.SAMPLE_FOLDER.listFiles())
//					copyFiles.add(f);
//				while(!copyFiles.isEmpty()){
//					File f = copyFiles.removeLast();
//					if(f.isDirectory()){
//						File newFolder = new File(workDir, f.getName());
//						if(!newFolder.exists())
//							if(!fileControl.makeDir(newFolder))throw new Exception();
//						for(File child : f.listFiles()){
//							copyFiles.add(child);
//						}
//					}else if(f.isFile()){
//						File abs = FileUtility.makeAbstractPath(FileResource.SAMPLE_FOLDER, f);
//						if(!fileControl.copyFile(f, new File(workDir, abs.getPath()), FileControl.OVERWRITE_ASK, false))
//							throw new Exception();
//					}
//				}
			}catch(Exception e){
				JOptionPane.showMessageDialog(parent, Messages.getString("Error.sample_copy"),  //$NON-NLS-1$
						Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			}
		}
		

		Rflow.setting.setProperty(Setting.Key.DEFAULT_WORKING_DIRECTORY, workDir.getAbsolutePath());
		return true;
	}

	public static void copySamples(FileControl fileControl, File workDir)
			throws IOException {
		for(File f : FileResource.SAMPLE_FILE){
			File abs = FileUtility.makeAbstractPath(FileResource.SAMPLE_FOLDER, f).getParentFile();
			File outFolder = new File(workDir, abs.getPath());

			if(!outFolder.exists())
				if(!fileControl.makeDir(outFolder))throw new IOException();
			
			if(!fileControl.copyFile(f, outFolder, FileControl.OVERWRITE_ASK, false))
				throw new IOException();
		}
	}

	private static class AskMakeDirDialog extends JDialog{
		
		File workDir;
		boolean copySample;		
		
		AskMakeDirDialog(final Frame parent, File dir){
			super(parent, Messages.getString("WorkingDirSetter.set_def_workdir_title"), true); //$NON-NLS-1$
			
			
			JLabel ask = new JLabel(Messages.getString("WorkingDirSetter.set_def_workdir_label")); //$NON-NLS-1$
			Box labelBox = Box.createHorizontalBox();
			labelBox.add(ask);
			labelBox.add(Box.createHorizontalGlue());
			
//			ask.setMaximumSize(new Dimension(Integer.MAX_VALUE, 100));
//			ask.setPreferredSize(new Dimension(500, 100));
			
			final JTextField directoryInput = new RflowTextField(dir.getAbsolutePath());
			JButton ref = new JButton("..."); //$NON-NLS-1$
			ref.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					File file = FileChooserUtility.chooseDir(parent, 
							new File(directoryInput.getText()));
					if(file == null)return;
					directoryInput.setText(file.getAbsolutePath());
				}
			});
			
			
			Box inputBox = Box.createHorizontalBox();
			
			inputBox.setMaximumSize(new Dimension(Integer.MAX_VALUE, 20));
			inputBox.setPreferredSize(new Dimension(400, 20));
			
			inputBox.add(directoryInput);
			inputBox.add(ref);
			
			
			final JCheckBox sample = new JCheckBox(Messages.getString("WorkingDirSetter.copy_sample"), true); //$NON-NLS-1$
			Box sampleBox = Box.createHorizontalBox();
			sampleBox.add(Box.createHorizontalStrut(20));
			sampleBox.add(sample);
			sampleBox.add(Box.createHorizontalGlue());
			
			ActionListener yesAction = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					File d = new File(directoryInput.getText());
					workDir = d;
					copySample = sample.isSelected();
					AskMakeDirDialog.this.dispose();
				}
			};
			
			ActionListener noAction = new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					workDir = null;
					AskMakeDirDialog.this.dispose();
				}
			};
			
			ButtonPanel buttonPanel = new ButtonPanel(yesAction, noAction);
			
			Box main = Box.createVerticalBox();
			main.setBorder(new EmptyBorder(15,15,15,15));
			main.add(labelBox);
			main.add(Box.createVerticalStrut(10));
			main.add(inputBox);
			main.add(Box.createVerticalStrut(10));
			main.add(sampleBox);
			main.add(Box.createVerticalStrut(15));
			main.add(buttonPanel);
			
			getContentPane().add(main);
			
			pack();
			setLocationRelativeTo(parent);
		}
	}
}
