package com.ef_prime.rflow.util;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.ef_prime.rflow.resource.Messages;

public class FileChooserUtility {
	
	/**
	 * 通常のファイル選択に加え以下を行う
	 * 　拡張子の自動追加（MyFileFilterクラスのフィルターが選択された場合のみ）
	 * 　上書きの確認（上書きキャンセル後ファイル選択に戻る）
	 * キャンセルされた場合にはnullがかえる
	 * @param chooserTitle
	 * @param currentDirectory
	 * @param filters
	 * @param parent
	 * @return キャンセルされた場合にはnull
	 */
	public static File chooseSaveFile(
			String chooserTitle,
			File currentDirectory,
			FileFilter[] filters,
			Component parent
			)
	{
		JFileChooser chooser = new JFileChooser(currentDirectory);
		chooser.setDialogTitle(chooserTitle);
		for(FileFilter filter : filters){
			chooser.addChoosableFileFilter(filter);
		}
		return chooseSaveFile(chooser, parent);
	}

	/**
	 * 通常のファイル選択に加え以下を行う
	 * 　拡張子の自動追加（MyFileFilterクラスのフィルターが選択された場合のみ）
	 * 　上書きの確認（上書きキャンセル後ファイル選択に戻る）
	 * キャンセルされた場合にはnullがかえる
	 * @param chooser
	 * @param parent
	 * @return　キャンセルされた場合にはnull
	 */
	public static File chooseSaveFile(
			JFileChooser chooser, 
			Component parent)
	{
		int ret = chooser.showSaveDialog(parent);
		if(ret != JFileChooser.APPROVE_OPTION)return null;
		

		File file = chooser.getSelectedFile();

		//拡張子の追加
		if(chooser.getFileFilter() instanceof MyFileFilter){
			MyFileFilter filter = (MyFileFilter)chooser.getFileFilter();
			file = ExtensionUtility.addExtension(file, filter.getExtension(), filter.getDelegateExtension());
		}

		//上書き確認
		if(!ConfirmOverwrite.confirm(file, parent)){
			return chooseSaveFile(chooser, parent);
		}
		
		return file;
	}
	
	public static File chooseOpenFile(
			String chooserTitle,
			File currentDirectory,
			FileFilter[] filters,
			Component parent
			)
	{
		JFileChooser chooser = new JFileChooser(currentDirectory);
		chooser.setDialogTitle(chooserTitle);
		for(FileFilter filter : filters){
			chooser.addChoosableFileFilter(filter);
		}
		//chooser.setCurrentDirectory(currentDirectory);
		return chooseOpenFile(chooser, parent);
	}


	private static File chooseOpenFile(
			JFileChooser chooser, 
			Component parent)
	{
		int ret = chooser.showOpenDialog(parent);
		if(ret != JFileChooser.APPROVE_OPTION)return null;
		
		File file = chooser.getSelectedFile();
		
		return file;
	}
	
	/**
	 * 
	 * @param parent
	 * @param currentDir
	 * @return chosen file; null if canceled
	 */
	public static File chooseDir(Component parent, File currentDir){
		JFileChooser chooser = new JFileChooser();
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		chooser.setDialogTitle(Messages.getString("Message.select_dir")); //$NON-NLS-1$
		if(currentDir != null && currentDir.exists() && currentDir.isDirectory())
			chooser.setCurrentDirectory(currentDir);
		
		int ret = chooser.showDialog(parent, Messages.getString("Button.ok")); //$NON-NLS-1$
		if(ret != JFileChooser.APPROVE_OPTION)return null;
		
		return chooser.getSelectedFile();
	}
}
