package com.ef_prime.rflow.util;

import java.awt.Window;
import java.io.File;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import com.ef_prime.rflow.resource.Messages;
import com.ef_prime.rflow.tool.TextGetter;

public class FileControl {

	private final int MAX_PATH;
	public static final int OVERWRITE_YES = 0;
	public static final int OVERWRITE_NO = 1;
	public static final int OVERWRITE_ASK = 2;
	private static final String COPY_SUFFIX = "_copy"; //$NON-NLS-1$
	private static final String COPY_PREFIX_REG0 = "(.*)(_copy)(\\.[^.]*)?$"; //$NON-NLS-1$
	private static final String COPY_PREFIX_REG = "(.*_copy)(\\d*)(\\.[^.]*)?$"; //$NON-NLS-1$
	private static final int DIR_OVERWRITE_OPTION_YES = 0;
	private static final int DIR_OVERWRITE_OPTION_YES_TO_ALL = 1;
	private static final int DIR_OVERWRITE_OPTION_NO = 2;
	private static final int DIR_OVERWRITE_OPTION_CANCEL = 3;
	private static final String[] dirOverwriteOptions = new String[] { Messages.getString("Button.yes"), Messages.getString("Button.yes_to_all"), Messages.getString("Button.no"), Messages.getString("Button.cancel") }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

	private Window parent;
	

	public FileControl(final int max_path, Window parent) {
		super();
		MAX_PATH = max_path;
		this.parent = parent;
	}

	public boolean delete_file(File file){
		return delete_file(file, true);
	}
	
	public boolean delete_file(File file, boolean errorMessage){
		if(!checkFileExists(file, errorMessage))
			return false;
		
		File failed = null;
		if(file.isFile()){
			if(file.delete())
				return true;
			failed = file;
		}else if(file.isDirectory()){
			for(File child : file.listFiles())
				if(!delete_file(child, errorMessage)){
					failed = child;
					break;
				}
			if(failed == null){
				if(file.delete())
					return true;
				failed = file;
			}
		}
		if(errorMessage)
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.delete")+": "+failed.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	
		return false;
	}

	public void rename(File file){
		if(!checkFileExists(file))
			return;
		
		if(file.getParentFile() == null)
			return;
		
		String newName = TextGetter.getText(parent, Messages.getString("Action.rename"), Messages.getString("Message.name"), file.getName()); //$NON-NLS-1$ //$NON-NLS-2$
		if(newName == null)return;
		File renamed = new File(file.getParentFile(), newName);
	
		if(!checkFileLength(renamed))
			return;		
		
		if(!file.renameTo(renamed))
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.rename")+": "+file.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	
	}

	/**
	 * 
	 * @param source
	 * @param dest
	 * @param overwriteType
	 * @return false if failed or canceled
	 */
	public boolean moveFile(File source, File dest, int overwriteType, boolean cancelOption){
		if(!checkFileExists(source) || !checkFileLength(dest))
			return false;
		
		if(source.isDirectory())
			return copyOrMoveDirectory(source, dest, false, overwriteType);
		
		if(source.isDirectory() || dest.isDirectory())
			throw new IllegalArgumentException("both source and dest must be file"); //$NON-NLS-1$
		
		int state = processOverwrite(dest, overwriteType, cancelOption);
		switch(state){
		case PROCESS_OVERWRITE_SKIPPED:return true;
		case PROCESS_OVERWRITE_CANCELED:return false;
		}
		
		if(!source.renameTo(dest)){
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.move")+": "+source.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return false;
		}
		
			return true;
	}
	
	public boolean makeDir(File newDir){
		if(checkFileLength(newDir)){
			if(newDir.mkdirs())
				return true;
		}
		JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.create_folder"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
		return false;
	}

	public void createFolderAskName(File parent){
		if(!checkFileExists(parent))
			return;
		
		if(parent.isFile() && parent.getParentFile() == null)
			return;
		
		String newName = TextGetter.getText(this.parent, Messages.getString("Action.new_folder"), Messages.getString("Message.name"), ""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if(newName == null)return;
		File newDir;
		if(parent.isDirectory()){
			newDir = new File(parent, newName);
		}else{
			newDir = new File(parent.getParentFile(), newName);
		}
		if(!newDir.mkdir()){
			JOptionPane.showMessageDialog(this.parent, Messages.getString("Error.file.create_folder"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
	}

	/**
	 * Copy file. 
	 * @param file
	 * @param parent
	 * @param overwriteType
	 * @return false if canceled or failed
	 * //TODO arg src and dest is better
	 */
	public boolean copyFile(File file, File parent, int overwriteType, boolean cancelOption){
		if(!checkFileExists(parent) || !checkFileExists(file))
			return false;
		
		if(file.getParentFile() == null)
			return false;
		if(parent.isFile() && parent.getParentFile() == null)
			return false;
		
		File newFile;
		if(parent.isDirectory()){
			newFile = new File(parent, file.getName());
		}else{
			newFile = new File(parent.getParentFile(), file.getName());
		}
		
		if(!checkFileLength(newFile))
			return false;
	
		if(file.isDirectory())
			return copyOrMoveDirectory(file, newFile, true, overwriteType);
		
		File failed = null;
		if(newFile.equals(file)){
			//同じフォルダへのコピーの場合
			File copyFile = getCopyFile(newFile);
			if(copyFile == null){
				failed = file;
			}else{
				newFile = copyFile;
				if(!checkFileLength(newFile))
					return false;
			}
		}else{
			int state = processOverwrite(newFile, overwriteType, cancelOption);
			switch(state){
			case PROCESS_OVERWRITE_SKIPPED:return true;
			case PROCESS_OVERWRITE_CANCELED:return false;
			}
		}
	
		if(failed == null)
			try {
				FileUtility.copy(file, newFile);
			} catch (IOException e) {
				failed = file;
			}
		
		if(failed != null){
			JOptionPane.showMessageDialog(this.parent, Messages.getString("Error.file.copy")+": "+failed.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return false;
		}
		
		return true;
	}

	private static final int PROCESS_OVERWRITE_SUCCESS = 0;
	private static final int PROCESS_OVERWRITE_SKIPPED = 1;
	private static final int PROCESS_OVERWRITE_CANCELED = 2;
	/**
	 * 
	 * @param targetFile
	 * @param overwriteType
	 * @return true if success;false if failed;null if skipped
	 */
	private int processOverwrite(File targetFile, int overwriteType, boolean cancelOption) {
		if(targetFile.exists()){
			if(overwriteType == OVERWRITE_ASK){
				int ret = ConfirmOverwrite.confirmYesNoCancel(targetFile, parent, cancelOption);
				switch(ret){
				case ConfirmOverwrite.YES_OPTION:
					overwriteType = OVERWRITE_YES;break;
				case ConfirmOverwrite.NO_OPTION:
					overwriteType = OVERWRITE_NO;break;
				case ConfirmOverwrite.CANCEL_OPTION:
					return PROCESS_OVERWRITE_CANCELED;
				}
			}
	
			if(overwriteType == OVERWRITE_YES){
				//消す
				if(!delete_file(targetFile)){
					JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.overwrite")+": "+targetFile.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					return PROCESS_OVERWRITE_CANCELED;
				}
			}
			else if(overwriteType == OVERWRITE_NO)
				return PROCESS_OVERWRITE_SKIPPED;
		}
		return PROCESS_OVERWRITE_SUCCESS;
	}

	private File getCopyFile(File copyFile){
		if(copyFile.exists()){
			File parent = copyFile.getParentFile();
			if(parent == null)
				throw new RuntimeException("can not copy root"); //$NON-NLS-1$
			String name = copyFile.getName();
			int dotIndex = name.lastIndexOf('.');
			if(dotIndex < 0)
				name = name + COPY_SUFFIX;
			else
				name = name.substring(0, dotIndex) + COPY_SUFFIX + name.substring(dotIndex, name.length());
	
			while(new File(parent, name).exists()){
				if(name.matches(COPY_PREFIX_REG0)){
					try{
						name = name.replaceFirst(COPY_PREFIX_REG0, "$1"+COPY_SUFFIX+"2"+"$3"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}catch(Exception e){
						name = name.replaceFirst(COPY_PREFIX_REG0, "$1"+COPY_SUFFIX+"2"); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}else if(name.matches(COPY_PREFIX_REG)){
					String num = name.replaceFirst(COPY_PREFIX_REG, "$2"); //$NON-NLS-1$
					int nextNum = Integer.valueOf(num).intValue() + 1;
					if(nextNum > 255)
						return null;
					try{
						name = name.replaceFirst(COPY_PREFIX_REG, "$1"+nextNum+"$3"); //$NON-NLS-1$ //$NON-NLS-2$
					}catch(Exception e){
						name = name.replaceFirst(COPY_PREFIX_REG, "$1"+nextNum); //$NON-NLS-1$
					}
				}
			}
			return getCopyFile(new File(parent, name));
		}else{
			return copyFile;
		}
	}

	/**
	 * 
	 * @param srcDir
	 * @param destDir
	 * @return false if failed or canceled
	 */
	private boolean copyOrMoveDirectory(File srcDir, File destDir, 
			boolean isCopy, int overwriteType)
	{
		if(!checkFileLength(destDir))
			return false;
		
		if(!checkIsValidAction(srcDir, destDir))
			return false;
		
		int fileOverwriteType = OVERWRITE_NO;
		
		//同じ場所へのコピーの場合
		if(isCopy && srcDir.equals(destDir)){
			File copyDir = getCopyFile(destDir);
			if(copyDir == null){
				JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.copy")+": "+destDir.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				return false;
			}else{
				return copyOrMoveDirectory(srcDir, copyDir, isCopy, overwriteType);
			}
		}
		
		//上書きの確認
		if(destDir.exists()){
			switch(overwriteType){
			case OVERWRITE_ASK:
				JOptionPane optionPane = new JOptionPane(destDir.getName()+Messages.getString("Message.folder_overwrite"), JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
				optionPane.setOptions(dirOverwriteOptions);
				JDialog dialog = optionPane.createDialog(parent, Messages.getString("Message.confirm_overwrite")); //$NON-NLS-1$
				dialog.setVisible(true);
				dialog.dispose();
				String s = (String)optionPane.getValue();
				
				int i;
				if(s == null)
					i = DIR_OVERWRITE_OPTION_CANCEL;
				else
					for(i = 0; i < dirOverwriteOptions.length; i++){
						if(dirOverwriteOptions[i].equals(s))
							break;
					}
				
				switch(i){
				case DIR_OVERWRITE_OPTION_YES_TO_ALL:
					overwriteType = OVERWRITE_YES;
					fileOverwriteType = OVERWRITE_YES;break;
				case DIR_OVERWRITE_OPTION_YES:
					fileOverwriteType = OVERWRITE_YES;break;
				case DIR_OVERWRITE_OPTION_NO:
					fileOverwriteType = OVERWRITE_NO;break;
				case DIR_OVERWRITE_OPTION_CANCEL:
					return false;
				}
				break;
				
			case OVERWRITE_YES:
				fileOverwriteType = OVERWRITE_YES;break;
			case OVERWRITE_NO:
				fileOverwriteType = OVERWRITE_NO;break;
			}
		}
		//ディレクトリを作る
		else if(!destDir.mkdir()){
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.create_folder"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		//フォルダ内のフォルダおよびファイルの処理
		for(File file : srcDir.listFiles()){
			File newFile = new File(destDir, file.getName());
			if(file.isDirectory()){
				//再帰的にディレクトリコピー
				if(!copyOrMoveDirectory(file, newFile, isCopy, overwriteType))
					return false;
			}else{
				//ファイル
				if(isCopy){
					if(!copyFile(file, destDir, fileOverwriteType, false))//note that never ask overwrite
						return false;
				}else{
					if(!moveFile(file, newFile, fileOverwriteType, false))//note that never ask overwrite
						return false;
				}
			}
		}
	
		//移動の場合、元フォルダを消す
		if(!isCopy){
			if(!srcDir.delete()){
				JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.delete")+": "+srcDir.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				return false;
			}
		}
		
		return true;
	}

	public boolean checkFileLength(File file){
		if(file.getAbsolutePath().getBytes().length > MAX_PATH){
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.exceed_path_limit")+": "+file.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return false;
		}
		return true;
	}

	public boolean checkFileExists(File file){
		return checkFileExists(file, true);
	}
	
	public boolean checkFileExists(File file, boolean errorMessage){
		if(!file.exists()){
			if(errorMessage)
				JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.file_not_found")+": "+file.getPath(), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return false;
		}
		return true;
	}

	public boolean checkIsValidAction(File src, File dest){
		if(src.isFile())
			return true;
		
		if(FileUtility.isChild(src, dest)){
			JOptionPane.showMessageDialog(parent, Messages.getString("Error.file.folder_to_subfolder"), Messages.getString("Error.title.error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		return true;
	}

}
