package com.ef_prime.rflow.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import com.ef_prime.rflow.Rflow;
import com.ef_prime.rflow.exception.CanceledException;
import com.ef_prime.rflow.tool.ProgressMonitor;

public class FileUtility {
	/**
	 * ファイルをコピーします
	 * @param from
	 * @param to
	 * @throws IOException
	 */
	public static void copy(File from, File to) throws IOException{
		FileInputStream inStream = null;
		FileOutputStream outStream = null;
		FileChannel in = null;
		FileChannel out = null;
		
		try{
			inStream = new FileInputStream(from);
			outStream = new FileOutputStream(to);
			in = inStream.getChannel();
			out = outStream.getChannel();
	
	        in.transferTo(0, in.size(), out);
	        
		}finally{
			if(inStream != null)
				inStream.close();
			if(outStream != null)
				outStream.close();
			if(in != null)
				in.close();
			if(out != null)
				out.close();
		}
	}
	
	/**
	 * absoluteのパスをparentからの抽象パスに変換します
	 * @param parent
	 * @param absolute
	 * @return
	 */
	public static File makeAbstractPath(File parent, File absolute){
		if(parent == null)return absolute;
		
		String parentPath = parent.getAbsolutePath();
		String absPath = absolute.getAbsolutePath();
		File abstractFile = absolute;
		if(isChild(parent, absolute)){
			abstractFile = new File(absPath.substring(parentPath.length()+1, absPath.length()));
		}
		return abstractFile;
	}
	
	public static boolean isChild(File parent, File child){
		String parentPath = parent.getAbsolutePath().toLowerCase();
		String childPath = child.getAbsolutePath().toLowerCase();
		if(!childPath.startsWith(parentPath))
			return false;
		
		File f = child;
		while(f.getParentFile() != null){
			if(parent.equals(f))
				return true;
			f = f.getParentFile();
		}
		return false;
	}
	
	public static String getHash(File file, ProgressMonitor pm) throws IOException, CanceledException{
		long start = System.currentTimeMillis();
		MessageDigest md = null;
		try {
			md = MessageDigest.getInstance("MD5"); //$NON-NLS-1$
		} catch (NoSuchAlgorithmException e) {
		} 
		DigestInputStream input = new DigestInputStream(new BufferedInputStream(new FileInputStream(file)), md);

		try{
			byte[] b = new byte[2048];
			if(pm != null){
				int a;
				int sum = 0;
				while(true){
					a = input.read(b, 0, b.length);
					if(a == -1)break;
					sum += a;
					if(sum > 8191999){
						//System.out.println(sum);
						if(pm.isCanceled()){
							throw new CanceledException();
						}
						sum = 0;
					}
				}
			}else{
				while(input.read(b)!=-1){}
			}
			byte[] digest = md.digest();


			StringBuffer buf = new StringBuffer();
			for(int i = 0; i < digest.length; i++){
				String tmp = Integer.toHexString(digest[i] & 0xff);
				if(tmp.length() < 2)
					buf.append('0');
				buf.append(tmp);
			}

			if(Rflow.DEBUG > 0)
				System.out.println("elapsed time to compute hash :" + (System.currentTimeMillis() -start)); //$NON-NLS-1$
			return buf.toString();
		}finally{
			input.close();
		}
	}
}
