package com.ef_prime.rflow.util;

import java.io.File;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.prefs.Preferences;

/**
 * Windows版でのみ有効なメソッドもあるので注意。
 * インストーラでインストールされていることを前提とするメソッドもある。
 * （ディレクトリ構造がインストーラによってインストールされたもの前提になっている場合もある）
 * 
 * レジストリにアクセスするなどして、Rのディレクトリを取得するメソッドなどを提供する。
 * SunのJavaに依存。将来のSunのJavaのリリースにおいては使えない可能性もあるので注意。
 * 
 * @author <a href="mailto:taniguchi@ef-prime.com">Tomoya Taniguchi</a>
 *
 */
public class RFindUtility {
	
	
	/**
	 * Example output of this class.
	 * @param args
	 */
	public static void main(String[] args)
	{
		System.out.print("getRDirectory() -> "); //$NON-NLS-1$
		System.out.println(getRDirectoryFromRegistory());
		
		System.out.print("findRFromProgramFiles() -> "); //$NON-NLS-1$
		System.out.println(findRFromProgramFiles());
		
		System.out.print("Preferences.userRoot() -> "); //$NON-NLS-1$
		System.out.println(Preferences.userRoot().absolutePath());

		System.out.println(getValueFromHKeyCurrentUser(
				"Software\\JavaSoft\\Java Update\\Policy",  //$NON-NLS-1$
				"LastUpdateBeginTime")); //$NON-NLS-1$
		
		System.out.println(getValueFromHKeyLocalMachine(
				"Software\\JavaSoft\\Java Update\\Policy",  //$NON-NLS-1$
				"LastUpdateBeginTime")); //$NON-NLS-1$

	}
	
	
	/**
	 * このRAnalyticFlowが対応しているRのVersionを表します。
	 * 
	 * 実際にはlib\R\以下のフォルダ名の一覧を返します。
	 * Windows版でインストーラによるインストールをしているときのみ正常に動作します。
	 * 
	 * lib\Rが存在しない場合はnullを返します。
	 * @return
	 */
	/*private static String[] getFittedRVersions()
	{
		File f = new File("lib"); //$NON-NLS-1$
		
		if(!f.exists() || !f.isDirectory())
			return null;
		
		f = new File(f,"R"); //$NON-NLS-1$
		
		if(!f.exists() || !f.isDirectory())
			return null;
		
		return f.list();
	}*/
	
	
	/**
	 * レジストリから発見できない場合でもなんとか取得しようとがんばります。
	 * @return
	 */
	public static String getRDirecotry()
	{
		String str = getRDirectoryFromRegistory();
		if(str != null)
			return str;

		str = findRFromProgramFiles();
		if(str != null)
			return str;

		return null;
	}
	
	/**
	 * Program Filesのフォルダを返します。
	 * @return null if not found.
	 */
	private static String findRFromProgramFiles()
	{
		String str = getProgramFilesDirectory();
		
		if(str == null)return null;
		
		File f = new File(str);

		if(f.exists())
		{
			File g = new File(f,"R"); //$NON-NLS-1$
			if(g.exists())
			{
				File[] files = g.listFiles();
				Arrays.sort(files);
								
				//System.out.println(Arrays.toString(files));
				if(files.length == 0)
					return null;
				
				return files[files.length - 1].getAbsolutePath();
			}
		}
		return null;
	}
	
	
	/**
	 * Get R install directory from registory
	 * 
	 * @return R install path.null if not found.
	 */
	public static String getRDirectoryFromRegistory() {
		String ret = getValueFromHKeyLocalMachine("SOFTWARE\\R-core\\R", "InstallPath"); //$NON-NLS-1$ //$NON-NLS-2$
		if(ret != null)
			return ret;
		
		return getValueFromHKeyCurrentUser("SOFTWARE\\R-core\\R", "InstallPath"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private static String getProgramFilesDirectory()
	{
		String str = getValueFromHKeyLocalMachine(
				"SOFTWARE\\Microsoft\\Windows\\CurrentVersion", //$NON-NLS-1$
				"ProgramFilesDir"); //$NON-NLS-1$

		if(new File(str).exists())
			return str;
		
		if(new File("C:\\Programs").exists()) //$NON-NLS-1$
			return "C:\\Programs"; //$NON-NLS-1$

		if(new File("C:\\Program Files").exists()) //$NON-NLS-1$
			return "C:\\Program Files"; //$NON-NLS-1$
		
		return null;
	}
	
	/**
	 * レジストリにアクセスします。
	 * SunのJDKの実装に依存しています。
	 *
	 * http://www.jroller.com/lenkite/entry/use_pure_java_to_access
	 * http://www.velocityreviews.com/forums/t151365-java-accessing-windows-registry.html
	 * 
	 * @param subKey
	 * @param name
	 * @return
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private static String getValueFromHKeyLocalMachine(
			String subKey,
			String name
			)
	{
		//final int HKEY_CURRENT_USER = 0x80000001;
		//final int KEY_QUERY_VALUE = 1;
		//final int KEY_SET_VALUE = 2;
		final int KEY_READ = 0x20019;
		
		String value = null;
		
		//final Preferences userRoot = Preferences.userRoot();
		final Preferences userRoot = Preferences.systemRoot();
		final Class clz = userRoot.getClass();
		try {
			Class[] parms1 = {byte[].class, int.class, int.class};
			final Method mOpenKey = clz.getDeclaredMethod("openKey", //$NON-NLS-1$
			parms1);
			mOpenKey.setAccessible(true);
			
			Class[] parms2 = {int.class};
			final Method mCloseKey = clz.getDeclaredMethod("closeKey", //$NON-NLS-1$
			parms2);
			mCloseKey.setAccessible(true);
			
			Class[] parms3 = {int.class, byte[].class};
			final Method mWinRegQueryValue = clz.getDeclaredMethod(
			"WindowsRegQueryValueEx", parms3); //$NON-NLS-1$
			mWinRegQueryValue.setAccessible(true);
			
			Class[] parms4 = {int.class, int.class, int.class};
			final Method mWinRegEnumValue = clz.getDeclaredMethod(
			"WindowsRegEnumValue1", parms4); //$NON-NLS-1$
			mWinRegEnumValue.setAccessible(true);
			
			Class[] parms5 = {int.class};
			final Method mWinRegQueryInfo = clz.getDeclaredMethod(
			"WindowsRegQueryInfoKey1", parms5); //$NON-NLS-1$
			mWinRegQueryInfo.setAccessible(true);
			
			
			Object[] objects1 = {toByteArray(subKey), new
			Integer(KEY_READ), new Integer(KEY_READ)};
			Integer hSettings = (Integer) mOpenKey.invoke(userRoot,
			objects1);
			
			
			Object[] objects2 = {hSettings, toByteArray(name)};
			byte[] b = (byte[]) mWinRegQueryValue.invoke(userRoot,
			objects2);
			value = (b != null ? new String(b).trim() : null);
			
			Object[] objects3 = {hSettings};
			mCloseKey.invoke(Preferences.userRoot(), objects3);
		
		} catch (Throwable e) {
		}
		return value;		
	}

	/**
	 * レジストリにアクセスします。
	 * SunのJDKの実装に依存しています。
	 *
	 * http://www.jroller.com/lenkite/entry/use_pure_java_to_access
	 * http://www.velocityreviews.com/forums/t151365-java-accessing-windows-registry.html
	 * 
	 * @param subKey
	 * @param name
	 * @return
	 */
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	private static String getValueFromHKeyCurrentUser(
			String subKey,
			String name
			)
	{
		//final int HKEY_CURRENT_USER = 0x80000001;
		//final int KEY_QUERY_VALUE = 1;
		//final int KEY_SET_VALUE = 2;
		final int KEY_READ = 0x20019;
		
		String value = null;
		
		final Preferences userRoot = Preferences.userRoot();
		//final Preferences userRoot = Preferences.systemRoot();
		final Class clz = userRoot.getClass();
		try {
			Class[] parms1 = {byte[].class, int.class, int.class};
			final Method mOpenKey = clz.getDeclaredMethod("openKey", //$NON-NLS-1$
			parms1);
			mOpenKey.setAccessible(true);
			
			Class[] parms2 = {int.class};
			final Method mCloseKey = clz.getDeclaredMethod("closeKey", //$NON-NLS-1$
			parms2);
			mCloseKey.setAccessible(true);
			
			Class[] parms3 = {int.class, byte[].class};
			final Method mWinRegQueryValue = clz.getDeclaredMethod(
			"WindowsRegQueryValueEx", parms3); //$NON-NLS-1$
			mWinRegQueryValue.setAccessible(true);
			
			Class[] parms4 = {int.class, int.class, int.class};
			final Method mWinRegEnumValue = clz.getDeclaredMethod(
			"WindowsRegEnumValue1", parms4); //$NON-NLS-1$
			mWinRegEnumValue.setAccessible(true);
			
			Class[] parms5 = {int.class};
			final Method mWinRegQueryInfo = clz.getDeclaredMethod(
			"WindowsRegQueryInfoKey1", parms5); //$NON-NLS-1$
			mWinRegQueryInfo.setAccessible(true);
			
			
			Object[] objects1 = {toByteArray(subKey), new
			Integer(KEY_READ), new Integer(KEY_READ)};
			Integer hSettings = (Integer) mOpenKey.invoke(userRoot,
			objects1);
			
			
			Object[] objects2 = {hSettings, toByteArray(name)};
			byte[] b = (byte[]) mWinRegQueryValue.invoke(userRoot,
			objects2);
			value = (b != null ? new String(b).trim() : null);
			
			Object[] objects3 = {hSettings};
			mCloseKey.invoke(Preferences.userRoot(), objects3);
		
		} catch (Throwable e) {
		}
		return value;		
	}

	
	
	private static byte[] toByteArray(String str) {
		byte[] result = new byte[str.length() + 1];
		for (int i = 0; i < str.length(); i++) {
		result[i] = (byte) str.charAt(i);
		}
		result[str.length()] = 0;
		return result;
	}

	
	

}
