package com.ef_prime.rflow.util;

import java.util.ArrayList;
import java.util.List;

import com.ef_prime.rflow.Constants;

public class RTypeFilter extends RObjFilter{
	
	/** this filter pass everything*/
	public static final RTypeFilter NullFilter = new RTypeFilter();
	public static final RTypeFilter ListFilter = new RTypeFilter(new FilterEntry(FilterMethod.Is, "list")); //$NON-NLS-1$
	public static final RTypeFilter DataFrameFilter = new RTypeFilter(new FilterEntry(FilterMethod.Is, "data.frame")); //$NON-NLS-1$
	
	
	public static enum FilterMethod{
		Is("is"), //$NON-NLS-1$
		Class("class"), //$NON-NLS-1$
		Mode("mode"), //$NON-NLS-1$
		Typeof("typeof"); //$NON-NLS-1$
		
		private String name;
		
		private FilterMethod(String name){
			this.name = name;
		}
		
		@Override
		public String toString() {
			return name;
		}
		
		public static FilterMethod toFilterMethod(String name){
			if(name.equals("is")) //$NON-NLS-1$
				return Is;
			if(name.equals("class")) //$NON-NLS-1$
				return Class;
			if(name.equals("mode")) //$NON-NLS-1$
				return Mode;
			if(name.equals("typeof")) //$NON-NLS-1$
				return Typeof;
			return null;
		}
		
		public static String[] nameValues(){
			FilterMethod[] methods = values();
			String[] nameValues = new String[methods.length];
			for(int i = 0; i< nameValues.length; i++){
				nameValues[i] = methods[i].name;
			}
			return nameValues;
		}
	}
	
	public static class FilterEntry{
		
		private FilterMethod method;
		private String target;
		
		public FilterEntry(FilterMethod method, String target) {
			this.method = method;
			this.target = target;
		}
		
		public FilterMethod getMethod() {
			return method;
		}
		
		public String getTarget() {
			return target;
		}
		
		public String getEvalFormula(String object){
			switch(method){
			case Is:return "is."+target+"("+ object +")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			case Class:return "inherits("+object+", \""+ target +"\")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			case Mode:return "mode("+object+") == \"" + target +"\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			case Typeof:return "typeof("+object+") == \"" + target +"\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			default:return Constants.TRUE;
			}
		}
	}
	
	private List<FilterEntry> entrys = new ArrayList<FilterEntry>();
	
	public RTypeFilter(){
	}
	
	public RTypeFilter(FilterEntry entry){
		addFilter(entry);
	}
	
	
	public void addFilter(FilterEntry entry){
		entrys.add(entry);
	}
	
	
	public String getEvalFormula(String obj){
		if(entrys.size() < 1)return Constants.TRUE;
		String function = ""; //$NON-NLS-1$
		for(int i = 0; i < entrys.size(); i++){
			FilterEntry entry = entrys.get(i);
			function += entry.getEvalFormula(obj) + filterType;
		}
		function = function.substring(0, function.length()-filterType.length());
		return function;
	}
}
