package com.ef_prime.rflow.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * シリアライズ、デシリアライズのためのユーティリティメソッドです。
 * 
 * @author <a href="mailto:taniguchi@ef-prime.com">Tomoya Taniguchi</a>
 *
 * @param <T> 保存対象のオブジェクトの型
 */
public class SerializeUtility<T extends Serializable> {

	File file;
	
	/**
	 * 指定したファイルへのObjectStorageを返します。
	 * 
	 * @param file
	 */
	public SerializeUtility(File file)
	{
		this.file = file;
	}

	public T load() throws IOException, ClassNotFoundException
	{
		return load(file);
	}
	
	public void save(T obj)throws IOException
	{
		save(obj,file);
	}

	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public static <T extends Serializable> T load(File file)throws IOException, ClassNotFoundException
	{
	    FileInputStream inFile = new FileInputStream(file);
	    ObjectInputStream s = new ObjectInputStream(inFile);
	    
	    T ret = (T)s.readObject();

	    s.close();
	    inFile.close();
	    return ret;

	}
	
	public static void save(Object obj,File file)throws IOException
	{
	    FileOutputStream outFile = new FileOutputStream(file);
	    ObjectOutput s = new ObjectOutputStream(outFile);

	    s.writeObject(obj);
	    s.flush();

	    s.close();
	    outFile.close();				
	}
	
	
	
}
