package com.ef_prime.rflow.util;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.border.AbstractBorder;

public class SideBorder extends AbstractBorder{
	
	public static final int SIDE_TOP = 0;
	public static final int SIDE_LEFT = 1;
	public static final int SIDE_BOTTOM = 2;
	public static final int SIDE_RIGHT = 3;
	private int side = SIDE_TOP;
	private Color color;
	
	public SideBorder(int side, Color color){
		this.side = side;
		this.color = color;
	}
	
	@Override
	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
        Color oldColor = g.getColor();
		g.setColor(color);
		int offset = 0;
		switch(side){
		case SIDE_TOP:
			g.drawLine(x, y-offset, x+width-1, y-offset);
			break;
		case SIDE_LEFT:
			g.drawLine(x-offset, y, x-offset, y+height-1);
			break;
		case SIDE_BOTTOM:
			g.drawLine(x, y+height-1+offset, x+width-1, y+height-1+offset);
			break;
		case SIDE_RIGHT:
			g.drawLine(x+width-1+offset, y, x+width-1+offset, y+height-1);
			break;
		}
        g.setColor(oldColor);
	}
	
	@Override
	public Insets getBorderInsets(Component c) {
		return setInsets(new Insets(0,0,0,0));
	}
	
	@Override
	public Insets getBorderInsets(Component c, Insets insets) {
		return setInsets(insets);
	}
	
	private Insets setInsets(Insets insets){
        insets.left = insets.top = insets.right = insets.bottom = 0;
		int offset = 1;
		switch(side){
		case SIDE_TOP:
			insets.top += offset;
			break;
		case SIDE_LEFT:
			insets.left += offset;
			break;
		case SIDE_BOTTOM:
			insets.bottom += offset;
			break;
		case SIDE_RIGHT:
			insets.right += offset;
			break;
		}
		return insets;
	}
	
	@Override
	public boolean isBorderOpaque() {
		return true;
	}
}
