package com.ef_prime.rflow.util;

import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import javax.swing.undo.UndoManager;

import com.ef_prime.rflow.Constants;
import com.ef_prime.rflow.resource.Messages;

public class SwingUtils {
	
	public static void setDefaultTextComponentOption(final JTextComponent c){
		setTextComponentUndoable(c);
		setTextComponentPopupMenu(c);
	}
	
	public static UndoManager setTextComponentUndoable(final JTextComponent c){
		final UndoManager undoManager = new UndoManager();
		c.getDocument().addUndoableEditListener(undoManager);
		c.addKeyListener(new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
				if(e.getModifiers() == Constants.MENU_KEY_MASK){
					switch(e.getKeyCode()){
					case KeyEvent.VK_Z:
						if(undoManager.canUndo())undoManager.undo();
						break;
					case KeyEvent.VK_Y:
						if(undoManager.canRedo())undoManager.redo();
						break;
					}
					return;
				}
			}
		});
		return undoManager;
	}
	
	public static void setTextComponentPopupMenu(final JTextComponent c){

		//actions
		Action cut = new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				c.cut();
			}
		};
		cut.putValue(Action.NAME, Messages.getString("Action.cut")); //$NON-NLS-1$
		cut.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_X, Constants.MENU_KEY_MASK));

		Action copy = new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				c.copy();
			}
		};
		copy.putValue(Action.NAME, Messages.getString("Action.copy")); //$NON-NLS-1$
		copy.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, Constants.MENU_KEY_MASK));
		
		Action paste = new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				c.paste();
			}
		};
		paste.putValue(Action.NAME, Messages.getString("Action.paste")); //$NON-NLS-1$
		paste.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, Constants.MENU_KEY_MASK));

		Action selectAll = new AbstractAction(){
			public void actionPerformed(ActionEvent e) {
				c.selectAll();
			}
		};
		selectAll.putValue(Action.NAME, Messages.getString("Action.select_all")); //$NON-NLS-1$
		selectAll.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_A, Constants.MENU_KEY_MASK));
		
		
		//popup
		final JPopupMenu popup = new JPopupMenu();
		popup.add(cut);
		popup.add(copy);
		popup.add(paste);
		popup.addSeparator();
		popup.add(selectAll);
		
		
		//mouselistener
		c.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseClicked(MouseEvent e) {
				if(SwingUtilities.isRightMouseButton(e)){
					popup.show(c, e.getPoint().x, e.getPoint().y);
				}
			}
		});
	}
	
	public static Rectangle getScreenBounds(){
		return GraphicsEnvironment.getLocalGraphicsEnvironment()
			.getDefaultScreenDevice().getDefaultConfiguration().getBounds();
	}
	
	public static void insertMouseListener(Component c, MouseListener listener){
		MouseListener[] listeners = c.getMouseListeners();
		for(MouseListener l : listeners)
			c.removeMouseListener(l);
		
		c.addMouseListener(listener);

		for(MouseListener l : listeners)
			c.addMouseListener(l);
	}
	
	public static void setTabs(JTextPane textPane, int charactersPerTab)
	{
		FontMetrics fm = textPane.getFontMetrics(textPane.getFont());
		int charWidth = fm.charWidth('w');
		int tabWidth = charWidth * charactersPerTab;
 
		TabStop[] tabs = new TabStop[10];
 
		for (int j = 0; j < tabs.length; j++)
		{
			int tab = j + 1;
			tabs[j] = new TabStop( tab * tabWidth );
		}
 
		TabSet tabSet = new TabSet(tabs);
		SimpleAttributeSet attributes = new SimpleAttributeSet();
		//MutableAttributeSet attributes = (MutableAttributeSet)getParagraphAttributes();
		StyleConstants.setTabSet(attributes, tabSet);
		//attributes.setResolveParent(new SimpleAttributeSet(getParagraphAttributes()));
		
		textPane.setParagraphAttributes(attributes, false);
	}
	
	public static Box createGluedHorizontalBox(Component c){
		Box box = Box.createHorizontalBox();
		box.add(c);
		box.add(Box.createHorizontalGlue());
		return box;
	}

	public static JRadioButton[] createRadioButtons(String[] labels){
		JRadioButton[] buttons = new JRadioButton[labels.length];
		ButtonGroup group = new ButtonGroup(); 
		for(int i = 0; i < labels.length; i++){
			buttons[i] = new JRadioButton(labels[i]);
			group.add(buttons[i]);
		}
		return buttons;
	}
	
	public static void clearPopupSeparator(JPopupMenu popup){
		if(popup.getComponentCount() == 0)return;
		
		boolean isItem = false;
		Set<Component> removeSepSet = new HashSet<Component>();
		for(int i = 0; i < popup.getComponentCount(); i++){
			boolean sep = popup.getComponent(i) instanceof JSeparator;
			if(sep && !isItem)
				removeSepSet.add(popup.getComponent(i));
			isItem = !sep;
		}
		Component last = popup.getComponent(popup.getComponentCount()-1);
		if(last instanceof JSeparator)
			removeSepSet.add(last);
		
		for(Component c : removeSepSet){
			popup.remove(c);
		}
	}


	public static String replaceCharForHtml(String str){
		str = str.replaceAll("&", "&amp;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll("<", "&lt;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll(">", "&gt;");  //$NON-NLS-1$ //$NON-NLS-2$
		//str = str.replaceAll("'", "&apos;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll("\"", "&quot;"); //$NON-NLS-1$ //$NON-NLS-2$
		return str;
	}
}
