package com.ef_prime.rflow.xml;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

public class DefaultXMLWriter implements XMLWriter{
	
	private Writer out;
	
	private Stack<Element> openElements;
	
	public DefaultXMLWriter(Writer out){
		this.out = out;
		openElements = new Stack<Element>();
	}

	public void close() throws XMLException {
		try {
			out.close();
		} catch (IOException e) {
			throw new XMLException(e);
		}
	}

	public void writeAttribute(String localName, String value) throws XMLException {
		if(openElements.isEmpty())
			throw new XMLException();
		getCurrentOpenElement().addAttribute(new Attribute(localName, value));
	}

	public void writeCharacters(String text) throws XMLException {
		if(openElements.isEmpty())
			throw new XMLException();
		
		getCurrentOpenElement().isText = true;
		flushOpenElement(false);
		
		try {
			out.write(replaceChar(text));
		} catch (IOException e) {
			throw new XMLException(e);
		}
	}

	public void writeDTD(String dtd) throws XMLException {
		try {
			out.write(dtd);
			if(!dtd.endsWith("\n")) //$NON-NLS-1$
				newLine();
		} catch (IOException e) {
			throw new XMLException(e);
		}
	}

	public void writeEndDocument() throws XMLException {
		
	}

	public void writeEndElement() throws XMLException {
		if(openElements.isEmpty())
			throw new XMLException();
		try {
			if(!getCurrentOpenElement().doneWrite && !getCurrentOpenElement().isEmpty){
				//means not set empty but empty element 
				flushOpenElement(true);
			} else {
				if(getCurrentOpenElement().isEmpty)//flush if empty element remains
					flushOpenElement(true);
				
				Element element = getCurrentOpenElement();
				if(!element.isText)
					tabs();
				out.write("</"+element.name+">"); //$NON-NLS-1$ //$NON-NLS-2$
				newLine();
				
				popOpenElement();
			}
		} catch (IOException e) {
			throw new XMLException(e);
		}
	}
	
	public void writeEmptyElement(String localName) throws XMLException {
		flushOpenElement(false);
		pushOpenElement(new Element(localName, true));
	}

	public void writeStartDocument(String encoding, String version) throws XMLException {
		try {
			out.write("<?xml version=\""+version+"\" encoding=\""+encoding+"\"?>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			newLine();
		} catch (IOException e) {
			throw new XMLException(e);
		}
	}

	public void writeStartElement(String localName) throws XMLException {
		flushOpenElement(false);
		pushOpenElement(new Element(localName, false));
	}
	
	private void pushOpenElement(Element element){
		openElements.push(element);
	}
	
	private Element getCurrentOpenElement(){
		if(openElements.isEmpty())
			return null;
		return openElements.peek();
	}
	
	private void flushOpenElement(boolean empty) throws XMLException {
		Element openElement = getCurrentOpenElement();
		if(openElement != null && !openElement.doneWrite){
			try {
				tabs();
				out.write("<"+openElement.name); //$NON-NLS-1$
				for(Attribute attr : openElement.attrs){
					out.write(" "+attr.name+"=\""+replaceChar(attr.value)+"\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				if(openElement.isEmpty || empty)
					out.write("/>"); //$NON-NLS-1$
				else
					out.write(">"); //$NON-NLS-1$
				if(!openElement.isText)
					newLine();
				
				
				openElement.doneWrite = true;
				if(openElement.isEmpty || empty)
					openElements.pop();
				
			} catch (IOException e) {
				throw new XMLException(e);
			}
		}
	}
	
	private Element popOpenElement() throws XMLException {
		if(openElements.isEmpty())
			throw new XMLException();
		return openElements.pop();
	}
	
	private String replaceChar(String str){
		str = str.replaceAll("&", "&amp;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll("<", "&lt;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll(">", "&gt;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll("'", "&apos;");  //$NON-NLS-1$ //$NON-NLS-2$
		str = str.replaceAll("\"", "&quot;"); //$NON-NLS-1$ //$NON-NLS-2$
		return str;
	}
	
	private void tabs() throws IOException{
		for(int i = 0; i < openElements.size()-1; i++)
			out.write("\t"); //$NON-NLS-1$
	}
	
	private void newLine() throws IOException{
		out.write("\n"); //$NON-NLS-1$
	}

	private static class Element {
		String name;
		List<Attribute> attrs = new ArrayList<Attribute>();
		boolean doneWrite = false;
		
		/**
		 * element is set Empty
		 */
		boolean isEmpty;
		
		boolean isText = false;

		public Element(String name, boolean isEmpty) {
			this.name = name;
			this.isEmpty = isEmpty;
		}
		
		public void addAttribute(Attribute attr){
			attrs.add(attr);
		}
	}
	
	private static class Attribute {
		String name;
		String value;
		public Attribute(String name, String value) {
			this.name = name;
			this.value = value;
		}
		
	}
}
